// ARBAdvancedDlg.cpp : implementation file
//
/****************************************************************

Boston University data analysis (BUDA) package 

Description: Buda is a data analysis program for Fourier 
Transform Mass Spectrometry data.  

by Peter B. O'Connor

    Copyright (C) 2000  Peter B. O'Connor

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "stdafx.h"
#include "Buda.h"
#include "ARBAdvancedDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CARBAdvancedDlg dialog


CARBAdvancedDlg::CARBAdvancedDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CARBAdvancedDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CARBAdvancedDlg)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
}


void CARBAdvancedDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CARBAdvancedDlg)
	DDX_Control(pDX, IDC_SAMPLESCOMBO, m_NDPCombo);
	DDX_Control(pDX, IDC_FILTERCOMBO, m_FilterCombo);
	DDX_Control(pDX, IDC_DACRATECOMBO, m_DACCombo);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CARBAdvancedDlg, CDialog)
	//{{AFX_MSG_MAP(CARBAdvancedDlg)
	ON_CBN_SELCHANGE(IDC_SAMPLESCOMBO, OnSelchangeSamplescombo)
	ON_CBN_SELCHANGE(IDC_FILTERCOMBO, OnSelchangeFiltercombo)
	ON_CBN_SELCHANGE(IDC_DACRATECOMBO, OnSelchangeDacratecombo)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CARBAdvancedDlg message handlers

void CARBAdvancedDlg::OnSelchangeSamplescombo() 
{
	int samplenum=m_NDPCombo.GetCurSel() - 1;
	if(samplenum<0) {
		SamplesFixed = 1;
		Samples = SampleValues[0];
	}
	else {
		Samples = SampleValues[samplenum];
		SamplesFixed = 0;
	}
}

void CARBAdvancedDlg::OnSelchangeFiltercombo() 
{
	int filternum=m_FilterCombo.GetCurSel() - 1;
	if(filternum<0) {
		FilterFixed = 1;
		Filter = 5;
	}
	else {
		FilterFixed = 0;
		Filter = filternum;
	}
}

void CARBAdvancedDlg::OnSelchangeDacratecombo() 
{
	int dacnum=m_DACCombo.GetCurSel() - 1;
	if(dacnum<0) {
		DacFixed = 1;
		DACRate = DacRates[0];
	}
	else {
		DacFixed = 0;
		DACRate = DacRates[dacnum];
	}
}

BOOL CARBAdvancedDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	// setup combo boxes
	m_DACCombo.AddString("Auto-Calculate");
	m_DACCombo.AddString("16 MSPS");
	DacRates[0] = 16000000;
	if(DACRate == DacRates[0]) DacPtr = 0;
	m_DACCombo.AddString("8 MSPS");
	DacRates[1] = 8000000;
	if(DACRate == DacRates[1]) DacPtr = 1;
	m_DACCombo.AddString("4 MSPS");
	DacRates[2] = 4000000;
	if(DACRate == DacRates[2]) DacPtr = 2;
	m_DACCombo.AddString("2 MSPS");
	DacRates[3] = 2000000;
	if(DACRate == DacRates[3]) DacPtr = 3;
	m_DACCombo.AddString("1 MSPS");
	DacRates[4] = 1000000;
	if(DACRate == DacRates[4]) DacPtr = 4;
	m_DACCombo.AddString("500 KSPS");
	DacRates[5] = 500000;
	if(DACRate == DacRates[5]) DacPtr = 5;
	m_DACCombo.AddString("250 KSPS");
	DacRates[6] = 250000;
	if(DACRate == DacRates[6]) DacPtr = 6;
	m_DACCombo.AddString("125 KSPS");
	DacRates[7] = 125000;
	if(DACRate == DacRates[7]) DacPtr = 7;

	if(DacFixed) m_DACCombo.SetCurSel(0);
	else m_DACCombo.SetCurSel(DacPtr+1);

	// set up filter combo
	m_FilterCombo.AddString("Auto-Calculate");
	m_FilterCombo.AddString("100 KHz");
	m_FilterCombo.AddString("200 KHz");
	m_FilterCombo.AddString("500 KHz");
	m_FilterCombo.AddString("1 MHz");
	m_FilterCombo.AddString("> 10 MHz");
	if(FilterFixed) m_FilterCombo.SetCurSel(0);
	else m_FilterCombo.SetCurSel(Filter+1);

	// set up samples combo
	m_NDPCombo.AddString("Auto-Calculate");
	if(MaxSamples>512*1024) {
		m_NDPCombo.AddString("1024k");
		SampleValues[0] = 1024*1024;
		if(Samples == SampleValues[0]) SamplePtr = 0;
	}
	m_NDPCombo.AddString("512k");
	SampleValues[1] = 512*1024;
	if(Samples == SampleValues[1]) SamplePtr = 1;
	m_NDPCombo.AddString("256k");
	SampleValues[2] = 256*1024;
	if(Samples == SampleValues[2]) SamplePtr = 2;
	m_NDPCombo.AddString("128k");
	SampleValues[3] = 128*1024;
	if(Samples == SampleValues[3]) SamplePtr = 3;
	m_NDPCombo.AddString("64k");
	SampleValues[4] = 64*1024;
	if(Samples == SampleValues[4]) SamplePtr = 4;
	m_NDPCombo.AddString("32k");
	SampleValues[5] = 32*1024;
	if(Samples == SampleValues[5]) SamplePtr = 5;
	m_NDPCombo.AddString("16k");
	SampleValues[6] = 16*1024;
	if(Samples == SampleValues[6]) SamplePtr = 6;
	m_NDPCombo.AddString("8k");
	SampleValues[7] = 8*1024;
	if(Samples == SampleValues[7]) SamplePtr = 7;
	m_NDPCombo.AddString("4k");
	SampleValues[8] = 4*1024;
	if(Samples == SampleValues[8]) SamplePtr = 8;
	m_NDPCombo.AddString("2k");
	SampleValues[9] = 2*1024;
	if(Samples == SampleValues[9]) SamplePtr = 9;

	if(MaxSamples>512*1024) SamplePtr--;
	if(SamplesFixed) m_NDPCombo.SetCurSel(0);
	else m_NDPCombo.SetCurSel(SamplePtr+1);

	return TRUE;
}
