// ARBPlotBtn.cpp : implementation file
//
/****************************************************************

Boston University data analysis (BUDA) package 

Description: Buda is a data analysis program for Fourier 
Transform Mass Spectrometry data.  

by Peter B. O'Connor

    Copyright (C) 2000  Peter B. O'Connor

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "stdafx.h"
#include "Buda.h"
#include "ARBPlotBtn.h"
#include <math.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CARBPlotBtn

CARBPlotBtn::CARBPlotBtn()
{
}

CARBPlotBtn::~CARBPlotBtn()
{
}


BEGIN_MESSAGE_MAP(CARBPlotBtn, CButton)
	//{{AFX_MSG_MAP(CARBPlotBtn)
		ON_WM_DRAWITEM_REFLECT()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CARBPlotBtn message handlers

void CARBPlotBtn::DrawItem(LPDRAWITEMSTRUCT lpDrawItemStruct) 
{
	// this is the plotfunction for plotting the ARB frequency domain
	// waveform in the ARB Edit function.
	
	pDC = CDC::FromHandle(lpDrawItemStruct->hDC);

	pDC->GetWindow()->GetClientRect(&ViewSize);

	/* testing the drawing by drawing an 'x'
	pDC->MoveTo(ViewSize.left,ViewSize.top);
	pDC->LineTo(ViewSize.right,ViewSize.bottom);
	pDC->MoveTo(ViewSize.right,ViewSize.top);
	pDC->LineTo(ViewSize.left,ViewSize.bottom);
	*/

	// set background color to white
	pDC->SetBkMode(TRANSPARENT);
	pDC->SetBkColor(RGB(255,255,255));
	// not working don't know why

	// mass axis line position units in pixels
	AxisLineYPos = ViewSize.bottom-35;
	TicLabelsYPos = AxisLineYPos+5;
	CaptionYPos = AxisLineYPos+17;
	TicLength = 5;

	// intensity axis position variables
	IntAxisPos = ViewSize.left+25;

	// waveform position variables
	MaxYPos = ViewSize.top + 10;
	MinYPos = AxisLineYPos - 5;
	MaxXPos = ViewSize.right - 1;
	MinXPos = IntAxisPos+5;

	DrawMassAxis();
	DrawIntensityAxis();
	DrawARBEvents();
}

void CARBPlotBtn::DrawMassAxis()
{
	int AxisLineXstart = MinXPos;
	int AxisLineXend = MaxXPos;
	float Xstart = pARBPacket->WindowLowMass;
	float Xend = pARBPacket->WindowHighMass;

	long NumEvents = 0;
	//count the events
	for(NumEvents=0; NumEvents<MAX_ARB_EVENTS; NumEvents++) {
		if(pARBPacket->Event[NumEvents].Type == ARB_NONE) break;
	}

	// second draw the axis line 
	pDC->MoveTo(AxisLineXstart,AxisLineYPos);
	pDC->LineTo(AxisLineXend,AxisLineYPos);

	// place tics
	// there will be <=10 tics
	int ticpos[10];
	double ticmass[10];
	double massbase = Xend-Xstart;
	long i;
	for(i=0; i<10; i++) {
		ticpos[i] = 0;
		ticmass[i] = 0;
	}

	// there will be 5-10 tics  
	// ticsteps can be (order of magnitude of timebase) / 1, 2, 5 
	double a = log10(massbase);
	double b = floor(a);	// b = order of magnitude of timebase
	double ticstep = pow(10,b);
	double nt;	
	int numtics;
	nt = massbase / ticstep;

	if(nt>=5) numtics = (int)nt;
	if(nt>2 && nt<5) {
		ticstep=pow(10,b)/2;
		numtics = (int)(massbase/ticstep);
	}
	if(nt<=2) {
		ticstep = pow(10,b)/5;
		numtics = (int)(massbase/ticstep);
	}

	// find the position of the first tic.
	// if zero is in the range, I want zero to be one of the ticx
	double NumTixb4Zero = (double)Xstart/ticstep;
	if(NumTixb4Zero>0) {// GenPS starts after zero
		ticmass[0]=(floor(NumTixb4Zero)+1) * ticstep;
	}
	if (NumTixb4Zero == 0) ticmass[0] = 0;
	if (NumTixb4Zero < 0) { // GenPS starts before zero
		ticmass[0] = -1 * floor(-1*NumTixb4Zero) * ticstep;
	}

	// find the position of each tic relative to tictime[0]
	for(i=1; i<10; i++) {
		ticmass[i]=ticstep*i+ticmass[0];
		if(ticmass[i]>Xend) break;
	}
	numtics = i-1;

	// find tic x positions
	// remember that position 1000 = FirstEventStart
	// and 31768 = LastEventEnd;
				// create a linear eqn y = a*x + b
				// where y's are the plotspace #'s and 
				// x's are the timebase space #'s
	a = (AxisLineXend-AxisLineXstart)/(Xend - Xstart);
	b = AxisLineXstart - a*Xstart;
	for(i=0; i<numtics; i++) {
		ticpos[i] = int(a*ticmass[i] + b);
	}

	// now actually plot the tics
	for(i=0; i<numtics; i++) {
		pDC->MoveTo(ticpos[i],AxisLineYPos);
		pDC->LineTo(ticpos[i],AxisLineYPos+TicLength);
	}

	// lable tics
	CString str;
	CPoint pt;
	CSize sz;

	str.Format("m/z");
	pt.x = (ViewSize.right+ViewSize.left)/2;
	pt.y = CaptionYPos;
	Text(str,pt,0);

	for(i=0; i<numtics; i++) {
		str.Format("%1.1f",ticmass[i]);
		pt.x = ticpos[i];
		pt.y = TicLabelsYPos;
		Text(str,pt,0);
	}

	// end of draw axis
}

void CARBPlotBtn::DrawARBEvents()
{	// set drawing window and parms
	double ax,bx,ay,by;
	ax = (MaxXPos-MinXPos)/(pARBPacket->WindowHighMass - pARBPacket->WindowLowMass);
	bx = MinXPos - ax*pARBPacket->WindowLowMass;
	ay = double(MaxYPos-MinYPos)/100.0;
	by = MinYPos;

	// this function loops through all the individual events
	// in pARBPacket and draws them on the screen.  
	int i;

	// must restructure arb-events into up-down plot
	double APmass[2*MAX_ARB_EVENTS];
	float APint[2*MAX_ARB_EVENTS];
	for(i=0; i<2*MAX_ARB_EVENTS; i++) {
		APmass[i]=0;
		APint[i]=0;
	}
	int apctr=0;
	for(i=0; i<MAX_ARB_EVENTS; i++) {
		if(pARBPacket->Event[i].Type == ARB_NONE) break;
		if(pARBPacket->Event[i].Type == ARB_RANGE) {
			APmass[apctr] = pARBPacket->Event[i].MassA;
			APint[apctr] = pARBPacket->Event[i].Amplitude;
			apctr++;
			APmass[apctr] = pARBPacket->Event[i].MassB;
			APint[apctr] = 0;
			apctr++;
		}
		if(pARBPacket->Event[i].Type == ARB_NOTCH) {
			APmass[apctr] = pARBPacket->Event[i].MassA-pARBPacket->Event[i].MassB;
			APint[apctr] = pARBPacket->Event[i].Amplitude;
			apctr++;
			APmass[apctr] = pARBPacket->Event[i].MassA+pARBPacket->Event[i].MassB;
			APint[apctr] = -1; // to be set after sorting to the previous AP amplitude
			apctr++;
		}
	}
	// sort AP in order of mass and then set the -1 int values
	int j;
	double tempd;
	float tempf;
	for(i=1; i<apctr; i++) {
		for(j=i; j>0; j--) {
			if(APmass[j]<APmass[j-1]) {
				// swap AP[j] and AP[j-1];
				tempd = APmass[j];
				APmass[j]=APmass[j-1];
				APmass[j-1]=tempd;

				tempf = APint[j];
				APint[j]=APint[j-1];
				APint[j-1]=tempf;
			} else break;	// jump out of j when mass[j] >= mass[j-1]
		} // end loop j
	} // end loop i
	for(i=0; i<apctr; i++) {
		// set all unknown apint values to previous values
		if(APint[i]==-1) APint[i]=APint[i-2];
	}

	// now APmass and APint are set, plot them;
	pDC->MoveTo(MinXPos,MinYPos);
	int x,y;
	for(i=0; i<apctr; i++) {
		if(APmass[i]<pARBPacket->WindowLowMass) {
			y = int(ay*APint[i]+by);
			pDC->MoveTo(MinXPos,y);
			continue;
		}
		if(APmass[i]>pARBPacket->WindowHighMass) {
			y = int(ay*APint[i-1]+by);
			pDC->LineTo(MaxXPos,y);
			continue;
		}
		if(i>0) {
			x = int(ax*APmass[i]+bx);
			y = int(ay*APint[i-1]+by);
			pDC->LineTo(x,y);
		} else {
			x = int(ax*APmass[i]+bx);
			y = int(by);
			pDC->LineTo(x,y);
		}
		x = int(ax*APmass[i]+bx);
		y = int(ay*APint[i]+by);
		pDC->LineTo(x,y);
		if(i==apctr-1) {
			x = int(ax*APmass[i]+bx);
			y = int(by);
			pDC->LineTo(x,y);
			x = int(MaxXPos);
			y = int(by);
			pDC->LineTo(x,y);
		}
	}
}

void CARBPlotBtn::Text(CString str, CPoint pt, int just)
{
	CSize sz = pDC->GetTextExtent(str);

	if(just == 0) pt.x = pt.x-sz.cx/2;	// center-justified
	if(just == 1) pt.y = pt.y - sz.cy/2;	// left-justified

	pDC->TextOut(pt.x,pt.y,str);
}


void CARBPlotBtn::DrawIntensityAxis()
{
	// draw line
	pDC->MoveTo(IntAxisPos,MinYPos);
	pDC->LineTo(IntAxisPos,MaxYPos);

	// draw 11 tics
	int i;
	float y,dy = (float)(MinYPos-MaxYPos)/(float)10.0;
	for(i=0; i<11; i++) {
		y = float(MinYPos) - float(i)*dy;
		pDC->MoveTo(IntAxisPos,int(y));
		pDC->LineTo(IntAxisPos-TicLength,int(y));
	}

	// put on 0, 50, 100 labels
	pDC->MoveTo(ViewSize.left,MinYPos);
	CPoint pt;
	pt.y = MinYPos;
	pt.x = ViewSize.left+12;
	Text("0",pt,1);
	pt.y = int((float)MinYPos-(float)5*dy);
	pt.x -= 7;
	Text("50",pt,1);
	pt.y = MaxYPos;
	pt.x -= 5;
	Text("100",pt,1);
}
