// AddPeakDlg.cpp : implementation file
//
/*
AddPeakDlg.cpp is the code for the CAddPeakDlg class that controls
the dialog box that allows users to add peaks to their calibration 
calculation.
*/
/****************************************************************

Boston University data analysis (BUDA) package 

Description: Buda is a data analysis program for Fourier 
Transform Mass Spectrometry data.  

by Peter B. O'Connor

    Copyright (C) 2000  Peter B. O'Connor

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "BudaMaster.h"
#include "Buda.h"
#include "AddPeakDlg.h"
#include "AddToCalbrntFileDlg.h"
#include "Math.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


/////////////////////////////////////////////////////////////////////////////
// CAddPeakDlg dialog


CAddPeakDlg::CAddPeakDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CAddPeakDlg::IDD, pParent)
{
	CalibrntFileName = "Calibrnt.cbt";
	//{{AFX_DATA_INIT(CAddPeakDlg)
	m_ExactMass = 0.0;
	m_CalFileName = _T("Calibrnt.cbt");
	//}}AFX_DATA_INIT
}


void CAddPeakDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CAddPeakDlg)
	DDX_Control(pDX, IDC_OPEN_CALFILE, m_OpenCalFileBtn);
	DDX_Control(pDX, IDC_PEAK_LIST, m_PeakList);
	DDX_Control(pDX, IDC_CALFILE_LIST, m_CalFileList);
	DDX_Text(pDX, IDC_EXACT_MASS, m_ExactMass);
	DDV_MinMaxDouble(pDX, m_ExactMass, 0., 1000000.);
	DDX_Text(pDX, IDC_EDIT2, m_CalFileName);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CAddPeakDlg, CDialog)
	//{{AFX_MSG_MAP(CAddPeakDlg)
	ON_BN_CLICKED(IDC_APPLY, OnApply)
	ON_LBN_SELCHANGE(IDC_PEAK_LIST, OnSelchangePeakList)
	ON_BN_CLICKED(IDC_ADD_TO_CAL_FILE, OnAddToCalFile)
	ON_BN_CLICKED(IDC_CAL_SORT_MITABLE, OnCalSortMitable)
	ON_BN_CLICKED(IDC_OPEN_CALFILE, OnOpenCalfile)
	ON_BN_CLICKED(IDC_CANCEL, OnCancel)
	ON_LBN_SELCHANGE(IDC_CALFILE_LIST, OnSelchangeCalfileList)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CAddPeakDlg message handlers

void CAddPeakDlg::OnApply() 
{
	int CalFileindex;

	// set the flag to tell calibrate to use this point
	IsOK = TRUE; 

	// get the two current selections and set the return parms
	MIindex = m_PeakList.GetCurSel();
	CalFileindex = m_CalFileList.GetCurSel();
	m_ExactMass = Calibrant[CalFileindex].mass;

	OnOK(); // return
}

void CAddPeakDlg::OnCancel() 
{
	IsOK = FALSE;  // tell calibrate not to use this point
	OnOK();	
}

BOOL CAddPeakDlg::OnInitDialog()
{
	CString str, mstr, istr;
	long i;
	path = PDoc->pApp->ProgramDirectory + "\\Calibrnt.cbt";

	// force the DoDataExchange to initialize things
	UpdateData(FALSE);
	SetListBoxFont();

	// clear and initialize peaklist
	m_PeakList.ResetContent();
	m_PeakList.InitStorage(MAXNUMPEAKS,30);
	m_PeakList.SetTabStops(15);

	long NumPeaks = PS->Spec[PDoc->CurrentSpectrumFocus].NumPeaks;
	// first, find most intense peak in the MITable so we can calculate relative
	// intensities for the peaklist box
	double max = 0;
	for(i=0; i<NumPeaks; i++) {
		if(FITable[i].I>max) max = FITable[i].I;
	}

	// fill the peaklist ListBox
	double mass;
	for (i=0; i<NumPeaks; i++) {
		mass = PDoc->GetMITableMass(i);
		mstr.Format("%5.4f",mass);
		TRACE("GetMITableMass(%d) = %f\n",i,mass);
		istr.Format("%3.3f",100*FITable[i].I/max);
		while(mstr.GetLength()<10) mstr = " "+mstr;
		while(istr.GetLength()<7) istr = " "+istr;
		str = mstr + "\t" + istr;
		
		m_PeakList.AddString(str);
	}

	// clear and initialize CalFileList
	m_CalFileList.ResetContent();
	m_CalFileList.InitStorage(MAX_CALIBRANT_LIST,64);
	m_CalFileList.SetTabStops(10);

	// open and load current CalibrantFileName
	char* pathname = "";
	CString errorstr,line,massstr,descstr;
	CString filestr = "File Open Error";
	BOOL test;
	CFile f;

	test = TRUE;
	NumCalibrants = 0;

	//pathname = "c:\\Buda\\calibrnt.cbt"; //CalibrntFileName.GetBuffer(0);
	test = f.Open(path,  CFile::modeRead, NULL);
	if (!test) {
		errorstr.Format("Couldn't open Calibrant description file \n %s",pathname);
		MessageBox(errorstr,filestr);
		return TRUE;

	}

	CArchive ar(&f,CArchive::load,4096,NULL);
	ar.ReadString(line);
	ar.ReadString(line);
	int space;
	for (i=0; i<MAX_CALIBRANT_LIST; i++) {
		if (!ar.ReadString(line)) break;
		space = ParseCalibrantLine(&line);
		massstr = line.Left(space);
		str = line.Right(line.GetLength()-space);
		//descstr = line->Right(line->GetLength()-space+1);
		sscanf(massstr,"%lf",&(Calibrant[i].mass));
		Calibrant[i].str = massstr + "\t" + str;
	}
	ar.Close();
	f.Close();
	NumCalibrants = i;
	UpdateCalibrantList();

	// set the initial selection points for both listboxes
	m_PeakList.SetCurSel(0);
	m_CalFileList.SetCurSel(0);

	// set the initial value of m_ExactMass
	m_ExactMass = Calibrant[0].mass;
	UpdateData(FALSE);

	return TRUE;
}


int CAddPeakDlg::OpenCBTFile()
{

//*******************************************************************
//open a new calibrant file
	int retval;

	// use the CFileDialog to get the name of the file
	CFileDialog *dlg;
	dlg = new CFileDialog(TRUE,"*.cbt"," Bombesin.cbt",OFN_HIDEREADONLY | OFN_LONGNAMES,"*.cbt",NULL);
	dlg->m_ofn.lpstrFilter = "Calibration Files (*.cbt)\0*.cbt\0All Files (*.*)\0*.*\0";
	dlg->m_ofn.lpstrInitialDir = "C:\\Buda";

	retval = dlg->DoModal();
	path = dlg->GetPathName();
	m_CalFileName = dlg->GetFileName();
	UpdateData(FALSE);
	delete dlg;
	if (retval == IDCANCEL) return 0;


	return 1;

}

void CAddPeakDlg::OnSelchangePeakList() 
{
	double curmass, selmass;
	long i,curind;

	// get mass of this selection
	i=m_PeakList.GetCurSel();
	CString str;
	m_PeakList.GetText(i,str);
	selmass = PDoc->GetMITableMass(i);
	TRACE("GetMITableMass(%d) = %f\n",i,selmass);

	// find closest mass in m_CalFileList and select it
	curind=0;
	curmass=Calibrant[0].mass;
	for (i=0; i<NumCalibrants; i++) {
		if (fabs(curmass-selmass) > fabs(Calibrant[i].mass-selmass)) {
			curmass = Calibrant[i].mass;
			curind = i;
		}
	}

	m_CalFileList.SetCurSel(curind);
	m_ExactMass = curmass;
	UpdateData(FALSE);
}

void CAddPeakDlg::OnAddToCalFile() 
{
	CString pathname, pathname2;
	CString errorstr,line,str;
	CString filestr = "File Open Error";
	BOOL test;
	CFile fi,fo;

	// now get description from user via a dialog box
	CAddToCalbrntFileDlg dlg;
	dlg.m_ExactMass = m_ExactMass;
	dlg.m_Description = "";
	dlg.IsOK = TRUE;

	dlg.DoModal();

	if (dlg.IsOK == FALSE) return;
	m_ExactMass = dlg.m_ExactMass;
	Description = dlg.m_Description;


	// read in from fi and write out to fo
	test = TRUE;
	pathname = path;
	pathname2 = "c:\\Buda\\tempcal.tbl";
	test = fi.Open(pathname,  CFile::modeRead, NULL);
	if (!test) {
		errorstr.Format("Couldn't open Calibrant description file for appending\n %s",pathname);
		MessageBox(errorstr,filestr);
		return;
	}
	test = fo.Open(pathname2,   CFile::modeCreate | CFile::modeWrite, NULL);
	if (!test) {
		errorstr.Format("Couldn't open/create temp Calibrant description file \n %s",pathname2);
		MessageBox(errorstr,filestr);
		return;
	}

	CArchive ari(&fi,CArchive::load,4096,NULL);
	CArchive aro(&fo,CArchive::store,4096,NULL);
	int space;
	CString mstr;
	double f;

	// prepare the new string
	str.Format("%5.4f",m_ExactMass);
	while(str.GetLength()<10) str = " "+str;
	str = str + " " + Description;

	// read in each line from the infile and write it out
	// to the outfile until m_ExactMass< mass_in_current_line
	// then first write out this line.
	BOOL NotWritten = TRUE;
	ari.ReadString(line);
	aro.WriteString(line+"\r\n");
	ari.ReadString(line);
	aro.WriteString(line+"\r\n");
	while (ari.ReadString(line)) {
		space = ParseCalibrantLine(&line);
		mstr = line.Left(space);
		sscanf(mstr,"%lf",&f);
		if (m_ExactMass<f && NotWritten) {
			aro.WriteString(str+"\r\n");
			NotWritten = FALSE;
		}
		aro.WriteString(line+"\r\n");
	}

	//this means that mass should be at the end of the file
	if (NotWritten) {
		aro.WriteString(str+"\r\n");
	}


	// close files, close archives, copy outfile over infile
	ari.Close();
	aro.Close();
	fi.Close();
	fo.Close();
	fi.Remove(pathname);
	fo.Rename(pathname2, pathname);

	// reset the dialog box with the new values
	OnInitDialog();
}

void CAddPeakDlg::SetListBoxFont()
{
 /*  CFont* pFont = GetFont();
   LOGFONT lf;
   if (pFont != NULL)
	   pFont->GetObject(sizeof(LOGFONT), &lf);
   else
	   ::GetObject(GetStockObject(SYSTEM_FONT), sizeof(LOGFONT), &lf);

	lf.lfPitchAndFamily = FIXED_PITCH | FF_MODERN;
	lf.lfWidth = 7;
	lf.lfHeight = 16;
	lf.lfWeight = 10;
	//char fnt[32]="Courier\0";
	//lf.lfFaceName = "Courier                       \0";
		// switch to new font.
	m_font.DeleteObject();
	if (m_font.CreateFontIndirect(&lf))
	{
		//CWaitCursor wait;
		m_PeakList.SetFont(&m_font);
		m_CalFileList.SetFont(&m_font);
	}
	*/
}


void CAddPeakDlg::OnCalSortMitable() 
{
	PDoc->SetupOptionsInitPage = 2;
	PDoc->OnSetupOptions();
	OnInitDialog();
}

int CAddPeakDlg::ParseCalibrantLine(CString *line)
{
	int i;
	BOOL gotnum = FALSE;
	CString m, d;
	char* c;
	i=line->GetLength();
	c = line->GetBuffer(i);

	for (i=0; i<line->GetLength(); i++) {
		if (c[i]=='.') gotnum = TRUE;
		if (gotnum && c[i]==' ') break;
	} 
	// now i should be the position of the <space> in the line

	return i;
}

void CAddPeakDlg::OnOpenCalfile() 
{
	int retval;

	// use the CFileDialog to get the name of the file
	CFileDialog *dlg;
	dlg = new CFileDialog(TRUE,"*.cbt"," Bombesin.cbt",OFN_HIDEREADONLY | OFN_LONGNAMES,"*.cbt",NULL);
	dlg->m_ofn.lpstrFilter = "Calibration Files (*.cbt)\0*.cbt\0All Files (*.*)\0*.*\0";
	dlg->m_ofn.lpstrInitialDir = "C:\\Buda";

	retval = dlg->DoModal();
	m_CalFileName = dlg->GetFileName();
	UpdateData(FALSE);
	
	if (retval != IDCANCEL) {
		path = dlg->GetPathName();
		OnInitDialog();
	}
	delete dlg;
}


void CAddPeakDlg::UpdateCalibrantList()
{
	// first, sort calibrant data by increasing mass 
	int i,j;
	struct CalibrantData temp;

	for(i=1; i<NumCalibrants; i++) {
		if( Calibrant[i].mass < Calibrant[i-1].mass) {
			for(j=i; j>0 && Calibrant[j].mass < Calibrant[j-1].mass; j--) {
				temp = Calibrant[j];
				Calibrant[j] = Calibrant[j-1];
				Calibrant[j-1]=temp;
			}
		}			
	} // end sort

	for(i=0; i<NumCalibrants; i++) {
		m_CalFileList.AddString(Calibrant[i].str);
	}
}

void CAddPeakDlg::OnSelchangeCalfileList() 
{
	int i = m_CalFileList.GetCurSel();
	m_ExactMass = Calibrant[i].mass;
	UpdateData(FALSE);	
}
