// AddUserDlg.cpp : implementation file
//
/*

File Description: AddUserDlg.cpp is the implementation of the 
CAddUserDlg dialog class.  It accepts new user information from
the user, creates a new user entry in the UserID file, creates the
new user directory if it doesn't already exist, and creates the
new setup file - with instrument default parameters.

*/
/****************************************************************

Boston University data analysis (BUDA) package 

Description: Buda is a data analysis program for Fourier 
Transform Mass Spectrometry data.  

by Peter B. O'Connor

    Copyright (C) 2000  Peter B. O'Connor

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "stdafx.h"
#include "Buda.h"
#include "BudaMaster.h"
#include "AddUserDlg.h"
#include <direct.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CAddUserDlg dialog


CAddUserDlg::CAddUserDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CAddUserDlg::IDD, pParent)
{
	pApp = (CBudaApp*)AfxGetApp();
	//{{AFX_DATA_INIT(CAddUserDlg)
	m_UserDirectory = _T("");
	m_NewUserID = _T("");
	m_UserSetupFile = _T("");
	//}}AFX_DATA_INIT
}


void CAddUserDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CAddUserDlg)
	DDX_Text(pDX, IDC_ADD_USERDIRECTORY, m_UserDirectory);
	DDX_Text(pDX, IDC_ADD_USERID, m_NewUserID);
	DDV_MaxChars(pDX, m_NewUserID, 8);
	DDX_Text(pDX, IDC_ADD_USERSETUPFILE, m_UserSetupFile);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CAddUserDlg, CDialog)
	//{{AFX_MSG_MAP(CAddUserDlg)
	ON_EN_KILLFOCUS(IDC_ADD_USERID, OnKillfocusUserid)
	ON_EN_KILLFOCUS(IDC_ADD_USERDIRECTORY, OnKillfocusUserdirectory)
	ON_EN_KILLFOCUS(IDC_ADD_USERSETUPFILE, OnKillfocusUsersetupfile)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CAddUserDlg message handlers

void CAddUserDlg::OnOK() 
{
	UpdateData(TRUE);

	int create = CreateUserDirectory();
		//check if the file already exists
	int check = CheckFile();

	//if directory was created alright
	if (create) {
		if (check) {
			if (CreateUserSetupFile() ) {
				SaveUserIDFileEntry();
				CDialog::OnOK();
			}
		}
		
	}
	//if directory already was there
	else if (create == 2) {
		if (check) {
			
			if ( CreateUserSetupFile() ) {
				SaveUserIDFileEntry();
				CDialog::OnOK();
			}
		}
	}

	

}

BOOL CAddUserDlg::CheckFile()
{
	UpdateData(TRUE);

	//make sure this file does not exist for another user
	CFile	file;
	BOOL	testfile;
	CString	file_to_test = m_UserDirectory+"\\"+m_UserSetupFile;

	//try to open file to read
	testfile = file.Open(file_to_test, CFile::modeRead, NULL);

	//if it opened, this user can't use this file because it already exists
	if (testfile) {
		MessageBox("This file is already in use","File Error");
		m_UserSetupFile = OldUserSetupFile;
		UpdateData(FALSE);
		return 0;
	}

	return 1;

}

void CAddUserDlg::SaveUserIDFileEntry()
{
	UpdateData(TRUE);

	CString	UIDFile = *(pApp->UserIDFile);
	BOOL	testfile;
	CString file_error_msg;
	CString file_error_title = "File Open Error";
	CFile	file;
	
	testfile = file.Open(pApp->UserIDFile, CFile::modeWrite, NULL);
	//if didn't open, display error message box
	if (!testfile){
		file_error_msg.Format("Couldn't open User ID file: \n %s \n", *(pApp->UserIDFile));
		MessageBox(file_error_msg,file_error_title);
		return;
	}
	
	//go to end of file
	file.SeekToEnd();

	CArchive ar(&file,CArchive::store,4096,NULL);
	// write new user info 
	ar.WriteString(m_NewUserID+"\r\n");
	ar.WriteString(m_UserDirectory+"\r\n");
	ar.WriteString(m_UserDirectory+"\\"+m_UserSetupFile+"\r\n");
	ar.WriteString("\r\n");

	ar.Close();
	file.Close();
	return;

}//end CAddUserDlg::SaveUserIDFileEntry()

BOOL CAddUserDlg::CreateUserDirectory()
{
	UpdateData(TRUE);
	//check if directory exists
	CString	user_dir = m_UserDirectory;

	int dirtest = _chdir(user_dir);

	if(dirtest != 0) { // if dirtest != 0 then change directory failed
		// change dir failed, dir doesn't exist so create it
		if (m_UserDirectory.IsEmpty()) {
			MessageBox("Please enter a path","ERROR");
			return 0;
		}

		int mk_result = _mkdir(user_dir);
		if (mk_result != 0) {
			MessageBox("Directory Could not be created\n Possible invalid path","File Error");
			MessageBox("User was not added","NOTICE");
			create_flag = 0;
			m_UserDirectory = OldUserDirectory;
			m_NewUserID = OldUserID;
			m_UserSetupFile = OldUserSetupFile;
			UpdateData(FALSE);
			return 0;
		}
		return 1;

	}
	
	return 2;
}

BOOL CAddUserDlg::CreateUserSetupFile()
{
	//just create an empty file as of right now, if there is a filename

	UpdateData(TRUE);

	//there is a valid filename
	if (!m_UserSetupFile.IsEmpty()) {
		CFile   file;
		CString filename = m_UserDirectory+"\\"+m_UserSetupFile;
		BOOL	testfile;


		testfile = file.Open(filename, CFile::modeCreate, NULL);
		if (!testfile) {
			MessageBox("Couldn't Create User file","ERROR");
			return 0;
		}
		file.Close();
		return 1;
	}
	//filename field is not filled in
	else {
		MessageBox("Please enter a filename","ERROR");
		return 0;
	}

}

void CAddUserDlg::OnKillfocusUserdirectory() 
{
	// check to see if m_UserDirectory exists.  If not, ask user if
	// we need to create it.  If not, reset to OldUserDirectory


	UpdateData(TRUE);
	//check to see if directory has valid characters:
	CString	dir = m_UserDirectory;

	//loop for all characters in directory
	for (int i=0;i<dir.GetLength();i++) {
		//check if the character is illegal
		if ( (dir[i] == '/') | 
			 (dir[i] == '*') | 
			 (dir[i] == '?') | (dir[i] == '|') |
			 (dir[i] == '<') | (dir[i] == '>') | 
			 (dir[i] == '"') ) {

			MessageBox("Directory contains an illegal character","Illegal Directory");
			m_UserDirectory = OldUserDirectory;
			UpdateData(FALSE);
			return;
		}//endif
	}//endfor
	ChangePath = 1;

}//end CAddUserDlg::OnUpdateAddUserdirectory() 

void CAddUserDlg::OnKillfocusUsersetupfile() 
{
	// check to see if m_UserSetupFile exists.  If so, reset
	// to OldUserSetupFile after informing the user.
	UpdateData(TRUE);
	CString	filename = m_UserSetupFile;

	//make sure filename is valid length
	if (filename.GetLength() > 255) {
		MessageBox( "Filename must be less than 255 characters",
					"Illegal Filename");
		m_UserSetupFile = OldUserSetupFile;
		UpdateData(FALSE);
		return;
	}//end if

	
	//make sure filename has valid characters

	//loop for all characters in the file name
	for (int i=0;i<filename.GetLength();i++) {
		//check if the character is illegal
		if ( (filename[i] == '/') | (filename[i] == '\\') | 
			 (filename[i] == ':') | (filename[i] == '*') | 
			 (filename[i] == '?') | (filename[i] == '|') |
			 (filename[i] == '<') | (filename[i] == '>') | 
			 (filename[i] == '"') ) {

			CString name_error_str;
			name_error_str.Format("Filename contains an illegal character: \n %s",filename[i] );
			MessageBox(name_error_str,"Illegal Filename");
			m_UserSetupFile = OldUserSetupFile;
			UpdateData(FALSE);
			return;
		}
	}

}

BOOL CAddUserDlg::OnInitDialog()
{
	OldUserID = m_NewUserID;
	OldUserDirectory = m_UserDirectory;
	OldUserSetupFile = m_UserSetupFile;
	m_UserDirectory = "c:\\Buda\\";
	ChangePath = 0;
	UpdateData(FALSE);
	return CDialog::OnInitDialog();
}


void CAddUserDlg::OnKillfocusUserid() 
{
	//update variables from fields
	UpdateData(TRUE);

	//check if ID length is too long; if so, display error and return
	if ( m_NewUserID.GetLength() > 8 ) {
		MessageBox("User ID cannot exceed 8 characters",
					"Illegal ID Length");
		//restore old user ID in field and update
		m_NewUserID = OldUserID;
		UpdateData(FALSE);
		return;
	}

	//CString* UIDFile;
	//UIDFile = (pApp->UserIDFile) ;
	CString	file_error_msg;
	CString file_error_title = "File Open Error";
	BOOL    testfile;
	CFile	file;
	
	//open file for reading
	testfile = file.Open(pApp->UserIDFile, CFile::modeRead, NULL);

	//if didn't open, display error message box and return
	if (!testfile){
		file_error_msg.Format("Couldn't open User ID file: \n %s \n", *pApp->UserIDFile); //UIDFile);
		MessageBox(file_error_msg,file_error_title);
		return;
	}
	
	CString	line,check_userID;
	BOOL	testline = TRUE;
	CString user_error = "Duplicate User";
	CString user_error_str;

	//create archive 
	CArchive ar(&file,CArchive::load,4096,NULL);
	
	//loop till end of file or user match
	// UserIDFile entries should be 3 lines each separated by
	// a blank line
	while (testline) {
		//read line 
		testline = ar.ReadString(line); // read in UserID line
		//if a line was read...
		if (testline) {
			//set temp user ID and check if already exists in User File
			check_userID = line.Left(9);
			if (check_userID == m_NewUserID) {
				//display error
				user_error_str.Format("This user already exists: \n %s \n", check_userID);
				MessageBox(user_error_str,user_error);
				//restore old user ID in field and update
				m_NewUserID = OldUserID;
				UpdateData(FALSE);
				return;
			}//endif
		}//endif
		testline = ar.ReadString(line); // read in directory line
		testline = ar.ReadString(line); // read in setupfile line
		testline = ar.ReadString(line); // read in blank line
	}//end while

	ar.Close();
	file.Close();
/*
	if (!ChangePath) {
		m_UserDirectory = "c:\\Buda\\" + m_NewUserID;
	}
*/
	UpdateData(FALSE);

	return;	
}//void CAddUserDlg::OnKillfocusUserid() 



