// AutoSetup1.cpp : implementation file
//
/****************************************************************

Boston University data analysis (BUDA) package 

Description: Buda is a data analysis program for Fourier 
Transform Mass Spectrometry data.  

by Peter B. O'Connor

    Copyright (C) 2000  Peter B. O'Connor

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "stdafx.h"
#include "Buda.h"
#include "Instrument.h"
#include "AutoSetup1.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CAutoSetup1 dialog

CAutoSetup1::CAutoSetup1(CWnd* pParent /*=NULL*/)
	: CDialog(CAutoSetup1::IDD, pParent)
{
	//{{AFX_DATA_INIT(CAutoSetup1)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
}


void CAutoSetup1::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CAutoSetup1)
	DDX_Control(pDX, IDC_ESI, m_EsiBtn);
	DDX_Control(pDX, IDC_MALDI, m_MaldiBtn);
	DDX_Control(pDX, IDC_EXPERIMENT_COMBO, m_ExpCombo);
	DDX_Control(pDX, IDC_CATEGORY_COMBO, m_CatCombo);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CAutoSetup1, CDialog)
	//{{AFX_MSG_MAP(CAutoSetup1)
	ON_BN_CLICKED(IDC_ESI, OnEsi)
	ON_BN_CLICKED(IDC_MALDI, OnMaldi)
	ON_CBN_SELCHANGE(IDC_CATEGORY_COMBO, OnSelchangeCategoryCombo)
	ON_CBN_SELCHANGE(IDC_EXPERIMENT_COMBO, OnSelchangeExperimentCombo)
	ON_WM_CLOSE()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CAutoSetup1 message handlers

void CAutoSetup1::OnEsi() 
{
	m_vc = 1;
	m_EsiBtn.SetCheck(1);
	m_MaldiBtn.SetCheck(0);
	//sys = &(pInst->sys[2]);

	if(m_FileOpenFlag) {
		m_file.Close();
		m_FileOpenFlag = FALSE;
	}

	SetupCombos();		
}

void CAutoSetup1::OnMaldi() 
{
	m_vc = 0;
	m_EsiBtn.SetCheck(0);
	m_MaldiBtn.SetCheck(1);
	//sys = &(pInst->sys[1]);

	if(m_FileOpenFlag) {
		m_file.Close();
		m_FileOpenFlag = FALSE;
	}

	SetupCombos();		
}

void CAutoSetup1::OnSelchangeCategoryCombo() 
{
	int i;
	m_ExpCombo.ResetContent();
	m_CatPtr = m_CatCombo.GetCurSel();
	for(i=0; i<MAX_NUM_ENTRIES_PER_CATEGORY; i++) {
		if(m_EntryInfo[m_CatPtr][i].EntryStr == "") break;
		m_ExpCombo.AddString(m_EntryInfo[m_CatPtr][i].EntryStr);
	}	
	m_ExpCombo.SetCurSel(0);
	OnSelchangeExperimentCombo();
}

void CAutoSetup1::OnSelchangeExperimentCombo() 
{
	int i = m_ExpCombo.GetCurSel();
	m_EntryCodeString = "";
	m_EntryCodeString = m_EntryInfo[m_CatPtr][i].EntryCode;
	if(m_EntryCodeString == "" ) {
		MessageBox("Invalid entry\nPlease select a different setup","User Error");
		return;
	}
	if(!GetEntryInfo()) {
		CString str;
		str = "Cannot open entry: "+m_EntryCodeString + "\nin file: "+filename;
		MessageBox(str + "\nPlease select a different setup","File Error");
		return;
	}
}

BOOL CAutoSetup1::OnInitDialog() 
{
	CDialog::OnInitDialog();
	sys = &(pInst->sys);
	m_FileOpenFlag = FALSE;
	m_CatCombo.InitStorage(10,64);
	m_ExpCombo.InitStorage(10,64);
	m_SORI = 0;

	OnMaldi();	

	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

BOOL CAutoSetup1::SetupCombos()
{
	// read in the data file chosen by m_vc
	//		if m_vc = 0, file = "MALDI.CNF"
	//		if m_vc = 1, file = "ESI.cnf"

	m_CatCombo.ResetContent();
	m_ExpCombo.ResetContent();

	filename = "";

	if(m_vc == 0) filename = pInst->pDoc->pApp->ProgramDirectory + "\\MALDI.CNF";
	if(m_vc == 1) filename = pInst->pDoc->pApp->ProgramDirectory + "\\ESI.CNF";
	if(filename == "") {
		MessageBox("Invalid m_vc in CAutoSetup1::SetupCombos()","Error");
		return FALSE;
	}

	if(m_file.Open(filename,CFile::modeRead) == 0) {
		filename = "Couldn't open file: "+filename;
		MessageBox(filename,"File open error");
		return FALSE;
	}
	m_FileOpenFlag = TRUE;

	// ok, file is open so read it.
	CArchive ar(&m_file,CArchive::load,4096,NULL);
	// first, scan through until you find the string "[MENU]"
	CString str,outstr,tempstr;
	while(ar.ReadString(str)) {
		str = CompressString(str,0);
		if(str.IsEmpty()) continue;			// ignore empty lines
		if(str.Left(1) == "#") continue;	// ignore comment lines

		if(str.Left(6) == "[MENU]") break;
	}

	// now read all lines until we come to a "[" as the first character
	m_CatPtr = 0;
	int i,j;
	for(i=0; i<MAX_NUM_CATEGORIES; i++){
		m_CatInfo[i].CatPtr = 0;
		m_CatInfo[i].CatStr = "";
		m_EntryPtr[i] = 0;
		for(j=0; j<MAX_NUM_ENTRIES_PER_CATEGORY; j++) {
			m_EntryInfo[i][j].EntryCode = "";
			m_EntryInfo[i][j].EntryPtr = 0;
			m_EntryInfo[i][j].EntryStr = "";
		}
	}

	while(ar.ReadString(str)) {
		outstr = CompressString(str,0);
		if(outstr.IsEmpty()) continue;			// ignore empty lines
		if(outstr.Left(1) == "#") continue;	// ignore comment lines

		if(outstr.Left(1) == "[") break;

		if(outstr.Left(8) == "CATEGORY" || outstr.Left(8) == "CATAGORY" ) {
			tempstr = GetCategoryString(str);
			if(tempstr == "") continue;

			m_CatCombo.AddString(tempstr);
			m_CatInfo[m_CatPtr].CatPtr = m_CatPtr;
			m_CatInfo[m_CatPtr].CatStr = tempstr;
			m_EntryPtr[m_CatPtr] = 0;
			m_CatPtr++;
		}

		if(outstr.Left(5) == "ENTRY") {
			// if catptr==0, then no category is yet defined, so ignore entries
			if(m_CatPtr == 0) continue;
			tempstr = GetEntryString(str);
			if(tempstr == "") continue;

			m_EntryInfo[m_CatPtr-1][m_EntryPtr[m_CatPtr-1]].EntryStr = tempstr;
			tempstr = GetFileEntryString(str);
			if(tempstr == "") continue;

			m_EntryInfo[m_CatPtr-1][m_EntryPtr[m_CatPtr-1]].EntryCode = tempstr;
			m_EntryInfo[m_CatPtr-1][m_EntryPtr[m_CatPtr-1]].EntryPtr = 0;
			m_EntryPtr[m_CatPtr-1]++;
		}
	}
	ar.Close();

	// finally, do the addstrings for the entry combo
	m_CatCombo.SetCurSel(0);
	OnSelchangeCategoryCombo();
	m_ExpCombo.SetCurSel(0);
	OnSelchangeExperimentCombo();

	//done, close file and return TRUE
	return TRUE;
}

CString CAutoSetup1::CompressString(CString str, int mode)
{
	// removes leading whitespace (except a crlf)
	// if mode == 0, it then makes the string all upper case

	CString outstr = "";
	if(str.IsEmpty()) return str;

	outstr = str;
	outstr.TrimLeft();

	if (mode == 0) outstr.MakeUpper();

	return outstr;
}

CString CAutoSetup1::GetCategoryString(CString str)
{
	// find the = sign position, 
	// and take all characters right of the =
	int i = str.Find('=');
	if(i<0) return "";

	i++;
	str = str.Right(str.GetLength() - i);
	str.TrimLeft();
	return str;
}

CString CAutoSetup1::GetEntryString(CString str)
{
	// find the last "," position, 
	// and take all characters right of the comma
	int i = str.ReverseFind(',');
	if(i<0) return "";

	i++;
	str = str.Right(str.GetLength() - i);
	str.TrimLeft();
	return str;
}

CString CAutoSetup1::GetFileEntryString(CString str)
{
	// find the = sign position and find the last comma
	// take all characters between the two
	int i = str.Find('=');
	int j = str.Find(',',i);

	if(i<0) return "";
	if(j<0) return "";
	if(j<=i) return "";

	i++;
	str = str.Mid(i,j-i);
	str.TrimLeft();
	return str;
}

BOOL CAutoSetup1::GetEntryInfo()
{
	m_file.SeekToBegin();
	
	//TRACE("CAutoSetup1::GetEntryInfo()   m_EntryCodeString = :%s:\n",m_EntryCodeString);
	// ok, file is open so read it.
	CArchive ar(&m_file,CArchive::load,4096,NULL);
	// first, scan through until you find the EntryCodeString
	CString str,outstr;
	BOOL good = FALSE;
	while(TRUE) {
		good = ar.ReadString(str);
		if(!good) {
			MessageBox("Reached End of file without finding\nentry : "+m_EntryCodeString,"File Error");
			return FALSE;
		}
		str = CompressString(str,1);
		if(str.IsEmpty()) continue;			// ignore empty lines
		if(str.Left(1) == "#") continue;	// ignore comment lines

		if(str.Left(2+m_EntryCodeString.GetLength()) == "["+m_EntryCodeString+"]") break;
	}

	// found the entry, now read in the elements
	m_Description = "";
	m_SetupFileName = "";
	m_DetectWindow = 0;
	m_ExciteWindow = 0;
	m_LowMass = 100;
	m_HighMass = 10000;
	m_SORI = 0;
	int i;

	while(TRUE) {
		good = ar.ReadString(str);
		if(!good) {
			break;
		}
		str = CompressString(str,0);
		if(str.IsEmpty()) continue;			// ignore empty lines
		if(str.Left(1) == "#") continue;	// ignore comment lines
		if(str.Left(1) == "[") break;

		if(str.Left(11) == "DESCRIPTION") {
			i = str.Find('=');
			if(i<0) continue;
			i++;
			m_Description = str.Right(str.GetLength()-i);
			m_Description.TrimLeft();
		}

		if(str.Left(4) == "FILE") {
			i = str.Find('=');
			if(i<0) return FALSE;
			i++;
			m_SetupFileName = str.Right(str.GetLength()-i);
			m_SetupFileName.TrimLeft();
		}

		if(str.Left(12) == "DETECTWINDOW") {
			i = str.Find('=');
			if(i<0) continue;
			i++;
			str = str.Right(str.GetLength()-i);
			if(str == "FIXED") {
				m_DetectWindow = 1;
			} else {
				m_DetectWindow = 0;
			}
		}

		if(str.Left(12) == "EXCITEWINDOW") {
			i = str.Find('=');
			if(i<0) continue;
			i++;
			str = str.Right(str.GetLength()-i);
			if(str == "FIXED") {
				m_ExciteWindow = 1;
			} else {
				m_ExciteWindow = 0;
			}
		}

		if(str.Left(7) == "LOWMASS") {
			i = str.Find('=');
			if(i<0) continue;
			i++;
			str = str.Right(str.GetLength()-i);
			sscanf(str,"%lf",&m_LowMass);
		}

		if(str.Left(8) == "HIGHMASS") {
			i = str.Find('=');
			if(i<0) continue;
			i++;
			str = str.Right(str.GetLength()-i);
			sscanf(str,"%lf",&m_HighMass);
		}

		if(str.Left(4) == "SORI") {
			i = str.Find('=');
			if(i<0) continue;
			i++;
			str = str.Right(str.GetLength()-i);
			if(str == "NO") {
				m_SORI = 0;
			} else sscanf(str,"%d",&m_SORI);
		}
	}
	ar.Close();

	return TRUE;
}

void CAutoSetup1::OnClose() 
{
	m_file.Close();	
	m_FileOpenFlag = FALSE;
	CDialog::OnClose();
}
