// Buda.cpp : Defines the class behaviors for the application.
//
// File Description: Buda.cpp is the code for the CBudaApp class. 
/****************************************************************

Boston University data analysis (BUDA) package 

Description: Buda is a data analysis program for Fourier 
Transform Mass Spectrometry data.  

by Peter B. O'Connor

    Copyright (C) 2000  Peter B. O'Connor

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "BudaMaster.h"

#include "MainFrm.h"
#include "ChildFrm.h"
#include "BudaDoc.h"
#include "Buda.h"
#include "BudaView.h"
#include "MITableView.h"
#include "SetupView.h"
#include "TextView.h"
#include "ParmView.h"
#include "AddUserDlg.h"
#include "ChangeUserDlg.h"
#include "PeriodicTable.h"
#include "FFTWorkerThread1.h"
#include "SelectSourceDlg.h"
#include "ChangeInstruments.h"
#include "AutoSetup1.h"
#include "AutoSetup2.h"
#include "OscilloscopeDlg.h"
//#include "SwirlyView.h"
#include "ElementalCompDlg.h"

#define SWIRLY_VIEW_ACTIVE
#include <direct.h>
#include <stdio.h>
#include <stdlib.h>

#ifdef _DEBUG
	#define new DEBUG_NEW
	#undef THIS_FILE
	static char THIS_FILE[] = __FILE__;
	#define DEBUG_FFTCRASH
#endif

// Buda debug defines
#ifdef _DEBUG
	#define DEBUG_INSTRUMENT_THREAD
#endif

/////////////////////////////////////////////////////////////////////////////
// CBudaApp

BEGIN_MESSAGE_MAP(CBudaApp, CSingleInstanceApp)
	//{{AFX_MSG_MAP(CBudaApp)
	ON_COMMAND(ID_APP_ABOUT, OnAppAbout)
	ON_COMMAND(ID_MITable, OnMITable)
	ON_UPDATE_COMMAND_UI(ID_MITable, OnUpdateMITable)
	ON_COMMAND(ID_VIEW_AS_TEXT, OnViewAsText)
	ON_COMMAND(ID_ADD_USER, OnAddUser)
	ON_COMMAND(ID_CHANGE_USER, OnChangeUser)
	ON_COMMAND(ID_TOOLS_EXACTMASS, OnToolsExactmass)
	ON_COMMAND(ID_VIEW_PARAMETERS, OnViewParameters)
	ON_COMMAND(ID_SETUP_PULSESEQUENCE, OnSetupButton)
	ON_UPDATE_COMMAND_UI(ID_ADD_USER, OnUpdateAddUser)
	ON_UPDATE_COMMAND_UI(ID_CHANGE_USER, OnUpdateChangeUser)
	ON_UPDATE_COMMAND_UI(ID_TOOLS_EXACTMASS, OnUpdateToolsExactmass)
	ON_UPDATE_COMMAND_UI(ID_VIEW_DECON, OnUpdateViewDecon)
	ON_UPDATE_COMMAND_UI(ID_VIEW_PARAMETERS, OnUpdateViewParameters)
	ON_UPDATE_COMMAND_UI(ID_VIEW_STICKPLOT, OnUpdateViewStickplot)
	ON_COMMAND(ID_LOAD_SETUP, OnLoadSetup)
	ON_COMMAND(ID_SAVE_SETUP, OnSaveSetup)
	ON_UPDATE_COMMAND_UI(ID_SAVE_SETUP, OnUpdateSaveSetup)
	ON_COMMAND(ID_SETUP_WIZARD, OnSetupWizard)
	ON_COMMAND(ID_NEW_SETUP, OnNewSetup)
	ON_COMMAND(ID_SETUP_CHANGEINSTRUMENTS, OnSetupChangeinstruments)
	ON_UPDATE_COMMAND_UI(ID_SETUP_CHANGEINSTRUMENTS, OnUpdateSetupChangeinstruments)
	ON_COMMAND(ID_VIEW_PLOTMASSSPECTRUM, OnViewPlotmassspectrum)
	ON_UPDATE_COMMAND_UI(ID_VIEW_PLOTMASSSPECTRUM, OnUpdateViewPlotmassspectrum)
	ON_COMMAND(ID_VIEW_PLOTFREQUENCY, OnViewPlotfrequency)
	ON_UPDATE_COMMAND_UI(ID_VIEW_PLOTFREQUENCY, OnUpdateViewPlotfrequency)
	ON_COMMAND(ID_VIEW_PLOTTIME, OnViewPlottime)
	ON_UPDATE_COMMAND_UI(ID_VIEW_PLOTTIME, OnUpdateViewPlottime)
	ON_COMMAND(ID_STOPEXP_BTN, OnStopexpBtn)
	ON_COMMAND(ID_RUN_EXP_BUTTON, OnRunExpButton)
	ON_COMMAND(ID_TUNE_MODE, OnTuneMode)
	ON_UPDATE_COMMAND_UI(ID_TUNE_MODE, OnUpdateTuneMode)
	ON_UPDATE_COMMAND_UI(ID_STOPEXP_BTN, OnUpdateStopexpBtn)
	ON_UPDATE_COMMAND_UI(ID_RUN_EXP_BUTTON, OnUpdateRunExpButton)
	ON_COMMAND(ID_ABORT_BTN, OnAbortBtn)
	ON_UPDATE_COMMAND_UI(ID_ABORT_BTN, OnUpdateAbortBtn)
	//ON_COMMAND(ID_SWIRLYVIEW, OnSwirlyview)
	//ON_UPDATE_COMMAND_UI(ID_SWIRLYVIEW, OnUpdateSwirlyview)
	//ON_COMMAND(ID_VIEW_DECONVOLUTION, OnViewDeconvolution)
	//ON_UPDATE_COMMAND_UI(ID_VIEW_DECONVOLUTION, OnUpdateViewDeconvolution)
	ON_MESSAGE(WM_NEWDATAREADY, OnExperimentDone)
	ON_COMMAND(ID_SETUP_BUTTON, OnSetupButton)
	ON_COMMAND(ID_TOOLS_ELEMENTALCOMP, OnToolsElementalcomp)
	//}}AFX_MSG_MAP
	// Standard file based document commands
	ON_COMMAND(ID_FILE_NEW, CSingleInstanceApp::OnFileNew)
	ON_COMMAND(ID_FILE_OPEN, CSingleInstanceApp::OnFileOpen)
	// Standard print setup command
	ON_COMMAND(ID_FILE_PRINT_SETUP, CSingleInstanceApp::OnFilePrintSetup)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CBudaApp construction

CBudaApp::CBudaApp()
{
	// TODO: add construction code here,
	// Place all significant initialization in InitInstance
}

/////////////////////////////////////////////////////////////////////////////
// The one and only CBudaApp object

CBudaApp theApp;

/////////////////////////////////////////////////////////////////////////////
// CBudaApp initialization

BOOL CBudaApp::InitInstance()
{
	// check to see if an instance of BUDA is already running
    if ( !CheckSingleInstance( IDR_MAINFRAME ) ) { 
        // Return indicating that this instance 
        // of the app should be shut down 
        return FALSE; 
    }

	BudaRegistry = "BostonUniversity";

	// load the version string from the string table
	// should be something like "Buda v1.2"

	srand( (unsigned)time( NULL ) );
	char bv[64];
	LoadString(m_hInstance,IDS_VERSION_STRING,bv,64);
	BudaVersion.Format("%s",bv);

	BudaRegistrySection = BudaVersion;
	ProgramDirectory = "C:\\Buda\\";
	ConfigFileName = "Buda.cfg";

	DebugCMassCounter = 5;

	AfxInitRichEdit( );
	// Ok, normal setup
	// set up registry and strings
	InitializeParameters();

	AfxEnableControlContainer();

	// Standard initialization
	// If you are not using these features and wish to reduce the size
	//  of your final executable, you should remove from the following
	//  the specific initialization routines you do not need.

#ifdef _AFXDLL
	Enable3dControls();			// Call this when using MFC in a shared DLL
#else
	Enable3dControlsStatic();	// Call this when linking to MFC statically
#endif

	// Change the registry key under which our settings are stored.
	// You should modify this string to be something appropriate
	// such as the name of your company or organization.
	SetRegistryKey(BudaRegistry);
	LoadStdProfileSettings();  // Load standard INI file options (including MRU)

	// CInstrument class holds sys structure and all programs that
	//talk to or from the mass spectrometer.  
	ActiveSystemIndex = 0;
	Instrument.StartInstrumentThread();
	Instrument.DiagnosticsComplete = FALSE;

	// Splash Window and Diagnostics if decline button is pushed, 
	// kill the program here and now by returning FALSE
	//m_Splash.pInst = &(Instrument);
	m_Splash.m_VersionString = BudaVersion;
	m_Splash.DoModal();
	if(!m_Splash.Accept) return FALSE;

	// now create the one and only FFTThread
	pFFT = new CFFTWorkerThread;

	// Register the application's document templates.  Document templates
	//  serve as the connection between documents, frame windows and views.
	m_pBudaViewTemplate = new CMultiDocTemplate(
		IDR_IONSPETYPE,
		RUNTIME_CLASS(CBudaDoc),
		RUNTIME_CLASS(CChildFrame), // custom MDI child frame
		RUNTIME_CLASS(CBudaView));
	AddDocTemplate(m_pBudaViewTemplate);

	m_pMIViewTemplate = new CMultiDocTemplate(
		IDR_MITABLEFRAME,
		RUNTIME_CLASS(CBudaDoc),
		RUNTIME_CLASS(CChildFrame), // custom MDI child frame
		RUNTIME_CLASS(CMITableView));
	AddDocTemplate(m_pMIViewTemplate);

	m_pSetupTemplate = new CMultiDocTemplate(
		IDR_SETUPFRAME,
		RUNTIME_CLASS(CBudaDoc),
		RUNTIME_CLASS(CChildFrame), // custom MDI child frame
		RUNTIME_CLASS(CSetupView));
	AddDocTemplate(m_pSetupTemplate);

	m_pTextViewTemplate = new CMultiDocTemplate(
		IDR_TEXTVIEWFRAME,
		RUNTIME_CLASS(CBudaDoc),
		RUNTIME_CLASS(CChildFrame), // custom MDI child frame
		RUNTIME_CLASS(CTextView));
	AddDocTemplate(m_pTextViewTemplate);

	/*
	m_pSwirlyViewTemplate = new CMultiDocTemplate(
		IDR_TEXTVIEWFRAME,
		RUNTIME_CLASS(CBudaDoc),
		RUNTIME_CLASS(CChildFrame), // custom MDI child frame
		RUNTIME_CLASS(CSwirlyView));
	AddDocTemplate(m_pSwirlyViewTemplate);
	*/

	m_pDeconViewTemplate = new CMultiDocTemplate(
		IDR_TEXTVIEWFRAME,
		RUNTIME_CLASS(CBudaDoc),
		RUNTIME_CLASS(CChildFrame), // custom MDI child frame
		RUNTIME_CLASS(CBudaView));
	AddDocTemplate(m_pDeconViewTemplate);

	m_pParmViewTemplate = new CMultiDocTemplate(
		IDR_PARMVIEWFRAME,
		RUNTIME_CLASS(CBudaDoc),
		RUNTIME_CLASS(CChildFrame), // custom MDI child frame
		RUNTIME_CLASS(CParmView));
	AddDocTemplate(m_pParmViewTemplate);

	// create main MDI Frame window
	CMainFrame* pMainFrame = new CMainFrame;
	if (!pMainFrame->LoadFrame(IDR_MAINFRAME,WS_OVERLAPPEDWINDOW | FWS_ADDTOTITLE, NULL, NULL ))
		return FALSE;
	m_pMainWnd = pMainFrame;

	// Parse command line for standard shell commands, DDE, file open
	/*CCommandLineInfo cmdInfo;
	ParseCommandLine(cmdInfo);

	// Dispatch commands specified on the command line
	// startup by opening the calibration file
	CString DefaultStartupFile = "C:\\FTData\\Calibration.Spectrum";
	if(cmdInfo.m_strFileName == "") {
		if(FileExist(DefaultStartupFile)) {
			cmdInfo.m_strFileName=DefaultStartupFile;
			cmdInfo.m_nShellCommand = CCommandLineInfo::FileOpen;
		} else {
		cmdInfo.m_nShellCommand=CCommandLineInfo::FileNothing;
		}
	}
	if (!ProcessShellCommand(cmdInfo))
		return FALSE;
		*/

	// The main window has been initialized, so show and update it.
	pMainFrame->ShowWindow(m_nCmdShow);
	m_pMainWnd->UpdateWindow();

	//***
	// get the user ID file name string - or if it doesn't exist, create it
	// and write the id to the Registry - userID saved in Instrument.sys.UserID
	//GetLastUserID();

	// now verify that user is a real user and read in the user's 
	// parameters or set them = default.
	//VerifyValidUserID();

	SetEvent(pFFT->FFTThrdInfo.m_hStartThread);

	// select ionization mode MALDI or ESI
	//CSelectSourceDlg dlg;
	//dlg.m_UserID = Instrument.sys[0].UserID;
	//dlg.pSys = Instrument.sys;
	//dlg.DoModal();
	// copy userid to all there sys structures
	//Instrument.SetUserID(dlg.m_UserID);
	//ActiveSystemIndex = dlg.Source;
	//Instrument.SetActiveSystem(ActiveSystemIndex);
	Instrument.InitializePS();

	// now setup the scope
	Scope.Initialize();

	//pMainFrame->OnStartStatus();
	return TRUE;
}

/////////////////////////////////////////////////////////////////////////////
// CAboutDlg dialog used for App About

class CAboutDlg : public CDialog
{
public:
	CBudaApp* pApp;
	CAboutDlg();

// Dialog Data
	//{{AFX_DATA(CAboutDlg)
	enum { IDD = IDD_ABOUTBOX };
	CString	m_VersionString;
	//}}AFX_DATA

	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CAboutDlg)
	protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
	//}}AFX_VIRTUAL

// Implementation
protected:
	//{{AFX_MSG(CAboutDlg)
	virtual BOOL OnInitDialog();
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

CAboutDlg::CAboutDlg() : CDialog(CAboutDlg::IDD)
{
	//{{AFX_DATA_INIT(CAboutDlg)
	m_VersionString = _T("");
	//}}AFX_DATA_INIT
}

void CAboutDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CAboutDlg)
	DDX_Text(pDX, IDC_ABOUT_BUDA_VERSION, m_VersionString);
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CAboutDlg, CDialog)
	//{{AFX_MSG_MAP(CAboutDlg)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

// App command to run the dialog
void CBudaApp::OnAppAbout()
{
	CAboutDlg aboutDlg;
	aboutDlg.DoModal();
}

/////////////////////////////////////////////////////////////////////////////
// CBudaApp commands

CDocument* CBudaApp::OpenDocumentFile(LPCTSTR lpszFileName)
{
	CBudaDoc* pdoc = (CBudaDoc *)CSingleInstanceApp::OpenDocumentFile(lpszFileName);

	return pdoc;
}

/*void CBudaApp::OnUpdateMITable(CCmdUI* pCmdUI) 
{
	CBudaDoc* pdoc;

	pdoc = (CBudaDoc*) GetDocument();

	if (pdoc != NULL && pdoc->IsFFTed) 
		pCmdUI->Enable(TRUE);
	else pCmdUI->Enable(FALSE);	
}
*/

void CBudaApp::OnMITable() 
{
	// test to see if get first document position works, then 
	// test through the views to find one that is selected then
	// you know which document to use -- if getfirstdocument works right

	CDocument* curdoc;
	CFrameWnd* frame;
	CWnd* activewnd;

	//int i;
	activewnd=m_pMainWnd->GetFocus();
	frame = activewnd->GetParentFrame();
	curdoc = frame->GetActiveDocument();

	CFrameWnd *cframe = m_pMIViewTemplate->CreateNewFrame(curdoc,NULL);
	m_pMIViewTemplate->InitialUpdateFrame( cframe, curdoc, TRUE );	
}

void CBudaApp::OnUpdateMITable(CCmdUI* pCmdUI) 
{
	CBudaDoc* curdoc;
	CFrameWnd* frame;
	CWnd* activewnd;

	activewnd=m_pMainWnd->GetFocus();
	if (!activewnd) return;
	frame = activewnd->GetParentFrame();
	if (!frame) return;
	curdoc = (CBudaDoc*) frame->GetActiveDocument();

	if (curdoc != NULL && curdoc->IsFFTed()) 
		pCmdUI->Enable(TRUE);
	else pCmdUI->Enable(FALSE);	
}

BOOL CBudaApp::OnIdle(LONG lCount) 
{
	// check instrument status, if running an experiment, 
	// then get the instrument to check if the experiment is done
	//if(Instrument.ThreadInfo.ThreadStatus == THREAD_RUNNING_EXPERIMENT) {
	//	Instrument.ThreadInfo.InstrumentCommands = UPDATE_STATUS;
	//	SetEvent(Instrument.ThreadInfo.m_hProcessCommand);
	//}
	if(Scope.IsRunning()) {
		Scope.Update();
	}
	return CSingleInstanceApp::OnIdle(lCount);
}

inline void CBudaApp::DumpToDiagFile(CString str)
{
#ifdef DEBUG_DUMP_DIAGS_TO_FILE
	str=str+"\r\n";
	diagfile.Write(str,str.GetLength());
	TRACE(str);
#endif
}

int CBudaApp::ExitInstance() 
{
	// kill off the FFTThread
	if(pFFT != 0) {
		SetEvent(pFFT->FFTThrdInfo.m_hKillThread);
		SetEvent(pFFT->FFTThrdInfo.m_hStartFFT);
		WaitForSingleObject(pFFT->FFTThrdInfo.m_hThreadDead, INFINITE);
		delete pFFT;
	}

	// kill of the InstrumentThread
	Instrument.KillInstrumentThread();

	StoreLastUserID();
	return CSingleInstanceApp::ExitInstance();
}

void CBudaApp::OnViewAsText() 
{ 
	// first get the document associated with the current active view
	// then create a new frame and view associated with it using the
	// m_pTextViewTemplate document template;
	CDocument* curdoc;
	//CBudaDoc* pDoc;
	CFrameWnd* frame;
	CWnd* activewnd;

	activewnd=m_pMainWnd->GetFocus();
	m_pCurrentActiveView = (CBudaView *)activewnd;
	frame = activewnd->GetParentFrame();
	if(frame == NULL) return;
	curdoc = frame->GetActiveDocument();
	if(curdoc == NULL) return;
	
	// no "view-as-text" for setup views
	if (m_pCurrentActiveView->IsKindOf(RUNTIME_CLASS( CSetupView ) ) ) {
		m_pCurrentActiveView->MessageBox("Cannot view Setup as text","Error");
		return;
	}

	// no "view-as-text" for MI tables - they already are text
	if (m_pCurrentActiveView->IsKindOf(RUNTIME_CLASS( CMITableView ) ) ) {
		m_pCurrentActiveView->MessageBox("Operator Error : MITable is already a text view","Operator Error : Replace Operator");
		return;
	}

	// before creating the frame and view and all, check the number of 
	// data points that are to be displayed.  If it is > MAX_PTS_IN_TEXTVIEW,
	// ask before displaying them, and cancel if the user says no.
	CBudaView* vw = (CBudaView*)m_pCurrentActiveView;

	if (vw->PlotType == MASSSPEC) {
		double minx, maxx;
		minx=vw->zoomstack[vw->zoompointer].minx;
		maxx=vw->zoomstack[vw->zoompointer].maxx;

		long minxpt, maxxpt;
		minxpt = vw->pDoc->MassToPt(minx);
		maxxpt = vw->pDoc->MassToPt(maxx);

		CString str;
		if(abs(maxxpt-minxpt)>MAX_PTS_IN_TEXTVIEW) {
			str.Format("Number of data points to Display is : %d\r\n Do you really wish to display all these?",abs(maxxpt-minxpt));
			if(vw->MessageBox(str,"Too Many Points?",MB_YESNO | MB_ICONWARNING) == IDNO) return;
		}
	}
	
	CFrameWnd *cframe = m_pTextViewTemplate->CreateNewFrame(curdoc,NULL);
	m_pTextViewTemplate->InitialUpdateFrame( cframe, curdoc, TRUE );	
}

void CBudaApp::OnAddUser() 
{
/*	DumpToDiagFile("entering CBudaApp::OnAddUser()");
	CAddUserDlg dlg;
	//char TempUserID[8];
	//int i;
	//for (i=0; i<8; i++) TempUserID[i] = ;
	
	dlg.m_NewUserID = *(Instrument.sys->UserID);
	dlg.m_UserDirectory = *SaveDirectory;
	dlg.m_UserSetupFile = *UserSetupFile;

	dlg.DoModal();

	//*(Instrument.sys->UserID)=dlg.m_NewUserID;
	//SaveDirectory = dlg.m_UserDirectory;
	
	delete (Instrument.sys->UserID);
	Instrument.sys->UserID = new CString(dlg.m_NewUserID);
	delete (Instrument.sys->UserDir);
	Instrument.sys->UserDir = new CString(dlg.m_UserDirectory);
	delete (Instrument.sys->UserSetupFile);
	Instrument.sys->UserSetupFile = new CString(dlg.m_UserSetupFile);

	DumpToDiagFile("leaving CBudaApp::OnAddUser()");	
*/
}

void CBudaApp::OnChangeUser() 
{
/*	DumpToDiagFile("entering CBudaApp::OnChangeUser()");
	
	CChangeUserDlg dlg;

	dlg.DoModal();

	//set global variables
	delete (Instrument.sys->UserID);
    Instrument.sys->UserID = new CString(dlg.UserID);
	delete (Instrument.sys->UserDir);
    Instrument.sys->UserDir = new CString(dlg.UserDir);
	delete (Instrument.sys->UserSetupFile);
    Instrument.sys->UserSetupFile = new CString(dlg.UserFilename);

	//MessageBox("This User's Specifications have been implemented",
	//			"User Notice", MB_OK + MB_ICONINFORMATION);

	DumpToDiagFile("Leaving CBudaApp::OnChangeUser()");
*/
}

void CBudaApp::GetLastUserID()
{
	// first, get the User ID file from the Windows Registry or if not
	// there, assume default.  If file does not exist, create it with the
	// default user
	//CString DefaultUserIDFile = "C:\\Buda\\BudaUserIDs";
	CString LastUser, DefaultUser = "PeterOConnor";
	//CFile file;


	// get the last user from the registry.  If there is no last user, or if the
	// last user does not exist in the UserIDFile, then set sys.UserID = DefaultUser
	//CString DefaultProgramDirectory = "C:\\Buda\\";
	CString DefaultSetupFile = "C:\\Buda\\Setup";
	LastUser = GetProfileString(BudaRegistrySection,"Last UserID",DefaultUser);
	if (LastUser == "") { // can this occur?
		LastUser = DefaultUser;
		WriteProfileString(BudaRegistrySection,"Last UserID",DefaultUser);
	}
	Instrument.sys.UserID=LastUser;

} // end GetLastUser()

void CBudaApp::VerifyValidUserID()
{
	// check the UserIDFile for Instrument.sys.UserID.  If it exists, 
	// set UserSetupFile and SaveDirectory from the file.  If it doesn't
	// exist, set to defaults:
	//		Instrument.sys.UserID = "Buda"
	//		SaveDirectory = "C:\\Buda\\FTData"
	//		UserSetupFile =	"C:\\Buda\\Setup"

	// for now, don't do anything
	return;

	/*CBudaView* vw = (CBudaView*)m_pCurrentActiveView;

	CString	UIDFile = UserIDFile;
	CString	file_error_msg;
	CString file_error_title = "File Open Error";
	BOOL    testfile;
	CFile	file;
	
	//open file for reading
	testfile = file.Open(UIDFile, CFile::modeRead, NULL);

	//if didn't open, display error message box and return
	if (!testfile){
		file_error_msg.Format("Couldn't open User ID file for verify user: \n %s \n", UIDFile);
		vw->MessageBox(file_error_msg,file_error_title);
		return;
	}
	
	
	CString	UserID, path, filename, check_userID;
	BOOL	testline = TRUE;
	CString user_error = "Duplicate User";
	CString user_error_str;

	//create archive 
	CArchive ar(&file,CArchive::load,4096,NULL);
	
	int found_user = 0;

	//loop till end of file 
	// UserIDFile entries should be 3 lines each separated by
	// a blank line
	while (testline) {
		//read line 
		testline = ar.ReadString(UserID); // read in UserID line
		//if a line (UserID) was read...
		if (testline) {
			//check to see if the user matches the last user
			if (*(Instrument.sys->UserID) == UserID) {
				found_user = 1;
				ar.ReadString(path);
				ar.ReadString(filename);
				break;
			}
		}//endif

		testline = ar.ReadString(UserID); // read in directory line
		testline = ar.ReadString(UserID); // read in setupfile line
		testline = ar.ReadString(UserID); // read in blank line

	}//end while

	ar.Close();
	file.Close();

	if (found_user) {
		//set user's parameters
		Instrument.sys->UserID = UserID;
		Instrument.sys->UserDir = path;
		Instrument.sys->UserSetupFile = filename;
	}
	//set default parameters
	else {

		//vw->MessageBox("The LastUser has NOT been found--defaults are set","HEY NOW!");

	}
	*/

}

void CBudaApp::OnToolsExactmass() 
{
/*	CPeriodicTable PT;	

	PT.m_ElementalComposition = ElementalComposition;

	PT.DoModal();

	ElementalComposition = PT.m_ElementalComposition;
*/
}

BOOL CBudaApp::FileExist(LPCTSTR str)
{
	CFile f;
	CFileException e;

	if( !f.Open( str, CFile::modeRead | CFile::shareDenyWrite, &e ) ) {
		#ifdef _DEBUG
		afxDump << "File " << str << " could not be opened " << e.m_cause << "\n";
		#endif
		//MessageBox("File could not be opened\n"+e.m_cause,"File Error",MB_OK);
		return FALSE;
    }
	f.Close();
	return TRUE;
}


void CBudaApp::OnSetupPulsesequence() 
{
	Instrument.OnSetupPulsesequence();
}

void CBudaApp::OnViewParameters() 
{
// first get the document associated with the current active view
// then create a new frame and view associated with it using the
// m_pTextViewTemplate document template;
	CDocument* curdoc;
	CFrameWnd* frame;
	CWnd* activewnd;

	activewnd=m_pMainWnd->GetFocus();
	m_pCurrentActiveView = (CBudaView *)activewnd;
	frame = activewnd->GetParentFrame();
	curdoc = frame->GetActiveDocument();
	
	CFrameWnd *cframe = m_pTextViewTemplate->CreateNewFrame(curdoc,NULL);
	m_pTextViewTemplate->InitialUpdateFrame( cframe, curdoc, TRUE );	
}


void CBudaApp::OnSetupButton() 
{
	CDocument* curdoc;
	CBudaDoc* pDoc;
	CFrameWnd* frame;
	CWnd* activewnd;

	//int i;
	activewnd=m_pMainWnd->GetFocus();
	frame = activewnd->GetParentFrame();
	curdoc = frame->GetActiveDocument();
	pDoc = (CBudaDoc*) curdoc;

	if(curdoc == NULL) {
		// no document available, so make a new one
		OnNewSetup();
		return;
	}

	if(pDoc->SetupViewOpen) return;
	pDoc->SetupViewOpen = TRUE;
	CFrameWnd *cframe = m_pSetupTemplate->CreateNewFrame(curdoc,NULL);
	m_pSetupTemplate->InitialUpdateFrame( cframe, curdoc, TRUE );	
}

void CBudaApp::OnUpdateAddUser(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(FALSE);	
}

void CBudaApp::OnUpdateChangeUser(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(FALSE);	
	
}

/*void CBudaApp::OnUpdateFileSave(CCmdUI* pCmdUI) 
{
	// if there is no document available, disable this button		
	CBudaDoc* curdoc;
	CFrameWnd* frame;
	CWnd* activewnd;

	activewnd=m_pMainWnd->GetFocus();
	if (!activewnd) {
		pCmdUI->Enable(FALSE);
		return;
	}
	frame = activewnd->GetParentFrame();
	if (!frame){
		pCmdUI->Enable(FALSE);
		return;
	}
	curdoc = (CBudaDoc*) frame->GetActiveDocument();

	if (curdoc != NULL) 
		pCmdUI->Enable(TRUE);
	else pCmdUI->Enable(FALSE);	
}

void CBudaApp::OnUpdateFileSaveAs(CCmdUI* pCmdUI) 
{
	// if there is no document available, disable this button		
	CBudaDoc* curdoc;
	CFrameWnd* frame;
	CWnd* activewnd;

	activewnd=m_pMainWnd->GetFocus();
	if (!activewnd) {
		pCmdUI->Enable(FALSE);
		return;
	}
	frame = activewnd->GetParentFrame();
	if (!frame){
		pCmdUI->Enable(FALSE);
		return;
	}
	curdoc = (CBudaDoc*) frame->GetActiveDocument();

	if (curdoc != NULL) 
		pCmdUI->Enable(TRUE);
	else pCmdUI->Enable(FALSE);	
}*/

void CBudaApp::OnUpdateToolsExactmass(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(FALSE);	
	
}

void CBudaApp::OnUpdateViewDecon(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(FALSE);	
	
}

void CBudaApp::OnUpdateViewParameters(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(FALSE);	
	
}

void CBudaApp::OnUpdateViewStickplot(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(FALSE);	
	
}

void CBudaApp::OnNewSetup() 
{
	// create a new CBudaDoc.  Fill it with all default info.
	// fill it with a GenPS that has ONLY a broadband excite and detect
	// create a zeroed FreqArray and set Transient is loaded, etc to FALSE

	CBudaDoc* pDoc;
	CChildFrame* pFrame;

	// create an empty document
	pDoc = (CBudaDoc*)m_pSetupTemplate->CreateNewDocument();
	pDoc->PS.TransientIsStored = 0;
	pDoc->PS.FrequencyIsStored = 0;
	pDoc->PS.MITableIsStored = 0;
	pDoc->SetTitle("New file");
	pDoc->OnNewDocument();

	// initialize the setupview
	pFrame = (CChildFrame*)m_pSetupTemplate->CreateNewFrame(pDoc,NULL);
	m_pSetupTemplate->InitialUpdateFrame(pFrame,pDoc,TRUE);
}

CBudaDoc* CBudaApp::OnLoadSetup(CString FileName)
{
	// create a new CBudaDoc.  Fill it with all default info.
	// create a zeroed FreqArray and set Transient is loaded, etc to FALSE
	CBudaDoc* pDoc;
	struct PS* pPS;

	pDoc = (CBudaDoc*)m_pBudaViewTemplate->CreateNewDocument();
	pPS = &(pDoc->PS);
	CString title = "New file (" + FileName + ")";
	pDoc->SetTitle(title);

	// load in a GenPS array from disk
	if(!pDoc->ReadSetupFile(FileName)) {
		// couldn't read the file.  pdoc->ReadSetupFile has already
		// notified the user, so just kill the document, and return NULL
		pDoc->OnCloseDocument();
		return NULL;
	}

	// setup new ps parameters from sys
	pPS->TransientIsStored = 0;
	pPS->FrequencyIsStored = 0;
	pPS->MITableIsStored = 0;
	pPS->A = pDoc->sys->A;
	pPS->B = pDoc->sys->B;
	pPS->HighMassShift = pDoc->sys->HighMassShift;
	pPS->TrapShift = pDoc->sys->TrapShift;
	pPS->CalibrationVersion = pDoc->sys->CalibrationVersion;
	pPS->PeakMatching = FALSE;
	pPS->ExperimentMemo[0] = '\0';
	pPS->ExperimentNotes[0] = '\0';
	pPS->DataFileName[0] = '\0';
	pPS->TimeAcquired[0] = '\0';
	pPS->TimeProcessed[0] = '\0';
	int i;
	for(i=0; i<8 && i<pDoc->sys->UserID.GetLength() ; i++) pPS->UserID[i] = pDoc->sys->UserID.GetAt(i);
	for(i=0; i<MAX_DACS; i++) {
		pPS->dac[i] = pDoc->sys->dac[i];
	}
	if(pPS->ADCGain < 1) pPS->ADCGain = 1;
	if(pPS->ADCGain > 5) pPS->ADCGain = 5;

	return pDoc;
}

void CBudaApp::OnLoadSetup() 
{
	CString str = "";
	CBudaDoc* pDoc = OnLoadSetup(str);
	if(pDoc == NULL) return;

	// initialize the setupview
	CChildFrame* pFrame = (CChildFrame*)m_pSetupTemplate->CreateNewFrame(pDoc,NULL);
	m_pSetupTemplate->InitialUpdateFrame(pFrame,pDoc,TRUE);
}


void CBudaApp::OnSaveSetup() 
{
	// save the GenPS and PS into a setup file.
	
	CBudaDoc* pDoc;
	CFrameWnd* frame;
	CWnd* activewnd;

	activewnd=m_pMainWnd->GetFocus();
	if (!activewnd) return;
	frame = activewnd->GetParentFrame();
	if (!frame)return;
	pDoc = (CBudaDoc*) frame->GetActiveDocument();

	pDoc->SaveSetupFile();
}

void CBudaApp::OnUpdateSaveSetup(CCmdUI* pCmdUI) 
{
	// if there is no document available, disable this button		
	CBudaDoc* curdoc;
	CFrameWnd* frame;
	CWnd* activewnd;

	activewnd=m_pMainWnd->GetFocus();
	if (!activewnd) {
		pCmdUI->Enable(FALSE);
		return;
	}
	frame = activewnd->GetParentFrame();
	if (!frame){
		pCmdUI->Enable(FALSE);
		return;
	}
	curdoc = (CBudaDoc*) frame->GetActiveDocument();

	if (curdoc != NULL) 
		pCmdUI->Enable(TRUE);
	else pCmdUI->Enable(FALSE);	
}

void CBudaApp::OnSetupWizard() 
{
	CBudaDoc* pDoc;
	struct PS* pPS;

	// call the Setup Wizard to get the name of the loadsetup file
	// then call OnLoadSetup()
	CAutoSetup1 dlg1;
	CAutoSetup2 dlg2;

	dlg1.pInst = &(Instrument);
	if(dlg1.DoModal() != IDOK) return;

	pDoc = OnLoadSetup(dlg1.m_SetupFileName);
	if(pDoc == NULL) return;
	pPS = &(pDoc->PS);

	dlg2.m_Mass0 = pPS->MonitorMass;
	dlg2.m_Mass1 = pPS->MonitorMass;
	dlg2.m_SORI = dlg1.m_SORI;
	dlg2.pInst = &(Instrument);
	if(dlg2.DoModal() != IDOK) return;

	// otherwise, both dlg's returned ok, so get the filename, load the setup file,
	// and modify the parms as required by dlg2.
	double midfreq;

	// set monitormass and high/low mass
	pPS->MonitorMass = dlg2.m_Mass0;	//Mass of interest

	if(dlg1.m_DetectWindow > 0) {
		pPS->DetectLowMass = dlg2.m_Mass0 - double(dlg1.m_DetectWindow/2);
		pPS->DetectHighMass = pPS->DetectLowMass + double(dlg1.m_DetectWindow);
	} else if(dlg1.m_DetectWindow < 0) {
		midfreq = pDoc->MassToFreq(dlg2.m_Mass0);
		double lowfreq = midfreq - double(-dlg1.m_DetectWindow/2);
		double hifreq = lowfreq + double(-dlg1.m_DetectWindow);
		pPS->DetectLowMass  = pDoc->FreqToMass(hifreq);
		pPS->DetectHighMass = pDoc->FreqToMass(lowfreq);
		if(pPS->DetectMode == MixerMode) {
			pPS->DetectLowMass = dlg2.m_Mass0 - 3.0;
			pPS->DetectHighMass = dlg2.m_Mass0 + 3.0;
		}
	}

	// set excitewindow
	int i=0;
	long packet = 0;
	if(dlg1.m_ExciteWindow != 0) {
		for(i=0; i<pPS->NumEvents; i++) {
			if(pPS->GenPS[i].Channel != SYN_TRIGGER) continue;
			if(i == pPS->DetectEvent) break;
			packet++;
		}
		pPS->cfp[packet].StartMass = pPS->DetectLowMass;
		pPS->cfp[packet].EndMass = pPS->DetectHighMass;
	}

	// for ion isolation/ SORI experiments, adjust the isolation
	// notch and the SORI rf pulse for the synthesizer.

	if(dlg2.m_SORI > 0) {
		for(i=0; i<MAX_ARB_EVENTS; i++) {
			if(pPS->arb[0].Event[i].Type == ARB_NOTCH) {
				pPS->arb[0].Event[i].MassA = dlg2.m_Mass0;
				pPS->arb[0].Event[i].MassB = 10.0;
				break;
			} // if
		} // for

		if(dlg2.m_SORI > 1)  {
			for(i=0; i<MAX_ARB_EVENTS; i++) {
				if(pPS->arb[1].Event[i].Type == ARB_NOTCH) {
					pPS->arb[1].Event[i].MassA = dlg2.m_Mass1;
					pPS->arb[1].Event[i].MassB = 10.0;
					break;
				} // if
			} // for
		} // if

		// modify the dlg2.m_SORI burst
		for(i=0; i<MAX_SYN_TRIGGERS; i++) {
			if(pPS->cfp[i].Type == CFP_BURST) {
				midfreq = pDoc->MassToFreq(dlg2.m_Mass0);
				pPS->cfp[i].StartMass = pDoc->FreqToMass(midfreq - 3000.0);
				break;
			} // end if
		} // end for

		if(dlg2.m_SORI > 1) {
			for(i++ ; i<MAX_SYN_TRIGGERS; i++) {
				if(pPS->cfp[i].Type == CFP_BURST) {
					midfreq = pDoc->MassToFreq(dlg2.m_Mass1);
					pPS->cfp[i].StartMass = pDoc->FreqToMass(midfreq - 3000.0);
					break;
				} // end if
			} // end for
		} // end if
	} // if(dlg2.m_SORI > 0)

	// if mixer mode, select mixer frequency
	if(pPS->DetectMode == MixerMode) {
		pPS->MixerFrequency = pDoc->MassToFreq(dlg2.m_Mass0) + pPS->AdcRate/4 + 0.5;
	}

	// set the quad RF frequency as loaded in from config file
	// set quad RF amplitude
	int quads = 0, FirstQuadEvent = 0;
	for(i=0; i<pPS->NumEvents; i++) {
		if(pPS->GenPS[i].Type == NO_EVENT) break;
		if(pPS->GenPS[i].Channel != SYN_TRIGGER) continue;
		if(pPS->cfp[quads].Type == CFP_QUAD) {
			pPS->cfp[quads].Amplitude = Instrument.QuadRFNeeded(dlg2.m_Mass0);
			pPS->cfp[quads].StartMass = pDoc->sys->QuadFrequency;
			quads++;
			if(FirstQuadEvent == 0) FirstQuadEvent = i;
		}
	}

	if(quads<1) {
		MessageBox("Error detected in setup file: "+dlg1.m_SetupFileName+"\nNo quad events defined!","Error in setup file");
		return;
	}

	// if MALDI, do some additional stuff
	//		add a new Quad Trap plate pulse event for gated trapping
	if(pPS->HiResMALDI) {
		// find last laser pulse event
		int lastlaser = -1;
		for(i=0; i<pPS->NumEvents; i++) {
			if(pPS->GenPS[i].Channel != LASER_PULSE) continue;
			lastlaser = i;
		}

		if(lastlaser < 0) {
			MessageBox("Error detected in HiResMALDI setup file...\nNo laser events defined!!!","Setup File Error");
			return;
		}

		float start,finish;
		pDoc->GetEventEdges(lastlaser, &start, &finish);
		// add dac event
		pPS->GenPS[pPS->NumEvents].Channel = DAC_TRAP1;
		pPS->GenPS[pPS->NumEvents].Type = DAC_PULSE;
		pPS->GenPS[pPS->NumEvents].Link2Event = 0;
		pPS->GenPS[pPS->NumEvents].Linking = NO_LINK;
		pPS->GenPS[pPS->NumEvents].Value = 0;
		pPS->GenPS[pPS->NumEvents].Width = float(2.0 + Instrument.FlightTime(dlg2.m_Mass0, double(pPS->GenPS[FirstQuadEvent].Value)) + pDoc->sys->LaserTriggerDelay);
		pPS->GenPS[pPS->NumEvents].Delay = start - 2;
		pPS->NumEvents++;
		//resort the ps. 
		pDoc->SortGenPS();
	}	// if HiResMALDI

	if(dlg2.m_Mass0 < 0) { //negative ion mode
		// change all dac voltages to - dac voltages
		for(i=0; i<pPS->NumEvents; i++) {
			if(pPS->GenPS[i].Type == NO_EVENT) break;
			if(pPS->GenPS[i].Channel < FIRST_DAC) continue;
			// only dac events get here
			pPS->GenPS[i].Value *= -1;
		}
	}


	// finally, initialize the setupview
	CChildFrame* pFrame = (CChildFrame*)m_pSetupTemplate->CreateNewFrame(pDoc,NULL);
	m_pSetupTemplate->InitialUpdateFrame(pFrame,pDoc,TRUE);
}


void CBudaApp::OnSetupChangeinstruments() 
{
	CChangeInstruments dlg;
	
	dlg.m_CurrentInstrument = Instrument.CurrentInstrument;
	dlg.pInst = &(Instrument);

	if(dlg.DoModal() == IDOK) {
		Instrument.CurrentInstrument = dlg.m_CurrentInstrument;
	}

}

void CBudaApp::OnUpdateSetupChangeinstruments(CCmdUI* pCmdUI) 
{
	int num = 1;	
	if(Instrument.sys.HiResMALDI) num++;
	if(Instrument.sys.HiResESI) num++;
	if(num<=1) pCmdUI->Enable(FALSE);
	else pCmdUI->Enable(TRUE);
}

void CBudaApp::OnViewPlotmassspectrum() 
{
	// first get the active document
	// then create a new frame window with m_pBudaViewTemplate using that document
	// then call CBudaView::OnPlotMassSpec();
	CBudaDoc* pDoc;
	CFrameWnd* frame;
	CWnd* activewnd;
	CChildFrame* pFrame;

	activewnd=m_pMainWnd->GetFocus();
	if(activewnd == NULL) return;
	frame = activewnd->GetParentFrame();
	if(frame == NULL) return;
	pDoc = (CBudaDoc*)frame->GetActiveDocument();

	pFrame = (CChildFrame*)m_pBudaViewTemplate->CreateNewFrame(pDoc,NULL);
	m_pBudaViewTemplate->InitialUpdateFrame(pFrame,pDoc,TRUE);

	CBudaView* pView = (CBudaView*)pFrame->GetActiveView();
	pView->OnPlotMassSpectrum();
}

void CBudaApp::OnUpdateViewPlotmassspectrum(CCmdUI* pCmdUI) 
{
	CBudaDoc* pDoc;
	CFrameWnd* frame;
	CWnd* activewnd;

	activewnd=m_pMainWnd->GetFocus();
	if(activewnd == NULL) {
		pCmdUI->Enable(FALSE);
		return;
	}
	frame = activewnd->GetParentFrame();
	if(frame == NULL) {
		pCmdUI->Enable(FALSE);
		return;
	}
	pDoc = (CBudaDoc*)frame->GetActiveDocument();
	if(pDoc == NULL) {
		pCmdUI->Enable(FALSE);
		return;
	}
	if(!pDoc->PS.TransientIsStored && !pDoc->PS.FrequencyIsStored){
		pCmdUI->Enable(FALSE);
		return;
	}
	pCmdUI->Enable(TRUE);	
}


void CBudaApp::OnViewPlotfrequency() 
{
	// first get the active document
	// then create a new frame window with m_pBudaViewTemplate using that document
	// then call CBudaView::OnPlotMassSpec();
	CBudaDoc* pDoc;
	CFrameWnd* frame;
	CWnd* activewnd;
	CChildFrame* pFrame;

	activewnd=m_pMainWnd->GetFocus();
	if(activewnd == NULL) return;
	frame = activewnd->GetParentFrame();
	if(frame == NULL) return;
	pDoc = (CBudaDoc*)frame->GetActiveDocument();

	pFrame = (CChildFrame*)m_pBudaViewTemplate->CreateNewFrame(pDoc,NULL);
	m_pBudaViewTemplate->InitialUpdateFrame(pFrame,pDoc,TRUE);

	CBudaView* pView = (CBudaView*)pFrame->GetActiveView();
	pView->FirstPlot = FALSE;
	pView->OnPlotFrequency();
}

void CBudaApp::OnUpdateViewPlotfrequency(CCmdUI* pCmdUI) 
{
	CBudaDoc* pDoc;
	CFrameWnd* frame;
	CWnd* activewnd;

	activewnd=m_pMainWnd->GetFocus();
	if(activewnd == NULL) {
		pCmdUI->Enable(FALSE);
		return;
	}
	frame = activewnd->GetParentFrame();
	if(frame == NULL) {
		pCmdUI->Enable(FALSE);
		return;
	}
	pDoc = (CBudaDoc*)frame->GetActiveDocument();
	if(pDoc == NULL) {
		pCmdUI->Enable(FALSE);
		return;
	}
	if(!pDoc->PS.TransientIsStored && !pDoc->PS.FrequencyIsStored){
		pCmdUI->Enable(FALSE);
		return;
	}
	pCmdUI->Enable(TRUE);	
}

void CBudaApp::OnViewPlottime() 
{
	// first get the active document
	// then create a new frame window with m_pBudaViewTemplate using that document
	// then call CBudaView::OnPlotMassSpec();
	CBudaDoc* pDoc;
	CFrameWnd* frame;
	CWnd* activewnd;
	CChildFrame* pFrame;

	activewnd=m_pMainWnd->GetFocus();
	if(activewnd == NULL) return;
	frame = activewnd->GetParentFrame();
	if(frame == NULL) return;
	pDoc = (CBudaDoc*)frame->GetActiveDocument();

	pFrame = (CChildFrame*)m_pBudaViewTemplate->CreateNewFrame(pDoc,NULL);
	m_pBudaViewTemplate->InitialUpdateFrame(pFrame,pDoc,TRUE);

	CBudaView* pView = (CBudaView*)pFrame->GetActiveView();
	pView->FirstPlot = FALSE;
	pView->OnPlotTransient();
}

void CBudaApp::OnUpdateViewPlottime(CCmdUI* pCmdUI) 
{
	CBudaDoc* pDoc;
	CFrameWnd* frame;
	CWnd* activewnd;

	activewnd=m_pMainWnd->GetFocus();
	if(activewnd == NULL) {
		pCmdUI->Enable(FALSE);
		return;
	}
	frame = activewnd->GetParentFrame();
	if(frame == NULL) {
		pCmdUI->Enable(FALSE);
		return;
	}
	pDoc = (CBudaDoc*)frame->GetActiveDocument();
	if(pDoc == NULL) {
		pCmdUI->Enable(FALSE);
		return;
	}
	if(!pDoc->PS.TransientIsStored){
		pCmdUI->Enable(FALSE);
		return;
	}
	pCmdUI->Enable(TRUE);	
}

void CBudaApp::MessageBox(CString str, CString caption)
{
	::MessageBox(m_pMainWnd->m_hWnd,str,caption,MB_OK);
}

void CBudaApp::OnStopexpBtn() 
{
	if(Instrument.Running) {
		Instrument.Lock();
		Instrument.acqStop = TRUE;
		Instrument.Unlock();
	}
}

void CBudaApp::OnRunExpButton() 
{
	Instrument.TuneMode = FALSE;
	StartExperiment();
}

void CBudaApp::OnTuneMode() 
{
	Instrument.TuneMode = TRUE;
	StartExperiment();
}

void CBudaApp::OnUpdateTuneMode(CCmdUI* pCmdUI) 
{
	// make sure that we have a document and that the instrument
	// is available and that it isn't already running an experiment
	CBudaDoc *pDoc;
	CFrameWnd* frame;
	CWnd* activewnd;
	pCmdUI->Enable(FALSE);

	activewnd=m_pMainWnd->GetFocus();
	if(activewnd == NULL) {
		return;
	}
	frame = activewnd->GetParentFrame();
	if(frame == NULL) {
		return;
	}
	pDoc = (CBudaDoc*)frame->GetActiveDocument();
	if(pDoc == NULL) {
		return;
	}
	if(Instrument.InstrumentAvailable && !Instrument.Running)
		pCmdUI->Enable(TRUE);
}

void CBudaApp::OnUpdateStopexpBtn(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(FALSE);
	if(Instrument.InstrumentAvailable && Instrument.Running)
		pCmdUI->Enable(TRUE);
}

void CBudaApp::OnUpdateRunExpButton(CCmdUI* pCmdUI) 
{
	// make sure that we have a document and that the instrument
	// is available and that it isn't already running an experiment
	CBudaDoc *pDoc;
	CFrameWnd* frame;
	CWnd* activewnd;
	pCmdUI->Enable(FALSE);

	activewnd=m_pMainWnd->GetFocus();
	if(activewnd == NULL) {
		return;
	}
	frame = activewnd->GetParentFrame();
	if(frame == NULL) {
		return;
	}
	pDoc = (CBudaDoc*)frame->GetActiveDocument();
	if(pDoc == NULL) {
		return;
	}
	if(Instrument.InstrumentAvailable && !Instrument.Running)
		pCmdUI->Enable(TRUE);
}

void CBudaApp::StartExperiment()
{
	// find the currently focused document;
	CBudaDoc *pDoc, *pNewDoc;
	CFrameWnd* pframe;
	CWnd* activewnd;
	long i;

	activewnd=m_pMainWnd->GetFocus();
	if(activewnd == NULL) {
		return;
	}
	pframe = activewnd->GetParentFrame();
	if(pframe == NULL) {
		return;
	}
	pDoc = (CBudaDoc*)pframe->GetActiveDocument();
	if(pDoc == NULL) {
		return;
	}

	// clone the document and open the setup window.
	// create an empty document
	pNewDoc = (CBudaDoc*)m_pSetupTemplate->CreateNewDocument();
	pNewDoc->DuplicateDoc(pDoc);
	pNewDoc->PS.TransientIsStored = 0;
	pNewDoc->PS.FrequencyIsStored = 0;
	pNewDoc->PS.MITableIsStored = 0;
	pDoc->CopyPS(&(pDoc->PS),&(pNewDoc->PS));
	pNewDoc->FreqArray = 0;
	for(i=0; i<pNewDoc->PS.SpectraRequested; i++) {
		pNewDoc->PS.Spec[i].TArrayFileOffset = 0;
		pNewDoc->PS.Spec[i].FITable = 0;
		pNewDoc->PS.Spec[i].IsMIed = FALSE;
		pNewDoc->PS.Spec[i].IsFFTed = FALSE;
		pNewDoc->PS.Spec[i].ElapsedTime = 0;
		pNewDoc->PS.Spec[i].NoiseLevel = 0;
		pNewDoc->PS.Spec[i].NumPeaks = 0;
		pNewDoc->PS.Spec[i].ScaleFactor = 0;
		pNewDoc->PS.Spec[i].TotalIonInt = 0;
	}
	pNewDoc->FITable = 0;
	pNewDoc->SetTitle("New file");

	// initialize the setupview
	pframe = (CChildFrame*)m_pSetupTemplate->CreateNewFrame(pNewDoc,NULL);
	m_pSetupTemplate->InitialUpdateFrame(pframe,pNewDoc,TRUE);
	
	Instrument.pDoc = pNewDoc;
	// run the experiment 
	Instrument.ThreadInfo.InstrumentCommands = RUN_EXPERIMENT;
	SetEvent(Instrument.ThreadInfo.m_hProcessCommand);
}

void CBudaApp::OnExperimentDone()
{
	// this function handles the windows message WM_NEWDATAREADY
	
	Instrument.pDoc->OnExperimentDone();
}

void CBudaApp::CheckMemState(CString str)
{
#ifdef _DEBUG
	//TRACE("****************************\n");
	//TRACE(str+"\n");
	//TRACE("****************************\n");
	ASSERT(_CrtCheckMemory());
	//ASSERT(!_CrtDumpMemoryLeaks());
	//TRACE("****************************\n");
#endif	// _DEBUG

}

void CBudaApp::StoreLastUserID()
{
	CString UserID = Instrument.sys.UserID;
	WriteProfileString(BudaRegistrySection,"Last UserID",UserID);
}

void CBudaApp::OnAbortBtn() 
{
	if(Instrument.Running) {
		Instrument.Lock();
		Instrument.acqAbort = TRUE;
		Instrument.acqTRASH = TRUE;
		Instrument.Unlock();
	}	
}

void CBudaApp::OnUpdateAbortBtn(CCmdUI* pCmdUI) 
{
	if(Instrument.Running) pCmdUI->Enable(TRUE);
	else pCmdUI->Enable(FALSE);
}

/*
void CBudaApp::OnSwirlyview() 
{
	CDocument* curdoc;
	CFrameWnd* frame;
	CWnd* activewnd;

	activewnd=m_pMainWnd->GetFocus();
	frame = activewnd->GetParentFrame();
	curdoc = frame->GetActiveDocument();
	if(curdoc == NULL) return;

	CFrameWnd *cframe = m_pSwirlyViewTemplate->CreateNewFrame(curdoc,NULL);
	m_pMIViewTemplate->InitialUpdateFrame( cframe, curdoc, TRUE );	
}

void CBudaApp::OnUpdateSwirlyview(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(FALSE);
	#ifdef SWIRLY_VIEW_ACTIVE 
		pCmdUI->Enable(TRUE);
	#endif
}
*/

void CBudaApp::InitializeParameters()
{
	UserSetupFile = "";
	SaveDirectory = "";
	UserIDFile = "";
	pFFT = 0;

	// default values for DisplayParms
	DisplayParms.allparms = FALSE;
	DisplayParms.massrange = FALSE;
	DisplayParms.acqrate = FALSE;
	DisplayParms.apodfunc = FALSE;
	DisplayParms.calconst = FALSE;
	DisplayParms.exciteamp = FALSE;
	DisplayParms.filename = FALSE;
	DisplayParms.notes = FALSE;
	DisplayParms.numpts = FALSE;
	DisplayParms.numscans = FALSE;
	DisplayParms.numzerofills = FALSE;
	DisplayParms.timedate = FALSE;
	DisplayParms.trapvolts = FALSE;
	DisplayParms.labelpoints = TRUE;
	DisplayParms.LabelPeakPos = FALSE;
	DisplayParms.noisethreshold = FALSE;
	DisplayParms.LabelCharge = FALSE;
	DisplayParms.PosNeg = FALSE;
	DisplayParms.ScaleFactor = FALSE;

	// initialize charge state info
	CSHighChargeLimit = CS_DEFAULT_MAX_CHARGE;
	CSDetMethod = CS_DEFAULT_DETERMINATION_METHOD;

	// initialize deconvolution info
	DeconHighMassLimit = DECON_DEFAULT_HIGH_MASS_LIMIT;
	DeconLowMassLimit = DECON_DEFAULT_LOW_MASS_LIMIT;
	DeconHighChargeLimit = DECON_DEFAULT_HIGH_CHARGE_LIMIT;
	DeconLowChargeLimit = DECON_DEFAULT_LOW_CHARGE_LIMIT;
	DeconMethod = DECON_DEFAULT;
	DeconResolution = DECON_DEFAULT_RESOLUTION;
	DeconAdductMass = DECON_DEFAULT_ADDUCTMASS;
	DeconIsotopicallyResolved = DECON_DEFAULT_ISOTOPICALLY_RESOLVED;


	// initializing MI table info
	AutoDetectThreshold = TRUE;
	ThresholdMethod = MI_DEFAULT_THRESHOLD_METHOD;
	MinimumAutoThreshold = 1.0;

	// initializing filter info
	BoxcarAvg = FALSE;
	BoxcarPoints = 5;

	ReadIniFile();
}

void CBudaApp::WriteIniFile()
{
	// open the ini file;
	CFile infile;
	CString str;
 
	char* pathname;
	pathname = new char[256];
	strcpy(pathname,ProgramDirectory);
	strcat(pathname,ConfigFileName);

    if(!infile.Open(pathname,CFile::modeCreate|CFile::modeWrite,NULL)) {
		::MessageBox(NULL,"Couldn't open Buda.cfg file for creation","File Error",MB_OK);
		delete pathname;
		return;
	}

    delete pathname;

	CArchive ar(&infile,CArchive::store,4096,NULL);
	str.Format("# Buda %s\r\n",BudaVersion);
	ar.WriteString(str);
	ar.WriteString("# Boston University Data Analysis configuration file\r\n");
	ar.WriteString("# \r\n");
	ar.WriteString("# by Peter B. O'Connor\r\n");
	ar.WriteString("# poconnor@bu.edu\r\n");
	ar.WriteString("# \r\n");
	ar.WriteString("# note: blank lines and lines starting with '#' are ignored\r\n");
	ar.WriteString("# \r\n");

	// display parms parameters
	ar.WriteString("\r\n");
	ar.WriteString("# Default Display Parameters\r\n");
	WriteIniInt(&ar, "DisplayParms.allparms",DisplayParms.allparms);
	WriteIniInt(&ar,"DisplayParms.massrange",DisplayParms.massrange);
	WriteIniInt(&ar,"DisplayParms.acqrate",DisplayParms.acqrate);
	WriteIniInt(&ar,"DisplayParms.apodfunc",DisplayParms.apodfunc);	
	WriteIniInt(&ar,"DisplayParms.calconst",DisplayParms.calconst);
	WriteIniInt(&ar,"DisplayParms.exciteamp",DisplayParms.exciteamp);
	WriteIniInt(&ar,"DisplayParms.filename",DisplayParms.filename);
	WriteIniInt(&ar,"DisplayParms.notes",DisplayParms.notes);
	WriteIniInt(&ar,"DisplayParms.numpts",DisplayParms.numpts);
	WriteIniInt(&ar,"DisplayParms.numscans",DisplayParms.numscans);
	WriteIniInt(&ar,"DisplayParms.numzerofills",DisplayParms.numzerofills);
	WriteIniInt(&ar,"DisplayParms.timedate",DisplayParms.timedate);
	WriteIniInt(&ar,"DisplayParms.trapvolts",DisplayParms.trapvolts);
	WriteIniInt(&ar,"DisplayParms.labelpoints",DisplayParms.labelpoints);
	WriteIniInt(&ar,"DisplayParms.LabelPeakPos",DisplayParms.LabelPeakPos);
	WriteIniInt(&ar,"DisplayParms.noisethreshold",DisplayParms.noisethreshold);
	WriteIniInt(&ar,"DisplayParms.LabelCharge",DisplayParms.LabelCharge);
	WriteIniInt(&ar,"DisplayParms.PosNeg",DisplayParms.PosNeg);
	WriteIniInt(&ar,"DisplayParms.ScaleFactor",DisplayParms.ScaleFactor);

	// Charge State Determination parameters
	ar.WriteString("\r\n");
	ar.WriteString("# Charge State Determination Parameters\r\n");
	WriteIniInt(&ar,"CS_Determination.HighChargeLimit",CSHighChargeLimit);
	WriteIniInt(&ar,"CS_Determination.CS_Determination_Method",CSDetMethod);

	// Deconvolution parameters
	ar.WriteString("\r\n");
	ar.WriteString("# Deconvolution Parameters\r\n");
	WriteIniFloat(&ar,"Decon.AdductMass",DeconAdductMass);
	WriteIniInt(&ar,"Decon.Decon_Method",DeconMethod);
	WriteIniInt(&ar,"Decon.HighChargeLimit",DeconHighChargeLimit);
	WriteIniInt(&ar,"Decon.LowChargeLimit",DeconLowChargeLimit);
	WriteIniFloat(&ar,"Decon.LowMassLimit",DeconLowMassLimit);
	WriteIniFloat(&ar,"Decon.HighMassLimit",DeconHighMassLimit);
	WriteIniFloat(&ar,"Decon.DeconResolution",DeconResolution);
	WriteIniInt(&ar, "Decon.IsotopicallyResolved", DeconIsotopicallyResolved);

	// MI table information
	ar.WriteString("\r\n");
	ar.WriteString("# MI Table Parameters\r\n");
	WriteIniInt(&ar, "MI_Table.AutoCalculateThreshold",AutoDetectThreshold);
	WriteIniInt(&ar, "MI_Table.ThresholdMethod",ThresholdMethod);
	WriteIniFloat(&ar, "MI_Table.MinimumAutoThreshold",MinimumAutoThreshold);

	// close the archive and the file;
	ar.Flush();
	infile.Close();
}

void CBudaApp::ReadIniFile()
{
	// open the ini file;
	CFile infile;
	if(_chdir(ProgramDirectory)==-1) {
		::MessageBox(NULL,"Couldn't find Buda program Directory","Directory Error!",MB_OK);
		return;
	}
 	char* pathname;
	pathname = new char[256];
	strcpy(pathname,ProgramDirectory);
	strcat(pathname,ConfigFileName);

	if(!infile.Open(pathname,CFile::modeRead,NULL)) {
		if(::MessageBox(NULL,"Couldn't open Buda.cfg file\nCreate New Buda.cfg file?","File Error",MB_YESNO) == IDYES) {
			WriteIniFile();
		} 
		delete pathname;
		return;
	}
	delete pathname;

	// get the default display parms values
	GetDefaultDisplayParms(&infile);

	// Charge State Determination parameters
	CSHighChargeLimit = GetIniInt(&infile, "CS_Determination.HighChargeLimit",CSHighChargeLimit);
	CSDetMethod = GetIniInt(&infile, "CS_Determination.CS_Determination_Method",CSDetMethod);

	// Deconvolution parameters
	DeconAdductMass = GetIniFloat(&infile, "Decon.AdductMass",DeconAdductMass);
	DeconMethod = GetIniInt(&infile, "Decon.Decon_Method",DeconMethod);
	DeconHighChargeLimit = GetIniInt(&infile, "Decon.HighChargeLimit",DeconHighChargeLimit);
	DeconLowChargeLimit = GetIniInt(&infile, "Decon.LowChargeLimit",DeconLowChargeLimit);
	DeconLowMassLimit = GetIniFloat(&infile, "Decon.LowMassLimit",DeconLowMassLimit);
	DeconHighMassLimit = GetIniFloat(&infile, "Decon.HighMassLimit",DeconHighMassLimit);
	DeconResolution = GetIniFloat(&infile, "Decon.DeconResolution",DeconResolution);
	DeconIsotopicallyResolved = GetIniInt(&infile, "Decon.IsotopicallyResolved",DeconIsotopicallyResolved);

	// MI table information
	AutoDetectThreshold = GetIniInt(&infile, "MI_Table.AutoCalculateThreshold",AutoDetectThreshold);
	ThresholdMethod = GetIniInt(&infile, "MI_Table.ThresholdMethod",ThresholdMethod);
	MinimumAutoThreshold = GetIniFloat(&infile, "MI_Table.MinimumAutoThreshold",MinimumAutoThreshold);
}

void CBudaApp::GetDefaultDisplayParms(CFile *infile)
{
	// display parms parameters
	DisplayParms.allparms = FALSE;
	DisplayParms.massrange = FALSE;
	DisplayParms.acqrate = FALSE;
	DisplayParms.apodfunc = FALSE;
	DisplayParms.calconst = FALSE;
	DisplayParms.exciteamp = FALSE;
	DisplayParms.filename = FALSE;
	DisplayParms.notes = FALSE;
	DisplayParms.numpts = FALSE;
	DisplayParms.numscans = FALSE;
	DisplayParms.numzerofills = FALSE;
	DisplayParms.timedate = FALSE;
	DisplayParms.trapvolts = FALSE;
	DisplayParms.labelpoints = TRUE;
	DisplayParms.LabelPeakPos = FALSE;
	DisplayParms.noisethreshold = FALSE;
	DisplayParms.LabelCharge = FALSE;
	DisplayParms.PosNeg = FALSE;
	DisplayParms.ScaleFactor = FALSE;

	// now load the defaults from the buda.cfg file
	DisplayParms.allparms = GetIniInt(infile, "DisplayParms.allparms",DisplayParms.allparms);
	DisplayParms.massrange = GetIniInt(infile,"DisplayParms.massrange",DisplayParms.massrange);
	DisplayParms.acqrate = GetIniInt(infile,"DisplayParms.acqrate",DisplayParms.acqrate);
	DisplayParms.apodfunc = GetIniInt(infile,"DisplayParms.apodfunc",DisplayParms.apodfunc);	
	DisplayParms.calconst = GetIniInt(infile,"DisplayParms.calconst",DisplayParms.calconst);
	DisplayParms.exciteamp = GetIniInt(infile,"DisplayParms.exciteamp",DisplayParms.exciteamp);
	DisplayParms.filename = GetIniInt(infile,"DisplayParms.filename",DisplayParms.filename);
	DisplayParms.notes = GetIniInt(infile,"DisplayParms.notes",DisplayParms.notes);
	DisplayParms.numpts = GetIniInt(infile,"DisplayParms.numpts",DisplayParms.numpts);
	DisplayParms.numscans = GetIniInt(infile,"DisplayParms.numscans",DisplayParms.numscans);
	DisplayParms.numzerofills = GetIniInt(infile,"DisplayParms.numzerofills",DisplayParms.numzerofills);
	DisplayParms.timedate = GetIniInt(infile,"DisplayParms.timedate",DisplayParms.timedate);
	DisplayParms.trapvolts = GetIniInt(infile,"DisplayParms.trapvolts",DisplayParms.trapvolts);
	DisplayParms.labelpoints = GetIniInt(infile,"DisplayParms.labelpoints",DisplayParms.labelpoints);
	DisplayParms.LabelPeakPos = GetIniInt(infile,"DisplayParms.LabelPeakPos",DisplayParms.LabelPeakPos);
	DisplayParms.noisethreshold = GetIniInt(infile,"DisplayParms.noisethreshold",DisplayParms.noisethreshold);
	DisplayParms.LabelCharge = GetIniInt(infile,"DisplayParms.LabelCharge",DisplayParms.LabelCharge);
	DisplayParms.PosNeg = GetIniInt(infile,"DisplayParms.PosNeg",DisplayParms.PosNeg);
	DisplayParms.ScaleFactor = GetIniInt(infile,"DisplayParms.ScaleFactor",DisplayParms.ScaleFactor);
}

int CBudaApp::GetIniInt(CFile *file, CString valuestr, int defaultvalue)
{
	int value;
	CString str,getstr,rightstr;
	CArchive ar(file,CArchive::load,4096,NULL);
	file->SeekToBegin();

	getstr=CompressString(valuestr);
	
	while(ar.ReadString(str)) {	// breaks if EOF
		str=CompressString(str);
		if (str.GetLength()==0) continue; // empty lines
		if (str.Left(1)=="#") continue; // comment lines
	
		if (str.Left(getstr.GetLength()) == getstr) {
			rightstr = GetRightString(str);
			sscanf(rightstr,"%d",&value);
			return value;
		}
	}
	return defaultvalue;
}

double CBudaApp::GetIniFloat(CFile *file, CString valuestr, double defaultvalue)
{
	double value;
	CString str,getstr,rightstr;
	CArchive ar(file,CArchive::load,4096,NULL);
	file->SeekToBegin();

	getstr=CompressString(valuestr);
	
	while(ar.ReadString(str)) {	// breaks if EOF
		str=CompressString(str);
		if (str.GetLength()==0) continue; // empty lines
		if (str.Left(1)=="#") continue; // comment lines
	
		if (str.Left(getstr.GetLength()) == getstr) {
			rightstr = GetRightString(str);
			sscanf(rightstr,"%lf",&value);
			return value;
		}
	}
	return defaultvalue;
}

CString CBudaApp::GetIniString(CFile *file, CString ValueStr, CString defaultvalue)
{
	CString value;
	int i;
	CString readstr,str,getstr,rightstr;
	CArchive ar(file,CArchive::load,4096,NULL);
	file->SeekToBegin();

	getstr=CompressString(ValueStr);
	
	while(ar.ReadString(readstr)) {	// breaks if EOF
		str=CompressString(readstr);
		if (str.GetLength()==0) continue; // empty lines
		if (str.Left(1)=="#") continue; // comment lines
	
		if (str.Left(getstr.GetLength()) == getstr) {
			rightstr = GetRightString(readstr);
			for (i=0; i<rightstr.GetLength(); i++) {
				if(rightstr.GetAt(i) == ' ' || rightstr.GetAt(i) == '\t') {
					// remove preceeding spaces and tabs from string
					rightstr= rightstr.Right(rightstr.GetLength()-1);
				} else break;
			}
			return rightstr;
		}
	}
	return defaultvalue;

}

CString CBudaApp::CompressString(CString str)
{
	CString strleft, strright, outstring;
	outstring = str;
	outstring.MakeUpper();	// make string uppercase
	int i;
	for (i=0; i<outstring.GetLength(); i++) {
		if(outstring.GetAt(i) == ' ' || outstring.GetAt(i) == '\t') {
			// remove spaces and tabs from string
			strleft = outstring.Left(i);
			strright= outstring.Right(outstring.GetLength()-i-1);
			outstring = strleft + strright;
		}
	}
	return outstring;
}

CString CBudaApp::GetRightString(CString str)
{
	int equalpos;
	equalpos = FindEqualSign(str)+1;
	return str.Right(str.GetLength()-equalpos);
}


int CBudaApp::FindEqualSign(CString str)
{
	int i=0;
	for(i=0; i<str.GetLength(); i++) {
		if(str.Mid(i,1)=="=") break;
	}
	return i;
}

void CBudaApp::WriteIniInt(CArchive *ar, CString ValueStr, int value)
{
	CString str;
	str.Format("%s = %d\r\n",ValueStr,value);
	ar->WriteString(str);
}

void CBudaApp::WriteIniString(CArchive *ar, CString ValueStr, CString value)
{
	CString str;
	str.Format("%s = %s\r\n",ValueStr,value);
	ar->WriteString(str);
}

void CBudaApp::WriteIniFloat(CArchive *ar, CString ValueStr, double value)
{
	CString str;
	str.Format("%s = %1.8f\r\n",ValueStr,value);
	ar->WriteString(str);
}

void CBudaApp::OnDeconView()
{
	CDocument* curdoc;
	CFrameWnd* frame;
	CWnd* activewnd;

	activewnd=m_pMainWnd->GetFocus();
	frame = activewnd->GetParentFrame();
	curdoc = frame->GetActiveDocument();
	if(curdoc == NULL) return;

	CFrameWnd *cframe = m_pDeconViewTemplate->CreateNewFrame(curdoc,NULL);
	m_pDeconViewTemplate->InitialUpdateFrame( cframe, curdoc, TRUE );	
}

void CBudaApp::OnViewDeconvolution() 
{
	OnDeconView();
}

void CBudaApp::OnUpdateViewDeconvolution(CCmdUI* pCmdUI) 
{
	CBudaDoc* pDoc;
	CFrameWnd* frame;
	CWnd* activewnd;

	activewnd=m_pMainWnd->GetFocus();
	if(activewnd == NULL) {
		pCmdUI->Enable(FALSE);
		return;
	}
	frame = activewnd->GetParentFrame();
	if(frame == NULL) {
		pCmdUI->Enable(FALSE);
		return;
	}
	pDoc = (CBudaDoc*)frame->GetActiveDocument();
	if(pDoc == NULL) {
		pCmdUI->Enable(FALSE);
		return;
	}
	if(!pDoc->PS.TransientIsStored && !pDoc->PS.FrequencyIsStored){
		pCmdUI->Enable(FALSE);
		return;
	}
	pCmdUI->Enable(TRUE);	
}

void CBudaApp::OnToolsElementalcomp() 
{
	CElementalCompDlg dlg;
	dlg.DoModal();
}

BOOL CAboutDlg::OnInitDialog() 
{
	CDialog::OnInitDialog();
	
	pApp = (CBudaApp*)AfxGetApp();
	m_VersionString = pApp->BudaVersion;
	UpdateData(FALSE);

	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}
