/****************************************************************

Boston University data analysis (BUDA) package 

Description: Buda is a data analysis program for Fourier 
Transform Mass Spectrometry data.  

by Peter B. O'Connor

    Copyright (C) 2000  Peter B. O'Connor

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#if !defined(AFX_Buda_H__23A26EEA_6AF3_11D1_9106_44455354616F__INCLUDED_)
#define AFX_Buda_H__23A26EEA_6AF3_11D1_9106_44455354616F__INCLUDED_

#include "OscilloscopeDlg.h"	// Added by ClassView
#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#ifndef __AFXWIN_H__
	#error include 'stdafx.h' before including this file for PCH
#endif

#include "StatusControl.h"
#include "StatusProgress.h"
#include "resource.h"       // main symbols
#include "FFTWorkerThread1.h"
#include "BudaDoc.h"
#include "Instrument.h"
#include "SplashWnd.h"
#include "SingleInstance.h"

#define MAX_PTS_IN_TEXTVIEW 200

/////////////////////////////////////////////////////////////////////////////
// CBudaApp:
// See Buda.cpp for the implementation of this class
//

class CBudaApp : public CSingleInstanceApp
{
public:
	BOOL DeconIsotopicallyResolved;
	UINT DeconLowChargeLimit;
	double DeconAdductMass;
	BOOL BoxcarAvg;
	int BoxcarPoints;
	void OnDeconView();
	double GetIniFloat(CFile *file, CString valuestr, double defaultvalue);
	void WriteIniFloat(CArchive *ar, CString ValueStr, double value);
	double MinimumAutoThreshold;
	int AutoDetectThreshold;
	int ThresholdMethod;
	CString BudaVersion;
	CString ConfigFileName;
	int FindEqualSign(CString str);
	CString GetIniString(CFile *file, CString ValueStr, CString defaultvalue);
	CString GetRightString(CString str);
	CString CompressString(CString str);
	void WriteIniInt(CArchive *ar, CString ValueStr, int value);
	void WriteIniString(CArchive *ar, CString ValueStr, CString value);
	int GetIniInt(CFile* file,CString str, int defaultvalue);
	void GetDefaultDisplayParms(CFile* infile);
	struct DispParms DisplayParms;
	void ReadIniFile();
	void WriteIniFile();
	void InitializeParameters();
	int CSDetMethod;
	int CSHighChargeLimit;
	int DeconMethod;
	double DeconResolution;
	UINT DeconHighChargeLimit;
	double DeconHighMassLimit;
	double DeconLowMassLimit;
	void StoreLastUserID();
	int ActiveSystemIndex;
	void CheckMemState(CString str);
	COscilloscopeDlg Scope;
	void OnExperimentDone();
	void StartExperiment();
	void MessageBox(CString str, CString caption);
	CBudaDoc* OnLoadSetup(CString FileName);
#ifdef _DEBUG
	int DebugCMassCounter;
#endif;
	CFFTWorkerThread* pFFT;
	//CWinThread* FFTThrd;
	//struct FFTThreadInfo FFTThrdInfo;
	//CBudaDoc* m_pdoc;
	BOOL FileExist(LPCTSTR);
    //CSingleInstanceApp m_singleInstance;
	CString ElementalComposition;
	void VerifyValidUserID();
	void GetLastUserID();
	CString BudaRegistry;
	CString BudaRegistrySection;
	CString UserSetupFile;
	CString ProgramDirectory;
	CString SaveDirectory;
	CString UserIDFile;
	CWnd* m_pCurrentActiveView;
	void DumpToDiagFile(CString);
#ifdef DEBUG_DUMP_DIAGS_TO_FILE
	CFile diagfile;
#endif
	DWORD m_SplashTime;
	CSplashWnd m_Splash;
	CInstrument Instrument;
	CMultiDocTemplate* m_pMIViewTemplate;
	CMultiDocTemplate* m_pSetupTemplate;
	CMultiDocTemplate* m_pTextViewTemplate;
	CMultiDocTemplate* m_pParmViewTemplate;
	CMultiDocTemplate* m_pSwirlyViewTemplate;
	CMultiDocTemplate* m_pDeconViewTemplate;
	CMultiDocTemplate* m_pBudaViewTemplate;
	CBudaApp();

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CBudaApp)
	public:
	virtual BOOL InitInstance();
	virtual CDocument* OpenDocumentFile(LPCTSTR lpszFileName);
	virtual BOOL OnIdle(LONG lCount);
	virtual int ExitInstance();
	//}}AFX_VIRTUAL

// Implementation
	COleTemplateServer m_server;

	//{{AFX_MSG(CBudaApp)
	afx_msg void OnAppAbout();
	afx_msg void OnMITable();
	afx_msg void OnUpdateMITable(CCmdUI* pCmdUI);
	afx_msg void OnViewAsText();
	afx_msg void OnAddUser();
	afx_msg void OnChangeUser();
	afx_msg void OnToolsExactmass();
	afx_msg void OnSetupPulsesequence();
	afx_msg void OnViewParameters();
	afx_msg void OnSetupButton();
	afx_msg void OnUpdateAddUser(CCmdUI* pCmdUI);
	afx_msg void OnUpdateChangeUser(CCmdUI* pCmdUI);
	afx_msg void OnUpdateToolsExactmass(CCmdUI* pCmdUI);
	afx_msg void OnUpdateViewDecon(CCmdUI* pCmdUI);
	afx_msg void OnUpdateViewParameters(CCmdUI* pCmdUI);
	afx_msg void OnUpdateViewStickplot(CCmdUI* pCmdUI);
	afx_msg void OnLoadSetup();
	afx_msg void OnSaveSetup();
	afx_msg void OnUpdateSaveSetup(CCmdUI* pCmdUI);
	afx_msg void OnSetupWizard();
	afx_msg void OnNewSetup();
	afx_msg void OnSetupChangeinstruments();
	afx_msg void OnUpdateSetupChangeinstruments(CCmdUI* pCmdUI);
	afx_msg void OnViewPlotmassspectrum();
	afx_msg void OnUpdateViewPlotmassspectrum(CCmdUI* pCmdUI);
	afx_msg void OnViewPlotfrequency();
	afx_msg void OnUpdateViewPlotfrequency(CCmdUI* pCmdUI);
	afx_msg void OnViewPlottime();
	afx_msg void OnUpdateViewPlottime(CCmdUI* pCmdUI);
	afx_msg void OnStopexpBtn();
	afx_msg void OnRunExpButton();
	afx_msg void OnTuneMode();
	afx_msg void OnUpdateTuneMode(CCmdUI* pCmdUI);
	afx_msg void OnUpdateStopexpBtn(CCmdUI* pCmdUI);
	afx_msg void OnUpdateRunExpButton(CCmdUI* pCmdUI);
	afx_msg void OnAbortBtn();
	afx_msg void OnUpdateAbortBtn(CCmdUI* pCmdUI);
	afx_msg void OnSwirlyview();
	afx_msg void OnUpdateSwirlyview(CCmdUI* pCmdUI);
	afx_msg void OnViewDeconvolution();
	afx_msg void OnUpdateViewDeconvolution(CCmdUI* pCmdUI);
	afx_msg void OnToolsElementalcomp();
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

/////////////////////////////////////////////////////////////////////////////

//{{AFX_INSERT_LOCATION}}
// Microsoft Developer Studio will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_Buda_H__23A26EEA_6AF3_11D1_9106_44455354616F__INCLUDED_)
