/****************************************************************

Boston University data analysis (BUDA) package 

Description: Buda is a data analysis program for Fourier 
Transform Mass Spectrometry data.  

by Peter B. O'Connor

    Copyright (C) 2000  Peter B. O'Connor

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/
// BudaDoc.h : interface of the CBudaDoc class
//
/////////////////////////////////////////////////////////////////////////////

#if !defined(AFX_BudaDOC_H__23A26EF2_6AF3_11D1_9106_44455354616F__INCLUDED_)
#define AFX_BudaDOC_H__23A26EF2_6AF3_11D1_9106_44455354616F__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include "BudaMaster.h"
#include "Buda.h"
//#include "Diagnostics.h"	// Added by ClassView
#include "Instrument.h"
#include <afxmt.h>
#include "FFTWorkerThread1.h"	// Added by ClassView


// declare classes that will be needed so that I
// can use interlinked includes
class CBudaApp;
class CInstrument;
//class CBudaDoc;
class CMainFrame;

class CBudaDoc : public CDocument
{
protected: // create from serialization only
	DECLARE_DYNCREATE(CBudaDoc)

// Attributes
public:
	CBudaDoc();

// Operations
public:

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CBudaDoc)
	public:
	//virtual void CBudaDoc::OnCloseDocument();
	virtual void OnCloseDocument();
	virtual BOOL OnNewDocument();
	virtual void Serialize(CArchive& ar);
	virtual void DeleteContents();
	virtual BOOL OnOpenDocument(LPCTSTR lpszPathName);
	virtual BOOL CanCloseFrame(CFrameWnd* pFrame);
	//}}AFX_VIRTUAL

// Implementation
public:
	long ConvertBigEndian(long *in);
	long ExtractLongFromACQUFile(CFile *acqu, CString *str);
	double ExtractDoubleFromACQUFile(CFile* acqu, CString* str);
	void ReadBrukerFIDFile(CFile* arf, int mode);
	BOOL asserted;
	UINT DeconLowChargeLimit;
	int BoxcarPoints;
	BOOL BoxcarAvg;
	double DeconResolution;
	double DeconAdductMass;
	int DeconMethod;
	UINT DeconHighChargeLimit;
	double DeconLowMassLimit;
	double DeconHighMassLimit;
	int ThresholdMethod;
	int CSHighChargeLimit;
	int CSDetMethod;
	//void CS_Fourier(float* Patt, double minx, double maxx, int Max_Charge);
	//void CS_Patterson(float* Patt, double minx, double maxx, int Max_Charge);
	//void GetChargeState(double minx, double maxx);
	float GetPeakArea(double *Ypts, double *Xpts, int numpts);
	float InterpolateForRes(double *y, double *x, int ptr, double maxx);
	double GetDeterminant(double a1, double a2, double a3, double b1, double b2, double b3, double c1, double c2, double c3);
	BOOL ParabolicLeastSquares(double *y, int numpts, double *ret, double PeakMaxAmplitude, double PeakPickerLevel);
	void PeakFit(double *PeakStack, double *PeakXStack, int numpts);
	void PeakPick(long HighMassPt, long LowMassPt, float* data);
	void GetTotalIonInt();
	void GetMITable();
	void StoreDefaultDisplayParms();
	void StoreDefaultDisplayParms(struct DispParms* pDp);
	void GetDefaultDisplayParms();
	void CheckMemState(CString str);
	void SetIsMIed(BOOL tf);
	BOOL IsMIed();
	void SetIsMIed(int s, BOOL tf);
	BOOL IsFFTed();
	void SetIsFFTed(BOOL tf);
	void SetIsFFTed(int s, BOOL tf);
	BOOL SetCurrentSpectrumFocus(int s);
	void GetTimeArray(int s);
	long ReadTimeArray(int s, CFile* arf);
	long* TimeArray;
	void OnExperimentDone();
	void DuplicateDoc(CBudaDoc* pDoc);
	void SaveSetupFile();
	BOOL ReadSetupFile(CString FileName);
	BOOL CyclicLoopCheck[MAX_PS_EVENTS];
	CString GetLinkedToStr(struct PS* pPS, int Event);
	void RecalcEventTime(struct PS* pPS, int Event, int Linking);
	void RecalcEventTime(struct PS* pPS, int Event, int Linking, int Link2Event);
	CView* pSetupView;
	BOOL SetupViewOpen;
	void ExperimentDone();
	BOOL TDisDirty;
	BOOL PSisDirty;
	void CopyPS(struct PS* from, struct PS* to);
	double CyclotronFrequency(double freq, float Vtrap, double TotInt, long NumScans, int ADCgain, int direction);
	BOOL FFTIsDirty;
	BOOL CalculatingFFT;
	void ShiftDetectEvent();
	BOOL TwoDFFTDone;
	CFFTWorkerThread* pFFT;
	HWND m_hwndNotifyFFTDone;
	HANDLE m_hFFTDone;
	void InitializePS();

#ifdef INCLUDE_DIAGNOSTICS
	//CDiagnostics Diags;
#endif

	// function prototypes
	void AddToFFTQueue(int s);
	void UpdateSystemPS();
	int SwapEvents(struct PS* pPS, int EventA, int EventB);
	float EventTime(int event);
	float EventTime(struct PS* pPS, int event);
	int GetV6ChannelNumber(int);
	void SortGenPS();
	void CBudaDoc::SortGenPS(struct PS* pPS);
	BOOL SetCurrentMIFocus(int);
	void UpdateFFTStatus(int);
	BOOL TestForBigFFTs();
	float DacValueAtTime(long, float);
	void GetEventEdges(long Event, float* start, float* finish);
	void GetEventEdges(struct PS*, long Event, float* start, float* finish);
	void GetEventEdges(struct PS*, long Event, float* start, float* finish, int EventLoopCtr);
	void Setup2DDataMenu();
	void Update2DDataMenu();
	void ResetMenus();
	void RecalcAllViews();
	void GetCurrentFreqArray(int, BOOL);
	//void AssignToGroupZero(struct groupstruct*, int, int);
	double FreqToMass(double);
	//void ComputeCS(struct groupstruct*);
	//void GroupAssignment(struct groupstruct*);
	void DumpToDiagFile(CString);
	void SetupParmsOnRead();
	void SetStatusBar(CStatusBar*);
	int GetCurrentSpectrumFocus();
	BOOL SetCurrentSpectrumFocus(int Index, int mode);
	double GetMITableMass(long);
	int GetNextFileNumber();
	CString GetDefaultFileName();
	void GetTimeDate(CString*);
	//void ExtractOverlappingCSDists(struct groupstruct*, int);
	//void RoughGroupAssignment(struct groupstruct*, int*);
	void SortFITableOnAbundance();
	//void ChargeStateDetermination();
	double PtToTime(long);
	double PtToMass(long);
	long TimeToPt(double);
	long MassToPt(double);
	long FreqToPt(double);
	double MassToFreq(double);
	double MassToFreq(double, double, double);
	double FreqToMass(double, double, double);
	double PtToFreq(long);
	double CalculateIntShift();
	double GetTrapVoltsOnDetect();
	void NullTerminate(char *, int);
	virtual ~CBudaDoc();
	afx_msg void OnSetupOptions();
	void FileError(CString);
	void StoreVersion8File(CFile* arf,int mode);
	void ReadVersion8File(CFile* arf,int mode);
	void ReadVersion6File(CFile* arf,int mode);
	struct sys* GetSys();
	CInstrument* GetInstrument();
	double CyclotronFrequency(double, int);
	void SortMITable();
	void SortFITableOnFrequency();
	void SortFITableOnMass();
	void MessageBox(CString, CString);
	void InitializeParameters();
	double GetExciteVoltage();
	void OnDebugDialogMasstofreq();
	CString CompressString(CString str);
	CString GetRightString(CString str);
	int FindEqualSign(CString str);

	// data elements
	BOOL FFTThreadOn;
	double * temp;
	int order;
	int CurrentFreqArrayPtr;
	BOOL SingleFFTFlag;
	double SICWidth;
	double SICMass;
	BOOL AutoDetectThreshold;
	BOOL SICmode;
	BOOL IsCSed;
	BOOL HaltFFTFlag;
	BOOL EnableStopFFTBtn;
	double CurrentScaleFactor;
	CString UserSetupFile;
	CMenu TwoDDataMenu;
	BOOL TransientIsDefined;
	int CurrentSpectrumFocus;
	CStatusBar* StatusBar;
	long DefaultFileNameType;
	CString DefaultSaveDirectory;
	long AutoSave;
	long SaveTruncatedTransient;
	int SetupOptionsInitPage;
	BOOL FileWasRead;
	CMainFrame* pWnd;
	BOOL IsPeakMatching;
	CBudaApp* pApp;
	struct sys* sys;
	CInstrument* Instrument;
	int SortOnProperty;
	BOOL SortIncreasing;
	//BOOL IsMIed;
	struct DispParms DisplayParms;
	struct FITableElement* FITable;
	BOOL AssignChargStates;
	long DataPointSize;
	struct PS PS;
	struct CalibrationPoint RefPeak;
	double ScaleFactor;
	BOOL IsPeakPicked;
	BOOL PeakPickFlag;
	int ADCGain;
	double TrapVoltsOnDetect;
	double PeakMatchFactor;
	int IFFTFlag;  // = 1 for FFT, -1 for IFFT
	int PeakMatching;
	double MaxIntensity;
	//BOOL IsFFTed;
	BOOL IsDefined;
	double duration;
	float* FreqArray;

	// **** vestigial data elements
	//long* lTArray;
	//short* sTArray;
	//char * cTArray;
	//CWinThread* FFTThrd;
	//struct FFTThreadInfo FFTThrdInfo;
	//int PeakFitType;
	//unsigned long FrequencyPoints;
	//double NoiseLevel;
	//int NumPeaks;
	//int ApodFunc;
	//struct DataStructure* DS;
	//double* TimeArray;
	//double TotalIonInt;

	// **** vestigial function prototypes
	//void GetMITable(int,double*);
	//void KillFFTThread();
	//void StartFFTThread();
	//BOOL RemoveTempDiskFile(int);
	//BOOL StoreTempDiskFile(int *);
	//BOOL ReadTempDiskFile(int, double*, double*);
	//void Truncate();
	//void GetTotalIonInt();
	//double PeakToGrpFit(int i, int j, BOOL *);
	//double InterpolateForRes(double*, double*, int);
	//double GetDeterminant(double, double, double, double, double, double, double, double, double);
	//BOOL ParabolicLeastSquares(double*, double*, int, double*);
	//void EstimateNoiseLevel(double *, double *);
	//void PeakFit(double *, double *, int);
	//void PeakPick();
	//void Apodize(long);
	//void Zerofill(int, long);
	//int NumZeroFills;
	//void Magnitude();
	//void RealFFT(double *data, long n, int isign);
	//void Fourier(double *data, long nn, int isign);
	//void FFT();

#ifdef _DEBUG
	virtual void AssertValid() const;
	virtual void Dump(CDumpContext& dc) const;
#endif

protected:

// Generated message map functions
protected:
	//{{AFX_MSG(CBudaDoc)
	afx_msg void OnTiminginfo();
	afx_msg void OnTestApodization();
	afx_msg void OnChargeStates();
	afx_msg void OnCalibrate();
	afx_msg void OnPeakMatch();
	afx_msg void OnUpdatePeakMatch(CCmdUI* pCmdUI);
	afx_msg void OnUpdateCalibrate(CCmdUI* pCmdUI);
	afx_msg void OnDebugWritecalfile();
	afx_msg void OnFileSaveAs();
	afx_msg void OnFileSave();
	afx_msg void OnDebugTools();
	afx_msg void OnDebugSine();
	afx_msg void OnReadTestFftFile();
	afx_msg void OnDebugMakeSineWave();
	afx_msg void OnStopfftbtn();
	afx_msg void OnUpdateStopfftbtn(CCmdUI* pCmdUI);
	afx_msg void OnRunExpButton();
	afx_msg void OnUpdateRunExpButton(CCmdUI* pCmdUI);
	afx_msg void OnTuneMode();
	afx_msg void OnUpdateTuneMode(CCmdUI* pCmdUI);
	afx_msg void OnUpdateSetupButton(CCmdUI* pCmdUI);
	afx_msg void OnChangeMemo();
	afx_msg void OnAdjustVoltages();
	afx_msg void OnToolsChefcalculator();
//	afx_msg void OnFileExport();
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};


/////////////////////////////////////////////////////////////////////////////

//{{AFX_INSERT_LOCATION}}
// Microsoft Developer Studio will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_BudaDOC_H__23A26EF2_6AF3_11D1_9106_44455354616F__INCLUDED_)
