/****************************************************************

Boston University data analysis (BUDA) package 

Description: Buda is a data analysis program for Fourier 
Transform Mass Spectrometry data.  

by Peter B. O'Connor

    Copyright (C) 2000  Peter B. O'Connor

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/
// BudaMaster.h : global constants definition.  
//					 included into most classes
/*
/****************************************************************

Boston University data analysis (BUDA) package 

Description: Buda is a data analysis program for Fourier 
Transform Mass Spectrometry data.  

by Peter B. O'Connor

    Copyright (C) 2000  Peter B. O'Connor

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#if !defined(BUDAMASTER_H) // so it only compiles once
#define BUDAMASTER_H

#define VC_EXTRALEAN
#include "stdafx.h"
#include <afx.h>

// Buda WM_User messages.  other messages are in Pemessag.h
// FFTthread messages
#define	WM_FFTDONE		(WM_USER + 2975)	// message from FFTThread to View
#define WM_FFTPROGRESS	(WM_USER + 2976)	// message from FFTThread to mainframe
#define WM_DIAGDATAUPDATE (WM_USER + 2977)	// message from InstrumenThread to Diag test Dialogs

// Instrument messages
#define WM_NEWDATAREADY	(WM_USER + 2978)

// define Serial Numbers for this system
#define SERIAL_NUMBER_SCRAMBLE_FACTOR 6
#define ENCODE_SERIAL_NUMBER

// include diagnostics into this build
#define INCLUDE_DIAGNOSTICS

// definitions used in BudaDoc.cpp
// hardcoded constant definitions
#define pi 3.141592653589793
#define MAXNUMPEAKS 10000
#define PARABOLIC_LEAST_SQUARES 0
#define MAXIMUM_PT 1
#define WEIGHTED_AVERAGE 2
#define NUM_NOISE_LEVEL_ITERATION 6
#define NOISE_LEVEL_INCLUDE_FACTOR 3
#define NOISE_LEVEL_OVERCORRECTION_FACTOR 1.1
#define PEAK_FIT_THRESHOLD 0.60
// definitions for charge state determination
#define MAX_PEAKS_PER_GROUP 100
#define MAX_NUMBER_OF_GROUPS 1000

// DEBUG FLAGS
//#define DEBUG_DUMP_DIAGS_TO_FILE

// definitions used in BudaView.cpp
// plottypes
#define NO_PLOT							0
#define TRANSIENT						1
#define FREQUENCY						2
#define MASSSPEC						3
#define TOTAL_ION_CHROMATOGRAM			4
#define STICKPLOT						5
#define RECONSTRUCTED_ION_CHROMATOGRAM	6
#define DECONVOLVED_MASS_DATA			7

// hardcoded constants
#define Buda_VIEWLINELENGTH 4096
#define ZOOMSTACKLENGTH 20
#define PLOTMINMAX 0
#define AUTOSCALE 1
#define MAXPTS 0
#define MINMAXPTS 1
#define MAXCOMBOPTS 2

// celltypes
#define ELONGATED_CUBIC 0
#define CYLINDRICAL 1

// ******* Version 6 GenPS position information *******
// now define the version 6 GenPS TTL and DAC event info.
// in version 6, the GenPS array is encoded by position.
// these parameters define those positions.

// first define the hardware bit masks for each trigger
// buildPS bit codes for each trigger.
#define EXCITE_DISABLE_BITS	0x0001
#define	EXCITE_SOURCE_BITS	0x0002
#define	RF_SOURCE_BITS		0x0004
#define	RF_PHASE_BITS		0x0008
#define	IMP_TRIGGER_BITS	0x0010
#define	SYN_TRIGGER_BITS	0x0020
#define	ARB_TRIGGER_BITS	0x0040
#define	VSP_TRIGGER_BITS	0x0080
#define	CS_GUN_GATE_BITS	0x0100
#define	QUADRF_GATE_BITS	0x0200
#define	SHUTTER_GATE_BITS	0x0400
#define	LASER_PULSE_BITS	0x0800
#define	VALVE_PULSE_BITS	0x1000
#define	USER1_PULSE_BITS	0x2000
#define USER2_PULSE_BITS	0x4000
#define	USER3_PULSE_BITS	0x8000
#define ULTIMA_SWITCH_BITS	RF_PHASE_BITS

// TTL info first
// first, define the channel numbers
#define EXCITE_DISABLE	0	// TTL's 0-7 are not user definable they
#define	EXCITE_SOURCE	1	// are used internally by the Buda electronics
#define	RF_SOURCE		2
#define	ULTIMA_SWITCH	3	// High is ESI, Low is MALDI
#define RF_PHASE		ULTIMA_SWITCH	// Vestige from prior versions
#define	IMP_TRIGGER		4
#define	SYN_TRIGGER		5
#define	ARB_TRIGGER		6
#define	VSP_TRIGGER		7
#define	LASER_PULSE		8	// TTL's 8-15 are user definable in Buda.cnf
#define	QUADRF_GATE		9	// TTL's 8-11 will have 2 symbols here
#define VALVE_PULSE		10
#define	SHUTTER_GATE	11
#define	USER1_PULSE		8
#define	USER2_PULSE		9
#define	USER3_PULSE		10
#define USER4_PULSE		11
#define	USER5_PULSE		12
#define USER6_PULSE		13
#define USER7_PULSE		14
#define USER8_PULSE		15

// now define the maximum number of GenPS entrys for each channel
#define MAX_EXCITE_DISABLE	10
#define MAX_EXCITE_SOURCE	34
#define MAX_RF_SOURCE		24
#define MAX_RF_PHASE		24	// this TTL has been reassigned to ULTIMA_SWITCH
#define MAX_IMP_TRIGGER		10
#define MAX_SYN_TRIGGER		20
#define MAX_ARB_TRIGGER		4
#define MAX_VSP_TRIGGER		240
#define MAX_LASER_PULSE		10
#define MAX_QUADRF_GATE		10
#define MAX_VALVE_PULSE		10
#define MAX_SHUTTER_GATE	10
#define MAX_USER5_PULSE		10
#define MAX_USER6_PULSE		10
#define MAX_USER7_PULSE		10
#define MAX_USER8_PULSE		10

// now define the first and last positions for each TTL
#define	FIRST_EXCITE_DISABLE	0
#define FIRST_EXCITE_SOURCE		FIRST_EXCITE_DISABLE + MAX_EXCITE_DISABLE
#define FIRST_RF_SOURCE			FIRST_EXCITE_SOURCE + MAX_EXCITE_SOURCE
#define FIRST_RF_PHASE			FIRST_RF_SOURCE + MAX_RF_SOURCE
#define FIRST_IMP_TRIGGER		FIRST_RF_PHASE + MAX_RF_PHASE
#define FIRST_SYN_TRIGGER		FIRST_IMP_TRIGGER + MAX_IMP_TRIGGER
#define	FIRST_ARB_TRIGGER		FIRST_SYN_TRIGGER + MAX_SYN_TRIGGER
#define FIRST_VSP_TRIGGER		FIRST_ARB_TRIGGER + MAX_ARB_TRIGGER
#define FIRST_LASER_PULSE		FIRST_VSP_TRIGGER + MAX_VSP_TRIGGER
#define FIRST_QUADRF_GATE		FIRST_LASER_PULSE + MAX_LASER_PULSE
#define FIRST_VALVE_PULSE		FIRST_QUADRF_GATE + MAX_QUADRF_GATE
#define FIRST_SHUTTER_GATE		FIRST_VALVE_PULSE + MAX_VALVE_PULSE
#define FIRST_USER5_PULSE		FIRST_SHUTTER_GATE + MAX_SHUTTER_GATE
#define FIRST_USER6_PULSE		FIRST_USER5_PULSE + MAX_USER5_PULSE
#define FIRST_USER7_PULSE		FIRST_USER6_PULSE + MAX_USER6_PULSE 
#define FIRST_USER8_PULSE		FIRST_USER7_PULSE + MAX_USER7_PULSE

#define LAST_EXCITE_DISABLE		FIRST_EXCITE_DISABLE + MAX_EXCITE_DISABLE - 1
#define LAST_EXCITE_SOURCE		FIRST_EXCITE_SOURCE + MAX_EXCITE_SOURCE - 1
#define LAST_RF_SOURCE			FIRST_RF_SOURCE + MAX_RF_SOURCE - 1
#define LAST_RF_PHASE			FIRST_RF_PHASE + MAX_RF_PHASE - 1
#define LAST_IMP_TRIGGER		FIRST_IMP_TRIGGER + MAX_IMP_TRIGGER - 1
#define LAST_SYN_TRIGGER		FIRST_SYN_TRIGGER + MAX_SYN_TRIGGER - 1
#define	LAST_ARB_TRIGGER		FIRST_ARB_TRIGGER + MAX_ARB_TRIGGER - 1
#define LAST_VSP_TRIGGER		FIRST_VSP_TRIGGER + MAX_VSP_TRIGGER - 1
#define LAST_LASER_PULSE		FIRST_LASER_PULSE + MAX_LASER_PULSE - 1
#define LAST_QUADRF_GATE		FIRST_QUADRF_GATE + MAX_QUADRF_GATE - 1
#define LAST_VALVE_PULSE		FIRST_VALVE_PULSE + MAX_VALVE_PULSE - 1
#define LAST_SHUTTER_GATE		FIRST_SHUTTER_GATE + MAX_SHUTTER_GATE - 1
#define LAST_USER5_PULSE		FIRST_USER5_PULSE + MAX_USER5_PULSE - 1
#define LAST_USER6_PULSE		FIRST_USER6_PULSE + MAX_USER6_PULSE - 1
#define LAST_USER7_PULSE		FIRST_USER7_PULSE + MAX_USER7_PULSE - 1
#define LAST_USER8_PULSE		FIRST_USER8_PULSE + MAX_USER8_PULSE - 1

// ok, now we must do the same thing for the DAC's
// first define the dac channels from 16 - 39
#define FIRST_DAC	16
#define	DAC_XMIT1	16
#define	DAC_XMIT2	17
#define DAC_RCVR1	18
#define DAC_RCVR2	19
#define DAC_TRAP1	20
#define DAC_TRAP2	21
#define DAC_EE		22
#define DAC_GAIN	23
#define DAC_USER1	24
#define DAC_USER2	25
#define DAC_USER3	26
#define DAC_USER4	27
#define DAC_USER5	28
#define DAC_USER6	29
#define DAC_USER7	30
#define DAC_USER8	31
#define DAC_USER9	32
#define DAC_USER10	33
#define DAC_USER11	34
#define DAC_USER12	35
#define DAC_USER13	36
#define DAC_USER14	37
#define DAC_USER15	38
#define DAC_USER16	39
#define NO_EVENT_CHANNEL 40

// double names for HiResMALDI
#define DAC_LEFT		DAC_USER1
#define DAC_RIGHT		DAC_USER2
#define DAC_UPPER		DAC_USER3
#define DAC_LOWER		DAC_USER4
#define DAC_REPELLER	DAC_USER5
#define DAC_EXTRACTOR	DAC_USER6
#define DAC_QOFFSET		DAC_USER7

//now define the number of GenPS entries available for each DAC
#define MAX_XMIT1		10
#define MAX_XMIT2		10
#define MAX_RCVR1		10
#define MAX_RCVR2		10
#define MAX_TRAP1		10
#define MAX_TRAP2		10
#define MAX_EE			10
#define MAX_GAIN		10
#define MAX_USER1_DAC	10
#define MAX_USER2_DAC	10
#define MAX_USER3_DAC	10
#define MAX_USER4_DAC	10
#define MAX_USER5_DAC	10
#define MAX_USER6_DAC	10
#define MAX_USER7_DAC	10
#define MAX_USER8_DAC	10
#define MAX_USER9_DAC	10
#define MAX_USER10_DAC	10
#define MAX_USER11_DAC	10
#define MAX_USER12_DAC	10
#define MAX_USER13_DAC	10
#define MAX_USER14_DAC	10
#define MAX_USER15_DAC	10
#define MAX_USER16_DAC	10

#define FIRST_XMIT1			FIRST_USER8_PULSE + MAX_USER8_PULSE
#define FIRST_XMIT2			FIRST_XMIT1 + MAX_XMIT1
#define FIRST_RCVR1			FIRST_XMIT2 + MAX_XMIT2
#define FIRST_RCVR2			FIRST_RCVR1 + MAX_RCVR1
#define FIRST_TRAP1			FIRST_RCVR2 + MAX_RCVR2
#define FIRST_TRAP2			FIRST_TRAP1 + MAX_TRAP1
#define FIRST_EE			FIRST_TRAP2 + MAX_TRAP2
#define FIRST_GAIN			FIRST_EE + MAX_EE
#define FIRST_USER1_DAC		FIRST_GAIN + MAX_GAIN
#define FIRST_USER2_DAC		FIRST_USER1_DAC + MAX_USER1_DAC
#define FIRST_USER3_DAC		FIRST_USER2_DAC + MAX_USER2_DAC
#define FIRST_USER4_DAC		FIRST_USER3_DAC + MAX_USER3_DAC
#define FIRST_USER5_DAC		FIRST_USER4_DAC + MAX_USER4_DAC
#define FIRST_USER6_DAC		FIRST_USER5_DAC + MAX_USER5_DAC
#define FIRST_USER7_DAC		FIRST_USER6_DAC + MAX_USER6_DAC
#define FIRST_USER8_DAC		FIRST_USER7_DAC + MAX_USER7_DAC
#define FIRST_USER9_DAC		FIRST_USER8_DAC + MAX_USER8_DAC
#define FIRST_USER10_DAC	FIRST_USER9_DAC + MAX_USER9_DAC
#define FIRST_USER11_DAC	FIRST_USER10_DAC + MAX_USER10_DAC
#define FIRST_USER12_DAC	FIRST_USER11_DAC + MAX_USER11_DAC
#define FIRST_USER13_DAC	FIRST_USER12_DAC + MAX_USER12_DAC
#define FIRST_USER14_DAC	FIRST_USER13_DAC + MAX_USER13_DAC
#define FIRST_USER15_DAC	FIRST_USER14_DAC + MAX_USER14_DAC
#define FIRST_USER16_DAC	FIRST_USER15_DAC + MAX_USER15_DAC

#define LAST_XMIT1			FIRST_XMIT1 + MAX_XMIT1 - 1
#define LAST_XMIT2			FIRST_XMIT2 + MAX_XMIT2 - 1
#define LAST_RCVR1			FIRST_RCVR1 + MAX_RCVR1 - 1
#define LAST_RCVR2			FIRST_RCVR2 + MAX_RCVR2 - 1
#define LAST_TRAP1			FIRST_TRAP1 + MAX_TRAP1 - 1
#define LAST_TRAP2			FIRST_TRAP2 + MAX_TRAP2 - 1
#define LAST_EE				FIRST_EE + MAX_EE - 1
#define LAST_GAIN			FIRST_GAIN + MAX_GAIN - 1
#define LAST_USER1_DAC		FIRST_USER1_DAC + MAX_USER1_DAC - 1
#define LAST_USER2_DAC		FIRST_USER2_DAC + MAX_USER2_DAC - 1
#define LAST_USER3_DAC		FIRST_USER3_DAC + MAX_USER3_DAC - 1
#define LAST_USER4_DAC		FIRST_USER4_DAC + MAX_USER4_DAC - 1
#define LAST_USER5_DAC		FIRST_USER5_DAC + MAX_USER5_DAC - 1
#define LAST_USER6_DAC		FIRST_USER6_DAC + MAX_USER6_DAC - 1
#define LAST_USER7_DAC		FIRST_USER7_DAC + MAX_USER7_DAC - 1
#define LAST_USER8_DAC		FIRST_USER8_DAC + MAX_USER8_DAC - 1
#define LAST_USER9_DAC		FIRST_USER9_DAC + MAX_USER9_DAC - 1
#define LAST_USER10_DAC		FIRST_USER10_DAC + MAX_USER10_DAC - 1
#define LAST_USER11_DAC		FIRST_USER11_DAC + MAX_USER11_DAC - 1
#define LAST_USER12_DAC		FIRST_USER12_DAC + MAX_USER12_DAC - 1
#define LAST_USER13_DAC		FIRST_USER13_DAC + MAX_USER13_DAC - 1
#define LAST_USER14_DAC		FIRST_USER14_DAC + MAX_USER14_DAC - 1
#define LAST_USER15_DAC		FIRST_USER15_DAC + MAX_USER15_DAC - 1
#define LAST_USER16_DAC		FIRST_USER16_DAC + MAX_USER16_DAC - 1
// ******* end Version 6 GenPS informaiton *******

// Periodic Table Structures
struct Isotope
{
	CString Isoname;
	double Isomass;
	double IsoAbund;
	int IsoNum;
};

struct Element
{
	CString ElementName;
	struct Isotope* Isos;
	int NumIsos;
	CString ElementSymbol;
	double AverageMass;
};

// define CalibrationPoint structure
struct CalibrationPoint {
	double ActualMass;
	double MeasuredFrequency;
	float TrapVoltage;
	float TotalIntensity;
	long IncludeInCalc;
	char TimeEntered[40];
};

// define CalibrationPoint structure
struct PeakMatchPoint {
	double ActualMass;
	double MeasuredFrequency;
	float TrapVoltage;
	float TotalIntensity;
	char TimeEntered[40];
};

/* This section is the description of the pulse sequence structure

	The pulse sequence is an array of struct PSDesc.  Each element
	in this array is a description of an event with.  The names 
	self explanatory.
*/
// definition of the pulse sequence event struct
// version 8 files store the pulse sequence with each event 
// denoted by a channel number, older versions denote channel
// number by position in the events array.
struct PSDesc {
	long Channel;	// channels 0-15 are TTL pulses, channels 16-39 are DACs
	long Type;
	long Linking;
	long Link2Event;
	float Delay;
	float Width;
	float Value;
};

// the older version 6 structure
struct PSDesc6 {
	long Type;
	long Linking;
	long Link2Event;
	float Delay;
	float Width;
	float Value;
};

// "Type"s used in PSDesc pulse sequence event struct
#define NO_EVENT	0
#define TTL_PULSE	1	// this event is a ttl
#define	DAC_INIT	2	// this event initializes the DAC, delay is meaningless
#define DAC_STEP	3	// this event is an simple DAC voltage change, width is meaningless
#define DAC_RAMP	4	// this event is a DAC voltage ramp from current voltage to Value
#define DAC_PULSE	5	// this event is a DAC voltage step, with a return to the current voltage after Width
#define DAC_QUENCH	6	// this event is a vestige, but was intended to do the quench pulses
#define BLANK_EVENT	7	// this event is a signal for Setup part of the program
#define NO_CHANNEL	NO_EVENT_CHANNEL

// Linking constants
#define NO_LINK		0
#define LINK_BEG	1
#define LINK_END	2

// Internal DAC and TTL channel numbers and names.
//	The first 0-7 TTL pulses are used internal to the Buda
//	electronics, the 8-15 TTL pulses are user definable, but 
//	8-11 have default names
/* this stuff is defined above already
#define	FIRST_PULSE		0
#define	EXCITE_DISABLE	0
#define	EXCITE_SOURCE	1
#define	RF_SOURCE		2
#define	RF_PHASE		3
#define	IMP_TRIGGER		4
#define	ARB_TRIGGER		5
#define	SYN_TRIGGER		6
#define	VSP_TRIGGER		7
#define	LASER_PULSE		8
#define	QUADRF_GATE		9
#define	VALVE_PULSE		10
#define	SHUTTER_GATE	11
#define	USER5_PULSE		12
#define	USER6_PULSE		13
#define	USER7_PULSE		14
#define	USER8_PULSE		15
#define	LAST_PULSE		15

// DAC channel names and numbers
#define	FIRST_DAC		16
#define	DAC_XMIT1		16
#define	DAC_XMIT2		17
#define	DAC_RCVR1		18
#define	DAC_RCVR2		19
#define	DAC_TRAP1		20
#define	DAC_TRAP2		21
#define	DAC_EE			22
#define	DAC_GAIN		23
#define	DAC_USER1		24
#define	DAC_USER2		25
#define	DAC_USER3		26
#define	DAC_USER4		27
#define	DAC_USER5		28
#define	DAC_USER6		29
#define	DAC_USER7		30
#define	DAC_USER8		31
#define	DAC_USER9		32
#define	DAC_USER10		33
#define	DAC_USER11		34
#define	DAC_USER12		35
#define	DAC_USER13		36
#define	DAC_USER14		37
#define	DAC_USER15		38
#define	DAC_USER16		39
#define	LAST_DAC		39

// duplicate definitions
#define	DAC_LEFT		DAC_USER1
#define	DAC_RIGHT		DAC_USER2
#define	DAC_UPPER		DAC_USER3
#define	DAC_LOWER		DAC_USER4
#define	DAC_REPELLER	DAC_USER5
#define	DAC_EXTRACTOR	DAC_USER6
#define	DAC_QUADOFFSET	DAC_USER7
*/ 

// Calibration of the dac voltages to specific base2 values
struct DacPacket {
	long Units;
	float MaxValue;
	float MinValue;
	float MaxDac;
	float MinDac;
	float MaxGraph;
	float MinGraph;
	long DacAvailable;	// 0 = not available, 1 = available
};

// end of Pulse Sequence information

// define flagpacket needed by Cinstrument during data acquisition
struct FlagPacket {
	BOOL SetupValid;	// can do an experiment
	BOOL PSGLoaded;		// pulse sequence generator hardwere loaded
	BOOL VSPLoaded;		// voltage hardware loaded
	BOOL FSPLoaded;		// synthesizer hardware loaded
	BOOL ARBLoaded;		// arb generator loaded

		// most of the rest of these flags are vestigial - no longer used in v8.0.
		// just keep them for temporary compatibility's sake
	BOOL HaveMITable;	//mi table in memory
	BOOL HaveTransient;	// time domain data in memory
	BOOL HaveData;		// mi or analog
	BOOL DataStored;	// data stored to disk
	BOOL PlotOnScreen;	// valid graph on screen
	BOOL SplitAxis;		// double axis mode for spectra
	BOOL TuneUp;		// true when in tune up mode
	BOOL SortByAmplitude;	// otherwise, sort by mass
	BOOL scopeMode;
};
struct DispParms {
	BOOL massrange;
	BOOL acqrate;
	BOOL apodfunc;
	BOOL calconst;
	BOOL exciteamp;
	BOOL filename;
	BOOL notes;
	BOOL numpts;
	BOOL numscans;
	BOOL numzerofills;
	BOOL allparms;
	BOOL timedate;
	BOOL trapvolts;
	BOOL noisethreshold;
	BOOL labelpoints;
	BOOL LabelPeakPos;
	BOOL LabelCharge;
	BOOL PosNeg;
	BOOL ScaleFactor;
};

// define Chirp Function Packet constants and structure
#define CFP_NONE 0
#define CFP_BURST CFP_NONE+1
#define CFP_SWEEP CFP_BURST+1
#define CFP_IMPTRIG CFP_SWEEP+1
#define CFP_QUAD CFP_IMPTRIG+1

struct ChirpPacket {
	long Type;
	float Amplitude;
	double StartMass;
	double EndMass;
};

// define Arbitrary Waveform Packet constants and structure
#define MAX_ARB_EVENTS 10
#define ARB_NONE 0
#define ARB_CENTER ARB_NONE+1
#define ARB_RANGE  ARB_CENTER+1
#define ARB_NOTCH  ARB_RANGE+1
#define ARB_BURST  ARB_NOTCH+1
#define ARB_SWEEP  ARB_BURST+1
#define ARB_ARB    ARB_SWEEP+1
#define ARB_USER   ARB_ARB+1
#define ARB_UNKNOWN 0
#define ARB_GOOD ARB_UNKNOWN+1
#define ARB_FAIR ARB_GOOD+1
#define ARB_POOR ARB_FAIR+1
#define ARB_UNDEFINED 0
#define ARB_CALCULATED ARB_UNDEFINED+1
#define ARB_FILE ARB_CALCULATED+1

struct ArbEvent {
	long Type;		// as above
	long fit;	
	double MassA;	// either begin or center
	double MassB;	// either end or delta m/z
	float Amplitude;	// 0-100%
	long reserved;		// only here to preserve 8-byte boundary
};

struct ArbPacket {
	float WindowLowMass;	// display window for screen
	float WindowHighMass;
	long DacFixed;			// boolean : is the user defining the DAC rate or is this done by the computer
	long SamplesFixed;		// boolean : is the user defining the number of samples or is this done by the computer
	long FilterFixed;		// boolean : is the user defining the filter setting or is this done by the computer
	long AmplitudeFixed;	// boolean : is the user defining the Amplitude or is this done by the computer
	double DacRate;			// dac rate at which to clock out waveform
	long Samples;			// number of points in waveform
	long Filter;			// filter setting 0-4
	float Amplitude;		// Vbp of the maxima of the waveform
	long Setup;	
	long PacketSize;		// used in the stepped phaseing algorithm
	float PacketSpacing;	// used in the stepped phasing algorithm
	struct ArbEvent Event[MAX_ARB_EVENTS];	// the array of notches, windows, etc as defined above
};

struct FITableElement {  // elements of FITable
public:
	double F;	// frequency of the peak
	double I;	// peak Intensity
	double A;	// peak area
	double RP;	// peak resolving power
	int CS;		// peak charge state
};

struct groupstruct {
	double average;
	double stdev;
	double mass;
	int chargestate;
	int numpeaks;
	int MITableIndex[MAX_PEAKS_PER_GROUP];
}; 

// constants for file types
#define SETUP_FILE					0xeee1
#define SINGLE_TRANSIENT_FILE		0xeea1
#define MULTIPLE_TRANSIENT_FILE		0xeea2
#define SPECTRUM_FILE				0xeec1
#define MASS_INTENSITY_FILE			0xeeb1
#define VERSION_8_COMPOSITE_FILE	0xeed1
#define CHROMATOGRAPHYTEMPFILE		0xeed2
#define BRUKERFILE					0xbbbb

#define CURRENT_MAJOR_VERSION_NUMBER 8;
#define CURRENT_MINOR_VERSION_NUMBER 0;

//****************************************************
//
//	PS experiment data structure
//	this is the structure of the header to experiment
//  data files
//
//****************************************************
// constants used in PS
#define MAX_SPECTRA			200
#define MAX_REF_PEAK		2
#define MAX_CALIB_POINTS	20
#define MAX_CALIBRANT_LIST  1000
#define MAX_SYN_TRIGGERS	20
#define MAX_DACS			24
#define MAX_ARB_TRIGGERS	4
#define TOTAL_PS_EVENTS		687
#define MAX_PS_EVENTS		687

// PS Version 6.x Structure size
#define PS6_NUM_BYTES 21996  //number of bytes in V6.x PS structure
#define NO_LOAD_TIMEARRAY	1
#define LOAD_TIMEARRAY 		0

// ScanModes
#define	NOSCAN		0
#define DELAYSCAN	1
#define WIDTHSCAN	2
#define	PROFILE		3

// PS Structure
struct PS {
	long	FileType;
	long	MajorVersion;
	long	MinorVersion;
	char	SystemName[128];
	char	SystemSerialNumber[32];
	char	UserID[8];
	char	DataFileName[64];
	char	TimeAcquired[40];
	char	TimeProcessed[32];
	long	HiResMALDI;
	long	HiResESI;
	char	ExperimentMemo[60];
	char	ExperimentNotes[600];
	short	DetectMode;
	short	ADCGain;
	long	DetectEvent;
	double	DetectLowMass;
	double	DetectHighMass;
	double	MonitorMass;
	double	MixerFrequency;
	double	AdcRate;
	long	ScanMode;
	long	TransientPoints;
	long	SpectraRequested;
	long	SpectraAcquired;
	long	CurrentSpectrum;
	long	ScansPerSpectrum;
	long	ScansRequested;
	long	TriggerMode;
	long	CalibrationVersion;
	double	A;
	double	B;
	double	HighMassShift;
	double	TrapShift;
	double	IntensityShift;
	double	NoiseThreshold; // in % of base peak
	double	TotalIonIntensity; // in m/z units
	long	PeakMatching;
	double	PeakMatchFactor;
	struct	PeakMatchPoint RefPeak[MAX_REF_PEAK];
	long	SpectrometerType;
	long	DCRemoval;
	long	WindowFunction;
	long	ZeroFills;
	long	TruncateTransient;
	long	TruncatePoints;
	long	TruncateOffset;
	long	PeakFitType;
	long	FFTSize;
	long	SpectrumPoints;
	long	FrequencyDataType;
	long	ZoomFactor;
	double	ZoomLowMass;
	double	ZoomHighMass;
	double	ZoomFrequencyShift;
	long	ZoomTransientPoints;
	long	ZoomPointShift;
	long	TransientIsStored; // = 0, 8, 16, 32 (if > 0 then = num bits per entry
	long	FrequencyIsStored; // = 0, 8, 16, 32 (if > 0 then = num bits per entry
	long	MITableIsStored; // = 0 if not stored, = 1 if stored
	//long	IsFFTed;	// 0 if false, 1 if true
	//long	IsMIed;		// 0 if false, 1 if true
	long	NumEvents;	// total number of pulse sequence events in GenPS
	long	NumCFP;		// total number of cfp events
	long	NumDAC;		// number of dacs - usually 24
	long	NumARB;		// total number of ARB events
	struct ChirpPacket*	cfp;
	struct DacPacket*	dac;
	struct ArbPacket*	arb;
	struct PSDesc*		GenPS;
	struct Spectrum*	Spec;
}; // end PS8

// define structure that holds the data
// each spectrum in the document is defined by one of these
// note: this structure was chosen for 2dimensionality of data
// all elements in here are those needed by each spectrum of a 2-d
// data set anything that is true for all spectra in the set will
// be defined in PS
struct Spectrum {
	double ScaleFactor;  // = 100/highest point in spectrum domain
	double TotalIonInt;
	double NoiseLevel;
	long NumPeaks;
	long ElapsedTime;
	long IsMIed;
	long TArrayFileOffset;
	//void* TimeArray;
	struct FITableElement* FITable;
public:
	BOOL IsFFTed;
};

// Detection Modes
#define DirectMode 0
#define MixerMode 1
#define DIRECT_MODE DirectMode
#define MIXER_MODE  MixerMode

// Acquisition Modes
#define Single	0
#define Chromatography	1
#define TimeScan	2
#define VaryParm	3

// Instrument definitions
#define DEFAULT_INSTRUMENT	0
#define EI_INSTRUMENT		0
#define MALDI_INSTRUMENT	1
#define ESI_INSTRUMENT		2

#define MAX_NUM_VACUUM_SYSTEMS 3
// note: VS 0 is the hard-coded program defaults
//		 VS 1 is the MALDI defaults from the Buda.cnf file
//       VS 2 is the ESI defaults from the Buda.cnf file

// SYS structure definition
struct sys {
public:
	long UltimaSwitch;
	long CellType;
	long MajorVersion;			// of the program
	long MinorVersion;			// of the program
	CString UserID;				// who is logged in now
	CString SystemName;			// the unique name of this system
	CString SystemSerialNumber;
	long HaveImpulse;			// is there an impulse amplifier on this system?
	long HaveSynthesizer;		// is there a frequency synthesizer
	long HaveArbGenerator;		// is there an ARB
	long ProgrammableFilter;	// before ADC
	long ProgrammableGain;		// multiple gain settings for amp
	long ArbGeneratorMemory;	// in MB
	long ArrayProcessorMemory;	// in MB
	long AdcBits;				// 9, 10, 12, 16, ...
	double MaxAdcRate;			// samples per second
	long DigitizerMemory;		// in K samples
	CString DataPath;		// data directory
	CString BudaPath;		// program is here
	CString TempPath;		// for OMEGA temp work files
	CString UserIDFile;		// UserIDFile
	CString UserSetupFile;  // the current user's setup file
	CString DefaultSetupFile;	// the file to be automatically loaded on startup
	long DCRemoval;			// default mode
	long WindowFunction;	// default apodization function
	long TransientPointsUsed; // default # (-1 = maximum possible)
	long ZeroFills;			// default # (-1 = use table)
	long FrequencyDataType;	// default (complex or magnitude)
	double SystemLowMass;	// lowest for this config
	double SystemHighMass;	
	double SystemLowFrequency; // Lowest Frequency 
	double SystemHighFrequency;
	long TransientMemoryLimit; // largest transient that will fit in memory (in pts) 
	long TransientFFTLimit;	// largest transient that can be fft'ed
	long APFFTLimit;			// largest FFT size (on array processor)
	double PeakRFAmpVoltage;	// e.g. 200.0 V(b-p)
	double MinRFAmpVoltage;	// e.g. 1.2 V(b-p)
	double PeakQuadRFVoltage; // e.g. 600.0 V(b-p)
	double RFAmpFixedGain;	// e.g. 202.2 Gain (v/v)
	double QuadRFFixedGain;	// e.g. 667.1 Gain (v/v)
	double GainControlVoltageDivider;	// probably ~ 6.0
	double SynOutputVoltage;	// probably ~0.75 V(b-p)
	double ArbOutputVoltage;	// probably ~0.6 V(b-p)
	double QuadFrequency; // Quad freqs for this system
	double MixerCorrection;	// in ppm shift
	long PsgTicksPerMillisecond;
	long CalibrationLoaded;	// from startup disk file
	double A;				// system cal constants - not the same as the ones in PS
	double B;				// as the user can change those without changing these defaults
	double HighMassShift;	// in Hz/sqrt(m)
	double TrapShift;		// in Hz/Volt
	double IntensityShift;	// in Hz/(volt*intensity)
	long CalibrationVersion; // from the file
	long CalibrationPoints;	// # pts in cal file
	CString CalibrationFile;	// name of the file
	CString CalDate;		// time and date calibration saved
	CString CalDescription;
	struct CalibrationPoint CalData[MAX_CALIB_POINTS];
	double PeakPickerThreshold;	// negative means autoselect
	double PeakPickerMeanFactor;	
	long PeakPickerLoopCount;	// # loops to find threshold
	double PeakPickerLevel;	// how much of each peak to use in fitting (usually 60%)
	double PeakPickerLowMass; // note 0 and 0 means auto
	double PeakPickerHighMass;
	double MinimumAutoThreshold; // not below this value
	long AutoLabelPeaks;		// for spectra
	long AutoLabelCharge;		// for spectra
	long LabelCharge1;			// lable singly charged
	long ChargeLimit;			// for charge state determination
	long FlipFlopRFPhase;		// alternate RF +/- phase
	double MagnetDrift;			// in ppm/hour
	long FSPType;				// type of programmer
	long HaveNBMonitor;			// adc narrowband monitor
	long HiResMALDI;			// MALDI Flag special setup
	long HiResESI;				// ESI special setup
	long BootMALDI;				// startup in MALDI mode
	long BootESI;				// startup in ESI mode
	double LaserTriggerDelay;	// delay of laser firing from trigger
	struct DacPacket dac[MAX_DACS];
	long CorrectTotalIntensity; // adjust total intensity?
	long ExcludeNoResPeaks;		// exclude peaks that have undefined resolution from total intensity
	CString ChannelNames[40];	// LaserTrigger, Pulsed Valve, etc.
	CString UserDir;			
	long NumVacuumSystems;		// usually 1 or 2 for ESI, MALDI.  this is also the number of sys structures
	BOOL ADCRateAvailable[16];	// is rate[x] available or not?
	CString ADCRateString[16];	// the display string for this rate e.g. "8 MHz"
	double ADCRateList[16];		// the actual double-floating point rate values e.g. 8000000.000
	BOOL DacAvailable[24];		// is the dac available or not?
	BOOL TTLAvailable[16];		// is the TTL available or not?
	//struct VSSpecificInfo VSInfo[MAX_NUM_VACUUM_SYSTEMS];
}; // end sys structure

//////////////////////////////////////////
//	data structures for FFTWorkerThread
//////////////////////////////////////////

class CBudaDoc;

struct FFTThreadInfo
{
	struct FFTQueueEntry* CurrentFFT;
	int FFTsInQueue;
	int ThreadStatus;	// used for thread to communicate with BudaApp
						// 0 = idle
						// 1 = calc in stage 1 (setup, truncate, zerofill, apodize)
						// 2 = calc in stage 2 (FFT)
						// 3 = calc in stage 3 (magnitude mode)
						// 4 = calc in stage 4 (GetMITable, cleanup)

	// define FFT control events
	HANDLE m_hStartThread;	// event used to start the thread from its initial stopped state
	HANDLE m_hKillThread;	// event used to kill FFTthread from ExitInstance
	HANDLE m_hStartFFT;		// event used to start the fft calc from BudaApp/Doc
	HANDLE m_hStopFFT;		// event used to stop the fft calc and clear the current fft queue
	HANDLE m_hThreadDead;	// event used by thread to notify BudaApp that it is dead
	//HWND m_hwndNotifyFFTProgress;  // NotifyFFTDone will be sent to the view, NotifyFFTProgress will be sent to MainWnd
	CWnd* pMainWnd;
};

struct FFTQueueEntry
{
	HANDLE m_hFFTDone;		// event used by thread to notify BudaApp that it is done 
							//	with an FFT so that the App can use the data
	HWND m_hwndNotifyFFTDone;	// hwnds for postmessages - will be set to appropriate windows
	struct PS* pPS;
	CBudaDoc* pDoc;
	float* FTData;
	int DSIndex;
	long HighMassPt;
	long LowMassPt;
	struct FFTQueueEntry* NextFFT;
	BOOL EndofQueue;				// if TRUE, NextFFT has no meaning
									// if FALSE, NextFFT holds the data for the next FFT on the queue
};

// Charge State Determination algorithm definitions.

#define CSDET_DELTAMASS			0
#define CSDET_MULTIPLICATIVE	1
#define CSDET_FOURIER			2
#define CSDET_COMBO				3
#define CSDET_AUTOCORR			4
#define CSDET_OCONNOR			5
#define CSDET_ENTROPY			6
#define CSDET_SPARE1			7

#define CS_DEFAULT_MAX_CHARGE 50
#define CS_DEFAULT_DETERMINATION_METHOD CSDET_FOURIER

// Noise Level determination algorithm definitions
#define ITERATIVE_AVERAGING_METHOD	1
#define HISTOGRAM_METHOD			2
#define MI_DEFAULT_THRESHOLD_METHOD ITERATIVE_AVERAGING_METHOD

// Deconvolution definitions
#define DECON_FENN		0
#define DECON_REINHOLD	1
#define DECON_OCONNOR	2
#define DECON_ZHANG		3
#define DECON_DEFAULT   DECON_FENN
#define DECON_DEFAULT_HIGH_MASS_LIMIT 20000.0
#define DECON_DEFAULT_LOW_MASS_LIMIT 5000.0
#define DECON_DEFAULT_HIGH_CHARGE_LIMIT 20
#define DECON_DEFAULT_LOW_CHARGE_LIMIT 1
#define DECON_DEFAULT_RESOLUTION 0.1
#define MAX_DECON_SIZE 1048576
#define DECON_DEFAULT_ADDUCTMASS 1.007276465
#define DECON_DEFAULT_ISOTOPICALLY_RESOLVED 1

#endif // define BUDAMASTER_H
