/****************************************************************

Boston University data analysis (BUDA) package 

Description: Buda is a data analysis program for Fourier 
Transform Mass Spectrometry data.  

by Peter B. O'Connor

    Copyright (C) 2000  Peter B. O'Connor

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/
// BudaView.h : interface of the CBudaView class
//
/////////////////////////////////////////////////////////////////////////////

#if !defined(AFX_BUDAVIEW_H__23A26EF4_6AF3_11D1_9106_44455354616F__INCLUDED_)
#define AFX_BUDAVIEW_H__23A26EF4_6AF3_11D1_9106_44455354616F__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

// constants specific to CBudaView
#define VSCALE_FUDGE_FACTOR 20;

// declare classes that will be needed so that I
// can use interlinked includes
class CDeconvolution;

// end constants
class CBudaView : public CView
{
protected: // create from serialization only
	CBudaView();
	DECLARE_DYNCREATE(CBudaView)

// Attributes
public:
	CBudaDoc* GetDocument();

// Operations
public:

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CBudaView)
	public:
	virtual void OnDraw(CDC* pDC);  // overridden to draw this view
	virtual BOOL PreCreateWindow(CREATESTRUCT& cs);
	virtual void OnInitialUpdate();
	virtual BOOL OnScroll(UINT nScrollCode, UINT nPos, BOOL bDoScroll = TRUE);
	protected:
	virtual BOOL OnPreparePrinting(CPrintInfo* pInfo);
	virtual void OnBeginPrinting(CDC* pDC, CPrintInfo* pInfo);
	virtual void OnEndPrinting(CDC* pDC, CPrintInfo* pInfo);
	virtual void OnActivateView(BOOL bActivate, CView* pActivateView, CView* pDeactiveView);
	virtual void OnUpdate(CView* pSender, LPARAM lHint, CObject* pHint);
	//}}AFX_VIRTUAL

// Implementation
public:
	void OnDebugDialogMasstofreq();
	void SetScrollbarPos();
	void SetScrollbarSize();
	void SetupScrollbar();
	SCROLLINFO Scroll_Info;
	CDeconvolution* Decon;
	int FlagCSDetState;
	BOOL AnnotationsInRange(double mass, double intensity, double textwidth);
	double GlobalMin;
	double GlobalMax;
	BOOL UpdateScaleFactor;
	void GetScaleFactor();
	BOOL AddAnnotation(double mass, double intensity, int MIindex, CString text);
	void PlotCSLabels();
	double VScale;
	int TwoDFFTStatus;
	int TwoDFFTPlotType;
	BOOL TwoDFFT;
	// copied from the protected region below
	afx_msg void OnPlotMassSpectrum();
	afx_msg void OnPlotFrequency();
	afx_msg void OnPlotTransient();

	void OnPEFilePrint();
	BOOL Printing;
	void AddTextAnnotation(double x, double y, long Num, CString* text);
	BOOL FirstPlot;
	void CreateNBMIList();
	int NumMI;
	struct MIList* NBMIList;
	BOOL InitialView;
	int PostFFTPlotType;
	HANDLE m_hFFTDone;
	void OnFFTDone();
	int CurrentFreqArrayPtr;
	void CalcDeconViewLine();
	double TwoDSpecnumToMin(int);
	int OldPlotType;
	BOOL ViewAsStickplot;
	void OnPlotStickPlot();
	int TwoDMinToSpecnum(double);
	int TICLineFocus;
	CMenu TwoDDataMenu;
	void Update2DDataMenu();
	void ResetMenus();
	void CalcStickPlotViewLine();
	void CalcTICViewLine();
	CFrameWnd* pFrame;
	CStatusBar* StatusBar;
	CBudaApp* pApp;
	void CurrentXYPos();
	void OnPlot();
	double GetMITableMass(long);
	double GetMITableMass(struct FITableElement *);
	void PlotPeakPos();
	void DumpToDiagFile(CString);
	BOOL PeaksInRange(int ptind, double x, double y, double width);
	BOOL PtsInRange(double mass, double intensity, double textwidth);
	long NumAnnotations;
	void PlotPointLabels();
	double ViewLineMiny;
	double ViewLineMaxy;
	void PlotDisplayParameters();
	void GraphClicked();
	double scale;
	BOOL IsZoomingY;
	void WriteParms(CDC*, CRect);
	void CreatePlotWindow();
	int ViewLineType;
	void SetViewLineLength();
	void CalcMassSpecViewLine();
	void CalcFrequencyViewLine();
	void CalcTransientViewLine();
	void ScaleViewLine();
	int ViewLineScaling;
	bool ViewIsSetup;
	void ZoomViewLineOut();
	void ZoomViewLineIn();
	void InitializeZoomStack();
	int zoompointer;
	struct zoomwindow {
		//unsigned long minx;
		//unsigned long maxx;
		double minx;
		double maxx;
		double miny;
		double maxy;
	} zoomstack[20];
	BOOL OnCommand(WPARAM wp, LPARAM lp);
	//void OnInitialUpdate();
	int PlotType;
	BOOL ViewIsDirty;
	float * ViewLineXPts;
	BOOL PEIsDefined;
	BOOL ViewLineIsDirty;
	int ViewLineLength;
	int OldViewLineLength;
	float* ViewLine;
	void PlotViewLine();
	HWND hWndPE;
	void CalculateViewLine();
	CBudaDoc* pDoc;
	virtual ~CBudaView();
#ifdef _DEBUG
	virtual void AssertValid() const;
	virtual void Dump(CDumpContext& dc) const;
#endif

protected:

// Generated message map functions
protected:
	//{{AFX_MSG(CBudaView)
	afx_msg void GetFreqData(BOOL Do2DFFT);
	afx_msg void OnSize(UINT nType, int cx, int cy);
	afx_msg void OnNewwindow();
	afx_msg void OnSetDisplayRange();
	afx_msg void OnFilePrint();
	afx_msg void OnUpdateZoomY(CCmdUI* pCmdUI);
	afx_msg void OnZoomY();
	afx_msg void OnZoomOut();
	afx_msg void OnEditCopy();
	afx_msg void OnMouseMove(UINT nFlags, CPoint point);
	afx_msg void OnViewChromatogram();
	afx_msg void On2dSetSpec();
	afx_msg void OnDiagsKeytest();
	afx_msg void On2dSetSpecNext();
	afx_msg void On2dSetSpecPrev();
	afx_msg void OnViewStickplot();
	afx_msg void OnUpdateViewStickplot(CCmdUI* pCmdUI);
	afx_msg void OnUpdateViewChromatogram(CCmdUI* pCmdUI);
	afx_msg void OnUpdateViewDecon(CCmdUI* pCmdUI);
	afx_msg void OnViewDecon();
	afx_msg void OnAnalyzeSetscalefactor();
	afx_msg void OnAnalyzeSetpeakpickthreshold();
	afx_msg void OnUpdateAnalyzeSetpeakpickthreshold(CCmdUI* pCmdUI);
	afx_msg void OnUpdateCSDet(CCmdUI* pCmdUI);
	afx_msg void OnCSDet();
	afx_msg void OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar);
	afx_msg void OnViewCopyCursorXPos();
	afx_msg void OnFileExport();
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

#ifndef _DEBUG  // debug version in BudaView.cpp
inline CBudaDoc* CBudaView::GetDocument()
   { return (CBudaDoc*)m_pDocument; }
#endif

/////////////////////////////////////////////////////////////////////////////

//{{AFX_INSERT_LOCATION}}
// Microsoft Developer Studio will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_BUDAVIEW_H__23A26EF4_6AF3_11D1_9106_44455354616F__INCLUDED_)
