// CSDetParmsDlg.cpp : implementation file
//
/****************************************************************

Boston University data analysis (BUDA) package 

Description: Buda is a data analysis program for Fourier 
Transform Mass Spectrometry data.  

by Peter B. O'Connor

    Copyright (C) 2000  Peter B. O'Connor

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "stdafx.h"
#include "buda.h"
#include "BudaMaster.h"
#include "CSDetParmsDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CCSDetParmsDlg property page

IMPLEMENT_DYNCREATE(CCSDetParmsDlg, CPropertyPage)

CCSDetParmsDlg::CCSDetParmsDlg() : CPropertyPage(CCSDetParmsDlg::IDD)
{
	//{{AFX_DATA_INIT(CCSDetParmsDlg)
	m_HighChargeLimit = 0;
	//}}AFX_DATA_INIT
}

CCSDetParmsDlg::~CCSDetParmsDlg()
{
}

void CCSDetParmsDlg::DoDataExchange(CDataExchange* pDX)
{
	CPropertyPage::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CCSDetParmsDlg)
	DDX_Text(pDX, IDC_Highest_CS_Charge, m_HighChargeLimit);
	DDX_Control(pDX, IDC_CSMETHOD_DELTAMASS, m_DeltaMassBtn);
	DDX_Control(pDX, IDC_CSMETHOD_PATTERSON, m_PattersonBtn);
	DDX_Control(pDX, IDC_CSMETHOD_FOURIER, m_FourierBtn);
	DDX_Control(pDX, IDC_CSMETHOD_COMBO, m_ComboBtn);
	DDX_Control(pDX, IDC_CSMETHOD_AUTOCORRELATION, m_AutoCorrBtn);
	DDX_Control(pDX, IDC_CSMETHOD_CROSSCORR, m_CrossCorrBtn);
	DDX_Control(pDX, IDC_CSMETHOD_ENTROPY, m_EntropyBtn);
	DDX_Control(pDX, IDC_CSMETHOD_SPARE1, m_Spare1Btn);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CCSDetParmsDlg, CPropertyPage)
	//{{AFX_MSG_MAP(CCSDetParmsDlg)
	ON_BN_CLICKED(IDC_CSMETHOD_COMBO, OnCsmethodCombo)
	ON_BN_CLICKED(IDC_CSMETHOD_AUTOCORRELATION, OnCsmethodAutocorr)
	ON_BN_CLICKED(IDC_CSMETHOD_DELTAMASS, OnCsmethodDeltamass)
	ON_BN_CLICKED(IDC_CSMETHOD_FOURIER, OnCsmethodFourier)
	ON_BN_CLICKED(IDC_CSMETHOD_PATTERSON, OnCsmethodPatterson)
	ON_EN_KILLFOCUS(IDC_Highest_CS_Charge, OnKillfocusHighestCSCharge)
	ON_BN_CLICKED(IDC_SAVE_AS_DEFAULTS, OnSaveAsDefaults)
	ON_BN_CLICKED(IDC_CSMETHOD_CROSSCORR, OnCsmethodCrosscorr)
	ON_BN_CLICKED(IDC_CSMETHOD_ENTROPY, OnCsmethodEntropy)
	ON_BN_CLICKED(IDC_CSMETHOD_SPARE1, OnCsmethodSpare1)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CCSDetParmsDlg message handlers

void CCSDetParmsDlg::OnCsmethodCombo() 
{
	m_CSDetMethod = CSDET_COMBO;
	m_DeltaMassBtn.SetCheck(FALSE);
	m_PattersonBtn.SetCheck(FALSE);
	m_FourierBtn.SetCheck(FALSE);
	m_ComboBtn.SetCheck(TRUE);
	m_AutoCorrBtn.SetCheck(FALSE);
	m_CrossCorrBtn.SetCheck(FALSE);
	m_EntropyBtn.SetCheck(FALSE);
	m_Spare1Btn.SetCheck(FALSE);
}

void CCSDetParmsDlg::OnCsmethodDeltamass() 
{
	m_CSDetMethod = CSDET_DELTAMASS;
	m_DeltaMassBtn.SetCheck(TRUE);
	m_PattersonBtn.SetCheck(FALSE);
	m_FourierBtn.SetCheck(FALSE);
	m_ComboBtn.SetCheck(FALSE);
	m_AutoCorrBtn.SetCheck(FALSE);
	m_CrossCorrBtn.SetCheck(FALSE);
	m_EntropyBtn.SetCheck(FALSE);
	m_Spare1Btn.SetCheck(FALSE);
}

void CCSDetParmsDlg::OnCsmethodFourier() 
{
	m_CSDetMethod = CSDET_FOURIER;
	m_DeltaMassBtn.SetCheck(FALSE);
	m_PattersonBtn.SetCheck(FALSE);
	m_FourierBtn.SetCheck(TRUE);
	m_ComboBtn.SetCheck(FALSE);
	m_AutoCorrBtn.SetCheck(FALSE);
	m_CrossCorrBtn.SetCheck(FALSE);
	m_EntropyBtn.SetCheck(FALSE);
	m_Spare1Btn.SetCheck(FALSE);
}

void CCSDetParmsDlg::OnCsmethodPatterson() 
{
	m_CSDetMethod = CSDET_MULTIPLICATIVE;
	m_DeltaMassBtn.SetCheck(FALSE);
	m_PattersonBtn.SetCheck(TRUE);
	m_FourierBtn.SetCheck(FALSE);
	m_ComboBtn.SetCheck(FALSE);
	m_AutoCorrBtn.SetCheck(FALSE);
	m_CrossCorrBtn.SetCheck(FALSE);
	m_EntropyBtn.SetCheck(FALSE);
	m_Spare1Btn.SetCheck(FALSE);
}

void CCSDetParmsDlg::OnKillfocusHighestCSCharge() 
{
	if (m_HighChargeLimit > 100) m_HighChargeLimit=100;
	if (m_HighChargeLimit < 0) m_HighChargeLimit*=-1;
	if (m_HighChargeLimit == 0) m_HighChargeLimit=100;
}

void CCSDetParmsDlg::OnOK() 
{
	// TODO: Add extra validation here
	
	CPropertyPage::OnOK();
}

void CCSDetParmsDlg::OnUpdate()
{
	switch(m_CSDetMethod) {
	case CSDET_DELTAMASS:
		OnCsmethodDeltamass();
		break;
	case CSDET_MULTIPLICATIVE:
		OnCsmethodPatterson();
		break;
	case CSDET_FOURIER:
		OnCsmethodFourier();
		break;
	case CSDET_COMBO:
		OnCsmethodCombo();
		break;
	case CSDET_AUTOCORR:
		OnCsmethodAutocorr();
		break;
	case CSDET_OCONNOR:
		OnCsmethodCrosscorr();
		break;
	case CSDET_ENTROPY:
		OnCsmethodEntropy();
		break;
	case CSDET_SPARE1:
		OnCsmethodSpare1();
		break;
	}

	UpdateData(FALSE);
}

BOOL CCSDetParmsDlg::OnSetActive() 
{
	OnUpdate();
	return CPropertyPage::OnSetActive();
}

void CCSDetParmsDlg::ValidateParms()
{

}

void CCSDetParmsDlg::OnSaveAsDefaults() 
{
	CBudaApp* pApp;

	pApp = (CBudaApp*)AfxGetApp();
	pApp->CSHighChargeLimit = m_HighChargeLimit;
	pApp->CSDetMethod = m_CSDetMethod;
	pDoc->CSDetMethod = m_CSDetMethod;
	pDoc->CSHighChargeLimit = m_HighChargeLimit;
	pApp->WriteIniFile();
}

void CCSDetParmsDlg::OnCsmethodAutocorr() 
{
	m_CSDetMethod = CSDET_AUTOCORR;
	m_DeltaMassBtn.SetCheck(FALSE);
	m_PattersonBtn.SetCheck(FALSE);
	m_FourierBtn.SetCheck(FALSE);
	m_ComboBtn.SetCheck(FALSE);
	m_AutoCorrBtn.SetCheck(TRUE);
	m_CrossCorrBtn.SetCheck(FALSE);
	m_EntropyBtn.SetCheck(FALSE);
	m_Spare1Btn.SetCheck(FALSE);
}

void CCSDetParmsDlg::OnCsmethodCrosscorr() 
{
	m_CSDetMethod = CSDET_OCONNOR;
	m_DeltaMassBtn.SetCheck(FALSE);
	m_PattersonBtn.SetCheck(FALSE);
	m_FourierBtn.SetCheck(FALSE);
	m_ComboBtn.SetCheck(FALSE);
	m_AutoCorrBtn.SetCheck(FALSE);
	m_CrossCorrBtn.SetCheck(TRUE);
	m_EntropyBtn.SetCheck(FALSE);
	m_Spare1Btn.SetCheck(FALSE);
}

void CCSDetParmsDlg::OnCsmethodEntropy() 
{
	m_CSDetMethod = CSDET_ENTROPY;
	m_DeltaMassBtn.SetCheck(FALSE);
	m_PattersonBtn.SetCheck(FALSE);
	m_FourierBtn.SetCheck(FALSE);
	m_ComboBtn.SetCheck(FALSE);
	m_AutoCorrBtn.SetCheck(FALSE);
	m_CrossCorrBtn.SetCheck(FALSE);
	m_EntropyBtn.SetCheck(TRUE);
	m_Spare1Btn.SetCheck(FALSE);
}

void CCSDetParmsDlg::OnCsmethodSpare1() 
{
	m_CSDetMethod = CSDET_SPARE1;
	m_DeltaMassBtn.SetCheck(FALSE);
	m_PattersonBtn.SetCheck(FALSE);
	m_FourierBtn.SetCheck(FALSE);
	m_ComboBtn.SetCheck(FALSE);
	m_AutoCorrBtn.SetCheck(FALSE);
	m_CrossCorrBtn.SetCheck(FALSE);
	m_EntropyBtn.SetCheck(FALSE);
	m_Spare1Btn.SetCheck(TRUE);
}
