// CSDetermination.cpp : implementation of the chargestate determination routines
//
/****************************************************************

Boston University data analysis (BUDA) package 

Description: Buda is a data analysis program for Fourier 
Transform Mass Spectrometry data.  

by Peter B. O'Connor

    Copyright (C) 2000  Peter B. O'Connor

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "BudaMaster.h"
#include "Buda.h"
#include "BudaDoc.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
//#include "TimingDialog.h"
//#include "ViewFFTOptions.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define MAX_GROUPCHK_ITERCOUNT 10

void CBudaDoc::ChargeStateDetermination()
{
	/*
	// this function decides the charge state of all peaks in the MI table
	// it works by assigning a group value to each peak
	// extraneous or ridiculous peaks are given group -1
	// unassigned peaks are given group 0 (and hence are charge 1)
	//		first, roughly classify peaks into groups as all those within 2 Da
	//			of each other go into a group
	//		then we only have to deal with overlapping distributions
	//		remove peaks that are probably not consistent
	//			for each group, 
	//				find highest peak, find all spacings, find median spacing
	//		test groups for self consistency
	//			for each group
	//				for each peak
	//					if peak spacing +/- not within group stdev of one member
	//						reassign peak to group 0;
	//		for each peak left in group 0
	//			if similar to spacing of grp x, add to group x
	//				(similar = within stdev and within 0.55 Da of one member of x
	//					test all grpx's before adding to one with best fit)
	//			if not similar, start new group.	
	//		rerun chargestate determination until no change

	int i, maxgrp=0;
	struct groupstruct *Grps;

	Grps = new struct groupstruct[1000];
	// first sort FITable in Abundance order
	SortFITableOnAbundance();

	// rough assignment - group together all data within 2 amu
	RoughGroupAssignment(Grps, &maxgrp);

	// iteratively check all groups for extraneous peaks and 
	// assign them to group zero.  then assign group zero peaks
	// to a best fit group or new group.
	int itercount;
	BOOL changed = FALSE;
	for(itercount = 0; itercount<MAX_GROUPCHK_ITERCOUNT; itercount++) {
		changed = FALSE;
		for(i=0; i<=maxgrp; i++) {
			ExtractOverlappingCSDists(Grps,i);
		} // endfor i
		
		GroupAssignment(Grps);
		if(!changed) break;
	}

	// now all groups are assigned, just calculate cs.
	for(i=0; i<=maxgrp; i++) {
		ComputeCS(&Grps[i]);
	}
	*/
} // end CSDetermination


void CBudaDoc::RoughGroupAssignment(struct groupstruct *grps, int *maxgrpval)
{
	/*
	int i,j;
	int maxgrp;

	// assign all peaks to groups.  If a peak is within 2 Da of the 
	// most intense member of the group, it is added to the group.  
	// group zero is the leftover peaks
	maxgrp=0; // create new group
	// first peak goes in new group
	AddToCSGroup(0,0,&maxgrp);

	// now iterate through the peaks assigning them to the group with
	// the first largest peak within 2 Da (largest peak will always 
	//be first member of a group since the MITable is sorted on abundance)
	double currmass;
	BOOL fit = FALSE;
	long NumPeaks = PS.Spec[CurrentSpectrumFocus].NumPeaks;
	for(i=1; i<NumPeaks; i++) {
		currmass = FreqToMass(FITable[i].F);
		for(j=0;j<maxgrp;j++) {
			if (fabs(currmass-grp[j].mass)<2) { // is it within 2 Da of 1st peak in grp j?
				AddToCSGroup(j,i,&maxgrp);
				fit = TRUE;
				break; // break out of j loop
			} 
		}
		if (!fit) {  // if not start a new group
			AddToCSGroup(maxgrp,i,&maxgrp);
		}
	}	// end i loop
	*maxgrpval=maxgrp;
	*/
}

void CBudaDoc::ExtractOverlappingCSDists(struct groupstruct *grps, int grpnum)
{
	//The initial version of this function uses the spacing
	//between the highest peak and the next highest peak in the 
	//distribution as the initial estimate of the CS spacing.
	//any peak not a multiple of that spacing is set to group
	//zero.

	/*
	struct groupstruct* grp;
	grp = &(grps[grpnum]);
	int maxpeakindex = 0;
	int nextpeakindex = 1;
	int i=0;
	double spacing, maxpeakmass;

	maxpeakindex = grp->MITableIndex[0];
	nextpeakindex = grp->MITableIndex[1];

	maxpeakmass = FreqToMass(FITable[maxpeakindex].F);
	spacing = fabs(maxpeakmass-FreqToMass(FITable[nextpeakindex].F));

	double currmass, remainder, accuracy;

	// assume mass accuracy >= 5* peak width FWHM
	accuracy = (maxpeakmass/FITable[maxpeakindex].RP)/5;

	for(i=0; i<grp->numpeaks; i++){
		// test each peak for being multiple of spacing from max
		currmass = FreqToMass(FITable[i].F);
		remainder = fabs(currmass - maxpeakmass) / spacing;
		remainder = remainder - int(remainder);
		if (remainder > accuracy) {
			// assign this peak to group zero
			AssignToGroupZero(grps, grpnum, i);
		}
	}
	*/
}

void CBudaDoc::GroupAssignment(struct groupstruct *grp)
{
	/*
	grp = grp;
	// This finction takes each member of group zero and
	// assigns it to its best fit group, or if no best fit group, 
	// creates a new group
	*/
}

void CBudaDoc::ComputeCS(struct groupstruct *grp)
{
	// This function takes the current group and finds
	// the average and stdev of the mass spacings - delta(m). 
	// It then assigns the cs to 1/average delta(m)

/*	int i, max;
	double mass1, mass2;
	max = grp->numpeaks;

	double *deltam;
	deltam = new double[max];

	// first sort group elements by mass
	for(i=0; i<max; i++) {
		mass1 = FreqToMass(FITable[grp->MITableIndex[i]].F);
		mass2 = FreqToMass(FITable[grp->MITableIndex[i+1]].F);

		deltam[i]=fabs(mass2-mass1);
	}

	// now get average and stdev of deltam
	double average=0, stdev=0;
	for(i=0; i<max; i++) {
		average+=deltam[i];
		stdev+=deltam[i];
	}
	grp->average=average/(double)max;
	grp->chargestate=int((double)max/average);
	*/
}

void CBudaDoc::AssignToGroupZero(struct groupstruct *grps, int grpnum, int peaknum)
{
	// this function assigns element n to group zero and
	// then moves all the later elements up in the list and
	// decrements numpeaks
	/*

	grps[0].MITableIndex[grps[0].numpeaks]=grps[grpnum].MITableIndex[peaknum];
	grps[0].numpeaks++;

	int i;
	for(i=peaknum; i<grps[grpnum].numpeaks; i++) {
		grps[grpnum].MITableIndex[i]=grps[grpnum].MITableIndex[i+1];
	}
	grps[grpnum].numpeaks--;
	*/
}

void CBudaDoc::AddToCSGroup(int groupnum, int MIIndexNum, int* maxgrp)
{
	// if grp[groupnum] exists, add peak to it.  Otherwise,
	// create it.
	/*
	if(groupnum > *maxgroup) {
		MessageBox("Error 34 : invalid charge state peak grouping number","Charge state determination error");
		return;
	}
	if(groupnum == *maxgroup) {
		// new group - create it
		grp[groupnum].mass = FreqToMass(FITable[MIIndexNum].F);
		grp[groupnum].average = 1;
		grp[groupnum].chargestate = 1;
		grp[groupnum].numpeaks = 1;
		grp[groupnum].stdev = 100; // effectively infinite
		grp[groupnum].MITableIndex[0]=MIIndexNum;
		*maxgroup++;
	} else {
		// old group - add to it
		grp[groupnum].MITableIndex[grp[groupnum].numpeaks]=MIIndexNum;
		grp[groupnum].numpeaks++; 
	}
	*/
}
