// CSMapBtn.cpp : implementation file
//

#include "stdafx.h"
#include "buda.h"
#include <math.h>
#include "CSMapBtn.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CCSMapBtn

CCSMapBtn::CCSMapBtn()
{
}

CCSMapBtn::~CCSMapBtn()
{
}


BEGIN_MESSAGE_MAP(CCSMapBtn, CButton)
	//{{AFX_MSG_MAP(CCSMapBtn)
	ON_WM_DRAWITEM()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CCSMapBtn message handlers

void CCSMapBtn::DrawItem(LPDRAWITEMSTRUCT lpDrawItemStruct) 
{
	// this is the plotfunction for plotting the ARB frequency domain
	// waveform in the ARB Edit function.
	
	pDC = CDC::FromHandle(lpDrawItemStruct->hDC);

	pDC->GetWindow()->GetClientRect(&ViewSize);

	 //testing the drawing by drawing an 'x'
/*	pDC->MoveTo(ViewSize.left,ViewSize.top);
	pDC->LineTo(ViewSize.right,ViewSize.bottom);
	pDC->MoveTo(ViewSize.right,ViewSize.top);
	pDC->LineTo(ViewSize.left,ViewSize.bottom);
*/
	
	// mass axis line position units in pixels
	AxisLineYPos = ViewSize.bottom-35;
	TicLabelsYPos = AxisLineYPos+5;
	CaptionYPos = AxisLineYPos+17;
	TicLength = 5;

	// intensity axis position variables
	IntAxisPos = ViewSize.left+25;

	// waveform position variables
	MaxYPos = ViewSize.top + 10;
	MinYPos = AxisLineYPos - 5;
	MaxXPos = ViewSize.right - 1;
	MinXPos = IntAxisPos+5;

	DrawZAxis();
	DrawIntensityAxis();
	DrawZMap();
}

void CCSMapBtn::DrawZAxis()
{
	int AxisLineXstart = MinXPos;
	int AxisLineXend = MaxXPos;
	float Xstart = pCSDet->CSValue[0];
	float Xend = (float)pCSDet->ChargeLimit;

	// second draw the axis line 
	pDC->MoveTo(AxisLineXstart,AxisLineYPos);
	pDC->LineTo(AxisLineXend,AxisLineYPos);

	// place tics
	// there will be <=10 tics
	int ticpos[10];
	double ticZ[10];
	double ZBase = Xend-Xstart;
	long i;
	for(i=0; i<10; i++) {
		ticpos[i] = 0;
		ticZ[i] = 0;
	}

	// there will be 5-10 tics  
	// ticsteps can be (order of magnitude of timebase) / 1, 2, 5 
	double a = log10(ZBase);
	double b = floor(a);	// b = order of magnitude of timebase
	double ticstep = pow(10,b);
	double nt;	
	int numtics;
	nt = ZBase / ticstep;

	if(nt>=5) numtics = (int)nt;
	if(nt>2 && nt<5) {
		ticstep=pow(10,b)/2;
		numtics = (int)(ZBase/ticstep);
	}
	if(nt<=2) {
		ticstep = pow(10,b)/5;
		numtics = (int)(ZBase/ticstep);
	}

	// find the position of the first tic.
	// if zero is in the range, I want zero to be one of the ticx
	double NumTixb4Zero = (double)Xstart/ticstep;
	if(NumTixb4Zero>0) {// GenPS starts after zero
		ticZ[0]=(floor(NumTixb4Zero)+1) * ticstep;
	}
	if (NumTixb4Zero == 0) ticZ[0] = 0;
	if (NumTixb4Zero < 0) { // GenPS starts before zero
		ticZ[0] = -1 * floor(-1*NumTixb4Zero) * ticstep;
	}

	// find the position of each tic relative to tictime[0]
	for(i=1; i<10; i++) {
		ticZ[i]=ticstep*i+ticZ[0];
		if(ticZ[i]>Xend) break;
	}
	numtics = i-1;

	// find tic x positions
	// remember that position 1000 = FirstEventStart
	// and 31768 = LastEventEnd;
				// create a linear eqn y = a*x + b
				// where y's are the plotspace #'s and 
				// x's are the timebase space #'s
	a = (AxisLineXend-AxisLineXstart)/(Xend - Xstart);
	b = AxisLineXstart - a*Xstart;
	for(i=0; i<numtics; i++) {
		ticpos[i] = int(a*ticZ[i] + b);
	}

	// now actually plot the tics
	for(i=0; i<numtics; i++) {
		pDC->MoveTo(ticpos[i],AxisLineYPos);
		pDC->LineTo(ticpos[i],AxisLineYPos+TicLength);
	}

	// lable tics
	CString str;
	CPoint pt;
	CSize sz;

	str.Format("Z");
	pt.x = (ViewSize.right+ViewSize.left)/2;
	pt.y = CaptionYPos;
	Text(str,pt,0);

	for(i=0; i<numtics; i++) {
		str.Format("%1.1f",ticZ[i]);
		pt.x = ticpos[i];
		pt.y = TicLabelsYPos;
		Text(str,pt,0);
	}

	// end of draw axis

}

void CCSMapBtn::DrawIntensityAxis()
{
	// draw line
	pDC->MoveTo(IntAxisPos,MinYPos);
	pDC->LineTo(IntAxisPos,MaxYPos);

	// draw 11 tics
	int i;
	float y,dy = (float)(MinYPos-MaxYPos)/(float)10.0;
	for(i=0; i<11; i++) {
		y = float(MinYPos) - float(i)*dy;
		pDC->MoveTo(IntAxisPos,int(y));
		pDC->LineTo(IntAxisPos-TicLength,int(y));
	}

	// put on 0, 50, 100 labels
	pDC->MoveTo(ViewSize.left,MinYPos);
	CPoint pt;
	pt.y = MinYPos;
	pt.x = ViewSize.left+12;
	Text("0",pt,1);
	pt.y = int((float)MinYPos-(float)5*dy);
	pt.x -= 7;
	Text("50",pt,1);
	pt.y = MaxYPos;
	pt.x -= 5;
	Text("100",pt,1);
}

void CCSMapBtn::DrawZMap()
{
	int i,x,y;
	double ax,bx,ay,by;
	ax = (MaxXPos-MinXPos)/(pCSDet->ChargeLimit - pCSDet->CSValue[0]);
	bx = MinXPos - ax*pCSDet->CSValue[0];

	by=0;
	for(i=0; i<pCSDet->ChargeLimit*10; i++) {
		if(pCSDet->CSMap[i]>by) {
			by = pCSDet->CSMap[i];
		}
	}
	ay = double(MaxYPos-MinYPos)/by;
	by = MinYPos;

	// now pCSDet->CSValue and pCSDet->CSMap are set, plot them;
	pDC->MoveTo(MinXPos,MinYPos);
	for(i=0; i<pCSDet->ChargeLimit*10; i++) {
		x = int(ax*pCSDet->CSValue[i]+bx);
		y = int(ay*pCSDet->CSMap[i]+by);
		pDC->LineTo(x,y);
		if(i==pCSDet->ChargeLimit*10-1) {
			x = int(ax*pCSDet->CSValue[i]+bx);
			y = int(by);
			pDC->LineTo(x,y);
			x = int(MaxXPos);
			y = int(by);
			pDC->LineTo(x,y);
		}
	}
}

void CCSMapBtn::Text(CString str, CPoint pt, int just)
{
	CSize sz = pDC->GetTextExtent(str);

	if(just == 0) pt.x = pt.x-sz.cx/2;	// center-justified
	if(just == 1) pt.y = pt.y - sz.cy/2;	// left-justified

	pDC->TextOut(pt.x,pt.y,str);
}
