// CalibrateDlg.cpp : implementation file
//
/*
CalibrateDlg.cpp is the code that controls the CCalibrateDlg Class.
This class is the one that works on calibration, calls up the addpeak
dialog, add to calibrant file dialog, reads in the calibrant file, 
calculates the new calibrations, and allows the user to read/store
calibration files, and apply it to the system or the current file.
*/

/****************************************************************

Boston University data analysis (BUDA) package 

Description: Buda is a data analysis program for Fourier 
Transform Mass Spectrometry data.  

by Peter B. O'Connor

    Copyright (C) 2000  Peter B. O'Connor

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include <afxdlgs.h>
#include "BudaMaster.h"
#include "BudaDoc.h"
#include "CalibrateDlg.h"
#include "AddPeakDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CCalibrateDlg dialog


CCalibrateDlg::CCalibrateDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CCalibrateDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CCalibrateDlg)
	m_CalParmA = _T("");
	m_CalParmB = _T("");
	m_CalDate = _T("");
	//}}AFX_DATA_INIT
}


void CCalibrateDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CCalibrateDlg)
	DDX_Control(pDX, IDC_PEAKMATCHSTR, m_PeakMatchStatic);
	DDX_Control(pDX, IDC_CALIBRATION_LIST, m_CalibrationList);
	DDX_Control(pDX, IDC_EXTERNAL_CALIBRATION, m_ExternalCalBtn);
	DDX_Text(pDX, IDC_CAL_PARM_A, m_CalParmA);
	DDV_MaxChars(pDX, m_CalParmA, 20);
	DDX_Text(pDX, IDC_CAL_PARM_B, m_CalParmB);
	DDV_MaxChars(pDX, m_CalParmB, 20);
	DDX_Text(pDX, IDC_CAL_DATE, m_CalDate);
	DDV_MaxChars(pDX, m_CalDate, 100);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CCalibrateDlg, CDialog)
	//{{AFX_MSG_MAP(CCalibrateDlg)
	ON_BN_CLICKED(IDC_ADD_PEAK, OnAddPeak)
	ON_BN_CLICKED(IDC_APPLY_SPECTRUM, OnApplySpectrum)
	ON_BN_CLICKED(IDC_EXTERNAL_CALIBRATION, OnExternalCalibration)
	ON_BN_CLICKED(IDC_LOAD_CALIBRATION_FROM_FILE, OnLoadCalibrationFromFile)
	ON_BN_CLICKED(IDC_SAVE_CALIBRATION_TO_FILE, OnSaveCalibrationToFile)
	ON_BN_CLICKED(IDC_SAVE_DEFAULT_CALIBRATION, OnSaveDefaultCalibration)
	ON_BN_CLICKED(IDC_REMOVE_PEAK, OnRemovePeak)
	ON_LBN_DBLCLK(IDC_CALIBRATION_LIST, OnDblclkCalibrationList)
	ON_BN_CLICKED(IDC_EXIT, OnExit)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CCalibrateDlg message handlers

void CCalibrateDlg::OnAddPeak() 
{
	if(IsPeakMatching) {
		if(RefPeak[1].ActualMass >= 1 && RefPeak[0].ActualMass >= 1) {
			MessageBox("Maximum 2 Internal Calibrant Peaks allowed","Maximum peaks!");
			return;
		}
	}
	CAddPeakDlg addpeak;

	// load addpeak with appropriate variables
	//   addpeak will fill in it's list boxes in OnInitDialog
	// Give it the PS structure for FITable's NumPeaks
	addpeak.PS = PS;
	// Give it the FITable
	addpeak.FITable = FITable;
	addpeak.PDoc = pDoc;

	// do modal
	addpeak.DoModal();

	// retrieve addpeak parms
	if (!addpeak.IsOK) return;
	MIindex = addpeak.MIindex;
	ExactMass = addpeak.m_ExactMass;

	if(IsPeakMatching) {
		int ptr = 0;
		if(RefPeak[0].ActualMass !=0 && RefPeak[1].ActualMass==0) ptr = 1;
		if(ptr == 0) RefPeak[1].ActualMass = 0;
		RefPeak[ptr].ActualMass = ExactMass;
		RefPeak[ptr].MeasuredFrequency = float(FITable[MIindex].F);
		RefPeak[ptr].TotalIntensity = float(FITable[MIindex].I);
		RefPeak[ptr].TrapVoltage = (float)pDoc->TrapVoltsOnDetect;

		// getting time string
		CTime t;
		t.GetCurrentTime();
		CString timestr = t.Format("%H:%M:%S on %m-%d-%Y");
		int j;
		for(j=0; j<39 && j<timestr.GetLength(); j++) {
			RefPeak[ptr].TimeEntered[j] = timestr.GetAt(j);
			RefPeak[ptr].TimeEntered[j+1] = '\0';
		}
	} else {
		int i = pDoc->sys->CalibrationPoints;
		pDoc->sys->CalData[i].ActualMass = ExactMass;
		pDoc->sys->CalData[i].MeasuredFrequency = FITable[MIindex].F;
		double TIT = pDoc->PS.Spec[pDoc->CurrentSpectrumFocus].TotalIonInt;
		pDoc->sys->CalData[i].TotalIntensity = (float)TIT/(float)PS->ScansPerSpectrum/(float)PS->ADCGain;
		pDoc->sys->CalData[i].IncludeInCalc = 1;
		pDoc->sys->CalData[i].TrapVoltage = (float)pDoc->TrapVoltsOnDetect;
		//sys.CalData[i].TimeEntered = timestr;
		pDoc->sys->CalibrationPoints++;
	}

	UpdateCalibrationDialog();
}

void CCalibrateDlg::OnApplySpectrum() 
{
	PS->A = A;
	PS->B = B;
	if(IsPeakMatching) {
		int i;
		PS->RefPeak[0].ActualMass = RefPeak[0].ActualMass;
		PS->RefPeak[0].MeasuredFrequency = RefPeak[0].MeasuredFrequency;
		for(i=0; i<40; i++) PS->RefPeak[0].TimeEntered[i] = RefPeak[0].TimeEntered[i];
		PS->RefPeak[0].TotalIntensity = RefPeak[0].TotalIntensity;
		PS->RefPeak[0].TrapVoltage = RefPeak[0].TrapVoltage;

		PS->RefPeak[1].ActualMass = RefPeak[1].ActualMass;
		PS->RefPeak[1].MeasuredFrequency = RefPeak[1].MeasuredFrequency;
		for(i=0; i<40; i++) PS->RefPeak[1].TimeEntered[i] = RefPeak[1].TimeEntered[i];
		PS->RefPeak[1].TotalIntensity = RefPeak[1].TotalIntensity;
		PS->RefPeak[1].TrapVoltage = RefPeak[1].TrapVoltage;
	}
	PS->PeakMatching = IsPeakMatching;

	pDoc->UpdateAllViews(NULL,0,NULL);
}

void CCalibrateDlg::OnExternalCalibration() 
{
	if (IsPeakMatching) IsPeakMatching = FALSE;
	else IsPeakMatching = TRUE;

	UpdateCalibrationDialog();
}

void CCalibrateDlg::OnLoadCalibrationFromFile() 
{	
	int i,j;
	BOOL test;
	double f, m;
	float amp, v;
	char date[40];
	CFile calfile;
	long calhdr,incl;
	CString errorstr;
	CString filestr = "File Open Error";
	int retval;

	// use the CFileDialog to get the name of the file
	CFileDialog *dlg;
	dlg = new CFileDialog(TRUE,"*.cal","MALDI.cal",OFN_HIDEREADONLY | OFN_LONGNAMES,"*.cal",NULL);
	dlg->m_ofn.lpstrFilter = "Calibration Files (*.cal)\0*.cal\0All Files (*.*)\0*.*\0";
	dlg->m_ofn.lpstrInitialDir = pDoc->pApp->ProgramDirectory;

	retval = dlg->DoModal();
	CString pathname = dlg->GetPathName();
	delete dlg;
	if (retval == IDCANCEL) return;

	// ok, open the file and try to read it

	test = calfile.Open(pathname,  CFile::modeRead, NULL);
	if (!test) {
		errorstr.Format("Couldn't open Calibration file \n %s",pathname);
		MessageBox(errorstr,filestr);
	}

	CArchive ar(&calfile,CArchive::load,4096,NULL);

	ar.Read(&calhdr,4);
	if (calhdr != 0xcafd) {
		errorstr.Format("File %s is not a valid calibration file",pathname);
		MessageBox(errorstr,filestr);
	}

	ar.Read(&(pDoc->sys->CalibrationVersion),4);
	ar.Read(&(pDoc->sys->A),8);
	ar.Read(&(pDoc->sys->B),8);
	ar.Read(&(pDoc->sys->HighMassShift),8);
	ar.Read(&(pDoc->sys->TrapShift),8);
	ar.Read(&(pDoc->sys->IntensityShift),8);
	ar.Read(&(pDoc->sys->CalibrationPoints),4);
	ar.Read(&(pDoc->sys->CalDate),40);
	ar.Read(&(pDoc->sys->CalDescription),64);

	test = TRUE;
	for(i=0; i<pDoc->sys->CalibrationPoints; i++) {
		if(ar.Read(&m,8)<8) {
			test = FALSE;
			break;
		}
		if(ar.Read(&f,8)<8){
			test = FALSE;
			break;
		}
		if(ar.Read(&v,4)<4) {
			test = FALSE;
			break;
		}
		if(ar.Read(&amp,4)<4) {
			test = FALSE;
			break;
		}
		if(ar.Read(&incl,4)<4) {
			test = FALSE;
			break;
		}
		if(ar.Read(&date,40)<40) {
			test = TRUE;
			break;
		}
		pDoc->sys->CalData[i].ActualMass = m;
		pDoc->sys->CalData[i].MeasuredFrequency = f;
		pDoc->sys->CalData[i].TrapVoltage = v;
		pDoc->sys->CalData[i].TotalIntensity = amp;
		pDoc->sys->CalData[i].IncludeInCalc = incl;
		for (j=0; j<40; j++) {
			pDoc->sys->CalData[i].TimeEntered[j] = date[j];
			if (date[j]==NULL) break;
		}
	}
	if (!test) {
		errorstr.Format("Premature End of File :\n %s",pathname);
		MessageBox(errorstr,filestr);
	}
	ar.Close();
	UpdateCalibrationDialog();
}

void CCalibrateDlg::OnSaveCalibrationToFile() 
{
	int i;
	BOOL test;
	CFile calfile;
	long calhdr;
	CString errorstr;
	CString filestr = "File Open Error";
	int retval;

	// use the CFileDialog to get the name of the file
	CFileDialog *dlg;
	dlg = new CFileDialog(FALSE,"*.cal","MALDI.cal",OFN_HIDEREADONLY | OFN_LONGNAMES,"*.cal",NULL);
	dlg->m_ofn.lpstrFilter = "Calibration Files (*.cal)\0*.cal\0All Files (*.*)\0*.*\0";
	dlg->m_ofn.lpstrInitialDir = pDoc->pApp->ProgramDirectory;

	retval = dlg->DoModal();
	CString pathname = dlg->GetPathName();
	delete dlg;
	if (retval == IDCANCEL) return;

	// ok, try open the file and then dump to it
	test = calfile.Open(pathname,  CFile::modeWrite | CFile::modeCreate, NULL);
	if (!test) {
		errorstr.Format("Couldn't open Calibration file \n %s",pathname);
		MessageBox(errorstr,filestr);
	}

	CArchive ar(&calfile,CArchive::store,4096,NULL);

	calhdr = 0xcafd;
	ar.Write(&calhdr,4);
	ar.Write(&(pDoc->sys->CalibrationVersion),4);
	ar.Write(&(pDoc->sys->A),8);
	ar.Write(&(pDoc->sys->B),8);
	ar.Write(&(pDoc->sys->HighMassShift),8);
	ar.Write(&(pDoc->sys->TrapShift),8);
	ar.Write(&(pDoc->sys->IntensityShift),8);
	ar.Write(&(pDoc->sys->CalibrationPoints),4);
	ar.Write(&(pDoc->sys->CalDate),40);
	ar.Write(&(pDoc->sys->CalDescription),64);

	test = TRUE;
	for(i=0; i<pDoc->sys->CalibrationPoints; i++) {
		ar.Write(&(pDoc->sys->CalData[i].ActualMass),8);
		ar.Write(&(pDoc->sys->CalData[i].MeasuredFrequency),8); 
		ar.Write(&(pDoc->sys->CalData[i].TrapVoltage),4); 
		ar.Write(&(pDoc->sys->CalData[i].TotalIntensity),4);
		ar.Write(&(pDoc->sys->CalData[i].IncludeInCalc),4);
		ar.Write(&(pDoc->sys->CalData[i].TimeEntered),40);
	}
	ar.Close();
}

void CCalibrateDlg::OnSaveDefaultCalibration() 
{
	pDoc->sys->A = A;
	pDoc->sys->B = B;
}

void CCalibrateDlg::OnRemovePeak() 
{
	long i,j;

	j = (long)m_CalibrationList.GetCurSel();

	if(j == LB_ERR) {
		// nothing selected or nothing in listbox
		return;
	}

	if(IsPeakMatching) {
		if(j == 0) {
			RefPeak[0].ActualMass = RefPeak[1].ActualMass;
			RefPeak[0].MeasuredFrequency = RefPeak[1].MeasuredFrequency;
			for(i=0; i<40; i++) RefPeak[0].TimeEntered[i] = RefPeak[1].TimeEntered[i];
			RefPeak[0].TotalIntensity = RefPeak[1].TotalIntensity;
			RefPeak[0].TrapVoltage = RefPeak[1].TrapVoltage;
			j = 1;
		}
		RefPeak[j].ActualMass = 0;
		RefPeak[j].MeasuredFrequency = 0;
		for(i=0; i<40; i++) RefPeak[j].TimeEntered[i] = '\0';
		RefPeak[j].TotalIntensity = 0;
		RefPeak[j].TrapVoltage = 0;
	} else {
		for (i=j; i<pDoc->sys->CalibrationPoints-1; i++) {
			pDoc->sys->CalData[i] = pDoc->sys->CalData[i+1];
		}
	}
	pDoc->sys->CalibrationPoints--;

	UpdateCalibrationDialog();
}

BOOL CCalibrateDlg::OnInitDialog()
{
	A = sys->A;
	B = sys->B;
	if(IsPeakMatching) {
		int i;
		RefPeak[0].ActualMass = PS->RefPeak[0].ActualMass;
		RefPeak[0].MeasuredFrequency = PS->RefPeak[0].MeasuredFrequency;
		for(i=0; i<40; i++) RefPeak[0].TimeEntered[i] = PS->RefPeak[0].TimeEntered[i];
		RefPeak[0].TotalIntensity = PS->RefPeak[0].TotalIntensity;
		RefPeak[0].TrapVoltage = PS->RefPeak[0].TrapVoltage;

		RefPeak[1].ActualMass = PS->RefPeak[1].ActualMass;
		RefPeak[1].MeasuredFrequency = PS->RefPeak[1].MeasuredFrequency;
		for(i=0; i<40; i++) RefPeak[1].TimeEntered[i] = PS->RefPeak[1].TimeEntered[i];
		RefPeak[1].TotalIntensity = PS->RefPeak[1].TotalIntensity;
		RefPeak[1].TrapVoltage = PS->RefPeak[1].TrapVoltage;
	}

	// force the DoDataExchange - otherwise the listboxes aren't initialized
	UpdateData(FALSE);

	SetCalListFont();

	UpdateCalibrationDialog();

	//if(IsPeakMatching) {
	//	MessageBox("Peak Match Mode\n\nPeak match constants are continually updated, but\nThe new calibration constants will not be updated","For Your Information...");
	//}
	return TRUE;
}

void CCalibrateDlg::UpdateCalibrationDialog()
{
	long i;
	CString str, massstr, actualmass, ppmstr,incstr;
	double mass,ppm;

	CalculateNewCalibration();

	// Update the Internal/External Calibration Button
	CString caption;
	if (IsPeakMatching) {
		m_ExternalCalBtn.SetCheck(1);
		caption = "Internal Calibration";
		m_PeakMatchStatic.ShowWindow(SW_SHOW);
	} else {
		m_ExternalCalBtn.SetCheck(0);
		caption = "External Calibration";
		m_PeakMatchStatic.ShowWindow(SW_HIDE);
	}
	m_ExternalCalBtn.SetWindowText(caption);

	// set the m_CalParms strings
	m_CalParmA.Format("A = %10.5f",A);
	m_CalParmB.Format("B = %10.5f",B);
	m_CalDate = pDoc->sys->CalDate;
	m_CalDate = "Current Calibration was set on " + m_CalDate.Left(20);

	// load the calibrationlist
	m_CalibrationList.ResetContent();
	m_CalibrationList.InitStorage(MAX_CALIB_POINTS,64);

	int tabstops[3];
	tabstops[0]=5;
	tabstops[1]=10;
	tabstops[2]=15;
	m_CalibrationList.SetTabStops(8);

	// fill the calibration list with the calibration points 
	//   or if in peakmatch mode with the peak match points
	int numpts = pDoc->sys->CalibrationPoints;
	if(IsPeakMatching) {
		numpts = 2;
		if(RefPeak[1].ActualMass < 1) numpts = 1;
		if(RefPeak[0].ActualMass < 1) numpts = 0;
	}

	float actmass;

	for (i=0; i<numpts; i++) {
		mass = pDoc->FreqToMass(pDoc->sys->CalData[i].MeasuredFrequency, A, B);
		actmass = float(pDoc->sys->CalData[i].ActualMass);
		if((pDoc->sys->CalData[i].IncludeInCalc == 1)) incstr = "yes ";
		else incstr = " no ";
		if(IsPeakMatching) {
			mass = float(pDoc->FreqToMass(RefPeak[i].MeasuredFrequency));
			actmass = float(RefPeak[i].ActualMass);
			incstr = "yes";
		}
		ppm = 1e6*(mass - actmass)/mass;
		if (ppm < 0.001 && ppm > -0.001) ppm = 0;
		actualmass.Format("%5.4f",actmass);
		massstr.Format("%5.4f",mass);
		ppmstr.Format("%3.3f",ppm);

		SetCourierStrWidth(&massstr,10);
		SetCourierStrWidth(&actualmass,10);
		SetCourierStrWidth(&ppmstr,10);

		str = massstr + "\t" + actualmass + "\t\t" + incstr + "\t" + ppmstr;
		m_CalibrationList.AddString(str); 
	}

	// ok, now copy all this out to the dialog
	UpdateData(FALSE);
	/*
		if (IsPeakMatching) { // define the peakmatch refpeaks
			if(pDoc->sys->CalData[i].IncludeInCalc == 1) {
				if(NumPtsInCal<2) {
					PS->RefPeak[NumPtsInCal].ActualMass = pDoc->sys->CalData[i].ActualMass;
					PS->RefPeak[NumPtsInCal].MeasuredFrequency = pDoc->sys->CalData[i].MeasuredFrequency;
					//PS->RefPeak[NumPtsInCal].TimeEntered = sys->CalData[i].TimeEntered;
					PS->RefPeak[NumPtsInCal].TotalIntensity = pDoc->sys->CalData[i].TotalIntensity;
					PS->RefPeak[NumPtsInCal].TrapVoltage = pDoc->sys->CalData[i].TrapVoltage;
					NumPtsInCal++;
				}
			}
		}
	*/
}

void CCalibrateDlg::CalculateNewCalibration()
{
/* 
This routine is the fundamental calibration routine for IonSpec's
FTMS.  Thus, I include comments from CLBLSQ.FOR that this is 
derived from.  CLBLSQ.FOR is part of the original IonSpec Omega 
data system program created by Rick Hunter

C    This subroutine is used to compute the least squares best fit value
C    for the calibration constants A and B based on input frequencies
C    and masses.  The data are fit to the calibration equation:
C
C                Mi*Fo = A + B*Mi
C
C    where Mi are known masses of the calibration peaks and Fo are the
C    true cyclotron frequencies for the masses, Mi calculated from the
C    measured resonance frequencies, Fi, using the equation:
C
C                Fo = Fi + C*Vi + D*Vi*Ii
C
C    where C and D are system constants, Vi are the trapping voltages used
C    to measure the Fi frequencies, and Ii are total ion intensity values
C    for the spectra.

*/

// first, calculate the number of real calibration points.  That is the 
// number of entries in sys.CalData that have the IncludeInCalc Flag set to 1

	// PeakMatchMode : calculate new peakmatchfactor
	if(IsPeakMatching) {
		PS->PeakMatchFactor = RefPeak[0].ActualMass * pDoc->CyclotronFrequency
			 (RefPeak[0].MeasuredFrequency,  RefPeak[0].TrapVoltage,  RefPeak[0].TotalIntensity,long(1),1,1);
		return;
	}


	long i, calpts = 0;
	long indices[MAX_CALIB_POINTS], indptr = 0;

	for (i=0; i<pDoc->sys->CalibrationPoints; i++) {
		if (pDoc->sys->CalData[i].IncludeInCalc == 1) {
			calpts++;
			indices[indptr] = i;
			indptr++;
		}
	}

// if we have no calibration points, we cannot calibrate.
	if (calpts <= 0) {
		A = 0.0;
		B = 0.0;
		return;
	}

// if we have only one calibration point, ...
	double m, f;
	struct CalibrationPoint temp;

	if (calpts == 1) {
		temp = pDoc->sys->CalData[indices[0]];
		m = temp.ActualMass;
		f = pDoc->CyclotronFrequency(temp.MeasuredFrequency,1);
		A = m*f;
		B = 0;
		return;
	}

// Otherwise, we have to do least squares
/*
C -- Derive the least squares algorithm for this equation:
C
C                f  = A/m + B
C
C    to find the constants A and B.  The m values are actual known
C    masses for peaks in the calibration mass spectra.  The f values
C    in equation (1) are cyclotron frequencies calculated from the
C    measured resonance frequencies in the calibration mass spectra.
C    The approximate cyclotron frequencies are derived from the resonace
C    frequencies by correcting for the trapping voltage and total ion
C    intensity in the spectra.  This is calculated using system constants.
C    The double precision function "CyclotronFrequency" is used.
C
C    Define an error function:    E = sum (A/m + B - f)**2
C    and minimize in terms of the constants A and B:
C
C    dE/dA = 0 = sum [ (2) (A/m + B - f) (1/m) ]
C    dE/dB = 0 = sum [ (2) (A/m + B - f) (1) ]
C
C    Group terms and rearrange to give:
C
C            A * sum(1/m*m) +   B * sum(1/m)  =   sum(f/m)
C            A * sum(1/m)   +   B * N         =   sum(f)
C
C    To solve, use a 2x2 determinate:
C
C        | sumMM sumM  |     | sumFM  |
C        |             |  =  |        |
C        | sumM    N   |     | sumF   |
C
C   where:
C            N      = number of data pairs
C            sumM   = sum(1/m)
C            sumMM  = sum(1/m*m)
C            sumFM  = sum(f/m)
C            sumF   = sum(f)
C
C   Calculate the A and B terms:
C
C       A = (sumFM * N    - sumF * sumM)  /  (N * sumMM - sumM * sumM)
C       B = (sumMM * sumF - sumFM * sumM) /  (N * sumMM - sumM * sumM)
*/
	double N = (double) calpts;
	double sumM = 0.0;
	double sumMM = 0.0;
	double sumFM = 0.0;
	double sumF = 0.0;

	for (i=0; i<calpts; i++) {
		temp = pDoc->sys->CalData[indices[i]];
		m=temp.ActualMass;
		f=pDoc->CyclotronFrequency(temp.MeasuredFrequency,1);

		sumM = sumM + 1.0/m;
		sumMM = sumMM + 1.0/(m*m);
		sumFM = sumFM + f/m;
		sumF = sumF + f;
	}

	double d;
	d = N * sumMM - sumM*sumM;
	A = (sumFM * N - sumF * sumM) / d;
	B = (sumMM * sumF - sumFM * sumM) / d;

	return;
}

void CCalibrateDlg::OnDblclkCalibrationList() 
{
	if(IsPeakMatching) return;

	int index = m_CalibrationList.GetCurSel();
	if (pDoc->sys->CalData[index].IncludeInCalc == 1) pDoc->sys->CalData[index].IncludeInCalc = 0;
	else pDoc->sys->CalData[index].IncludeInCalc = 1;

	UpdateCalibrationDialog();
}

void CCalibrateDlg::SetCalListFont()
{
/*   CFont* pFont = GetFont();
   LOGFONT lf;
   if (pFont != NULL)
	   pFont->GetObject(sizeof(LOGFONT), &lf);
   else
	   ::GetObject(GetStockObject(SYSTEM_FONT), sizeof(LOGFONT), &lf);

	lf.lfPitchAndFamily = FIXED_PITCH;
	lf.lfWidth = 7;
	lf.lfHeight = 16;
	lf.lfWeight = 10;
	char FaceName[32] = "Courier New                   \0";;
	lf.lfFaceName = FaceName;
		// switch to new font.
	m_font.DeleteObject();
	if (m_font.CreateFontIndirect(&lf))
	{
		//CWaitCursor wait;
		m_CalibrationList.SetFont(&m_font);
	}
	*/
	//m_font.CreatePointFont(1,"Courier",m_CalibrationList.GetDC());
	//m_font.CreateFont(7,5,0,0,0,0,0,0,0,0,0,0,FIXED_PITCH,"Courier");
	//m_CalibrationList.SetFont(&m_font);
}


void CCalibrateDlg::SetCourierStrWidth(CString *str, int len)
{
	// changes str so that the length of str is = 10 by padding in spaces
	// note that spaces only count for 1/2 a char width in courier font.
	int spaces = 0;
	while(str->GetLength()<(len+spaces)) {
		*str = "  "+*str;
		spaces++;
	}
}

void CCalibrateDlg::OnExit() 
{
	// should I automatically update any calibrations here?
	//		e.g. the system calibration or any of the doc calibrations?

	CDialog::OnOK();
}
