// DACChannelList.cpp : implementation file
//
/****************************************************************

Boston University data analysis (BUDA) package 

Description: Buda is a data analysis program for Fourier 
Transform Mass Spectrometry data.  

by Peter B. O'Connor

    Copyright (C) 2000  Peter B. O'Connor

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "stdafx.h"
#include "Buda.h"
#include "DACChannelList.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CDACChannelList dialog


CDACChannelList::CDACChannelList(CWnd* pParent /*=NULL*/)
	: CDialog(CDACChannelList::IDD, pParent)
{
	//{{AFX_DATA_INIT(CDACChannelList)
	//}}AFX_DATA_INIT
}


void CDACChannelList::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CDACChannelList)
	DDX_Control(pDX, IDC_LIST1, m_DACList);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CDACChannelList, CDialog)
	//{{AFX_MSG_MAP(CDACChannelList)
	ON_LBN_DBLCLK(IDC_LIST1, OnDblclkList1)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CDACChannelList message handlers

BOOL CDACChannelList::OnInitDialog()
{
	BOOL retval = CDialog::OnInitDialog();
	int i;
	BOOL ChannelsListed[40];

	for(i=0; i<40; i++) {
		ChannelsListed[i]=FALSE;
		ChannelList[i]=0;
	}

	for(i=0; i<pPS->NumEvents; i++) {
		if(pPS->GenPS[i].Channel>15 && pPS->GenPS[i].Channel<40 && pPS->GenPS[i].Type != DAC_INIT) {
			ChannelsListed[pPS->GenPS[i].Channel]=TRUE;
		}
	}

	CString str;
	int chnum=0;
	for(i=16; i<40; i++) {
		if(InFlags == SELECT_UNUSED) {
			if(!ChannelsListed[i]) {
				str = pDoc->sys->ChannelNames[i];
				if(str == "UNUSED") break;
				m_DACList.AddString((LPCTSTR)str);
				ChannelList[chnum]=i;
				chnum++;
			}
		}
		if(InFlags == SELECT_CURRENT) {
			if(ChannelsListed[i]) {
				str = pDoc->sys->ChannelNames[i];
				m_DACList.AddString((LPCTSTR)str);
				ChannelList[chnum]=i;
				chnum++;
			}
		}
	}

	if(chnum==0) {
		m_DACList.AddString("No DAC's Available");
		m_DACList.EnableWindow(FALSE);
		OutFlag = CANCEL;
	} else {
		if(Channel>=0 && Channel<chnum) {
			m_DACList.SetCurSel(Channel);
		} else {
			Channel = 0;
			m_DACList.SetCurSel(Channel);
		}
	}

	return retval;
}

void CDACChannelList::OnCancel() 
{
	OutFlag = CANCEL;	
	CDialog::OnCancel();
}

void CDACChannelList::OnOK() 
{
	Channel = ChannelList[m_DACList.GetCurSel()];	
	CDialog::OnOK();
}

void CDACChannelList::OnDblclkList1() 
{
	OnOK();	
}
