// DACSetupDlg.cpp : implementation file
//
/****************************************************************

Boston University data analysis (BUDA) package 

Description: Buda is a data analysis program for Fourier 
Transform Mass Spectrometry data.  

by Peter B. O'Connor

    Copyright (C) 2000  Peter B. O'Connor

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "stdafx.h"
#include "Buda.h"
#include "BudaMaster.h"
#include "DACSetupDlg.h"
#include "LinkingDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CDACSetupDlg dialog


CDACSetupDlg::CDACSetupDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CDACSetupDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CDACSetupDlg)
	m_DACName = _T("");
	m_Delay = 0.0f;
	m_Duration = 0.0f;
	m_Voltage = 0.0f;
	m_StartTimeStr = _T("");
	m_TrapLinkStr = _T("");
	//}}AFX_DATA_INIT
}


void CDACSetupDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CDACSetupDlg)
	DDX_Control(pDX, IDC_START_TIME_STR, m_DelayCtrl);
	DDX_Control(pDX, IDC_DURATIONSTR, m_DurationCtrl);
	DDX_Control(pDX, IDC_DELAY, m_DelayEdit);
	DDX_Control(pDX, IDC_LINK_CHK, m_LinkChk);
	DDX_Control(pDX, IDOK, m_OkBtn);
	DDX_Control(pDX, IDC_DURATION, m_DurationEdit);
	DDX_Control(pDX, IDC_VOLTAGE, m_VoltageEdit);
	DDX_Control(pDX, IDC_DACINIT, m_DACInitBtn);
	DDX_Control(pDX, IDC_DACSTEP, m_DACStepBtn);
	DDX_Control(pDX, IDC_DACPULSE, m_DACPulseBtn);
	DDX_Control(pDX, IDC_DACRAMP, m_DACRampBtn);
	DDX_Control(pDX, IDC_EVENTLIST, m_EventListBox);
	DDX_Text(pDX, IDC_DACNAME, m_DACName);
	DDX_Text(pDX, IDC_DELAY, m_Delay);
	DDX_Text(pDX, IDC_DURATION, m_Duration);
	DDX_Text(pDX, IDC_VOLTAGE, m_Voltage);
	DDX_Text(pDX, IDC_START_TIME_STR, m_StartTimeStr);
	DDX_Text(pDX, IDC_TRAP_LINK_STR, m_TrapLinkStr);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CDACSetupDlg, CDialog)
	//{{AFX_MSG_MAP(CDACSetupDlg)
	ON_BN_CLICKED(IDC_LINK_DAC, OnLinkDac)
	ON_LBN_SELCHANGE(IDC_EVENTLIST, OnSelchangeEventlist)
	ON_EN_CHANGE(IDC_DELAY, OnChangeDelay)
	ON_EN_CHANGE(IDC_DURATION, OnChangeDuration)
	ON_EN_CHANGE(IDC_VOLTAGE, OnChangeVoltage)
	ON_BN_CLICKED(IDC_DACINIT, OnDacinit)
	ON_BN_CLICKED(IDC_DACPULSE, OnDacpulse)
	ON_BN_CLICKED(IDC_DACRAMP, OnDacramp)
	ON_BN_CLICKED(IDC_DACSTEP, OnDacstep)
	ON_BN_CLICKED(IDC_DELETE_EVENT, OnDeleteEvent)
	ON_BN_CLICKED(IDC_ADD_EVENT, OnAddEvent)
	ON_BN_CLICKED(IDC_CLEAR_ALL_EVENTS, OnClearAllEvents)
	ON_BN_CLICKED(IDC_LINK_CHK, OnLinkChk)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CDACSetupDlg message handlers

void CDACSetupDlg::OnLinkDac() 
{
	if(PS.GenPS[EventList[CurrentEvent]].Type == DAC_INIT) {
		MessageBox("Rejected:\nCannot link an Initialization event","Error: Invalid Input");
		return;
	}
	CLinkingDlg dlg;
	dlg.Linking = PS.GenPS[EventList[CurrentEvent]].Linking;
	dlg.InEvent = PS.GenPS[EventList[CurrentEvent]].Link2Event;
	dlg.pPS = &PS;
	dlg.pSys = pSys;
	dlg.pDoc = pDoc;

	if( dlg.DoModal() == IDOK) {
		PS.GenPS[EventList[CurrentEvent]].Linking = dlg.Linking;
		PS.GenPS[EventList[CurrentEvent]].Link2Event = dlg.InEvent;
	}
}

BOOL CDACSetupDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	int i;
	for(i=0; i<	MAX_PS_EVENTS; i++) EventList[i]=0;

	// force the dialog to update the window
	UpdateData(FALSE);

	// make a copy of PS, and in OnOK, copy it back.
	PS.cfp = 0;
	PS.arb = 0;
	PS.dac = 0;
	PS.GenPS = 0;
	PS.Spec = 0;

	pDoc->SortGenPS();
	pDoc->CopyPS(pPS, &PS);

	//initialize the Event list
	m_EventListBox.ResetContent();
	m_EventListBox.InitStorage(40,30);

	// set up the list box
	CurrentEvent = 0;
	SetupEventListBox();

	InitEventInfo();

	//if (StartFlags == ADD_NEW_EVENT) 
	//	OnAddEvent();

	return TRUE;
}

void CDACSetupDlg::InitEventInfo()
{
	CString str;
	// uses the event listed in CurrentEvent 
	CurrentEvent = m_EventListBox.GetCurSel();
	m_DACInitBtn.EnableWindow(TRUE);

	if(eventnum==0) {
		str.Format("No Events");
		m_EventListBox.AddString(str);
		return;
	}
	if(CurrentEvent < 0 || CurrentEvent > eventnum-1) {
		MessageBox("Invalid Event Number","Error");
		return;
	}
	if(InitEventExist && EventList[CurrentEvent] != InitEvent) {
		m_DACInitBtn.EnableWindow(FALSE);
		if (PS.GenPS[EventList[CurrentEvent]].Type == DAC_INIT) {
			MessageBox("Invalid Event:\n There is more than one initialization event\nChanging event type to NEW event","Error: Invalid event");
			PS.GenPS[EventList[CurrentEvent]].Type = BLANK_EVENT;
		}
	}
	m_LinkChk.EnableWindow(TRUE);

	if(PS.GenPS[EventList[CurrentEvent]].Type == DAC_INIT) {
		m_DACInitBtn.SetCheck(1);
		m_DACStepBtn.EnableWindow(TRUE);
		m_DACStepBtn.SetCheck(0);
		m_DACStepBtn.EnableWindow(FALSE);
		m_DACRampBtn.SetCheck(0);
		m_DACRampBtn.EnableWindow(FALSE);
		m_DACPulseBtn.SetCheck(0);
		m_DACPulseBtn.EnableWindow(FALSE);

		m_VoltageEdit.EnableWindow(TRUE);
		m_DelayEdit.EnableWindow(FALSE);
		m_DurationEdit.EnableWindow(FALSE);

		m_VoltageEdit.ShowWindow(SW_SHOW);
		m_DelayEdit.ShowWindow(SW_HIDE);
		m_DelayCtrl.ShowWindow(SW_HIDE);
		m_DurationEdit.ShowWindow(SW_HIDE);
		m_DurationCtrl.ShowWindow(SW_HIDE);

		PS.GenPS[EventList[CurrentEvent]].Linking = NO_LINK;
		PS.GenPS[EventList[CurrentEvent]].Link2Event = 0;
		m_LinkChk.SetCheck(0);
		m_LinkChk.EnableWindow(FALSE);
		m_LinkChk.ShowWindow(SW_HIDE);
	}
	if(PS.GenPS[EventList[CurrentEvent]].Type == DAC_STEP) {
		m_DACStepBtn.EnableWindow(FALSE);
		m_DACStepBtn.EnableWindow(TRUE);
		m_DACRampBtn.EnableWindow(TRUE);
		m_DACPulseBtn.EnableWindow(TRUE);
		m_DACInitBtn.SetCheck(0);
		m_DACStepBtn.SetCheck(1);
		m_DACRampBtn.SetCheck(0);
		m_DACPulseBtn.SetCheck(0);

		m_VoltageEdit.EnableWindow(TRUE);
		m_VoltageEdit.ShowWindow(SW_SHOW);
		m_DelayEdit.EnableWindow(TRUE);
		m_DelayEdit.ShowWindow(SW_SHOW);
		m_DelayCtrl.ShowWindow(SW_SHOW);
		m_DurationEdit.EnableWindow(FALSE);
		m_DurationEdit.ShowWindow(SW_HIDE);
		m_DurationCtrl.ShowWindow(SW_HIDE);

		m_LinkChk.EnableWindow(TRUE);
		m_LinkChk.ShowWindow(SW_SHOW);
	}
	if(PS.GenPS[EventList[CurrentEvent]].Type == DAC_PULSE) {
		m_DACStepBtn.EnableWindow(FALSE);
		m_DACStepBtn.EnableWindow(TRUE);
		m_DACRampBtn.EnableWindow(TRUE);
		m_DACPulseBtn.EnableWindow(TRUE);
		m_DACInitBtn.SetCheck(0);
		m_DACStepBtn.SetCheck(0);
		m_DACRampBtn.SetCheck(0);
		m_DACPulseBtn.SetCheck(1);

		m_VoltageEdit.EnableWindow(TRUE);
		m_DelayEdit.EnableWindow(TRUE);
		m_DurationEdit.EnableWindow(TRUE);

		m_VoltageEdit.ShowWindow(SW_SHOW);
		m_DelayEdit.ShowWindow(SW_SHOW);
		m_DelayCtrl.ShowWindow(SW_SHOW);
		m_DurationEdit.ShowWindow(SW_SHOW);
		m_DurationCtrl.ShowWindow(SW_SHOW);

		m_LinkChk.EnableWindow(TRUE);
		m_LinkChk.ShowWindow(SW_SHOW);
	}
	if(PS.GenPS[EventList[CurrentEvent]].Type == DAC_RAMP) {
		m_DACStepBtn.EnableWindow(FALSE);
		m_DACStepBtn.EnableWindow(TRUE);
		m_DACRampBtn.EnableWindow(TRUE);
		m_DACPulseBtn.EnableWindow(TRUE);
		m_DACInitBtn.SetCheck(0);
		m_DACStepBtn.SetCheck(0);
		m_DACRampBtn.SetCheck(1);
		m_DACPulseBtn.SetCheck(0);

		m_VoltageEdit.EnableWindow(TRUE);
		m_DelayEdit.EnableWindow(TRUE);
		m_DurationEdit.EnableWindow(TRUE);

		m_VoltageEdit.ShowWindow(SW_SHOW);
		m_DelayEdit.ShowWindow(SW_SHOW);
		m_DelayCtrl.ShowWindow(SW_SHOW);
		m_DurationEdit.ShowWindow(SW_SHOW);
		m_DurationCtrl.ShowWindow(SW_SHOW);

		m_LinkChk.EnableWindow(TRUE);
		m_LinkChk.ShowWindow(SW_SHOW);
	}
	if(PS.GenPS[EventList[CurrentEvent]].Type == BLANK_EVENT) {
		m_DACInitBtn.SetCheck(0);
		m_DACStepBtn.SetCheck(0);
		m_DACRampBtn.SetCheck(0);
		m_DACPulseBtn.SetCheck(0);
		m_DACStepBtn.EnableWindow(FALSE);
		m_DACStepBtn.EnableWindow(TRUE);
		m_DACRampBtn.EnableWindow(TRUE);
		m_DACPulseBtn.EnableWindow(TRUE);

		m_VoltageEdit.EnableWindow(FALSE);
		m_DelayEdit.EnableWindow(FALSE);
		m_DurationEdit.EnableWindow(FALSE);

		m_VoltageEdit.ShowWindow(SW_HIDE);
		m_DelayEdit.ShowWindow(SW_HIDE);
		m_DelayCtrl.ShowWindow(SW_HIDE);
		m_DurationEdit.ShowWindow(SW_HIDE);
		m_DurationCtrl.ShowWindow(SW_HIDE);

		m_LinkChk.EnableWindow(FALSE);
		m_LinkChk.ShowWindow(SW_HIDE);
	}

	m_Delay = PS.GenPS[EventList[CurrentEvent]].Delay;
	str.Format("%1.3f",m_Delay);

	m_Duration = PS.GenPS[EventList[CurrentEvent]].Width;
	str.Format("%1.3f",m_Duration);

	m_Voltage = PS.GenPS[EventList[CurrentEvent]].Value;
	str.Format("%1.3f",m_Voltage);

	if(PS.GenPS[EventList[CurrentEvent]].Linking == NO_LINK) {
		m_LinkChk.SetCheck(0);
		m_LinkChk.SetWindowText("Not Linked");
		m_StartTimeStr = "Start Time (msec):";
		m_TrapLinkStr = "";
	} else {
		m_LinkChk.SetCheck(1);
		m_LinkChk.SetWindowText("Linked to");
		m_StartTimeStr = "DELTA Start Time (msec):";
		m_TrapLinkStr = pDoc->GetLinkedToStr(&PS,EventList[CurrentEvent]);
		if(m_TrapLinkStr == "" ) {
			MessageBox("Error 86: A Link is defined to or from an invalid event\nWe will try to remove the link.  Please repair link manually","Error: Link to/from invalid event");
			PS.GenPS[EventList[CurrentEvent]].Linking = NO_LINK;
			PS.GenPS[EventList[CurrentEvent]].Link2Event = 0;
			m_LinkChk.SetCheck(0);
			m_LinkChk.SetWindowText("Not Linked");
			m_StartTimeStr = "Start Time (msec):";
		}
	}

	UpdateData(FALSE);
}

void CDACSetupDlg::OnSelchangeEventlist() 
{
	InitEventInfo();
}

void CDACSetupDlg::OnChangeDelay() 
{
	char buffer[100];
	float delay=m_Delay;
	int i;
	for (i=0; i<100; i++) buffer[i] = '\0';

	m_DelayEdit.GetLine(0,buffer,100);

	if(sscanf(buffer,"%f",&delay) <= 0) delay = 0;

	//todo: verify valid values of delay
	m_Delay = delay;

	PS.GenPS[EventList[CurrentEvent]].Delay = m_Delay;

	OnUpdate();
}

void CDACSetupDlg::OnChangeDuration() 
{
	char buffer[100];
	int i;
	for (i=0; i<100; i++) buffer[i] = '\0';
	m_DurationEdit.GetLine(0,buffer,100);

	if(sscanf(buffer,"%f",&m_Duration) <= 0) m_Duration = 0;
	PS.GenPS[EventList[CurrentEvent]].Width = m_Duration;
	OnUpdate();
}

void CDACSetupDlg::OnChangeVoltage() 
{
	char buffer[100];
	int i;
	for (i=0; i<100; i++) buffer[i] = '\0';
	m_VoltageEdit.GetLine(0,buffer,100);

	if(sscanf(buffer,"%f",&m_Voltage) <= 0) m_Voltage = 0;
	// now check to be sure that this voltage is a valid one.
	// it must be within the range specified by the PS.dac info.
	CString str;
	if(m_Voltage > pSys->dac[Channel-16].MaxValue) {
		str.Format("This DAC is limited to %1.3f Volts\n Voltage adjusted",pSys->dac[Channel-16].MaxValue);
		MessageBox(str,"Overvoltage");
		m_Voltage = pSys->dac[Channel-16].MaxValue;
	}
	if(m_Voltage < pSys->dac[Channel-16].MinValue) {
		str.Format("This DAC is limited to %1.3f Volts\n Voltage adjusted",pSys->dac[Channel-16].MinValue);
		MessageBox(str,"Undervoltage");
		m_Voltage = pSys->dac[Channel-16].MinValue;
	}
	
	PS.GenPS[EventList[CurrentEvent]].Value = m_Voltage;
	OnUpdate();
}

void CDACSetupDlg::OnDacinit() 
{
	PS.GenPS[EventList[CurrentEvent]].Type = DAC_INIT;
	OnUpdate();	
}

void CDACSetupDlg::OnDacpulse() 
{
	PS.GenPS[EventList[CurrentEvent]].Type = DAC_PULSE;
	OnUpdate();	
}

void CDACSetupDlg::OnDacramp() 
{
	PS.GenPS[EventList[CurrentEvent]].Type = DAC_RAMP;
	OnUpdate();	
}

void CDACSetupDlg::OnDacstep() 
{
	PS.GenPS[EventList[CurrentEvent]].Type = DAC_STEP;
	OnUpdate();	
}

void CDACSetupDlg::SetupEventListBox()
{
	// this function clears the Eventlistbox and then
	// fills it with new events
	m_EventListBox.ResetContent();
	//pDoc->SortGenPS();

	CString str;
	int i;
	eventnum=0;
	InitEventExist = FALSE;
	InitEvent = 0;
	for(i=0; i<PS.NumEvents; i++) {
		if(PS.GenPS[i].Channel == Channel) {
			if(PS.GenPS[i].Type == DAC_INIT) {
				InitEventExist = TRUE;
				InitEvent = i;
				str.Format("Init. DAC to %1.1f Volts",PS.GenPS[i].Value);
			}
			if(PS.GenPS[i].Type == DAC_STEP)
				str.Format("Step DAC to %1.1f Volts",PS.GenPS[i].Value);
			if(PS.GenPS[i].Type == DAC_PULSE)
				str.Format("Pulse DAC to %1.1f Volts",PS.GenPS[i].Value);
			if(PS.GenPS[i].Type == DAC_RAMP)
				str.Format("Ramp DAC to %1.1f Volts",PS.GenPS[i].Value);
			if(PS.GenPS[i].Type == BLANK_EVENT)
				str.Format("New Event");
			m_EventListBox.AddString(str);
			EventList[eventnum]=i;
			eventnum++;
		}
	}

	m_EventListBox.SetCurSel(CurrentEvent);
}

void CDACSetupDlg::OnUpdate()
{
	SetupEventListBox();
	InitEventInfo();
}

void CDACSetupDlg::OnDeleteEvent() 
{
	CurrentEvent = m_EventListBox.GetCurSel();
	if(PS.GenPS[EventList[CurrentEvent]].Type == DAC_INIT) {
		InitEventExist = FALSE;
		InitEvent = -1;
	}
	PS.GenPS[EventList[CurrentEvent]].Delay=0;
	PS.GenPS[EventList[CurrentEvent]].Width=0;
	PS.GenPS[EventList[CurrentEvent]].Value=0;
	PS.GenPS[EventList[CurrentEvent]].Type=NO_EVENT;
	PS.GenPS[EventList[CurrentEvent]].Channel=NO_CHANNEL;
	PS.GenPS[EventList[CurrentEvent]].Linking=NO_LINK;
	PS.GenPS[EventList[CurrentEvent]].Link2Event=0;
	CurrentEvent = 0;
	m_EventListBox.SetCurSel(CurrentEvent);
	OnUpdate();
}

void CDACSetupDlg::OnAddEvent() 
{
	CurrentEvent = eventnum;
	eventnum++;

	// find a NO_EVENT event in the GenPS
	int i;
	for(i=0; i<PS.NumEvents; i++) {
		if(PS.GenPS[i].Type == NO_EVENT) break;
	}
	EventList[CurrentEvent] = i;

	// add a blank event, but set a flag telling of the existence
	// of a blank event
	PS.GenPS[EventList[CurrentEvent]].Delay=0;
	PS.GenPS[EventList[CurrentEvent]].Width=0;
	PS.GenPS[EventList[CurrentEvent]].Value=0;
	PS.GenPS[EventList[CurrentEvent]].Type=BLANK_EVENT;
	PS.GenPS[EventList[CurrentEvent]].Channel=Channel;
	PS.GenPS[EventList[CurrentEvent]].Linking=NO_LINK;
	PS.GenPS[EventList[CurrentEvent]].Link2Event=0;
	m_EventListBox.SetCurSel(CurrentEvent);
	PS.NumEvents++;
	OnUpdate();
}

void CDACSetupDlg::OnClearAllEvents() 
{
	for(CurrentEvent=0; CurrentEvent<eventnum; CurrentEvent++) {
		PS.GenPS[EventList[CurrentEvent]].Delay=0;
		PS.GenPS[EventList[CurrentEvent]].Width=0;
		PS.GenPS[EventList[CurrentEvent]].Value=0;
		PS.GenPS[EventList[CurrentEvent]].Type=NO_EVENT;
		PS.GenPS[EventList[CurrentEvent]].Channel=NO_CHANNEL;
		PS.GenPS[EventList[CurrentEvent]].Linking=NO_LINK;
		PS.GenPS[EventList[CurrentEvent]].Link2Event=0;
	}
	OnUpdate();
}

void CDACSetupDlg::OnOK() 
{
	//	search for BLANK events among the DACS.  If they
	// exist, return after message boxing the error
	int i;
	for(i=0; i<eventnum; i++) {
		if(PS.GenPS[EventList[i]].Type == BLANK_EVENT) {
			// error there is a blank event.
			MessageBox("One or more undefined event(s)\nYou must define this event before exiting",
				"Error : Invalid Event");
			return;
		}
	}

	// check for overlapping events in time.  This is not allowed
	if(	CheckOverlappingEvents()) {
		MessageBox("Invalid value: \nTwo or more events overlap in time\nPlease correct this and continue","Error: Overlapping Events");
		return;
	}


	// copy back the modified PS
	pDoc->CopyPS(&PS, pPS);
	pDoc->SortGenPS();
	pDoc->UpdateAllViews(NULL,NULL,NULL);
	CDialog::OnOK();
}


BOOL CDACSetupDlg::CheckOverlappingEvents()
{
	// return FALSE if no overlapping events
	// otherwise, return TRUE
	int i,j;
	float start2,stop2;
	float start1,stop1;

	for(i=0; i<eventnum; i++) {
		pDoc->GetEventEdges(pPS,EventList[i],&start2,&stop2);
		for(j=i+1; j<eventnum; j++) {
			pDoc->GetEventEdges(pPS,EventList[j],&start1,&stop1);
			if(start1>=start2 && start1<=stop2) return TRUE;
			if(stop1>=start2 && stop1<=stop2) return TRUE;
			if(start2>=start1 && start2<=stop1) return TRUE;
			if(stop2>=start1 && stop2<=stop1) return TRUE;
		}
	}
	return FALSE;
}

void CDACSetupDlg::OnLinkChk() 
{
	if(PS.GenPS[EventList[CurrentEvent]].Type == DAC_INIT) {
		MessageBox("Rejected:\nCannot link an Initialization event","Error: Invalid Input");
		return;
	}

	CLinkingDlg dlg;
	dlg.Linking = PS.GenPS[EventList[CurrentEvent]].Linking;
	dlg.InEvent = PS.GenPS[EventList[CurrentEvent]].Link2Event;
	dlg.CurrentEvent = EventList[CurrentEvent];
	dlg.pPS = &PS;
	dlg.pSys = pDoc->sys;
	dlg.pDoc = pDoc;

	if( dlg.DoModal() == IDOK) {
		if(dlg.Linking != NO_LINK && dlg.InEvent == EventList[CurrentEvent]) {
			MessageBox("Cannot link an event to itself\nRejecting the link","Error: Invalid link");
			PS.GenPS[EventList[CurrentEvent]].Link2Event=0;
			PS.GenPS[EventList[CurrentEvent]].Linking = NO_LINK;
			
		} else {
			pDoc->RecalcEventTime(&PS, EventList[CurrentEvent],dlg.Linking,dlg.InEvent);		

			m_Delay = PS.GenPS[EventList[CurrentEvent]].Delay;
			m_Voltage = PS.GenPS[EventList[CurrentEvent]].Value;	

			InitEventInfo();
			UpdateData(FALSE);
		}
	}

	if(PS.GenPS[EventList[CurrentEvent]].Linking == NO_LINK) m_LinkChk.SetCheck(0);
	else m_LinkChk.SetCheck(1);
}
