// DeconOptionsDlg.cpp : implementation file
//
/****************************************************************

Boston University data analysis (BUDA) package 

Description: Buda is a data analysis program for Fourier 
Transform Mass Spectrometry data.  

by Peter B. O'Connor

    Copyright (C) 2000  Peter B. O'Connor

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "stdafx.h"
#include "buda.h"
#include "BudaMaster.h"
#include "DeconOptionsDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CDeconOptionsDlg dialog


CDeconOptionsDlg::CDeconOptionsDlg()
	: CPropertyPage(CDeconOptionsDlg::IDD)
{
	//{{AFX_DATA_INIT(CDeconOptionsDlg)
	m_HighMass = 0.0;
	m_LowMass = 0.0;
	m_MaxCharge = 0;
	m_DeconRes = 0.0;
	m_AdductMass = 0.0;
	m_MinCharge = 0;
	m_IsotopicallyResolved = FALSE;
	//}}AFX_DATA_INIT
}


void CDeconOptionsDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CDeconOptionsDlg)
	DDX_Control(pDX, IDC_MINCHARGE, m_MinChargeEdit);
	DDX_Control(pDX, IDC_DECON_RESOLUTION, m_DeconResEdit);
	DDX_Control(pDX, IDC_MAXCHARGE, m_MaxChargeEdit);
	DDX_Control(pDX, IDC_LOWMASS, m_LowMassEdit);
	DDX_Control(pDX, IDC_HIGHMASS, m_HighMassEdit);
	DDX_Text(pDX, IDC_HIGHMASS, m_HighMass);
	DDX_Text(pDX, IDC_LOWMASS, m_LowMass);
	DDX_Text(pDX, IDC_MAXCHARGE, m_MaxCharge);
	DDX_Text(pDX, IDC_DECON_RESOLUTION, m_DeconRes);
	DDX_Text(pDX, IDC_ADDUCT_MASS, m_AdductMass);
	DDX_Text(pDX, IDC_MINCHARGE, m_MinCharge);
	DDX_Control(pDX, IDC_DECONMETHOD_FENN, m_DeconFennBtn);
	DDX_Control(pDX, IDC_DECONMETHOD_REINHOLD, m_DeconReinholdBtn);
	DDX_Control(pDX, IDC_DECONMETHOD_OCONNOR, m_DeconOConnorBtn);
	DDX_Control(pDX, IDC_DECONMETHOD_ZHANG, m_DeconZhangBtn);
	DDX_Control(pDX, IDC_REPORTED_Z_0, m_Z0);
	DDX_Control(pDX, IDC_REPORTED_Z_1, m_Z1);
	DDX_Check(pDX, IDC_ISOTOPICALLY_RESOLVED, m_IsotopicallyResolved);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CDeconOptionsDlg, CDialog)
	//{{AFX_MSG_MAP(CDeconOptionsDlg)
	ON_BN_CLICKED(IDC_DECONMETHOD_FENN, OnDeconmethodFenn)
	ON_BN_CLICKED(IDC_DECONMETHOD_REINHOLD, OnDeconmethodReinhold)
	ON_BN_CLICKED(IDC_DECONMETHOD_OCONNOR, OnDeconmethodOconnor)
	ON_BN_CLICKED(IDC_DECONMETHOD_ZHANG, OnDeconmethodZhang)
	ON_EN_KILLFOCUS(IDC_HIGHMASS, OnKillfocusHighmass)
	ON_EN_KILLFOCUS(IDC_LOWMASS, OnKillfocusLowmass)
	ON_EN_KILLFOCUS(IDC_MAXCHARGE, OnKillfocusMaxcharge)
	ON_EN_KILLFOCUS(IDC_DECON_RESOLUTION, OnKillfocusDeconResolution)
	ON_COMMAND(ID_TOOLS_ELEMENTALCOMP, OnToolsElementalcomp)
	ON_BN_CLICKED(IDC_SAVE_AS_DEFAULTS, OnSaveAsDefaults)
	ON_BN_CLICKED(IDC_ISOTOPICALLY_RESOLVED, OnIsotopicallyResolved)
	ON_BN_CLICKED(IDC_REPORTED_Z_0, OnReportedZ0)
	ON_BN_CLICKED(IDC_REPORTED_Z_1, OnReportedZ1)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CDeconOptionsDlg message handlers

void CDeconOptionsDlg::OnDeconmethodFenn() 
{
	m_DeconMethod = DECON_FENN;
	m_DeconFennBtn.SetCheck(TRUE);
	m_DeconReinholdBtn.SetCheck(FALSE);
	m_DeconOConnorBtn.SetCheck(FALSE);
	m_DeconZhangBtn.SetCheck(FALSE);
}

void CDeconOptionsDlg::OnDeconmethodReinhold() 
{
	m_DeconMethod = DECON_REINHOLD;
	m_DeconFennBtn.SetCheck(FALSE);
	m_DeconReinholdBtn.SetCheck(TRUE);
	m_DeconOConnorBtn.SetCheck(FALSE);
	m_DeconZhangBtn.SetCheck(FALSE);
}

void CDeconOptionsDlg::OnDeconmethodOconnor() 
{
	m_DeconMethod = DECON_OCONNOR;
	m_DeconFennBtn.SetCheck(FALSE);
	m_DeconReinholdBtn.SetCheck(FALSE);
	m_DeconOConnorBtn.SetCheck(TRUE);
	m_DeconZhangBtn.SetCheck(FALSE);
}


void CDeconOptionsDlg::OnDeconmethodZhang() 
{
	m_DeconMethod = DECON_ZHANG;
	m_DeconFennBtn.SetCheck(FALSE);
	m_DeconReinholdBtn.SetCheck(FALSE);
	m_DeconOConnorBtn.SetCheck(FALSE);
	m_DeconZhangBtn.SetCheck(TRUE);
}

void CDeconOptionsDlg::OnKillfocusHighmass() 
{	
	ValidateParms();
}

void CDeconOptionsDlg::OnKillfocusLowmass() 
{
	ValidateParms();
}

void CDeconOptionsDlg::OnKillfocusMaxcharge() 
{
	ValidateParms();
}

void CDeconOptionsDlg::OnUpdate()
{
	ValidateParms();

	switch(m_DeconMethod) {
	case DECON_FENN:
		OnDeconmethodFenn();
		break;
	case DECON_REINHOLD:
		OnDeconmethodReinhold();
		break;
	case DECON_OCONNOR:
		OnDeconmethodOconnor();
		break;
	case DECON_ZHANG:
		OnDeconmethodZhang();
		break;
	}

	switch(m_ReportedZ) {
	case 0:
		OnReportedZ0();
		break;
	case 1:
		OnReportedZ1();
		break;
	default:
		OnReportedZ0();
		break;
	}

	UpdateData(FALSE);
}

BOOL CDeconOptionsDlg::OnSetActive() 
{
	OnUpdate();
	return CPropertyPage::OnSetActive();
}

void CDeconOptionsDlg::ValidateParms()
{
	// if the decon method isn't defined as one of the
	// recognized types, define it as the default type
	switch(m_DeconMethod) {
	case DECON_FENN:
		break;
	case DECON_REINHOLD:
		break;
	case DECON_OCONNOR:
		break;
	case DECON_ZHANG:
		break;
	default:
		m_DeconMethod = DECON_DEFAULT;
		break;
	}

	// is the maximum charge state range reasonable?
	if(m_MaxCharge <1) m_MaxCharge = 1;
	if(m_MaxCharge > 100) m_MaxCharge = 100;

	// are the lowmasses and highmasses reasonable?


}

void CDeconOptionsDlg::OnOK() 
{
	// TODO: Add extra validation here
	
	CPropertyPage::OnOK();
}


void CDeconOptionsDlg::OnKillfocusDeconResolution() 
{
	ValidateParms();
}

void CDeconOptionsDlg::OnToolsElementalcomp() 
{
	// TODO: Add your command handler code here
	
}

void CDeconOptionsDlg::OnSaveAsDefaults() 
{
	CBudaApp* pApp;
	pApp = (CBudaApp*)AfxGetApp();

	pApp->DeconMethod = m_DeconMethod;
	pApp->DeconResolution = m_DeconRes;
	pApp->DeconHighMassLimit = m_HighMass;
	pApp->DeconLowMassLimit = m_LowMass;
	pApp->DeconHighChargeLimit = m_MaxCharge;
	pApp->DeconLowChargeLimit = m_MinCharge;
	pApp->DeconAdductMass = m_AdductMass;

	pDoc->DeconMethod = m_DeconMethod;
	pDoc->DeconResolution = m_DeconRes;
	pDoc->DeconHighMassLimit = m_HighMass;
	pDoc->DeconLowMassLimit = m_LowMass;
	pDoc->DeconHighChargeLimit = m_MaxCharge;
	pDoc->DeconLowChargeLimit = m_MinCharge;
	pDoc->DeconAdductMass = m_AdductMass;

	pApp->WriteIniFile();		
}

void CDeconOptionsDlg::OnIsotopicallyResolved() 
{
	// TODO: Add your control notification handler code here
	return;
}

void CDeconOptionsDlg::OnReportedZ0() 
{
	m_ReportedZ = 0;
	m_Z0.SetCheck(TRUE);
	m_Z1.SetCheck(FALSE);
}

void CDeconOptionsDlg::OnReportedZ1() 
{
	m_ReportedZ = 1;
	m_Z0.SetCheck(FALSE);
	m_Z1.SetCheck(TRUE);
}
