#if !defined(DeconStruct_h)
#define DeconStruct_h
#include "Deconvolution.h"

#define MAX_DECON_ARRAY_SIZE 8192

// Decon error values
#define DECON_PEAKS_TOO_LARGE -10
#define DECON_TRIED_TO_EXCEDE_DATA_RANGE -11
#define DECON_OVERFLOW_PEAK_BUFFER -12

typedef float CfReal;
typedef CfReal CfRealFunc(CfReal);

struct DeconStruct {
	CfReal *Data;	// pointer to raw data array from XMass
	long DataSize;  // length of frequency array from XMass

	//CfRealFunc *MassToPt;
	//CfRealFunc *PtToMass;

	// predefine these arrays to ~8k each and pass the pointers
	CfReal *DeconMass;	// mass array from Decon
	CfReal *DeconIntensity;  // data array from Decon
	long DeconSize;  // length of DeconMass and DeconIntensity arrays
	double NoiseLevel;
	double NoiseSigma;

	double LowestMZ;  // nyquist low mass limit (high freq limit)
	double HighestMZ; // defined by raw data.  What is the highest m/z the instrument can see?  

	long MaxCharge;		// user defined (typically defined = 30 ) 
	long MinCharge;		// user defined (typically defined = 1 )
	double MaxMass;		// user defined (typically defined = 20000 Da)
	double MinMass;		// user defined (typically defined = 1000 Da)
	double MassResolution;	// user defined (typically defined = 0.1 Da)
	double AdductMass;	// user defined  typically 1.0072 but could be Na+
	long IsotopicallyResolved;  // user defined  high/low res flag (1 = high, 0 = low)
	long DeconChargeState;	// user defined (typically either +0 or +1)
	
	// return status
	long DeconStatus;  // if 0, decon worked fine.  if < 0 an error occurred.
};

#endif // !defined(DeconStruct_h)