// DeconView.cpp : implementation file
//
/****************************************************************

Boston University data analysis (BUDA) package 

Description: Buda is a data analysis program for Fourier 
Transform Mass Spectrometry data.  

by Peter B. O'Connor

    Copyright (C) 2000  Peter B. O'Connor

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "stdafx.h"
#include "buda.h"
#include "DeconView.h"
#include <math.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CDeconView

IMPLEMENT_DYNCREATE(CDeconView, CView)

CDeconView::CDeconView()
{
}

CDeconView::~CDeconView()
{
}


BEGIN_MESSAGE_MAP(CDeconView, CView)
	//{{AFX_MSG_MAP(CDeconView)
	ON_WM_RBUTTONDOWN()
	ON_WM_LBUTTONUP()
	ON_WM_LBUTTONDOWN()
	ON_WM_MOUSEMOVE()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CDeconView drawing

void CDeconView::OnDraw(CDC* pDeviceContext)
{
	pDC = pDeviceContext;

	if(flags.InitializeWindow) InitializeDrawParms();

	PrepareView();
	DrawAxis();
	DrawParms();
	//if(flags.Redraw) 
	DrawData();
	DrawVerticalAxis();
	LabelPeaks();

}

/////////////////////////////////////////////////////////////////////////////
// CDeconView diagnostics

#ifdef _DEBUG
void CDeconView::AssertValid() const
{
	CView::AssertValid();
}

void CDeconView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CDeconView message handlers

void CDeconView::InitializeZoomStack()
{
	int i;

	for(i=0; i<20; i++) {
		zoomstack[i].minx=(float)DeconLowMassLimit;
		zoomstack[i].maxx=(float)DeconHighMassLimit;
		zoomstack[i].miny=0;
		zoomstack[i].maxy=1.0;
	}

	flags.UpdateScaleFactor = TRUE;
	flags.ViewIsSetup=FALSE;
	zoompointer = 0;
}

void CDeconView::OnInitialUpdate() 
{
	CView::OnInitialUpdate();
	pDoc = (CBudaDoc*)GetDocument();
	pApp = (CBudaApp*)AfxGetApp();

	DeconMethod = pDoc->DeconMethod;
	DeconHighMassLimit = pDoc->DeconHighMassLimit;
	DeconLowMassLimit = pDoc->DeconLowMassLimit;
	DeconHighChargeLimit = pDoc->DeconHighChargeLimit;
	DeconRes = pDoc->DeconResolution;

	ViewSize.right=0;
	ViewSize.left=0;
	ViewSize.top=0;
	ViewSize.bottom=0;

	flags.Zooming = FALSE;
	flags.ClearViewLine = TRUE;
	flags.CalcViewLine = TRUE;
	flags.RemoveDC = TRUE;
	flags.RecalcData = TRUE;
	flags.IsZoomingY = FALSE;
	flags.PlotYAxis = TRUE;
	flags.PlotParameters = FALSE;
	flags.InitializedWindowSize = FALSE;
	flags.UpdateScaleFactor = TRUE;
	flags.ViewIsSetup = TRUE;
	flags.Redraw = TRUE;
	flags.InitializeWindow = TRUE;

	InitializeZoomStack();
	VScale = 1.0;
}

void CDeconView::DrawVerticalAxis()
{
	if(!flags.PlotYAxis) return;

	// this function draws a time axis out to the last event
	// in the pulse sequence
	int AxisLineYstart = PlotArea.bottom;
	int AxisLineYend = PlotArea.top;
	int AxisLineXpos = PlotArea.left;
	long i;

	double ybase = (double)ViewLineMaxy - (double)ViewLineMiny;

	// second draw the axis line 
	MoveTo(AxisLineXpos,PlotArea.bottom);
	LineTo(AxisLineXpos,PlotArea.top);

	// place tics
	// there will be <=10 tics
	int ticpos[10];
	double ticy[10];
	for(i=0; i<10; i++) {
		ticpos[i] = 0;
		ticy[i] = 0;
	}

	// there will be 5-10 tics  
	// ticsteps can be (order of magnitude of timebase) / 1, 2, 5 
	double a = log10(ybase);
	double b = floor(a);	// b = order of magnitude of timebase
	double ticstep = pow(10,b);
	double nt;	
	int numtics;
	nt = ybase/ ticstep;

	if(nt>=5) numtics = (int)nt;
	if(nt>2 && nt<5) {
		ticstep=pow(10,b)/2;
		numtics = (int)(ybase/ticstep);
	}
	if(nt<=2) {
		ticstep = pow(10,b)/5;
		numtics = (int)(ybase/ticstep);
	}

	// find the position of the first tic.
	// if zero is in the range, I want zero to be one of the tics
	double NumTixb4Zero = (double)ViewLineMiny/ticstep;
	if(NumTixb4Zero>0) {// GenPS starts after zero
		ticy[0]=(floor(NumTixb4Zero)+1) * ticstep;
	}
	if (NumTixb4Zero == 0) ticy[0] = 0;
	if (NumTixb4Zero < 0) { // GenPS starts before zero
		ticy[0] = -1 * floor(-1*NumTixb4Zero) * ticstep;
	}

	// find the position of each tic relative to ticy[0]
	for(i=1; i<10; i++) {
		ticy[i]=ticstep*i+ticy[0];
		if(ticy[i]>ViewLineMaxy) break;
	}
	numtics = i-1;

	// find tic x positions
	// remember that position 1000 = FirstEventStart
	// and 31768 = LastEventEnd;
				// create a linear eqn y = a*x + b
				// where y's are the plotspace #'s and 
				// x's are the timebase space #'s
	a = (AxisLineYend-AxisLineYstart)/(ViewLineMaxy - ViewLineMiny);
	b = AxisLineYstart - a*ViewLineMiny;
	for(i=0; i<numtics; i++) {
		ticpos[i] = int(a*ticy[i] + b);
	}

	// now actually plot the tics
	for(i=0; i<numtics; i++) {
		MoveTo(AxisLineXpos,ticpos[i]);
		LineTo(AxisLineXpos-500,ticpos[i]);
	}

	// lable tics
	CString str;
	CPoint pt;
	CSize sz;

	str.Format("Raw ADC Value");
	pt.y = 16384;
	pt.y = AxisLineXpos - 2000;
	//VerticalText(str,pt,CENTER_JUSTIFIED);

	for(i=0; i<numtics; i++) {
		str.Format("%1.0f",ticy[i]);
		pt.y = ticpos[i];
		pt.x = AxisLineXpos - 600 ;
		Text(str,pt,RIGHT_JUSTIFIED);
	}

	// end of draw vertical axis
}

void CDeconView::Text(CString str, CPoint pt, int just)
{
	pDC = GetDC();
	CSize sz = pDC->GetTextExtent(str);

	CPoint pt1,pt2;
	pt1 = PTtoViewPT(pt);
	if(just == CENTER_JUSTIFIED) pt1.x = pt1.x-sz.cx/2;
	if(just == LEFT_JUSTIFIED) pt1.y = pt1.y - sz.cy/2;
	if(just == CENTER_JUSTIFIED2) {
		pt1.x = pt1.x-sz.cx/2;
		pt1.y = pt1.y - sz.cy/2;
	}
	if(just == RIGHT_JUSTIFIED) {
		pt1.x = pt1.x - sz.cx;
		pt1.y = pt1.y - sz.cy/2;
	}

	pDC->TextOut(pt1.x,pt1.y,str);
}

void CDeconView::DrawParms()
{

}

void CDeconView::DrawData()
{
	DrawDeconData();
	flags.Redraw = FALSE;
}

void CDeconView::LabelPeaks()
{

}

void CDeconView::DrawDeconAxis()
{
	DrawGenericAxis(PlotArea.left, PlotArea.right, float(zoomstack[zoompointer].minx), float(zoomstack[zoompointer].maxx), "mass (Da)", "%1.1f");
}

void CDeconView::DrawGenericAxis(int PlotStart, int PlotEnd, float DataStart, float DataEnd, CString caption, CString labelformat)
{
	// this function draws a time axis out to the last event
	// in the pulse sequence
	int AxisLineXstart = PlotStart;
	int AxisLineXend = PlotEnd;
	long i;
	double MassStart = DataStart;
	double MassEnd = DataEnd;

	double massbase = (double)MassEnd - (double)MassStart;

	// second draw the axis line 
	MoveTo(AxisLineXstart,AxisLineYpos);
	LineTo(AxisLineXend,AxisLineYpos);

	// place tics
	// there will be <=10 tics
	int ticpos[10];
	double ticmass[10];
	for(i=0; i<10; i++) {
		ticpos[i] = 0;
		ticmass[i] = 0;
	}

	// there will be 5-10 tics  
	// ticsteps can be (order of magnitude of massbase) / 1, 2, 5 
	double a = log10(massbase);
	double b = floor(a);	// b = order of magnitude of massbase
	double ticstep = pow(10,b);
	double nt;	
	int numtics;
	nt = massbase/ ticstep;

	if(nt>=5) numtics = (int)nt;
	if(nt>2 && nt<5) {
		ticstep=pow(10,b)/2;
		numtics = (int)(massbase/ticstep);
	}
	if(nt<=2) {
		ticstep = pow(10,b)/5;
		numtics = (int)(massbase/ticstep);
	}

	// find the position of the first tic.
	// if zero is in the range, I want zero to be one of the ticx
	double NumTixb4Zero = (double)MassStart/ticstep;
	if(NumTixb4Zero>0) {// GenPS starts after zero
		ticmass[0]=(floor(NumTixb4Zero)+1) * ticstep;
	}
	if (NumTixb4Zero == 0) ticmass[0] = 0;
	if (NumTixb4Zero < 0) { // GenPS starts before zero
		ticmass[0] = -1 * floor(-1*NumTixb4Zero) * ticstep;
	}

	// find the position of each tic relative to ticmass[0]
	for(i=1; i<10; i++) {
		ticmass[i]=ticstep*i+ticmass[0];
		if(ticmass[i]>MassEnd) break;
	}
	numtics = i-1;

	// find tic x positions
	// remember that position 1000 = FirstEventStart
	// and 31768 = LastEventEnd;
				// create a linear eqn y = a*x + b
				// where y's are the plotspace #'s and 
				// x's are the massbase space #'s
	a = (AxisLineXend-AxisLineXstart)/(MassEnd - MassStart);
	b = AxisLineXstart - a*MassStart;
	for(i=0; i<numtics; i++) {
		ticpos[i] = int(a*ticmass[i] + b);
	}

	// now actually plot the tics
	for(i=0; i<numtics; i++) {
		MoveTo(ticpos[i],AxisLineYpos);
		LineTo(ticpos[i],AxisLineYpos-500);
	}

	// lable tics
	CString str;
	CPoint pt;
	CSize sz;

	pt.x = 16384;
	pt.y = AxisLineYpos - 2000;
	Text(caption,pt,CENTER_JUSTIFIED);

	// probably will want to decide on how to print these numbers.
	// won't want %1.1f each time.
	for(i=0; i<numtics; i++) {
		str.Format(labelformat,ticmass[i]);
		pt.x = ticpos[i];
		pt.y = AxisLineYpos - 600;
		Text(str,pt,CENTER_JUSTIFIED);
	}
	// end of draw axis
}

void CDeconView::DrawDeconData()
{
	long i;
	CPoint pt;

	if(flags.CalcViewLine) {
		flags.CalcViewLine = FALSE;
		CalculateDeconViewLine();
	}

	ViewLineMaxx = zoomstack[zoompointer].maxx;
	ViewLineMinx = zoomstack[zoompointer].minx;

	float m1, b1, m2, b2;
	if(ViewLineMaxy != ViewLineMiny) 
		m1 = (PlotArea.top - PlotArea.bottom)/(ViewLineMaxy - ViewLineMiny);
	else m1 = 1;
//	b1 = -1*m1*ViewLineMiny + PlotArea.bottom;
	b1 = PlotArea.bottom;

	if(ViewLineMaxx != ViewLineMinx)
		m2 = (PlotArea.right - PlotArea.left)/(ViewLineMaxx - ViewLineMinx);
	else m2 = 1;
	//b2 = -1*m2*ViewLineMinx + PlotArea.left;
	b2 = PlotArea.left;

	pt.y = long(m1*(ViewLine[0].y-ViewLineMiny) + b1);
	pt.x = long(m2*(ViewLine[0].x-ViewLineMinx) + b2);

	//pt = DataToViewPoint(ViewLine[0], m1, b1, m2, b2);
	TRACE("PlotArea.top = %d\n",PlotArea.top);
	TRACE("PlotArea.bottom = %d\n",PlotArea.bottom);
	TRACE("PlotArea.right = %d\n",PlotArea.right);
	TRACE("PlotArea.left = %d\n",PlotArea.left);
	TRACE("ViewLineMaxy = %1.5f\n",ViewLineMaxy);
	TRACE("ViewLineMiny = %1.5f\n",ViewLineMiny);
	TRACE("ViewLineMaxx = %1.5f\n",ViewLineMaxx);
	TRACE("ViewLineMinx = %1.5f\n",ViewLineMinx);
	TRACE("m1=%1.5f\n",m1);
	TRACE("b1=%1.5f\n",b1);
	TRACE("m2=%1.5f\n",m2);
	TRACE("b2=%1.5f\n",b2);

	MoveTo(pt);
	TRACE("ViewLineLength = %d\n",ViewLineLength);
	TRACE("ViewLine[%d];ViewPt[%d] = %1.1f,%1.1f;%d,%d\n",0,0,ViewLine[0].x,ViewLine[0].y,pt.x,pt.y);
	for(i=1; i<ViewLineLength; i++) {
		pt.y = long(m1*(ViewLine[i].y-ViewLineMiny) + b1);
		pt.x = long(m2*(ViewLine[i].x-ViewLineMinx) + b2);
		//pt = DataToViewPoint(ViewLine[i], m1, b1, m2, b2);
		LineTo(pt);
		TRACE("ViewLine[%d];ViewPt[%d] = %1.1f,%1.1f;%d,%d\n",i,i,ViewLine[i].x,ViewLine[i].y,pt.x,pt.y);
	}
	zoomstack[zoompointer].maxy = ViewLineMaxy;
}

void CDeconView::CalculateDeconViewLine()
{
	long i;
	// clearing the viewline array.
	if(flags.ClearViewLine) {
		struct ViewLinePt NullPt;
		NullPt.x = 0;
		NullPt.y = 0;
		for(i=0; i<MAX_VIEWLINE_LENGTH; i++) ViewLine[i] = NullPt;
		flags.ClearViewLine = FALSE;
	}

	// call the appropriate viewline function
	if(flags.RecalcData) {
		switch(DeconMethod) {
		case DECON_FENN:
			CalculateFennDecon();
			break;
		case DECON_REINHOLD:
			CalculateReinholdDecon();
			break;
		case DECON_OCONNOR:
			CalculateOConnorDecon();
			break;
		case DECON_ZHANG:
			CalculateZhangDecon();
			break;
		}
		flags.RecalcData = FALSE;
	}
	
	DecimateDeconData();

	// set the datadirty to FALSE since we no longer need to recalculate
	flags.RecalcData = FALSE;
	flags.CalcViewLine = FALSE;
}

void CDeconView::PrepareView()
{
	CSize sz;
	CFont* pFnt;
	CPoint pt;

	pDC = GetDC();

	float FontScaleFactor = float(ViewSize.right - ViewSize.left)/float(InitialWindowSize.cx);
	if(FontScaleFactor > float(ViewSize.bottom - ViewSize.top)/float(InitialWindowSize.cy))
		FontScaleFactor = float(ViewSize.bottom - ViewSize.top)/float(InitialWindowSize.cy);
	// note: how do i rescale the size of bitmap and text when size of window changes?

	// now the MALDI/Esi Sign.
	CString str = "Buda v1.1: ESI-FTMS Deconvolution";
	sz.cx = 40;
	sz.cy = 40;
	pt.x = ViewSize.left;
	pt.y = ViewSize.top ;
	pt.x = pt.x + sz.cx;
	pt.y = pt.y ;
	pFnt = pDC->GetCurrentFont();

	CFont SignFont, TextFont;
	SignFont.CreatePointFont(int(FontScaleFactor*120),"Arial Black",pDC);
	pDC->SelectObject(&SignFont);
	pDC->TextOut(pt.x,pt.y,str);	// put the text on the page.
	sz = pDC->GetTextExtent(str);
	pDC->SelectObject(pFnt);

	TextFont.CreatePointFont(int(FontScaleFactor*70),"Times New Roman",pDC);
	pDC->SelectObject(&TextFont);

	// now add memo
	pt.y += sz.cy;
	str.Format("File : %s",pDoc->GetPathName());
	pDC->TextOut(pt.x,pt.y,str);	// put the text on the page.
	sz = pDC->GetTextExtent(str);
	pt.y += sz.cy;
	str.Format("%s",pDoc->PS.ExperimentMemo);
	pDC->TextOut(pt.x,pt.y,str);	// put the text on the page.

	// now add top parameters.
	sz = pDC->GetTextExtent(" 99-DEC-2000  ");	// dummy string to get the right spacing
	pt.x = ViewSize.right - sz.cx - 5;
	pt.y = ViewSize.top;
	CPoint pt1 = pt;	// for alignment of the ':' characters

	CString str1;
	str1.Format("%s",pDoc->PS.TimeAcquired);
	int space = str1.Find(" ",0);

	str = "Date : ";
	sz = pDC->GetTextExtent(str);	// right align the field name.
	pt1.x = pt.x - sz.cx;
	pt1.y = pt.y;
	pDC->TextOut(pt1.x,pt1.y,str);	// put the text on the page.
	pDC->TextOut(pt.x,pt.y,str1.Left(space));	// put the text on the page.
	sz = pDC->GetTextExtent(str);	// shift down pt for the next one.
	pt.y += long(0.9*sz.cy);

	str = "Time : ";
	sz = pDC->GetTextExtent(str);	// right align the field name.
	pt1.x = pt.x - sz.cx;
	pt1.y = pt.y;
	pDC->TextOut(pt1.x,pt1.y,str);	// put the text on the page.
	pDC->TextOut(pt.x,pt.y,str1.Right(str1.GetLength() - space - 1));	// put the text on the page.
	sz = pDC->GetTextExtent(str);	// shift down pt for the next one.
	pt.y += long(0.9*sz.cy);
}

void CDeconView::MoveTo(int x, int y)
{
	pDC->MoveTo(PTtoViewPT(x,y));
}

void CDeconView::MoveTo(CPoint pt)
{
	pDC->MoveTo(PTtoViewPT(pt));
}

void CDeconView::LineTo(int x, int y)
{
	pDC->LineTo(PTtoViewPT(x,y));
}

void CDeconView::LineTo(CPoint pt)
{
	pDC->LineTo(PTtoViewPT(pt));
}

CPoint CDeconView::PTtoViewPT(CPoint pt)
{
	// this function maps the window specified by ViewSize
	// into a logical window from lower-left = 0,0 to 
	// upper right = 32767,32767

	// the input variable pt is assumed to be in the 0-32767
	// window, while the output is in the viewsize window.

	if (pt.x>32767) pt.x = 32767;
	if (pt.y>32767) pt.y = 32767;
	if (pt.x<0) pt.x = 0;
	if (pt.y<0) pt.y = 0;

	CPoint outpt;
	double x,y,mx,my,by;
	my = ((double)ViewSize.bottom-(double)ViewSize.top)/32768.0;
	mx = ((double)ViewSize.right-(double)ViewSize.left)/32768.0;
	by = (double)ViewSize.bottom-(double)ViewSize.top;
	y = by - my*(double)pt.y;
	x = mx*(double)pt.x;

	outpt.y = (short)y;
	outpt.x = (short)x;
	return outpt;
}

CPoint CDeconView::ViewPTtoPT(CPoint pt)
{
	CPoint outpt;
	double x,y,mx,my,by;

	if (pt.x>ViewSize.right) pt.x = ViewSize.right;
	if (pt.y<ViewSize.top) pt.y = ViewSize.top;
	if (pt.x<ViewSize.left) pt.x = ViewSize.left;
	if (pt.y>ViewSize.bottom) pt.y = ViewSize.bottom;

	my = 32768.0/((double)ViewSize.bottom-(double)ViewSize.top);
	mx = 32768.0/((double)ViewSize.right-(double)ViewSize.left);
	by = (double)ViewSize.bottom-(double)ViewSize.top;
	y = my*(double)(pt.y - ViewSize.top);
	x = mx*(double)(pt.x - ViewSize.left) ;

	outpt.y = (short)y;
	outpt.x = (short)x;

	return outpt;
}

CPoint CDeconView::PTtoViewPT(int x, int y)
{
	CPoint pt;
	pt.x=x;
	pt.y=y;
	return PTtoViewPT(pt);
}
	
void CDeconView::DrawAxis()
{
	DrawDeconAxis();
}

void CDeconView::CalculateFennDecon()
{
	long i,z;
	long TotalPoints;
	double CurrentMass;
	double CurrentIntensity;
	double TempDeconRes;

	TempDeconRes = DeconRes;
	TotalPoints = (long)((DeconHighMassLimit - DeconLowMassLimit) / TempDeconRes);
	while(TotalPoints > MAX_DECON_SIZE) {
		TempDeconRes*=2;
		TotalPoints = (long)((DeconHighMassLimit - DeconLowMassLimit) / TempDeconRes);
	}
	if(TempDeconRes != DeconRes) {
		MessageBox("Deconvolution requested too many data points.  Adjusting resolution to compensate","Memory Error",MB_OK);
		DeconRes = TempDeconRes;
		pDoc->DeconResolution = DeconRes;
		pApp->DeconResolution = DeconRes;
	}

	for(i=0; i<TotalPoints; i++) {
		// this is the main loop.
		CurrentMass = (double)i*DeconRes + DeconLowMassLimit;
		CurrentIntensity=0;
		// Fenn deconvolution is the sum of the data points at mass/z 
		// where z is the charges expected .... regardless of noise problems
		for(z=DeconHighChargeLimit; z>0; z--) {
			CurrentIntensity += GetIntensityatMass(CurrentMass/(double)z);
		}
		DeconData[i] = (float)CurrentIntensity;
	} // end for i
}

void CDeconView::CalculateReinholdDecon()
{

}

void CDeconView::CalculateOConnorDecon()
{

}

void CDeconView::CalculateZhangDecon()
{

}

void CDeconView::DecimateDeconData()
{
	long i,j;

	// this function decimates the raw deconvolution data into 
	// an acceptable viewline.
	long minx,maxx,temp;
	double beginmass, endmass, tempmass, massstep;
	long lowpt, highpt;
	double rangepts;
	float MaxIntensity, MinIntensity;

	ViewLineLength = MAX_VIEWLINE_LENGTH;

	// get current mass window limits
	beginmass = zoomstack[zoompointer].minx;
	endmass = zoomstack[zoompointer].maxx;
	if(beginmass < DeconLowMassLimit) beginmass = DeconLowMassLimit;
	if(endmass < DeconLowMassLimit) endmass = DeconLowMassLimit;
	if(beginmass > DeconHighMassLimit) beginmass = DeconHighMassLimit;
	if(endmass > DeconHighMassLimit) endmass = DeconHighMassLimit;
	if(endmass == beginmass) {
		beginmass = DeconLowMassLimit;
		endmass = DeconHighMassLimit;
	}
	if(beginmass > endmass) {
		tempmass = beginmass;
		beginmass = endmass;
		endmass = tempmass;
	}
	ViewLineMaxx = (float)endmass;
	ViewLineMinx = (float)beginmass;

	// Decon Mass to point conversion
	minx = (long)((beginmass-DeconLowMassLimit)*DeconRes);
	maxx = (long)((endmass-DeconLowMassLimit)*DeconRes);
	if(minx>maxx) {
		temp=minx;
		minx=maxx;
		maxx=temp;
	}

	massstep=(endmass-beginmass)/ViewLineLength;
	rangepts = (maxx-minx)/float(ViewLineLength);
	if ((maxx-minx)<ViewLineLength) {  
		// if zoomed in far enough, plot all the points
		ViewLineLength = maxx-minx;
		for(i=0; i<ViewLineLength; i++) {
			ViewLine[i].x = (float)(beginmass + (double)i*massstep);
			ViewLine[i].y = DeconData[minx+i];
			if(i==0) {
				ViewLineMaxy = ViewLine[0].y;
				ViewLineMiny = ViewLine[0].y;
			}
			if(ViewLineMaxy < ViewLine[i].y) ViewLineMaxy = ViewLine[i].y;
			if(ViewLineMiny > ViewLine[i].y) ViewLineMiny = ViewLine[i].y;
		}
	} else { 
		// not zoomed in far enough.  gotta decimate.
		for(i=0; i<ViewLineLength; i+=2) {
			lowpt = minx + (long)((float)i*rangepts);
			highpt = minx + (long)((float)(i+2)*rangepts);
			MaxIntensity = 0;
			MinIntensity = DeconData[0];
			for(j=lowpt; j<highpt; j++) {
				if(DeconData[j] > MaxIntensity) MaxIntensity = DeconData[j];
				if(DeconData[j] < MinIntensity) MinIntensity = DeconData[j];
			}
			ViewLine[i].x = (float)(beginmass + ((float)i+0.5)*massstep);
			ViewLine[i].y = MaxIntensity;
			ViewLine[i+1].x = (float)(beginmass + ((float)i+0.5)*massstep);
			ViewLine[i+1].y = MinIntensity;
			if(i==0) {
				ViewLineMaxy = MaxIntensity;
				ViewLineMiny = MinIntensity;
			}
			if(ViewLineMaxy < MaxIntensity) ViewLineMaxy = MaxIntensity;
			if(ViewLineMiny > MinIntensity) ViewLineMiny = MinIntensity;
		}
	}
}

double CDeconView::GetIntensityatMass(double mass)
{
	// this function returns the FTData value at a particular
	// mass to charge ratio (mz).  If this falls between to data
	// points in the FT domain, it interpolates linearly.
	long pt0,pt1,pt2;
	double int1,int2,intensity,slope;

	if(mass < pDoc->PS.DetectLowMass) return 0;
	if(mass > pDoc->PS.DetectHighMass) return 0;

	pt0 = pDoc->MassToPt(mass);
	double tempmass;

	// check if mass of pt1 is above or below mass
	tempmass = pDoc->PtToMass(pt0);
	if (mass > tempmass) {
		pt1 = pt0;
		pt2 = pt1+1;
	} else {
		pt2=pt0;
		pt1=pt2-1;
	}
	int1 = pDoc->FreqArray[pt1];
	int2 = pDoc->FreqArray[pt2];

	// interpolate
	tempmass = pDoc->PtToMass(pt1);
	slope = double(double(int2-int1)/(pDoc->PtToMass(pt2)-tempmass));
	intensity = int1+(slope*double(mass - tempmass));
	return intensity;
}

CPoint CDeconView::DataToViewPoint(struct ViewLinePt inpt, float yslope, float yintercept, float xslope, float xintercept)
{
	CPoint outpt;
	// rescale ViewLinePt to PlotArea
	outpt.y = long(yslope*inpt.y + yintercept);
	outpt.x = long(xslope*inpt.x + xintercept);

	return outpt;
}

void CDeconView::OnLButtonUp(UINT nFlags, CPoint point) 
{
	long temp;
	CPoint pt;

	if(flags.Zooming) {
		flags.Zooming = FALSE;	
		flags.Redraw = TRUE;
		ZoomEndPt = point;
		// now calculate the new zoomstack points, reset flag.CalcViewLine, and redraw.
		CView::OnLButtonUp(nFlags, point);

		// check that ZoomEndPt > ZoomInitPt;
		if(ZoomEndPt.x < ZoomInitPt.x) {
			temp = ZoomEndPt.x;
			ZoomEndPt.x = ZoomInitPt.x;
			ZoomInitPt.x = temp;
		}
		if(ZoomEndPt.y < ZoomInitPt.y) {
			temp = ZoomEndPt.y;
			ZoomEndPt.y = ZoomInitPt.y;
			ZoomInitPt.y = temp;
		}

		// we need to convert the zoompts to mass to put on the zoomstack.
		zoompointer++;
		pt = ViewPTtoPT(ZoomEndPt);
		zoomstack[zoompointer].maxx = PTtoMass(pt.x, zoomstack[zoompointer-1].maxx, zoomstack[zoompointer-1].minx);
		zoomstack[zoompointer].maxy = (float)pt.y;
		pt = ViewPTtoPT(ZoomInitPt);
		zoomstack[zoompointer].minx = PTtoMass(pt.x, zoomstack[zoompointer-1].maxx, zoomstack[zoompointer-1].minx);
		zoomstack[zoompointer].miny = (float)pt.y;

		flags.CalcViewLine = TRUE;
		pDoc->UpdateAllViews(NULL,0,NULL);
	}
}

void CDeconView::OnLButtonDown(UINT nFlags, CPoint point) 
{
	flags.Zooming = TRUE;	
	ZoomInitPt = point;
	CView::OnLButtonDown(nFlags, point);
}

void CDeconView::OnMouseMove(UINT nFlags, CPoint point) 
{
	if(flags.Zooming) {
		pDC = GetDC();
		CPoint pt;
		CPen ErasePen(PS_SOLID, 1, pDC->GetBkColor() );
		CPen* pDrawPen;
		pDrawPen = pDC->GetCurrentPen();

		// first, remove the previous MouseMove box
		// how do i switch pens?  make it a dotted pen with color = background color
		pDC->SelectObject(&ErasePen);
		pt = ZoomInitPt;
		pDC->MoveTo(pt);
		pt.y = ZoomEndPt.y;
		pDC->LineTo(pt);
		pt.x = ZoomEndPt.x;
		pDC->LineTo(pt);
		pt.y = ZoomInitPt.y;
		pDC->LineTo(pt);
		pt.x = ZoomInitPt.x;
		pDC->LineTo(pt);

		// second, draw a new MouseMove box
		// change pens to color = foreground color
		pDC->SelectObject(pDrawPen);
		ZoomEndPt = point;
		pt.y = ZoomEndPt.y;
		pDC->LineTo(pt);
		pt.x = ZoomEndPt.x;
		pDC->LineTo(pt);
		pt.y = ZoomInitPt.y;
		pDC->LineTo(pt);
		pt.x = ZoomInitPt.x;
		pDC->LineTo(pt);
	}

	// finally, clean up and save this point.
	CView::OnMouseMove(nFlags, point);
}

void CDeconView::OnRButtonDown(UINT nFlags, CPoint point) 
{
	// TODO: Add your message handler code here and/or call default
	
	CView::OnRButtonDown(nFlags, point);
}

float CDeconView::PTtoMass(long xpt, double highmass, double lowmass)
{
	// this takes a point already converted to the 0..32767 range
	// and converts it to a mass point.
	if (xpt>32767) xpt = 32767;
	if (xpt<0) xpt = 0;

	CPoint outpt;
	double x,mx;
	long zp;
	zp = zoompointer -1;

	mx = ((double)highmass-(double)lowmass)/32768.0;
	x = mx*(double)xpt + lowmass;

	return (float)x;
}

void CDeconView::InitializeDrawParms()
{
	flags.InitializeWindow = FALSE; // don't come here again ;-)
	// Define drawing limits and constants
	PlotArea.top = 32768-6000;
	PlotArea.right = 32768 - 1000;
	PlotArea.bottom = 4000;
	PlotArea.left = 1000;
	AxisLineYpos = PlotArea.bottom;
	if(flags.PlotYAxis) PlotArea.left = 4000;
	if(flags.PlotParameters) PlotArea.right = 32768 - 8000;

	pDC->GetWindow()->GetClientRect(&ViewSize);
	if(!flags.InitializedWindowSize) {
		InitialWindowSize.cx = ViewSize.right - ViewSize.left;
		InitialWindowSize.cy = ViewSize.bottom - ViewSize.top;
		flags.InitializedWindowSize = TRUE;
	}

/*		// test the ranging by drawing an x in the screen
	MoveTo(0,0);			// lower right
	LineTo(16384,16384);	// center
	LineTo(32768,0);		// upper left
	LineTo(16384,16384);	// center
	LineTo(0,32768);		// lower right
	LineTo(16384,16384);	// center
	LineTo(32768,32768);	// upper right
	MoveTo(PlotArea.right, PlotArea.top);
	LineTo(PlotArea.left, PlotArea.top);
	LineTo(PlotArea.left, PlotArea.bottom);
	LineTo(PlotArea.right, PlotArea.bottom);
	LineTo(PlotArea.right, PlotArea.top);
*/	
}
