/****************************************************************

Boston University data analysis (BUDA) package 

Description: Buda is a data analysis program for Fourier 
Transform Mass Spectrometry data.  

by Peter B. O'Connor

    Copyright (C) 2000  Peter B. O'Connor

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/
#if !defined(AFX_DECONVIEW_H__24680560_DEFF_11D3_AFFA_0090278ECA2A__INCLUDED_)
#define AFX_DECONVIEW_H__24680560_DEFF_11D3_AFFA_0090278ECA2A__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000
// DeconView.h : header file
//
#define CENTER_JUSTIFIED 0
#define LEFT_JUSTIFIED 1
#define CENTER_JUSTIFIED2 2
#define RIGHT_JUSTIFIED 3
#define MAX_VIEWLINE_LENGTH 6000

#if !defined(STRUCT_VIEWLINEPT_DEFINED)
#define STRUCT_VIEWLINEPT_DEFINED
struct ViewLinePt {
	float x;
	float y;
};
#endif

/////////////////////////////////////////////////////////////////////////////
// CDeconView view

class CDeconView : public CView
{
protected:
	CDeconView();           // protected constructor used by dynamic creation
	DECLARE_DYNCREATE(CDeconView)

// Attributes
public:
	int DeconMethod;
	UINT DeconHighChargeLimit;
	double DeconLowMassLimit;
	double DeconHighMassLimit;
	CPoint ZoomEndPt;
	CPoint ZoomInitPt;
	float ViewLineMiny;
	float ViewLineMaxy;
	float ViewLineMinx;
	float ViewLineMaxx;
	float scale;
	float VScale;
	long ViewLineLength;
	float DeconData[MAX_DECON_SIZE];
	struct ViewLinePt ViewLine[MAX_VIEWLINE_LENGTH];
	CSize InitialWindowSize;
	int AxisLineYpos;
	CRect ViewSize;
	CRect PlotArea;
	CDC* pDC;
	CBudaDoc* pDoc;
	int zoompointer;
	struct zoomwindow {
		//unsigned long minx;
		//unsigned long maxx;
		float minx;
		float maxx;
		float miny;
		float maxy;
	} zoomstack[20];
	struct FlagValues {
	public:
		BOOL Zooming;
		BOOL ViewIsSetup;
		BOOL UpdateScaleFactor;
		BOOL ClearViewLine;
		BOOL CalcViewLine;
		BOOL PlotParameters;
		BOOL RemoveDC;
		BOOL RecalcData;
		BOOL PlotYAxis;
		BOOL IsZoomingY;
		BOOL InitializedWindowSize;
		BOOL Redraw;
		BOOL InitializeWindow;
	} flags;

// Operations
public:
	void InitializeDrawParms();
	float PTtoMass(long, double highmass, double lowmass);
	CPoint DataToViewPoint(struct ViewLinePt inpt, float yslope, float yintercept, float xslope, float xintercept);
	double GetIntensityatMass(double mass);
	double DeconRes;
	void DecimateDeconData();
	CBudaApp* pApp;
	void CalculateFennDecon();
	void CalculateReinholdDecon();
	void CalculateOConnorDecon();
	void CalculateZhangDecon();
	void InitializeZoomStack();
	void ScaleViewLine();
	void CalculateDeconViewLine();
	void Text(CString str, CPoint pt, int just);
	void DrawVerticalAxis();
	void DrawGenericAxis(int PlotStart, int PlotEnd, float DataStart, float DataEnd, CString caption, CString labelformat);
	void DrawDeconAxis();
	void DrawDeconData();
	void LabelPeaks();
	void DrawData();
	void DrawParms();
	void DrawAxis();
	void PrepareView();
	void MoveTo(int x, int y);
	void MoveTo(CPoint pt);
	void LineTo(int x, int y);
	void LineTo(CPoint pt);
	CPoint PTtoViewPT(CPoint pt);
	CPoint PTtoViewPT(int x, int y);
	CPoint ViewPTtoPT(CPoint pt);

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CDeconView)
	public:
	virtual void OnInitialUpdate();
	protected:
	virtual void OnDraw(CDC* pDC);      // overridden to draw this view
	//}}AFX_VIRTUAL

// Implementation
protected:
	virtual ~CDeconView();
#ifdef _DEBUG
	virtual void AssertValid() const;
	virtual void Dump(CDumpContext& dc) const;
#endif

	// Generated message map functions
protected:
	//{{AFX_MSG(CDeconView)
		// NOTE - the ClassWizard will add and remove member functions here.
	afx_msg void OnRButtonDown(UINT nFlags, CPoint point);
	afx_msg void OnLButtonUp(UINT nFlags, CPoint point);
	afx_msg void OnLButtonDown(UINT nFlags, CPoint point);
	afx_msg void OnMouseMove(UINT nFlags, CPoint point);
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

/////////////////////////////////////////////////////////////////////////////

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_DECONVIEW_H__24680560_DEFF_11D3_AFFA_0090278ECA2A__INCLUDED_)
