// DetectDlg.cpp : implementation file
//
/****************************************************************

Boston University data analysis (BUDA) package 

Description: Buda is a data analysis program for Fourier 
Transform Mass Spectrometry data.  

by Peter B. O'Connor

    Copyright (C) 2000  Peter B. O'Connor

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "stdafx.h"
#include "Buda.h"
#include "DetectDlg.h"
#include "LinkExcite.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CDetectDlg dialog


CDetectDlg::CDetectDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CDetectDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CDetectDlg)
	m_HighMass = 0.0;
	m_LowMass = 0.0;
	m_MonitorMass = 0.0;
	m_DetectTime = _T("");
	m_MixerFreq = _T("");
	m_NumScansString = _T("");
	m_NumSpectraString = _T("");
	m_WarningString = _T("");
	//}}AFX_DATA_INIT
}


void CDetectDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CDetectDlg)
	DDX_Control(pDX, IDC_MONITORMASS, m_MonitorMassEdit);
	DDX_Control(pDX, IDC_HIGHMASS, m_HighMassEdit);
	DDX_Control(pDX, IDC_LOWMASS, m_LowMassEdit);
	DDX_Control(pDX, IDC_NUMSPECTITLESTR, m_NumSpecTitle);
	DDX_Control(pDX, IDC_WARNINGSTR, m_WarningStatic);
	DDX_Control(pDX, IDC_MIXERFREQTITLESTR, m_MixerFreqTitleStatic);
	DDX_Control(pDX, IDC_MIXERFREQSTR, m_MixerFreqStatic);
	DDX_Control(pDX, IDC_DETECTMODE_DIRECT, m_DirectModeBtn);
	DDX_Control(pDX, IDC_DETECTMODE_MIXER, m_MixerModeBtn);
	DDX_Control(pDX, IDC_ACQMODESINGLE, m_AcqModeSingle);
	DDX_Control(pDX, IDC_ACQMODECHROM, m_AcqModeChrom);
	DDX_Control(pDX, IDC_ACQMODETIMESCAN, m_AcqModeTimeScan);
	DDX_Control(pDX, IDC_ACQRATE_COMBO, m_AcqRateCombo);
	DDX_Control(pDX, IDC_NDPCOMBO, m_NDPCombo);
	DDX_Control(pDX, IDC_LINK_EXCITE, m_LinkBtn);
	DDX_Text(pDX, IDC_HIGHMASS, m_HighMass);
	DDX_Text(pDX, IDC_LOWMASS, m_LowMass);
	DDX_Text(pDX, IDC_MONITORMASS, m_MonitorMass);
	DDX_Text(pDX, IDC_DETECTTIMESTR, m_DetectTime);
	DDX_Text(pDX, IDC_MIXERFREQSTR, m_MixerFreq);
	DDX_Text(pDX, IDC_NUMSCANSSTR, m_NumScansString);
	DDX_Text(pDX, IDC_NUMSPECTRASTR, m_NumSpectraString);
	DDX_Text(pDX, IDC_WARNINGSTR, m_WarningString);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CDetectDlg, CDialog)
	//{{AFX_MSG_MAP(CDetectDlg)
	ON_BN_CLICKED(IDC_LINK_EXCITE, OnLink)
	ON_CBN_SELCHANGE(IDC_NDPCOMBO, OnSelchangeNdpcombo)
	ON_CBN_SELCHANGE(IDC_ACQRATE_COMBO, OnSelchangeAcqrateCombo)
	ON_BN_CLICKED(IDC_DETECTMODE_DIRECT, OnDetectmodeDirect)
	ON_BN_CLICKED(IDC_DETECTMODE_MIXER, OnDetectmodeMixer)
	ON_BN_CLICKED(IDC_ACQMODECHROM, OnAcqModeChrom)
	ON_BN_CLICKED(IDC_ACQMODESINGLE, OnAcqModeSingle)
	ON_BN_CLICKED(IDC_ACQMODETIMESCAN, OnAcqModeTimescan)
	ON_BN_CLICKED(IDC_LINK_CHK, OnLinkChk)
	ON_EN_KILLFOCUS(IDC_LOWMASS, OnKillfocusLowmass)
	ON_EN_KILLFOCUS(IDC_HIGHMASS, OnKillfocusHighmass)
	ON_EN_KILLFOCUS(IDC_MONITORMASS, OnKillfocusMonitormass)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CDetectDlg message handlers

BOOL CDetectDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	// force the DoDataExchange to initialize things
	UpdateData(FALSE);

	// set up the NumDataPoints combobox
	int i;
	NDPComboEntries[0]="1024 K";
	NDPComboEntries[1]="512 K";
	NDPComboEntries[2]="256 K";
	NDPComboEntries[3]="128 K";
	NDPComboEntries[4]="64 K";
	NDPComboEntries[5]="32 K";
	NDPComboEntries[6]="16 K";
	NDPComboEntries[7]="8 K";
	NDPComboEntries[8]="4 K";
	NDPComboEntries[9]="2 K";
	NDPComboEntries[10]="1 K";
	NDP[10]=1024;
	NDP[9]=2048;
	NDP[8]=4096;
	NDP[7]=8192;
	NDP[6]=16384;
	NDP[5]=32*1024;
	NDP[4]=64*1024;
	NDP[3]=128*1024;
	NDP[2]=256*1024;
	NDP[1]=512*1024;
	NDP[0]=1024*1024;
	
	for(i=0; i<11; i++) {
		m_NDPCombo.AddString(NDPComboEntries[i]);
		if(NumDataPoints == NDP[i]) {
			m_NDPCombo.SetCurSel(i);
		}
	}
	
	// set up the ACQRate combobox
	m_AcqRateCombo.InitStorage(16,16);
	for(i=0; i<16; i++) {
		if(pSys->ADCRateAvailable[i]) {
			ADCRates[i] = pSys->ADCRateList[i];
			m_AcqRateCombo.AddString(pSys->ADCRateString[i]);
		}
	}
	
	//ADCRateEntries = new CString[12];
	//ADCRates = new double[12];
	/*ADCRateEntries[0]="8000 KSPS";
	ADCRateEntries[1]="4000 KSPS";
	ADCRateEntries[2]="1000 KSPS";
	ADCRateEntries[3]="500 KSPS";
	ADCRateEntries[4]="250 KSPS";
	ADCRateEntries[5]="125 KSPS";
	ADCRateEntries[6]="10 KSPS";
	ADCRateEntries[7]="1 KSPS";

	ADCRates[0]=8000000.0;
	ADCRates[1]=4000000.0;
	ADCRates[2]=1000000.0;
	ADCRates[3]=500000.0;
	ADCRates[4]=250000.0;
	ADCRates[5]=125000.0;
	ADCRates[6]=10000.0;
	ADCRates[7]=1000.0;
	*/
	for(i=0; i<6; i++) {
		if(AdcRate == ADCRates[i]) {
			m_AcqRateCombo.SetCurSel(i);
		}
	}

	// initialize the DetectionMode buttons
	if(DirectMode == 0) {
		m_DirectModeBtn.SetCheck(1);
		m_MixerModeBtn.SetCheck(0);
	} else {
		m_DirectModeBtn.SetCheck(0);
		m_MixerModeBtn.SetCheck(1);
	}

	// initialize the AcquisitionMode buttons
	if(AcqMode!=0 && AcqMode!=1 && AcqMode!=2) {
		// Error! invalid AcqMode
		AcqMode = 0;
	}
	if(AcqMode == 0) {
		m_AcqModeSingle.SetCheck(1);
		m_AcqModeChrom.SetCheck(0);
		m_AcqModeTimeScan.SetCheck(0);
	}
	if(AcqMode == 1) {
		m_AcqModeSingle.SetCheck(0);
		m_AcqModeChrom.SetCheck(1);
		m_AcqModeTimeScan.SetCheck(0);
	}
	if(AcqMode == 2) {
		m_AcqModeSingle.SetCheck(0);
		m_AcqModeChrom.SetCheck(0);
		m_AcqModeTimeScan.SetCheck(1);
	}

	// set up detect range and monitor mass
	m_LowMass = pPS->DetectLowMass;
	m_HighMass = pPS->DetectHighMass;
	m_MonitorMass = pPS->MonitorMass;
	m_NumScans = pPS->ScansRequested;

	// everything was fine, so return TRUE
	OnUpdate();
	return TRUE;
}

void CDetectDlg::SetListBoxFont()
{
  /* CFont* pFont = GetFont();
   LOGFONT lf;
   if (pFont != NULL)
	   pFont->GetObject(sizeof(LOGFONT), &lf);
   else
	   ::GetObject(GetStockObject(SYSTEM_FONT), sizeof(LOGFONT), &lf);

	lf.lfPitchAndFamily = FIXED_PITCH | FF_MODERN;
	lf.lfWidth = 7;
	lf.lfHeight = 16;
	lf.lfWeight = 10;
	//char fnt[32]="Courier\0";
	//lf.lfFaceName = "Courier                       \0";
		// switch to new font.
	m_font.DeleteObject();
	if (m_font.CreateFontIndirect(&lf))
	{
		//CWaitCursor wait;
		DacPulseList.SetFont(&m_font);
		EventList.SetFont(&m_font);
	}*/
}


void CDetectDlg::OnLink() 
{
	CLinkExcite dlg;
	dlg.m_LinkEvent = pPS->DetectEvent;
	dlg.pPS = pPS;
	dlg.pSys = pSys;

	if( dlg.DoModal() == IDOK) {
		pPS->DetectEvent = dlg.m_LinkEvent;
	}	
}

void CDetectDlg::OnSelchangeNdpcombo() 
{
	int i = m_NDPCombo.GetCurSel();	
	NumDataPoints = NDP[i];
	OnUpdate();
}

void CDetectDlg::OnSelchangeAcqrateCombo() 
{
	int i = m_AcqRateCombo.GetCurSel();	
	AdcRate = ADCRates[i];
	OnUpdate();
}

void CDetectDlg::OnDetectmodeDirect() 
{
	DetectMode = DirectMode;	
	m_DirectModeBtn.SetCheck(TRUE);
	m_MixerModeBtn.SetCheck(FALSE);
	OnUpdate();
}

void CDetectDlg::OnDetectmodeMixer() 
{
	DetectMode = MixerMode;	
	m_DirectModeBtn.SetCheck(FALSE);
	m_MixerModeBtn.SetCheck(TRUE);
	OnUpdate();
}

void CDetectDlg::OnAcqModeChrom() 
{
	AcqMode = Chromatography;
	OnUpdate();
}

void CDetectDlg::OnAcqModeSingle() 
{
	AcqMode = Single;
	OnUpdate();
}

void CDetectDlg::OnAcqModeTimescan() 
{
	AcqMode = TimeScan;
	OnUpdate();
}

void CDetectDlg::UpdateAcqMode()
{
	if(AcqMode == Single) m_AcqModeSingle.SetCheck(TRUE);
	else  m_AcqModeSingle.SetCheck(FALSE);

	if(AcqMode == Chromatography) {
		m_AcqModeChrom.SetCheck(TRUE);
	}
	else  {
		m_AcqModeChrom.SetCheck(FALSE);
	}

	if(AcqMode == Single) m_AcqModeTimeScan.SetCheck(TRUE);
	else  m_AcqModeTimeScan.SetCheck(FALSE);
}

void CDetectDlg::OnLinkChk() 
{
	// TODO: Add your control notification handler code here
	
}

void CDetectDlg::OnUpdate()
{
	// update strings
	m_WarningString.Empty();
	m_NumScansString.Format("%ld",m_NumScans);
	m_DetectTime.Format("%1.3f",NumDataPoints/AdcRate);
	if(AcqMode != Chromatography) {
		m_NumSpectraString.Empty();
		m_NumSpecTitle.ShowWindow(SW_HIDE);
	} else {
		m_NumSpectraString.Format("%ld",m_NumSpectra);
		m_NumSpecTitle.ShowWindow(SW_SHOW);
	}
	if(DetectMode != MixerMode) {
		m_MixerFreq.Empty();
		m_MixerFreqStatic.ShowWindow(SW_HIDE);
		m_MixerFreqTitleStatic.ShowWindow(SW_HIDE);
	} else {
		m_MixerFreq.Format("%1.1f",pDoc->PS.MixerFrequency);
		m_MixerFreqStatic.ShowWindow(SW_SHOW);
		m_MixerFreqTitleStatic.ShowWindow(SW_SHOW);
	}
	if(pDoc->MassToFreq(m_LowMass) > AdcRate/2) {
		m_WarningString.Format("Warning!!  Acquisition rate must be at least %1.1f Hz \n to detect mass %1.3f",2*pDoc->MassToFreq(m_LowMass),m_LowMass);
		m_WarningStatic.ShowWindow(SW_SHOW);
	}

	if(m_WarningString.IsEmpty()) m_WarningStatic.ShowWindow(SW_HIDE);

	UpdateAcqMode();
	UpdateData(FALSE);
}

void CDetectDlg::OnKillfocusLowmass() 
{
	UpdateData(TRUE);
	OnUpdate();
}

void CDetectDlg::OnKillfocusHighmass() 
{
	UpdateData(TRUE);
	OnUpdate();
}

void CDetectDlg::OnKillfocusMonitormass() 
{
	UpdateData(TRUE);
	OnUpdate();
}
