// DisplayOptions.cpp : implementation file
//
/*
DisplayOptions.cpp is the code that controls the CDisplayOptions dialog
class.  This class is derived from CPropertyPage and is one of the 
tab-ctrl options on the options property sheet.  
This class allows the user to control the annotation of their pages
i.e. which properties to plot there, whether or not to put plus marks
at the MI table points, peak labeling, etc

*/
/****************************************************************

Boston University data analysis (BUDA) package 

Description: Buda is a data analysis program for Fourier 
Transform Mass Spectrometry data.  

by Peter B. O'Connor

    Copyright (C) 2000  Peter B. O'Connor

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "BudaMaster.h"
#include "Buda.h"
#include "DisplayOptions.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CDisplayOptions property page

IMPLEMENT_DYNCREATE(CDisplayOptions, CPropertyPage)

CDisplayOptions::CDisplayOptions() : CPropertyPage(CDisplayOptions::IDD)
{
	//{{AFX_DATA_INIT(CDisplayOptions)
	dp.massrange = FALSE;
	dp.acqrate = FALSE;
	dp.apodfunc = FALSE;
	dp.calconst = FALSE;
	dp.exciteamp = FALSE;
	dp.numpts = FALSE;
	dp.numscans = FALSE;
	dp.numzerofills = FALSE;
	dp.allparms = FALSE;
	dp.trapvolts = FALSE;
	dp.labelpoints = FALSE;
	dp.noisethreshold = FALSE;
	dp.PosNeg = FALSE;
	dp.timedate = FALSE;
	dp.LabelCharge = FALSE;
	dp.ScaleFactor = FALSE;
	//}}AFX_DATA_INIT
}

CDisplayOptions::~CDisplayOptions()
{
}

void CDisplayOptions::DoDataExchange(CDataExchange* pDX)
{
	CPropertyPage::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CDisplayOptions)
	DDX_Control(pDX, IDC_DISP_SCALEFACTOR, m_DispScalefactorBtn);
	DDX_Control(pDX, IDC_LABLECHARGE, m_disp_labelcharge_btn);
	DDX_Control(pDX, IDC_DISP_TIME, m_disp_time_btn);
	DDX_Control(pDX, IDC_DISP_POSNEG, m_disp_posneg_btn);
	DDX_Control(pDX, IDC_NOISETHRESHOLD, m_noisebtn);
	DDX_Control(pDX, IDC_LABEL_PTS, m_labelptsbtn);
	DDX_Control(pDX, IDC_LABEL_PEAK_POS, m_LablePeakPosbtn);
	DDX_Control(pDX, IDC_DISP_TRAPVOLTS, m_disptrapbtn);
	DDX_Control(pDX, IDC_DISP_NUMZEROFILLS, m_dispzerobtn);
	DDX_Control(pDX, IDC_DISP_NUMSCANS, m_dispscansbtn);
	DDX_Control(pDX, IDC_DISP_NUMPTS, m_dispnumptsbtn);
	DDX_Control(pDX, IDC_DISP_MASSRANGE, m_dispmassbtn);
	DDX_Control(pDX, IDC_DISP_EXCAMP, m_dispexcbtn);
	DDX_Control(pDX, IDC_DISP_CALCONSTS, m_dispcalbtn);
	DDX_Control(pDX, IDC_DISP_APODFUNC, m_dispapodbtn);
	DDX_Control(pDX, IDC_DISP_ACQRATE, m_dispacqbtn);
	DDX_Control(pDX, IDC_DISP_PARMS, m_dispparmsbtn);
	DDX_Check(pDX, IDC_DISP_MASSRANGE, dp.massrange);
	DDX_Check(pDX, IDC_DISP_ACQRATE, dp.acqrate);
	DDX_Check(pDX, IDC_DISP_APODFUNC, dp.apodfunc);
	DDX_Check(pDX, IDC_DISP_CALCONSTS, dp.calconst);
	DDX_Check(pDX, IDC_DISP_EXCAMP, dp.exciteamp);
	DDX_Check(pDX, IDC_DISP_NUMPTS, dp.numpts);
	DDX_Check(pDX, IDC_DISP_NUMSCANS, dp.numscans);
	DDX_Check(pDX, IDC_DISP_NUMZEROFILLS, dp.numzerofills);
	DDX_Check(pDX, IDC_DISP_PARMS, dp.allparms);
	DDX_Check(pDX, IDC_DISP_TRAPVOLTS, dp.trapvolts);
	DDX_Check(pDX, IDC_LABEL_PTS, dp.labelpoints);
	DDX_Check(pDX, IDC_NOISETHRESHOLD, dp.noisethreshold);
	DDX_Check(pDX, IDC_DISP_POSNEG, dp.PosNeg);
	DDX_Check(pDX, IDC_DISP_TIME, dp.timedate);
	DDX_Check(pDX, IDC_LABLECHARGE, dp.LabelCharge);
	DDX_Check(pDX, IDC_DISP_SCALEFACTOR, dp.ScaleFactor);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CDisplayOptions, CPropertyPage)
	//{{AFX_MSG_MAP(CDisplayOptions)
	ON_BN_CLICKED(IDC_DISP_PARMS, OnDispParmsClicked)
	ON_BN_CLICKED(IDC_LABEL_PEAK_POS, OnLabelPeakPos)
	ON_BN_CLICKED(IDC_DISP_ACQRATE, OnDispAcqrate)
	ON_BN_CLICKED(IDC_DISP_APODFUNC, OnDispApodfunc)
	ON_BN_CLICKED(IDC_DISP_CALCONSTS, OnDispCalconsts)
	ON_BN_CLICKED(IDC_DISP_EXCAMP, OnDispExcamp)
	ON_BN_CLICKED(IDC_DISP_MASSRANGE, OnDispMassrange)
	ON_BN_CLICKED(IDC_DISP_NUMPTS, OnDispNumpts)
	ON_BN_CLICKED(IDC_DISP_NUMSCANS, OnDispNumscans)
	ON_BN_CLICKED(IDC_DISP_NUMZEROFILLS, OnDispNumzerofills)
	ON_BN_CLICKED(IDC_DISP_TRAPVOLTS, OnDispTrapvolts)
	ON_BN_CLICKED(IDC_LABEL_PTS, OnLabelPts)
	ON_BN_CLICKED(IDC_NOISETHRESHOLD, OnNoisethreshold)
	ON_BN_CLICKED(IDC_LABLECHARGE, OnLablecharge)
	ON_BN_CLICKED(IDC_DISP_POSNEG, OnDispPosneg)
	ON_BN_CLICKED(IDC_DISP_TIME, OnDispTime)
	ON_BN_CLICKED(IDC_DISP_SCALEFACTOR, OnDispScalefactor)
	ON_BN_CLICKED(IDC_SAVEASDEFAULTS, OnSaveasdefaults)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CDisplayOptions message handlers

void CDisplayOptions::OnDispParmsClicked() 
{ 
	if (m_dispparmsbtn.GetCheck()) {
		// enable all other buttons on this page
		dp.allparms = TRUE;
		m_dispacqbtn.EnableWindow(TRUE);
		m_dispcalbtn.EnableWindow(TRUE);
		m_dispapodbtn.EnableWindow(TRUE);
		//m_dispdatebtn.EnableWindow(TRUE);
		m_dispexcbtn.EnableWindow(TRUE);
		//m_dispfilebtn.EnableWindow(TRUE);
		m_dispmassbtn.EnableWindow(TRUE);
		//m_dispnotesbtn.EnableWindow(TRUE);
		m_dispnumptsbtn.EnableWindow(TRUE);
		m_dispscansbtn.EnableWindow(TRUE);
		m_disptrapbtn.EnableWindow(TRUE);
		m_dispzerobtn.EnableWindow(TRUE);
		m_noisebtn.EnableWindow(TRUE);
		m_disp_time_btn.EnableWindow(TRUE);
		m_disp_posneg_btn.EnableWindow(TRUE);
		m_DispScalefactorBtn.EnableWindow(TRUE);
	} else {
		// disable all other buttons on this page
		dp.allparms = FALSE;
		m_dispacqbtn.EnableWindow(FALSE);
		m_dispcalbtn.EnableWindow(FALSE);
		m_dispapodbtn.EnableWindow(FALSE);
		//m_dispdatebtn.EnableWindow(FALSE);
		m_dispexcbtn.EnableWindow(FALSE);
		//m_dispfilebtn.EnableWindow(FALSE);
		m_dispmassbtn.EnableWindow(FALSE);
		//.EnableWindow(FALSE);
		m_dispnumptsbtn.EnableWindow(FALSE);
		m_dispscansbtn.EnableWindow(FALSE);
		m_disptrapbtn.EnableWindow(FALSE);
		m_dispzerobtn.EnableWindow(FALSE);
		m_noisebtn.EnableWindow(FALSE);
		m_disp_time_btn.EnableWindow(FALSE);
		m_disp_posneg_btn.EnableWindow(FALSE);
		m_DispScalefactorBtn.EnableWindow(FALSE);
	}
}

BOOL CDisplayOptions::OnSetActive() 
{
	m_dispparmsbtn.SetCheck(dp.allparms);
	m_dispacqbtn.SetCheck(dp.acqrate);
	m_dispapodbtn.SetCheck(dp.apodfunc);
	m_dispcalbtn.SetCheck(dp.calconst);
	m_dispexcbtn.SetCheck(dp.exciteamp);
	//m_dispfilebtn.SetCheck(dp.filename);
	m_labelptsbtn.SetCheck(dp.labelpoints);
	m_dispmassbtn.SetCheck(dp.massrange);
	//m_dispnotesbtn.SetCheck(dp.notes);
	m_dispnumptsbtn.SetCheck(dp.numpts);
	//m_dispdatebtn.SetCheck(dp.timedate);
	m_disptrapbtn.SetCheck(dp.trapvolts);
	m_dispzerobtn.SetCheck(dp.numzerofills);
	m_dispscansbtn.SetCheck(dp.numscans);
	m_dispparmsbtn.SetCheck(dp.allparms);
	m_noisebtn.SetCheck(dp.noisethreshold);
	m_LablePeakPosbtn.SetCheck(dp.LabelPeakPos);
	m_disp_time_btn.SetCheck(dp.timedate);
	m_disp_posneg_btn.SetCheck(dp.PosNeg);
	m_DispScalefactorBtn.SetCheck(dp.ScaleFactor);

	OnDispParmsClicked();
	return CPropertyPage::OnSetActive();
}

void CDisplayOptions::OnLabelPeakPos() 
{
	if(m_LablePeakPosbtn.GetCheck()) dp.LabelPeakPos = TRUE;
	else dp.LabelPeakPos = FALSE;
}

void CDisplayOptions::OnDispAcqrate() 
{
	if(m_dispacqbtn.GetCheck()) dp.acqrate = TRUE;
	else dp.acqrate = FALSE;	
}

void CDisplayOptions::OnDispApodfunc() 
{
	if(m_dispapodbtn.GetCheck()) dp.apodfunc = TRUE;
	else dp.apodfunc = FALSE;		
}

void CDisplayOptions::OnDispCalconsts() 
{
	if(m_dispcalbtn.GetCheck()) dp.calconst = TRUE;
	else dp.calconst = FALSE;		
}

void CDisplayOptions::OnDispExcamp() 
{
	if(m_dispexcbtn.GetCheck()) dp.exciteamp = TRUE;
	else dp.exciteamp = FALSE;		
}

void CDisplayOptions::OnDispFilename() 
{
//	if(m_dispfilebtn.GetCheck()) dp.filename = TRUE;
//	else dp.filename = FALSE;		
}

void CDisplayOptions::OnDispMassrange() 
{
	if(m_dispmassbtn.GetCheck()) dp.massrange = TRUE;
	else dp.massrange = FALSE;		
}

void CDisplayOptions::OnDispNotes() 
{
//	if(m_dispnotesbtn.GetCheck()) dp.notes = TRUE;
//	else dp.notes = FALSE;		
}

void CDisplayOptions::OnDispNumpts() 
{
	if(m_dispnumptsbtn.GetCheck()) dp.numpts = TRUE;
	else dp.numpts = FALSE;		
}

void CDisplayOptions::OnDispNumscans() 
{
	if(m_dispscansbtn.GetCheck()) dp.numscans = TRUE;
	else dp.numscans = FALSE;	
}

void CDisplayOptions::OnDispNumzerofills() 
{
	if(m_dispzerobtn.GetCheck()) dp.numzerofills = TRUE;
	else dp.numzerofills = FALSE;	
}

void CDisplayOptions::OnDispTimedate() 
{
//	if(m_dispdatebtn.GetCheck()) dp.timedate = TRUE;
//	else dp.timedate = FALSE;		
}

void CDisplayOptions::OnDispTrapvolts() 
{
	if(m_disptrapbtn.GetCheck()) dp.trapvolts = TRUE;
	else dp.trapvolts = FALSE;		
}

void CDisplayOptions::OnLabelPts() 
{
	if(m_labelptsbtn.GetCheck()) dp.labelpoints = TRUE;
	else dp.labelpoints = FALSE;			
}

void CDisplayOptions::OnNoisethreshold() 
{
	if(m_noisebtn.GetCheck()) dp.noisethreshold = TRUE;
	else dp.noisethreshold = FALSE;		
}

void CDisplayOptions::OnLablecharge() 
{
	if(m_disp_labelcharge_btn.GetCheck()) dp.LabelCharge = TRUE;
	else dp.LabelCharge = FALSE;			
}

void CDisplayOptions::OnDispPosneg() 
{
	if(m_disp_posneg_btn.GetCheck()) dp.PosNeg = TRUE;
	else dp.PosNeg = FALSE;		
}

void CDisplayOptions::OnDispTime() 
{
	if(m_disp_time_btn.GetCheck()) dp.timedate = TRUE;
	else dp.timedate = FALSE;		
}

void CDisplayOptions::OnDispScalefactor() 
{
	if(m_DispScalefactorBtn.GetCheck()) dp.ScaleFactor = TRUE;
	else dp.ScaleFactor = FALSE;		
}

void CDisplayOptions::OnSaveasdefaults() 
{
	pDoc->StoreDefaultDisplayParms(&dp);
}
