// EditARBDlg.cpp : implementation file
//
/****************************************************************

Boston University data analysis (BUDA) package 

Description: Buda is a data analysis program for Fourier 
Transform Mass Spectrometry data.  

by Peter B. O'Connor

    Copyright (C) 2000  Peter B. O'Connor

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "stdafx.h"
#include "Buda.h"
#include "EditARBDlg.h"
#include "ChangeARBDisplay.h"
#include "ARBAdvancedDlg.h"
#include "AddARBWindowDlg.h"
#include "AddARBNotchDlg.h"
#include "ARBPlotBtn.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CEditARBDlg dialog


CEditARBDlg::CEditARBDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CEditARBDlg::IDD, pParent)
{
	CurrentArbWaveform=0;
	//{{AFX_DATA_INIT(CEditARBDlg)
	m_DACText = _T("");
	m_DurationText = _T("");
	m_FilterText = _T("");
	m_AmplitudeText = _T("");
	m_MemoryText = _T("");
	m_WindowText = _T("");
	//}}AFX_DATA_INIT
	NullARBEvent.Amplitude = 0;
	NullARBEvent.Type = ARB_NONE;
	NullARBEvent.MassA = 0;
	NullARBEvent.MassB = 0;
	NullARBEvent.fit=0;
	NullARBEvent.reserved=0;
}


void CEditARBDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//DDX_Control(pDX, IDC_ARBPICTURE, m_ARBPicture1);
	//{{AFX_DATA_MAP(CEditARBDlg)
	DDX_Control(pDX, IDC_EVENTLISTBOX, m_EventListBox);
	DDX_Control(pDX, IDC_ARBPLOTBTN, m_ARBPlotBtn);
	DDX_Control(pDX, IDC_ADDNOTCH, m_AddNotchBtn);
	DDX_Control(pDX, IDC_ADDWINDOW, m_AddWindowBtn);
	DDX_Control(pDX, IDC_ARB_ADVANCED, m_AdvancedBtn);
	DDX_Control(pDX, IDC_CHANGE_ARB_DISPLAY, m_ChangeDisplayBtn);
	DDX_Control(pDX, IDC_ARB_FILES, m_ARBFilesBtn);
	DDX_Text(pDX, IDC_DACRATETEXT, m_DACText);
	DDX_Text(pDX, IDC_DURATIONTEXT, m_DurationText);
	DDX_Text(pDX, IDC_FILTERTEXT, m_FilterText);
	DDX_Text(pDX, IDC_AMPLITUDETEXT, m_AmplitudeText);
	DDX_Text(pDX, IDC_MEMORYTEXT, m_MemoryText);
	DDX_Text(pDX, IDC_WINDOWTEXT, m_WindowText);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CEditARBDlg, CDialog)
	//{{AFX_MSG_MAP(CEditARBDlg)
	ON_BN_CLICKED(IDC_ADDNOTCH, OnAddnotch)
	ON_BN_CLICKED(IDC_ADDWINDOW, OnAddwindow)
	ON_BN_CLICKED(IDC_ARB_ADVANCED, OnArbAdvanced)
	ON_BN_CLICKED(IDC_ARB_FILES, OnArbFiles)
	ON_BN_CLICKED(IDC_CHANGE_ARB_DISPLAY, OnChangeArbDisplay)
	ON_BN_CLICKED(IDC_REMOVEARBEVENT, OnRemovearbevent)
	ON_BN_CLICKED(IDC_EDITARBEVENT, OnEditarbevent)
	ON_LBN_SELCHANGE(IDC_EVENTLISTBOX, OnSelchangeEventlistbox)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CEditARBDlg message handlers

BOOL CEditARBDlg::OnInitDialog()
{
	CDialog::OnInitDialog();
	// save a copy of PS so that we can retrieve the old
	// one if people cancel
	PSold = *pPS;
	m_ARBPlotBtn.pARBPacket=&(pPS->arb[CurrentArbWaveform]);

	// plot the ARB function in m_ARBPicture
	UpdateARB();
	return TRUE;
}

void CEditARBDlg::UpdateARB()
{
	// redraw plot button
	m_ARBPlotBtn.Invalidate();

	// re-initialize the text strings
	if(pPS->arb[CurrentArbWaveform].AmplitudeFixed) CalculateAmplitude();
	if(pPS->arb[CurrentArbWaveform].DacFixed) CalculateDacRate();
	if(pPS->arb[CurrentArbWaveform].FilterFixed) CalculateFilter();
	if(pPS->arb[CurrentArbWaveform].SamplesFixed) CalculateSamples();

	m_DurationText.Format("Duration : %1.3f ms",1000*pPS->arb[CurrentArbWaveform].Samples/pPS->arb[CurrentArbWaveform].DacRate);
	m_MemoryText.Format("Memory : %d kw",pPS->arb[CurrentArbWaveform].Samples/1024);
	m_AmplitudeText.Format("Amplitude : %1.3f V",pPS->arb[CurrentArbWaveform].Amplitude);
	m_WindowText.Format("Window (m/z) : %1.1f to %1.1f",pPS->arb[CurrentArbWaveform].WindowLowMass,pPS->arb[CurrentArbWaveform].WindowHighMass);
	// set filter text
	if(pPS->arb[CurrentArbWaveform].Filter==0)
		m_FilterText.Format("Filter : 100 kHz");
	else if(pPS->arb[CurrentArbWaveform].Filter==1)
		m_FilterText.Format("Filter : 200 kHz");
	else if(pPS->arb[CurrentArbWaveform].Filter==2)
		m_FilterText.Format("Filter : 500 kHz");
	else if(pPS->arb[CurrentArbWaveform].Filter==3)
		m_FilterText.Format("Filter : 1 MHz");
	else if(pPS->arb[CurrentArbWaveform].Filter==4)
		m_FilterText.Format("Filter : 10 MHz");
	else m_FilterText.Format("ERROR! Invalid Filter Setting");
	// set DacRate Text
	if(pPS->arb[CurrentArbWaveform].DacRate==16000000)
		m_DACText.Format("DAC Rate : 16 Msps");
	else if(pPS->arb[CurrentArbWaveform].DacRate==8000000)
		m_DACText.Format("DAC Rate : 8 Msps");
	else if(pPS->arb[CurrentArbWaveform].DacRate==4000000)
		m_DACText.Format("DAC Rate : 4 Msps");
	else if(pPS->arb[CurrentArbWaveform].DacRate==2000000)
		m_DACText.Format("DAC Rate : 2 Msps");
	else if(pPS->arb[CurrentArbWaveform].DacRate==1000000)
		m_DACText.Format("DAC Rate : 1 Msps");
	else if(pPS->arb[CurrentArbWaveform].DacRate==500000)
		m_DACText.Format("DAC Rate : 500 ksps");
	else if(pPS->arb[CurrentArbWaveform].DacRate==250000)
		m_DACText.Format("DAC Rate : 250 ksps");
	else if(pPS->arb[CurrentArbWaveform].DacRate==125000)
		m_DACText.Format("DAC Rate : 125 ksps");
	else m_DACText.Format("ERROR! invalid Dac Rate");

	//reset the event list box and fill it's strings
	int i;
	CString str;
	m_EventListBox.ResetContent();
	m_EventListBox.InitStorage(40,30);
	struct ArbEvent* pEvent;
	for(i=0; i<MAX_ARB_EVENTS; i++) {
		if(pPS->arb[CurrentArbWaveform].Event[i].Type == ARB_NONE) break;
		pEvent = &(pPS->arb[CurrentArbWaveform].Event[i]);
		if(pEvent->Type == ARB_RANGE) {
			str.Format("%1.1f%% : %1.1f to %1.1f",pEvent->Amplitude,pEvent->MassA,pEvent->MassB);
		}
		if(pEvent->Type == ARB_NOTCH) {
			str.Format("Notch at %1.1f +/- %1.1f",pEvent->MassA,pEvent->MassB);
		}
		m_EventListBox.AddString(str);
	}
	if(i==0) m_EventListBox.AddString("No Events Defined");
	NumArbEvents=i;
	UpdateData(FALSE);
}

void CEditARBDlg::OnAddnotch() 
{
	CAddARBNotchDlg dlg;
	dlg.m_Amplitude = 0;
	dlg.m_CenterMass=1000;
	dlg.m_DeltaMass=10;
	dlg.m_HiMass=1010;
	dlg.m_LowMass=990;
	int NextEvent = GetNextARBEvent();

	if(dlg.DoModal() == IDOK) {
		pPS->arb[CurrentArbWaveform].Event[NextEvent].Amplitude = (float)dlg.m_Amplitude;
		pPS->arb[CurrentArbWaveform].Event[NextEvent].Type = ARB_NOTCH;
		pPS->arb[CurrentArbWaveform].Event[NextEvent].MassA = (float)dlg.m_CenterMass;
		pPS->arb[CurrentArbWaveform].Event[NextEvent].MassB = (float)dlg.m_DeltaMass;
		pPS->arb[CurrentArbWaveform].Event[NextEvent].fit = 0;
		pPS->arb[CurrentArbWaveform].Event[NextEvent].reserved = 0;
	}
	UpdateARB();
}

void CEditARBDlg::OnAddwindow() 
{
	CAddARBWindowDlg dlg;
	dlg.m_Amplitude = 100;
	dlg.m_HiMass=pPS->DetectHighMass;
	dlg.m_LowMass=pPS->DetectLowMass;
	dlg.m_CenterMass=(dlg.m_HiMass+dlg.m_LowMass)/2;
	dlg.m_DeltaMass=(dlg.m_HiMass-dlg.m_LowMass)/2;
	int NextEvent = GetNextARBEvent();

	if(dlg.DoModal() == IDOK) {
		pPS->arb[CurrentArbWaveform].Event[NextEvent].Amplitude = (float)dlg.m_Amplitude;
		pPS->arb[CurrentArbWaveform].Event[NextEvent].Type = ARB_RANGE;
		pPS->arb[CurrentArbWaveform].Event[NextEvent].MassA = (float)dlg.m_LowMass;
		pPS->arb[CurrentArbWaveform].Event[NextEvent].MassB = (float)dlg.m_HiMass;
		pPS->arb[CurrentArbWaveform].Event[NextEvent].fit = 0;
		pPS->arb[CurrentArbWaveform].Event[NextEvent].reserved = 0;
	}
	UpdateARB();
}

void CEditARBDlg::OnArbAdvanced() 
{
	CARBAdvancedDlg dlg;

	dlg.Samples = pPS->arb[CurrentArbWaveform].Samples;
	dlg.Filter = pPS->arb[CurrentArbWaveform].Filter;
	dlg.DACRate = pPS->arb[CurrentArbWaveform].DacRate;
	dlg.DacFixed = pPS->arb[CurrentArbWaveform].DacFixed;
	dlg.FilterFixed = pPS->arb[CurrentArbWaveform].FilterFixed;
	dlg.SamplesFixed = pPS->arb[CurrentArbWaveform].SamplesFixed;
	dlg.MaxSamples = pDoc->sys->ArbGeneratorMemory/2;

	if(dlg.DoModal() == IDOK) {
		pPS->arb[CurrentArbWaveform].Samples = long(dlg.Samples);
		pPS->arb[CurrentArbWaveform].Filter = dlg.Filter;
		pPS->arb[CurrentArbWaveform].DacRate = dlg.DACRate;
		pPS->arb[CurrentArbWaveform].DacFixed = dlg.DacFixed;
		pPS->arb[CurrentArbWaveform].FilterFixed = dlg.FilterFixed;
		pPS->arb[CurrentArbWaveform].SamplesFixed = dlg.SamplesFixed;
	}
	UpdateARB();
}

void CEditARBDlg::OnArbFiles() 
{
	// NADA for now.  work on this when I have time	
}

void CEditARBDlg::OnChangeArbDisplay() 
{
	CChangeARBDisplay dlg;
	dlg.m_LowMass = pPS->arb[CurrentArbWaveform].WindowLowMass;
	dlg.m_HiMass = pPS->arb[CurrentArbWaveform].WindowHighMass;
	if(dlg.DoModal() == IDOK) {
		pPS->arb[CurrentArbWaveform].WindowLowMass = float(dlg.m_LowMass);
		pPS->arb[CurrentArbWaveform].WindowHighMass = float(dlg.m_HiMass);
	}
	UpdateARB();
}

void CEditARBDlg::OnOK() 
{
	if(CheckOverlappingNotches()) {
		// there are overlapping notches.  Require user to
		// fix the problem before exiting
		MessageBox("There are multiple notches defined \n for the same m/z range.\n\nPlease correct before continuing","ARB error!");
		return;
	}
	if(CheckOverlappingWindows()) {
		// there are multiple excite windows defined for
		// the same m/z.  Which is correct?  Don't know.
		// so, require user to fix the problem before exiting.
		MessageBox("There are multiple excite windows defined\n for the same m/z range.\n\nPlease correct before continuing","ARB error!");
		return;
	}
	CDialog::OnOK();
}

void CEditARBDlg::OnCancel() 
{
	// retrieve old copy of PS
	*(pPS) = PSold;

	CDialog::OnCancel();
}

int CEditARBDlg::GetNextARBEvent()
{
	int i;
	for(i=0; i<MAX_ARB_EVENTS; i++) {
		if(pPS->arb[CurrentArbWaveform].Event[i].Type == ARB_NONE) break;
	}
	return i;
}

BOOL CEditARBDlg::CheckOverlappingNotches()
{
	int i,j;
	double massA1,massB1,massA2,massB2;
	for(i=0; i<MAX_ARB_EVENTS; i++) {
		if(pPS->arb[CurrentArbWaveform].Event[i].Type == ARB_NOTCH) {
			for(j=0; j<MAX_ARB_EVENTS; j++) {
				if(i==j) continue;
				if(pPS->arb[CurrentArbWaveform].Event[j].Type == ARB_NOTCH) {
					massA1 = pPS->arb[CurrentArbWaveform].Event[i].MassA;
					massB1 = pPS->arb[CurrentArbWaveform].Event[i].MassB;
					massA2 = pPS->arb[CurrentArbWaveform].Event[j].MassA;
					massB2 = pPS->arb[CurrentArbWaveform].Event[j].MassB;
					if(massA1<=massA2 && massB1>=massA2) return TRUE;
					if(massA1<=massB2 && massB1>=massB2) return TRUE;
				}
			}
		}
	}
	return FALSE;
}

BOOL CEditARBDlg::CheckOverlappingWindows()
{
	int i,j;
	double massA1,massB1,massA2,massB2;
	for(i=0; i<MAX_ARB_EVENTS; i++) {
		if(pPS->arb[CurrentArbWaveform].Event[i].Type == ARB_RANGE) {
			for(j=0; j<MAX_ARB_EVENTS; j++) {
				if(i==j) continue;
				if(pPS->arb[CurrentArbWaveform].Event[j].Type == ARB_RANGE) {
					massA1 = pPS->arb[CurrentArbWaveform].Event[i].MassA;
					massB1 = pPS->arb[CurrentArbWaveform].Event[i].MassB;
					massA2 = pPS->arb[CurrentArbWaveform].Event[j].MassA;
					massB2 = pPS->arb[CurrentArbWaveform].Event[j].MassB;
					if(massA1<=massA2 && massB1>=massA2) return TRUE;
					if(massA1<=massB2 && massB1>=massB2) return TRUE;
				}
			}
		}
	}
	return FALSE;
}


void CEditARBDlg::OnRemovearbevent() 
{
	if(NumArbEvents==0) return;
	int Event = m_EventListBox.GetCurSel();
	pPS->arb[CurrentArbWaveform].Event[Event] = NullARBEvent;
	int i;
	for(i=Event; i<MAX_ARB_EVENTS-1; i++) {
		pPS->arb[CurrentArbWaveform].Event[i]=pPS->arb[CurrentArbWaveform].Event[i+1];
		if(i==MAX_ARB_EVENTS-1) pPS->arb[CurrentArbWaveform].Event[MAX_ARB_EVENTS-1] = NullARBEvent;
	}
	UpdateARB();
}

void CEditARBDlg::OnEditarbevent() 
{
	if(NumArbEvents==0) return;
	// TODO: Add your control notification handler code here
	MessageBox("This function not yet implemented.\nTo modify an event, remove it and \nthen add a new one with the correct values","Function unavailable");
	UpdateARB();	
}

void CEditARBDlg::OnSelchangeEventlistbox() 
{
	if(NumArbEvents==0) return;
}

void CEditARBDlg::CalculateAmplitude()
{
	// not yet implemented.  does nothing right now
}

void CEditARBDlg::CalculateDacRate()
{
	// DacRate ideally = 2 * WindowLowMassFreq so we choose
	// DacRate the next available DacRate above ideal
	double freq = 2*pDoc->MassToFreq(pPS->arb[CurrentArbWaveform].WindowLowMass);
	double MaxDacRate,f;
	MaxDacRate=16000000;
	f = MaxDacRate;
	int i;
	for(i=0; i<8; i++) {
		if (f>freq) pPS->arb[CurrentArbWaveform].DacRate = f;
		f = f/2;
	}
}

void CEditARBDlg::CalculateFilter()
{
	// filter is next filter above DacRate
	double filters[5];
	filters[4]=10000000;
	filters[3]=1000000;
	filters[2]=500000;
	filters[1]=200000;
	filters[0]=100000;
	int i;
	for(i=4; i>=0; i--) {
		if(filters[i]>pPS->arb[CurrentArbWaveform].DacRate)
			pPS->arb[CurrentArbWaveform].Filter = i;
	}
}

void CEditARBDlg::CalculateSamples()
{
	// samples is next value above 1/deltafreq of narrowest notch

	// first find narrowest notch.
	int i;
	double df,deltafreq,freq1,freq2,MassA,MassB;
	df=0;
	deltafreq=0;
	for(i=0; i<MAX_ARB_EVENTS; i++) {
		if(pPS->arb[CurrentArbWaveform].Event[i].Type == ARB_NONE) break;
		MassA = pPS->arb[CurrentArbWaveform].Event[i].MassA;
		MassB = pPS->arb[CurrentArbWaveform].Event[i].MassB;
		if(pPS->arb[CurrentArbWaveform].Event[i].Type == ARB_NOTCH) {
			freq1=pDoc->MassToFreq(MassA-MassB);
			freq2=pDoc->MassToFreq(MassA+MassB);
			df = freq2-freq1;
		}
		if(pPS->arb[CurrentArbWaveform].Event[i].Type == ARB_RANGE) {
			freq1=pDoc->MassToFreq(MassA);
			freq2=pDoc->MassToFreq(MassB);
			df = freq2-freq1;
		}
		if(df<0) df=df*-1; 
		if(i==0) deltafreq = df;
		if(deltafreq>df) deltafreq=df; // find minimum deltafreq
	}
	long nsps = 2*long(pPS->arb[CurrentArbWaveform].DacRate/(deltafreq));

	long Maxnsps = pDoc->sys->ArbGeneratorMemory*1024*1024/2;
	pPS->arb[CurrentArbWaveform].Samples = -1;
	for(i=0; i<10; i++) {
		if(Maxnsps>nsps) pPS->arb[CurrentArbWaveform].Samples = Maxnsps;
		Maxnsps=Maxnsps/2;
	}
	if(pPS->arb[CurrentArbWaveform].Samples == -1) {
		MessageBox("ARB memory limit exceeded.  Defaulting waveform to \nmaximum ARB waveform size as specified in \nthe Buda.cnf file.\n\nThis means that the waveform you have chosen will show\nrounded corners and isolation will be less effective","ARB Memory Limit Reached.");
		pPS->arb[CurrentArbWaveform].Samples=pDoc->sys->ArbGeneratorMemory*1024*1024/2;
	}
}
