/****************************************************************

Boston University data analysis (BUDA) package 

Description: Buda is a data analysis program for Fourier 
Transform Mass Spectrometry data.  

by Peter B. O'Connor

    Copyright (C) 2000  Peter B. O'Connor

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/
// FFTWorkerThread1.h: interface for the CFFTWorkerThread class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_FFTWORKERTHREAD1_H__3021E680_4EDE_11D2_B907_BA5AB4399273__INCLUDED_)
#define AFX_FFTWORKERTHREAD1_H__3021E680_4EDE_11D2_B907_BA5AB4399273__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "BudaMaster.h"
#include "Buda.h"
#include <afxmt.h>
#include "nsp.h"
//#include "ChargeStateDetermination.h"	// Added by ClassView

// declare classes that will be needed so that I
// can use interlinked includes
class CBudaApp;
class CInstrument;
class CBudaDoc;
class CMainFrame;

class CFFTWorkerThread  
{
public:
	void CalculateNoiseThresholdbyHistogram(long HighMassPt, long LowMassPt, float* data );
	void CalculateNoiseThresholdbyIterativeAveraging(long HighMassPt, long LowMassPt, float* data );
	double CalculateRes(double a, double b, double c, double maxy, double maxx, double df);
	BOOL IsMIed;
	double ScaleFactor;
	int NumPeaks;
	double NoiseLevel;
	double TotalIonInt;
	void CopyFITable();
	struct FITableElement* FITable;
	int UpdateProgress(int);
	CBudaDoc* pDoc;
	CFFTWorkerThread* pFFT;
	BOOL Unlock();
	BOOL Lock();
	CSingleLock* DataLock;
	CCriticalSection* Access;
	CBudaApp* pMainApp;
	CWinThread* FFTThrd;
	struct FFTThreadInfo FFTThrdInfo;
	CFFTWorkerThread();
	virtual ~CFFTWorkerThread();

	// prototypes for calculation functions
	void Apodize(long WindowFunc, long Npts, float* FTData);
	void Magnitude(long TransientPoints, long FFTSize, SCplx* FTData, float ScaleFactor);
	void GetMITable(long HighMassPt, long LowMassPt, float* FTData);
	void EstimateNoiseLevel(long HighMassPt, long LowMassPt, float* data );
	void GetTotalIonInt();
	void PeakPick(long HighMassPt, long LowMassPt, float* data);
	double PtToFreq(long pt);
	void PeakFit(double *PeakStack, double *PeakXStack, int numpts);
	BOOL ParabolicLeastSquares(double *y, int numpts, double *ret, double PeakMaxAmplitude, double PeakPickerLevel);
	double GetDeterminant(double a1, double a2, double a3, double b1, double b2, double b3, double c1, double c2, double c3);
	float InterpolateForRes(double *y, double *x, int ptr, double maxx);
	void NewPeakPick(long Highpt, long Lowpt, float* data);
	float GetPeakArea(double* Ypts, double* Xpts, int numpts);
};

// prototype for the FFT worker thread function
UINT FFTThreadProc(LPVOID pFFTInfo /* FFTThreadInfo pointer*/);

#endif // !defined(AFX_FFTWORKERTHREAD1_H__3021E680_4EDE_11D2_B907_BA5AB4399273__INCLUDED_)
