// Fdm.cpp : implementation file
//

#include "stdafx.h"
#include "Buda.h"
#include "Fdm.h"
#include <math.h>
#include <complex>
using namespace std;

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif
#define dcplx complex<double>

/////////////////////////////////////////////////////////////////////////////
// CFdm dialog
CFdm::CFdm(CWnd* pParent /*=NULL*/)
	: CDialog(CFdm::IDD, pParent)
{
	//{{AFX_DATA_INIT(CFdm)
	m_CheatFactor = 0.0;
	m_Cheatmore = FALSE;
	m_ErrorLimit = 0.0;
	Gamm = 0.0;
	Gcut = 0.0;
	m_HighFreq = 0.0;
	m_LowFreq = 0.0;
	m_NPower = 0;
	rho = 0.0;
	//}}AFX_DATA_INIT
}


void CFdm::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CFdm)
	DDX_Text(pDX, IDC_FDM_CHEATFACTOR, m_CheatFactor);
	DDX_Text(pDX, IDC_FDM_CHEATMORE, m_Cheatmore);
	DDX_Text(pDX, IDC_FDM_ERROR, m_ErrorLimit);
	DDX_Text(pDX, IDC_FDM_GAMM, Gamm);
	DDX_Text(pDX, IDC_FDM_GCUT, Gcut);
	DDX_Text(pDX, IDC_FDM_HI_FREQ, m_HighFreq);
	DDX_Text(pDX, IDC_FDM_LOW_FREQ, m_LowFreq);
	DDX_Text(pDX, IDC_FDM_NPOWER, m_NPower);
	DDX_Text(pDX, IDC_FDM_RHO, rho);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CFdm, CDialog)
	//{{AFX_MSG_MAP(CFdm)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CFdm message handlers

void CFdm::FDM()
{
	// this is adapted from Vladimir Mandelshtam's fd_qz.f program
	long i,j;
	long Nskip,dim;
	dcplx temp;
	double theta;
    double t0,error;
    double wmin_old,wmax_old,shift;
    long choose_nb;

	delt = 2000*pi/pPS->AdcRate;
	t0 = 0.0;
	theta = 0.0;
	wmin_old = m_LowFreq;
	wmax_old = m_HighFreq;
	Nsig = pPS->TransientPoints; // number of signal points to use
	Nskip = 0;		// number of data points at beginning of transient to skip
	error = m_ErrorLimit;	// maximum error for an eigenvalue
	Npower = m_NPower;	// number of frequency points for plotting
	dim=1;

	if (Npower > Npowermax) Npower = Npowermax;

	// clear the spectral display array
	for(j=0; j<Npower; j++) {
		Spec[j] = dcplx(0.0,0.0);
	}
	PI = cos(-1.0);
	xi = dcplx(0.0,1.0);
	phase_corr = exp(complex<double>(0,theta));  
	wmin=wmin_old*delt;
	if(wmin < -1*PI) wmin = -1*PI;
	wmax = wmax_old*delt;
	if(wmax > PI) wmax = PI;
	wmin_old=wmin/delt;
	wmax_old = wmax/delt;
	if(Nsig > Nsigmax) Nsig=Nsigmax;

	// copy data to coeff array.
	for(i=0; i<Nsig; i++) {
		coef[i]=dcplx(pDoc->TimeArray[i],0.0)*phase_corr;
	}

	// size of U arrays is MxM.
	M = (Nsig-2)/2;
	
	if(rho > 0.0) choose_nb = TRUE;
	else Nb = (long)(-1.0*rho*1.00000001);

	// ispec = 0.  // do fdm, not dft.
	if(choose_nb) {
		Nb = (long)(((wmax-wmin)/(2*PI))*((double)(M+1))*rho);
	}
	shift = PI/(2*(M+1));
	if(Nb < 3) Nb = 3;
	if(Nb > M+1) Nb = M+1;
	if(Nb > Nbmax) {
		MessageBox("Nb > Nbmax","FDM Error",MB_OK);
		return;
	}
	for(i=1; i<=Nb; i++) {
		z[i].real(wmin+((double)i-0.5)*(wmax-wmin)/(double)Nb);
		z[i].imag(0.0);
		z[i]=exp(xi*z[i]);	// this is 1/z_j as defined in paper.
	}
	Nb2=Nb;

	// call the main Filter diagonalization routine.
	FD();

	// write the set of points.

	// sort peaks

	// write out the peak list.

}

void CFdm::InitializeParameters()
{
	pPS = &(pDoc->PS);
	m_CheatFactor = 1.0;
	m_Cheatmore = FALSE;
	m_ErrorLimit = 1.0;
	Gamm = 0.0001;
	Gcut = 0.01;
	m_HighFreq = 45000.0;
	m_LowFreq = 50000.0;
	m_NPower = 1000;
	rho = 1.20;
}

BOOL CFdm::OnInitDialog() 
{
	CDialog::OnInitDialog();
	
	InitializeParameters();
	UpdateData(FALSE);

	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CFdm::FD()
{
	// main Filter diagonalization function
	long i,j,n,k,k1,ierr,n0,i1,j1,l;
	complex<double> Z1,Z2,Power,ss,uu,cc,f1,f2,c1,s1,temp;
	double t0,yiter;

	// construct small U0, U1, U2 matrices
	temp = dcplx(0,0);
	for(l=0; l<2; l++) {
		for(i=0; i<Nb; i++) {
			for(j=0; j<Nb; j++) {
				U[j][i][l] = temp;
			}	
		}
	}

	for(i=1; i<=Nb2; i++) {
		f[i][1][1] = temp;
		f[i][2][1] = temp;
		cc = temp;
		ss = coef[0];
		s1 = temp;
		c1 = temp;
		f1 = temp;
		f2 = temp;
		for(k=1; k<=M; k++) {
			if(k % 100 == 1) Z1=z[i];
			else Z1=Z1*z[i];
			f1 = f1+(coef[k]*Z1);
			s1 = s1+dcplx(k,0)*(coef[k]*Z1);
			f2 = f2+(coef[k+M]*Z1);
			c1 = c1+dcplx(k,0)*(coef[k+M]*Z1);
			if(k % 100 == 1 || k % 100 == 0) {
				f[i][1][1] = f[i][1][1] + f1;
				f[i][2][1] = f[i][2][1] + f2;
				ss = ss + s1;
				cc = cc + c1;
				s1 = temp;
				c1 = temp;
				f1 = temp;
				f2 = temp;
			} // end if k
		} // end for k
	
		Z2=Z1*Z1;
		f[i][2][1] = f[i][2][1] * Z1;
		ss = ss - (cc*Z1) + dcplx(M+2,0)*f[i][2][1];
		f[i][1][2] = f[i][1][1]/z[i] - coef[2] + coef[M+2]*Z1;
		f[i][2][2] = f[i][2][1]/z[i] - coef[M+2]*Z1 + coef[2*M + 2]*Z2;
		f[i][1][0] = (f[i][1][1]+coef[1]-coef[M+1]*Z1)*z[i];
		f[i][2][0] = (f[i][2][1]-coef[2*M+1]*Z2 + coef[M+1]*Z1)*z[i];
		j=i;
		if(i>Nb)j=i-Nb;
		U[j][j][1] = U[j][j][1] + ss;
		U[j][j][2] = U[j][j][2] + (ss-coef[1]-f[i][1][1]+f[i][2][1])/z[i] + coef[2*M+2]*Z2;
		U[j][j][0] = U[j][j][0] + (ss+coef[1]-dcplx(2,0)*coef[M+1]*Z1+f[i][1][1] - f[i][2][1])*z[i]+coef[0];
	} //end for i

	for(i=1; i<=Nb2; i++) {
		i1 = i;
		if(i>Nb) i1 = i-Nb;
		for(j=1; j<=Nb2; j++) {
			if(j != i) {
				j1=j;
				if(j>Nb) j1=j-Nb;
				cc = z[i]/z[j];
				for(l=0; l<=2; l++) {
					U[i1][j1][l] = U[i1][j1][l]+coef[l] + (f[j][1][l] - pow(cc,dcplx(M+1,0)*f[j][2][1]) - cc*f[i][1][1] + pow(cc,dcplx(-1*M,0)*f[i][2][1]))/(dcplx(1,0)-cc);
				} //endfor
			} //endif
		} // endfor
	} // endfor

	// solving: u1 bk = uk U0 Bk

	//QZ

}