// LinkExcite.cpp : implementation file
//
/****************************************************************

Boston University data analysis (BUDA) package 

Description: Buda is a data analysis program for Fourier 
Transform Mass Spectrometry data.  

by Peter B. O'Connor

    Copyright (C) 2000  Peter B. O'Connor

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "stdafx.h"
#include "Buda.h"
#include "LinkExcite.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CLinkExcite dialog


CLinkExcite::CLinkExcite(CWnd* pParent /*=NULL*/)
	: CDialog(CLinkExcite::IDD, pParent)
{
	//{{AFX_DATA_INIT(CLinkExcite)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
}


void CLinkExcite::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CLinkExcite)
	DDX_Control(pDX, IDC_EVENTLINKLIST, m_EventListBox);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CLinkExcite, CDialog)
	//{{AFX_MSG_MAP(CLinkExcite)
	ON_LBN_SELCHANGE(IDC_EVENTLINKLIST, OnSelchangeEventlinklist)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CLinkExcite message handlers

BOOL CLinkExcite::OnInitDialog()
{
	CDialog::OnInitDialog();
	m_EventListBox.ResetContent();
	m_EventListBox.InitStorage(TOTAL_PS_EVENTS,64);

	EventList = new int[pPS->NumEvents];

	SelectedEvent=0;
	int i, sptr=0, aptr=0;
	CString str;
	NumEvents=0;
	for(i=0; i<pPS->NumEvents; i++) { 
		if(pPS->GenPS[i].Channel != ARB_TRIGGER && pPS->GenPS[i].Channel != SYN_TRIGGER && pPS->GenPS[i].Channel != IMP_TRIGGER) continue;
		EventList[NumEvents]=i;
		switch(pPS->GenPS[i].Channel) {
		case SYN_TRIGGER:
			if (sptr > pPS->NumCFP) {
				MessageBox("Error in pulse sequence.\n Number of synthesizer events has exceeded\nthe NumCFP count","Pulse Sequence Error!");
				return FALSE;
			}
			switch(pPS->cfp[sptr].Type) {
			case CFP_NONE :
				str.Format("CFP_NONE");
				break;
			case CFP_BURST:
				str.Format("Single Frequency %1.1f V Burst at %1.4f m/z",pPS->cfp[sptr].Amplitude,pPS->cfp[sptr].StartMass);
				sptr++;
				break;
			case CFP_SWEEP:
				str.Format("%1.1f V Frequency Sweep from %1.4f m/z to %1.4f m/z",pPS->cfp[sptr].Amplitude,pPS->cfp[sptr].StartMass,pPS->cfp[sptr].EndMass);
				sptr++;
				break;
			case CFP_IMPTRIG:
				str.Format("Impulse Excite at %1.1f msec",pPS->GenPS[i].Delay);
				sptr++;
				break;
			case CFP_QUAD:
				sptr++;
				continue;
				break;
			default:
				str.Format("");
				break;
			}
			break;
		case ARB_TRIGGER:
			if (aptr > pPS->NumARB) {
				MessageBox("Error in pulse sequence.\n Number of ARB events has exceeded\nthe NumARB count","Pulse Sequence Error!");
				return FALSE;
			}
			str.Format("ARB excitation at %1.1f msec",pPS->GenPS[i].Delay);
			aptr++;
			break;
		case IMP_TRIGGER:
			break;
		default:
			str.Format("Miscellaneous event at %1.1f msec",pPS->GenPS[i].Delay);
			break;
		}
		
		m_EventListBox.AddString(str);
		if(i==m_LinkEvent) SelectedEvent = NumEvents;
		NumEvents++;
	}
	m_EventListBox.SetCurSel(SelectedEvent);

	// everything init'd fine, so return TRUE
	return TRUE;
}

void CLinkExcite::OnSelchangeEventlinklist() 
{
	int i = m_EventListBox.GetCurSel();
	m_LinkEvent = EventList[i];
}

void CLinkExcite::OnOK() 
{
	int i = m_EventListBox.GetCurSel();
	m_LinkEvent = EventList[i];
	CDialog::OnOK();
}
