// LinkingDlg.cpp : implementation file
//
/****************************************************************

Boston University data analysis (BUDA) package 

Description: Buda is a data analysis program for Fourier 
Transform Mass Spectrometry data.  

by Peter B. O'Connor

    Copyright (C) 2000  Peter B. O'Connor

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "stdafx.h"
#include "Buda.h"
#include "LinkingDlg.h"

#ifdef _DEBUG
	#define new DEBUG_NEW
	#undef THIS_FILE
	static char THIS_FILE[] = __FILE__;
#endif

// Buda debug defines
#ifdef _DEBUG
	//#define DEBUG_CLINKINGDLG
#endif

/////////////////////////////////////////////////////////////////////////////
// CLinkingDlg dialog


CLinkingDlg::CLinkingDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CLinkingDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CLinkingDlg)
	//}}AFX_DATA_INIT
}


void CLinkingDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CLinkingDlg)
	DDX_Control(pDX, IDC_EVENTLINKLIST, m_EventList);
	DDX_Control(pDX, IDC_DPLINKLIST, m_ChannelList);
	DDX_Control(pDX, IDC_NOLINK, m_NoLinkBtn);
	DDX_Control(pDX, IDC_LINKSTART, m_LinkStartBtn);
	DDX_Control(pDX, IDC_LINKEND, m_LinkEndBtn);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CLinkingDlg, CDialog)
	//{{AFX_MSG_MAP(CLinkingDlg)
	ON_BN_CLICKED(IDC_NOLINK, OnNoLink)
	ON_BN_CLICKED(IDC_LINKSTART, OnLinkStart)
	ON_BN_CLICKED(IDC_LINKEND, OnLinkEnd)
	ON_LBN_SELCHANGE(IDC_DPLINKLIST, OnSelchangeChannellist)
	ON_LBN_SELCHANGE(IDC_EVENTLINKLIST, OnSelchangeEventlist)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CLinkingDlg message handlers

void CLinkingDlg::OnNoLink() 
{
	Linking = NO_LINK;	
	m_NoLinkBtn.SetCheck(1);
	m_LinkStartBtn.SetCheck(0);
	m_LinkEndBtn.SetCheck(0);
}

void CLinkingDlg::OnLinkStart() 
{
	Linking = LINK_BEG;	
	m_NoLinkBtn.SetCheck(0);
	m_LinkStartBtn.SetCheck(1);
	m_LinkEndBtn.SetCheck(0);
}

void CLinkingDlg::OnLinkEnd() 
{
	Linking = LINK_END;	
	m_NoLinkBtn.SetCheck(0);
	m_LinkStartBtn.SetCheck(0);
	m_LinkEndBtn.SetCheck(1);
}

void CLinkingDlg::OnSelchangeChannellist() 
{
	IndexChannel = m_ChannelList.GetCurSel();
	SelectedChannel = ChannelList[IndexChannel];
	InitEventList();
}

void CLinkingDlg::OnSelchangeEventlist() 
{
	int tempevent;
	
	tempevent = m_EventList.GetCurSel();	
	if(pPS->GenPS[EventList[tempevent]].Type == DAC_INIT) {
		MessageBox("Link Rejected:\nYou cannot link to an Initialization Event","Error: Invalid link");
		return;
	}
	IndexEvent = tempevent;
	SelectedEvent = EventList[tempevent];
}

BOOL CLinkingDlg::OnInitDialog()
{
	CDialog::OnInitDialog();
	SelectedChannel=0;
	SelectedEvent = 0;
	if(InEvent <0 || InEvent > pPS->NumEvents) {
		MessageBox("Error 87: Cannot link to an undefined event","Error on Link-To");
		return FALSE;
	}
	LinkChannel = pPS->GenPS[InEvent].Channel;

	switch (Linking) {
	case NO_LINK:
		OnNoLink();
		break;
	case LINK_BEG:
		OnLinkStart();
		break;
	case LINK_END:
		OnLinkEnd();
		break;
	}

	InitChannelList();
	InitEventList();
	UpdateData(FALSE);
	return TRUE;
}

void CLinkingDlg::InitChannelList()
{
	m_ChannelList.ResetContent();
	m_ChannelList.InitStorage(40,30);
	m_ChannelList.SetCurSel(0);

	int i;
	NumChannels=0;
	SelectedChannel = 0;
	IndexChannel = 0;
	BOOL ChannelsAlreadyListed[40];
	for(i=0; i<40; i++) ChannelsAlreadyListed[i]=FALSE;
	for(i=0; i<pPS->NumEvents; i++) {
		if(i == CurrentEvent) continue;
		if(InEvent == i) {
			SelectedChannel = pPS->GenPS[i].Channel;
			IndexChannel = NumChannels;
		}
		if(pPS->GenPS[i].Type == DAC_INIT) continue;
		if(pPS->GenPS[i].Channel < 8) continue;
		//if(pPS->GenPS[i].Channel == QUADRF_GATE) continue;
		if(pDoc->sys->ChannelNames[pPS->GenPS[i].Channel] == "UNUSED") continue;
		if(ChannelsAlreadyListed[pPS->GenPS[i].Channel]) continue;
		ChannelsAlreadyListed[pPS->GenPS[i].Channel] = TRUE;
		ChannelList[NumChannels]=pPS->GenPS[i].Channel;
		m_ChannelList.AddString(pDoc->sys->ChannelNames[pPS->GenPS[i].Channel]);
		NumChannels++;
	}
	
	m_ChannelList.SetCurSel(IndexChannel);
	SelectedChannel = ChannelList[IndexChannel];
	InitEventList();
#ifdef DEBUG_CLINKINGDLG
	CString str;
	str.Format("CLinkingDlg::InitChannelList() : Selected Channel = %d  : IndexChannel = %d\n",SelectedChannel, IndexChannel);
	TRACE(str);
#endif
}

void CLinkingDlg::InitEventList()
{
	m_EventList.ResetContent();
	m_EventList.InitStorage(TOTAL_PS_EVENTS,64);
	m_EventList.SetCurSel(0);

	SelectedEvent=0;
	IndexEvent = 0;
	int i;
	CString str;
	float start,finish;
	NumEvents=0;
	for(i=0; i<pPS->NumEvents; i++) { 
		if(pPS->GenPS[i].Channel != SelectedChannel) continue;
		if(pPS->GenPS[i].Type == DAC_INIT) continue;
		if(i == CurrentEvent) continue;
		EventList[NumEvents]=i;
		pDoc->GetEventEdges(pPS,i,&start,&finish);
		switch(pPS->GenPS[i].Type) {
		case DAC_INIT:
			str.Format("DAC initialized to %1.1f volts",pPS->GenPS[i].Value);
			break;
		case TTL_PULSE:
			str.Format("TTL pulse at %1.1f msec",start);
			break;
		case DAC_STEP:
			str.Format("DAC step at %1.1f msec to %1.1f volts",pPS->GenPS[i].Delay,pPS->GenPS[i].Value);
			break;
		case DAC_RAMP:
			str.Format("DAC ramp to %1.1f volts",pPS->GenPS[i].Value);
			break;
		case DAC_PULSE:
			str.Format("Pulse DAC to %1.1f volts at %1.1f msec for %1.1f msec",pPS->GenPS[i].Value,start,pPS->GenPS[i].Width);
			break;
		default:
			str.Format("Miscellaneous event at %1.1f msec",start);
			break;
		}
				
		m_EventList.AddString(str);
		if(i==InEvent) {
			SelectedEvent = i;
			IndexEvent = NumEvents;
		}
		NumEvents++;
	}
	m_EventList.SetCurSel(IndexEvent);
#ifdef DEBUG_CLINKINGDLG
	CString str1;
	str1.Format("CLinkingDlg::InitEventList() : Selected Event = %d   : Index Event = %d\n",SelectedEvent,IndexEvent);
	TRACE(str1);
#endif
}

void CLinkingDlg::OnOK() 
{
	if(Linking != NO_LINK) {
		OnSelchangeEventlist();
	} else {
		CDialog::OnOK();
		return;
	}

	InEvent = EventList[IndexEvent];
	if(InEvent != SelectedEvent) {
		MessageBox("Error 85: Event invalid\n Removing Link","Event Error");
		InEvent = 0;
		Linking = NO_LINK;
	}

	CDialog::OnOK();
}
