// MITableSetup.cpp : implementation file
//
/*
MITableSetup.cpp is the code for the CMITableSetup property page
class.  This page in the options property sheet allows users to 
control MITable options.

*/
/****************************************************************

Boston University data analysis (BUDA) package 

Description: Buda is a data analysis program for Fourier 
Transform Mass Spectrometry data.  

by Peter B. O'Connor

    Copyright (C) 2000  Peter B. O'Connor

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "BudaMaster.h"
#include "Buda.h"
#include "BudaDoc.h"
#include "MITableSetup.h"


#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CMITableSetup property page

IMPLEMENT_DYNCREATE(CMITableSetup, CPropertyPage)

CMITableSetup::CMITableSetup() : CPropertyPage(CMITableSetup::IDD)
{
	SortDirty=FALSE;
	//{{AFX_DATA_INIT(CMITableSetup)
	m_MinimumThreshold = 0.0f;
	m_NoiseThreshold = 0.0;
	//}}AFX_DATA_INIT
}

CMITableSetup::~CMITableSetup()
{
}

void CMITableSetup::DoDataExchange(CDataExchange* pDX)
{
	CPropertyPage::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CMITableSetup)
	DDX_Control(pDX, IDC_AUTOTHRESHOLDCHK, m_AutoNoiseThresholdBtn);
	DDX_Control(pDX, IDC_SORT_ON_MZ, m_sort_on_mz_btn);
	DDX_Control(pDX, IDC_SORT_ON_MASS, m_sort_on_mass_btn);
	DDX_Control(pDX, IDC_SORT_ON_FREQ, m_sort_on_freq_btn);
	DDX_Control(pDX, IDC_SORT_ON_INTENSITY, m_sort_on_int_btn);
	DDX_Control(pDX, IDC_SORT_INCREASING, m_sort_increasing_btn);
	DDX_Control(pDX, IDC_SORT_DECREASING, m_sort_decreasing_btn);
	DDX_Text(pDX, IDC_MINIMUM_AUTOTHRESHOLD, m_MinimumThreshold);
	DDV_MinMaxDouble(pDX, m_MinimumThreshold, 0., 100.);
	DDX_Text(pDX, IDC_NOISETHRESHOLD, m_NoiseThreshold);
	DDV_MinMaxDouble(pDX, m_NoiseThreshold, 0., 100.);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CMITableSetup, CPropertyPage)
	//{{AFX_MSG_MAP(CMITableSetup)
	ON_BN_CLICKED(IDC_AUTOTHRESHOLDCHK, OnAutoThreshold)
	ON_BN_CLICKED(IDC_SORT_ON_FREQ, OnSortOnFreq)
	ON_BN_CLICKED(IDC_SORT_ON_INTENSITY, OnSortOnIntensity)
	ON_BN_CLICKED(IDC_SORT_ON_MASS, OnSortOnMass)
	ON_BN_CLICKED(IDC_SORT_INCREASING, OnSortIncreasing)
	ON_BN_CLICKED(IDC_SORT_DECREASING, OnSortDecreasing)
	ON_BN_CLICKED(IDC_THRESHOLD_METHOD_IA, OnThresholdMethodIa)
	ON_BN_CLICKED(IDC_THRESHOLD_METHOD_HISTOGRAM, OnThresholdMethodHistogram)
	ON_BN_CLICKED(IDC_SORT_ON_MZ, OnSortOnMz)
	ON_BN_CLICKED(IDC_SAVE_AS_DEFAULTS, OnSaveAsDefaults)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CMITableSetup message handlers

void CMITableSetup::OnSortOnFreq() 
{
	SortMode=3;
	SetModified(TRUE);
}

void CMITableSetup::OnSortOnIntensity() 
{
	SortMode=2;	
	SetModified(TRUE);
}

void CMITableSetup::OnSortOnMass() 
{
	SortMode = 4;
	SetModified(TRUE);
}

void CMITableSetup::OnSortOnMz() 
{
	SortMode = 1;
	SetModified(TRUE);
}

void CMITableSetup::OnSortIncreasing() 
{
	SortIncreasing = TRUE;
	SetModified(TRUE);
}

void CMITableSetup::OnSortDecreasing() 
{
	SortIncreasing = FALSE;
	SetModified(TRUE);
}

BOOL CMITableSetup::OnSetActive() 
{
	SortDirty = FALSE;
	if (SortIncreasing) {
		m_sort_increasing_btn.SetCheck(1);
		m_sort_decreasing_btn.SetCheck(0);
	} else {
		m_sort_increasing_btn.SetCheck(0);
		m_sort_decreasing_btn.SetCheck(1);
	}

	switch (SortMode) {
	case 1:
		m_sort_on_mz_btn.SetCheck(1);
		m_sort_on_mass_btn.SetCheck(0);
		m_sort_on_freq_btn.SetCheck(0);
		m_sort_on_int_btn.SetCheck(0);
		break;
	case 2:
		m_sort_on_mz_btn.SetCheck(0);
		m_sort_on_mass_btn.SetCheck(0);
		m_sort_on_freq_btn.SetCheck(0);
		m_sort_on_int_btn.SetCheck(1);
		break;
	case 3:
		m_sort_on_mz_btn.SetCheck(0);
		m_sort_on_mass_btn.SetCheck(0);
		m_sort_on_freq_btn.SetCheck(1);
		m_sort_on_int_btn.SetCheck(0);
		break;
	case 4:
		m_sort_on_mz_btn.SetCheck(0);
		m_sort_on_mass_btn.SetCheck(1);
		m_sort_on_freq_btn.SetCheck(0);
		m_sort_on_int_btn.SetCheck(0);
		break;
	}
	SetModified(FALSE);

	//m_SICbtn.SetCheck(SICmode);
	m_AutoNoiseThresholdBtn.SetCheck(AutoDetectThreshold);
/*	switch(ThresholdMethod) {
	case ITERATIVE_AVERAGING_METHOD:
		m_ThreshMethod_IA_btn.SetCheck(1);
		m_ThreshMethod_Histogram_btn.SetCheck(0);	
		break;
	case HISTOGRAM_METHOD:
		m_ThreshMethod_IA_btn.SetCheck(0);
		m_ThreshMethod_Histogram_btn.SetCheck(1);	
		break;
	}
*/
	
	return CPropertyPage::OnSetActive();
}

BOOL CMITableSetup::OnApply() 
{
	SortDirty = TRUE;
	return CPropertyPage::OnApply();
}

/*
void CMITableSetup::OnSicmodechk() 
{
	if(m_SICbtn.GetCheck()) SICmode = TRUE;
	else SICmode = FALSE;
}
*/
void CMITableSetup::OnThresholdMethodIa() 
{
//	m_AutoNoiseThresholdBtn.SetCheck(1);
//	m_ThreshMethod_IA_btn.SetCheck(1);
//	m_ThreshMethod_Histogram_btn.SetCheck(0);	
}

void CMITableSetup::OnThresholdMethodHistogram() 
{
//	m_AutoNoiseThresholdBtn.SetCheck(1);
//	m_ThreshMethod_IA_btn.SetCheck(0);
//	m_ThreshMethod_Histogram_btn.SetCheck(1);	
}

void CMITableSetup::OnAutoThreshold() 
{
	UpdateData(TRUE);
	if(AutoDetectThreshold) {
		AutoDetectThreshold = FALSE;
		Threshold = m_NoiseThreshold;
	} else {
		AutoDetectThreshold = TRUE;
		Threshold = m_MinimumThreshold;
	}

	m_AutoNoiseThresholdBtn.SetCheck(AutoDetectThreshold);
}


void CMITableSetup::OnSaveAsDefaults() 
{
	CBudaApp* pApp;
	pApp = (CBudaApp*)AfxGetApp();

	pApp->AutoDetectThreshold = AutoDetectThreshold;
//	pApp->ThresholdMethod = ThresholdMethod;
	pApp->MinimumAutoThreshold = m_MinimumThreshold;

	pApp->WriteIniFile();
}
