// MITableView.cpp : implementation file
//
/****************************************************************

Boston University data analysis (BUDA) package 

Description: Buda is a data analysis program for Fourier 
Transform Mass Spectrometry data.  

by Peter B. O'Connor

    Copyright (C) 2000  Peter B. O'Connor

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include <afxwin.h>
#include "BudaMaster.h"
#include "Buda.h"
#include "BudaDoc.h"
#include "MITableView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CMITableView

IMPLEMENT_DYNCREATE(CMITableView, CEditView)

CMITableView::CMITableView()
{
}

CMITableView::~CMITableView()
{
}


BEGIN_MESSAGE_MAP(CMITableView, CEditView)
	//{{AFX_MSG_MAP(CMITableView)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CMITableView drawing

void CMITableView::OnDraw(CDC* pDC)
{
	CDocument* pDoc = GetDocument();
	SetWindowText(m_ScreenBuffer);
	pDC=pDC; // this line doesn't do anything.  It just prevents a compile warning
	pDoc = pDoc;
}

/////////////////////////////////////////////////////////////////////////////
// CMITableView diagnostics

#ifdef _DEBUG
void CMITableView::AssertValid() const
{
	CEditView::AssertValid();
}

void CMITableView::Dump(CDumpContext& dc) const
{
	CEditView::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CMITableView message handlers
/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////
// MITableView message handlers

BOOL CMITableView::Create(LPCTSTR lpszClassName, LPCTSTR lpszWindowName, DWORD dwStyle, const RECT& rect, CWnd* pParentWnd, UINT nID, CCreateContext* pContext) 
{
	dwStyle |= (WS_VSCROLL | ES_MULTILINE | ES_READONLY | ES_AUTOHSCROLL | ES_AUTOVSCROLL);
	return CWnd::Create(lpszClassName, lpszWindowName, dwStyle, rect, pParentWnd, nID, pContext);
}

void CMITableView::OnInitialUpdate() 
{

	pDoc = (CBudaDoc *) GetDocument();

	CEditView::OnInitialUpdate();

	// if MITable not calc'd, return
	if (!pDoc->PS.Spec[pDoc->CurrentSpectrumFocus].IsMIed) {
		//pDoc->PeakPickFlag = TRUE;
		//pDoc->PeakPick();
		return;
	}

	//FITable = pDoc->PS.Spec[pDoc->CurrentSpectrumFocus].FITable;
	pDoc->SortMITable();
	SetMIFont();
}



void CMITableView::CreateMITableStringBuffer()
{
	CString str;
	CString crlf="\r\n";
	CBudaDoc* pDoc;
	pDoc=(CBudaDoc*) GetDocument();
	int i;
	float mass,freq,amp,rel,res,mass2;
	int cs;
	CString massstr,freqstr,ampstr,relstr,resstr,csstr,mass2str;

	// first get the scale factor to set the highest peak = 100.00%
	double scale;
	scale = 0.0;
	for(i=0; i<pDoc->PS.Spec[pDoc->CurrentSpectrumFocus].NumPeaks;i++) {
		// find highest peak
		if(scale<pDoc->FITable[i].I) scale = pDoc->FITable[i].I;
	}

	// then create string
	str="Mass/Intensity Table for ";
	str+=pDoc->PS.DataFileName;
	str+=crlf;
	m_ScreenBuffer+=str;

	str="Memo: ";
	str+=pDoc->PS.ExperimentMemo;
	str+=crlf;
	m_ScreenBuffer+=str;

	str.Format("Spectrum %d of %d                     Accumulated Scans: %d ",1+pDoc->CurrentSpectrumFocus, pDoc->PS.SpectraAcquired,pDoc->PS.ScansPerSpectrum);
	//str=     "-----------    -----------    ----------     ----------    ----------";
	str+=crlf;
	m_ScreenBuffer+=str;

	str.Format("Threshold = %1.2f = %1.2f%%         ",pDoc->PS.Spec[pDoc->CurrentSpectrumFocus].NoiseLevel, pDoc->PS.Spec[pDoc->CurrentSpectrumFocus].NoiseLevel*100.0/scale);
	//str=     "-----------    -----------    ----------     ----------    ----------";
	str+=pDoc->PS.TimeAcquired;
	str+=crlf;
	m_ScreenBuffer+=str;

	str.Format("Corrected Total Ion Intensity = %1.3f",pDoc->PS.Spec[pDoc->CurrentSpectrumFocus].TotalIonInt);
	str += crlf;
	m_ScreenBuffer += str;

	str.Format("Number of peaks found = %d",pDoc->PS.Spec[pDoc->CurrentSpectrumFocus].NumPeaks);
	str += crlf;
	m_ScreenBuffer += str;

	m_ScreenBuffer+=crlf;

	str="   M/z        Frequency     Abs.Int.   Rel.Int.    Res.";
	if(pDoc->sys->AutoLabelCharge) {
		str+="     Charge     Mass";
	}
	str+=crlf;
	m_ScreenBuffer+=str;

	str="---------------------------------------------------------";
	if(pDoc->sys->AutoLabelCharge) {
		str+="--------------------";
	}
	str+=crlf;
	m_ScreenBuffer+=str;

	//if(!pDoc->IsPeakPicked) {
	//	pDoc->EstimateNoiseLevel(pDoc->FreqArray,&(pDoc->NoiseLevel));
	//	pDoc->PeakPick();
	//}

	for(i=0;i<pDoc->PS.Spec[pDoc->CurrentSpectrumFocus].NumPeaks;i++) {
		freq=(float)pDoc->PS.Spec[pDoc->CurrentSpectrumFocus].FITable[i].F;
		mass=(float)pDoc->FreqToMass(freq, pDoc->PS.A, pDoc->PS.B);
		amp=(float)pDoc->PS.Spec[pDoc->CurrentSpectrumFocus].FITable[i].I;
		rel=(float)(double(amp)*100.0/scale);
		res=(float)pDoc->PS.Spec[pDoc->CurrentSpectrumFocus].FITable[i].RP;
		cs = (int)pDoc->PS.Spec[pDoc->CurrentSpectrumFocus].FITable[i].CS;
		// mass not mz assume protons as charge carriers
		mass2 = (mass*cs) - float(1.00728*cs);

		massstr.Format("%5.5f",mass);
		freqstr.Format("%6.3f",freq);
		ampstr.Format("%3.3f",amp);
		relstr.Format("%3.2f",rel);
		resstr.Format("%5.0f",res);
		csstr.Format("%d",cs);
		if(pDoc->TrapVoltsOnDetect>=0) csstr="+"+csstr;
		mass2str.Format("%5.3f",mass2);

		while(massstr.GetLength()<10) massstr = " "+massstr;
		while(freqstr.GetLength()<10) freqstr = " "+freqstr;
		while(ampstr.GetLength()<8) ampstr = " "+ampstr;
		while(relstr.GetLength()<6) relstr = " "+relstr;
		if (res>0) {
			while(resstr.GetLength()<7) resstr = " "+resstr;
		} else {
			resstr = "  ---  ";
		}

		str=massstr+"   "+freqstr+"   "+ampstr+"    "+relstr+"    "+resstr;
		if(pDoc->sys->AutoLabelCharge) {
			str+="       "+csstr+"      "+mass2str;
		}
		str+=crlf;

		m_ScreenBuffer+=str;
		if(m_ScreenBuffer.GetLength() >= 65535) {
			MessageBox("This data has reached the window's text limit\nCannot display all peaks in Window\nHowever, you can save the peaks to a text\nfile using File|Export","Warning, Too many peaks");
			break;
		}
	}
}

void CMITableView::SetMIFont()
{
   CFont* pFont = GetFont();
   LOGFONT lf;
   if (pFont != NULL)
	   pFont->GetObject(sizeof(LOGFONT), &lf);
   else
	   ::GetObject(GetStockObject(SYSTEM_FONT), sizeof(LOGFONT), &lf);

	lf.lfPitchAndFamily = FIXED_PITCH | FF_MODERN;
	lf.lfWidth = 5;
	lf.lfHeight = 12;
	lf.lfWeight = 10;
	//char fnt[32]="Courier\0";
	//lf.lfFaceName = "Courier                       \0";
		// switch to new font.
	m_font.DeleteObject();
	if (m_font.CreateFontIndirect(&lf))
	{
		CWaitCursor wait;
		SetFont(&m_font);
		m_lfDefFont = lf;
	}
}

void CMITableView::OnUpdate(CView* pSender, LPARAM lHint, CObject* pHint) 
{
	if(!pDoc->IsFFTed()) return;
	pDoc->SortMITable();

	m_ScreenBuffer = "";
	CreateMITableStringBuffer();
	SetWindowText(m_ScreenBuffer);
	CView::OnUpdate(pSender, lHint, pHint);
}
