// MainFrm.cpp : implementation of the CMainFrame class
//
/****************************************************************

Boston University data analysis (BUDA) package 

Description: Buda is a data analysis program for Fourier 
Transform Mass Spectrometry data.  

by Peter B. O'Connor

    Copyright (C) 2000  Peter B. O'Connor

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "StatusControl.h"
#include "StatusProgress.h"

#include "BudaMaster.h"
#include "Buda.h"
#include "MainFrm.h"
#include "Instrument.h"
#include "BudaDoc.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CMainFrame

IMPLEMENT_DYNAMIC(CMainFrame, CMDIFrameWnd)

BEGIN_MESSAGE_MAP(CMainFrame, CMDIFrameWnd)
	//{{AFX_MSG_MAP(CMainFrame)
	//ON_MESSAGE(WM_NEWDATAREADY, OnExperimentDone)
	ON_WM_CREATE()
	ON_MESSAGE(WM_FFTPROGRESS, OnFFTProgress)
	ON_WM_PAINT()
	ON_WM_ERASEBKGND()
	ON_WM_MOUSEMOVE()
	//ON_UPDATE_COMMAND_UI(ID_USERID_PANE, OnUpdateUserIDPane)
	ON_UPDATE_COMMAND_UI(IDS_FFTSTATUS, OnUpdateFFTStatusPane)
	ON_UPDATE_COMMAND_UI(IDS_FFTSINQUEUE, OnUpdateFFTQueuePane)
	//ON_UPDATE_COMMAND_UI(IDS_INSTRUMENTSTATUS, OnUpdateSysStatusPane)
	//ON_COMMAND(IDA_TEST_PROGRESS_BAR, OnTestProgressBar)
	//}}AFX_MSG_MAP

END_MESSAGE_MAP()

static UINT indicators[] =
{
	ID_SEPARATOR,           // status line indicator

	//ID_INDICATOR_PROGRESS,
	//IDS_INSTRUMENTSTATUS,
	IDS_FFTSTATUS,
	IDS_FFTSINQUEUE,
	//ID_USERID_PANE,
	//ID_INDICATOR_CAPS,
	//ID_INDICATOR_NUM,
	//ID_INDICATOR_SCRL,
};

/////////////////////////////////////////////////////////////////////////////
// CMainFrame construction/destruction

CMainFrame::CMainFrame()
{
	pApp = (CBudaApp*)AfxGetApp();
	pFFT = pApp->pFFT;
	sys = &(pApp->Instrument.sys);
}

CMainFrame::~CMainFrame()
{
}

int CMainFrame::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
	if (CMDIFrameWnd::OnCreate(lpCreateStruct) == -1)
		return -1;
	
	if (!m_wndToolBar.Create(this) ||
		!m_wndToolBar.LoadToolBar(IDR_MAINFRAME))
	{
		TRACE0("Failed to create toolbar\n");
		return -1;      // fail to create
	}

	if (!m_wndStatusBar.Create(this) ||
		!m_wndStatusBar.SetIndicators(indicators,
		  sizeof(indicators)/sizeof(UINT)))
	{
		TRACE0("Failed to create status bar\n");
		return -1;      // fail to create
	}

	// TODO: Remove this if you don't want tool tips or a resizeable toolbar
	m_wndToolBar.SetBarStyle(m_wndToolBar.GetBarStyle() |
		CBRS_TOOLTIPS | CBRS_FLYBY | CBRS_SIZE_DYNAMIC);

	// TODO: Delete these three lines if you don't want the toolbar to
	//  be dockable
	m_wndToolBar.EnableDocking(CBRS_ALIGN_ANY);
	EnableDocking(CBRS_ALIGN_ANY);
	DockControlBar(&m_wndToolBar);

	// set up toggle zoom y button and toggle peak match button
	/*int index;
	index = m_wndToolBar.CommandToIndex(ID_ZOOM_Y);
	m_wndToolBar.SetButtonStyle(index,TBBS_CHECKBOX);
	index = m_wndToolBar.CommandToIndex(ID_PEAK_MATCH_TBTN);
	m_wndToolBar.SetButtonStyle(index,TBBS_CHECKBOX);
	*/

	m_Menu = GetMenu();
	return 0;
}

BOOL CMainFrame::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	// modify class name so that it is always the same
	// this way, the singleinstance stuff can identify the 
	// instances of this app
	cs.lpszClass = ((CBudaApp*)AfxGetApp())->GetClassName();

	return CMDIFrameWnd::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CMainFrame diagnostics

#ifdef _DEBUG
void CMainFrame::AssertValid() const
{
	CMDIFrameWnd::AssertValid();
}

void CMainFrame::Dump(CDumpContext& dc) const
{
	CMDIFrameWnd::Dump(dc);
}

#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CMainFrame message handlers

/*
void CMainFrame::OnPaint() 
{
	CPaintDC dc(this); // device context for painting
	// Do not call CMDIFrameWnd::OnPaint() for painting messages

	CBitmap bmp;
	CPoint pt;
	CSize sz;

	bmp.LoadBitmap(IDB_BLUE_BUDA);
	pt=CPoint(0,0);
	sz=CSize(352,224);
	dc.DrawState(pt,sz,&bmp,DST_BITMAP | DSS_NORMAL);
}

BOOL CMainFrame::OnEraseBkgnd(CDC* pDC) 
{
	CBitmap bmp;
	CPoint pt;
	CSize sz;

	bmp.LoadBitmap(IDB_BLUE_BUDA);
	pt=CPoint(0,0);
	sz=CSize(352,224);
	pDC->DrawState(pt,sz,&bmp,DST_BITMAP | DSS_NORMAL);
	
	return CMDIFrameWnd::OnEraseBkgnd(pDC);
}
*/

CStatusBar* CMainFrame::GetStatusBar()
{
	return &m_wndStatusBar;
}

void CMainFrame::OnUpdateUserIDPane(CCmdUI *pCmdUI)
{
	pCmdUI = pCmdUI;
//    pCmdUI->Enable(); 
//	CString str;

//    str.Format( "%8s", sys[0].UserID ); 
//   pCmdUI->SetText(str);
}

void CMainFrame::OnUpdateFFTStatusPane(CCmdUI *pCmdUI)
{
    pCmdUI->Enable(); 
	CString str,str1;

	// this pane should say:
	// if FFTStatus = 0 : "FFT: Idle"
	// if FFTStatus = 1 : "FFT: Setup"
	// if FFTStatus = 2 : "FFT: FFT"
	// if FFTStatus = 3 : "FFT: Magn."
	// if FFTStatus = 4 : "FFT: MI"

	pFFT->Lock();
	switch (pFFT->FFTThrdInfo.ThreadStatus) {
	case 0:
		str = "FFT: Idle";
		break;
	case 1:
		str = "FFT: Setup";
		break;
	case 2:
		str = "FFT: FFT";
		break;
	case 3:
		str = "FFT: Magn";
		break;
	case 4:
		str = "FFT: MI";
		break;
	default:
		str = "FFT: Unkn.";
		break;
	}
	pFFT->Unlock();
		
    str1.Format( "%10s", str ); 
    pCmdUI->SetText( str1 );
}

void CMainFrame::OnUpdateFFTQueuePane(CCmdUI *pCmdUI)
{
    pCmdUI->Enable(); 
	CString str,str1;

	// this pane should say:
	//  "FFTs : ",number

	pFFT->Lock();
    str1.Format( "FFT's : %d", pFFT->FFTThrdInfo.FFTsInQueue ); 
    pFFT->Unlock();
	
	str.Format( "%8s", str1 ); 
    pCmdUI->SetText( str );
}

void CMainFrame::OnFFTProgress()
{
	int i;
	i=0;
}

/*void CMainFrame::OnUpdateSysStatusPane(CCmdUI *pCmdUI)
{
    /*pCmdUI->Enable(); 
	CString str,str1;
	CInstrument* pInst = &(pApp->Instrument);

	// this pane should say:
	// if InstrumentStatus = 0 : "Sys: Idle"
	// if InstrumentStatus = 1 : "Sys: Kill"
	// if InstrumentStatus = 2 : "Sys: Run"
	// if InstrumentStatus = 3 : "Sys: Diag."
	// if InstrumentStatus = 4 : "Sys: Reset"

	pInst->Lock();
	switch (pInst->ThreadInfo.ThreadStatus) {
	case 0:
		str = "Sys: Idle";
		break;
	case 1:
		str = "Sys: Kill";
		break;
	case 2:
		str = "Sys: Run";
		break;
	case 3:
		str = "Sys: Diag.";
		break;
	case 4:
		str = "Sys: Reset";
		break;
	default:
		str = "Sys: n/a";
		break;
	}
	pInst->Unlock();
		
    str1.Format( "%10s", str ); 
    pCmdUI->SetText( str1 );
}

void CMainFrame::OnStartStatus()
{
     /*c_StatusProgress.Create(&m_wndStatusBar, ID_INDICATOR_PROGRESS, WS_VISIBLE | PBS_SMOOTH);
     c_StatusProgress.SetRange(0, 100);
     c_StatusProgress.SetPos(0);
	 setting = 0;
}

void CMainFrame::OnFinishStatus()
{
	// c_StatusProgress.DestroyWindow();
}

void CMainFrame::OnUpdateStatus(int progress)
{
	 //c_StatusProgress.SetPos(progress);
}

void CMainFrame::OnTestProgressBar() 
{
	/*setting += 5;
	if(setting >= 100) setting = 1;
	OnUpdateStatus(setting);
}


void CMainFrame::OnExperimentDone()
{
	//OnUpdateStatus(0);
	((CBudaApp*)AfxGetApp())->OnExperimentDone();
}
*/