// PSSetupDlg.cpp : implementation file
//
/****************************************************************

Boston University data analysis (BUDA) package 

Description: Buda is a data analysis program for Fourier 
Transform Mass Spectrometry data.  

by Peter B. O'Connor

    Copyright (C) 2000  Peter B. O'Connor

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "stdafx.h"
#include "Buda.h"
#include "BudaMaster.h"
#include "PSSetupDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CPSSetupDlg dialog


CPSSetupDlg::CPSSetupDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CPSSetupDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CPSSetupDlg)
	Duration = 0.0;
	FinalValue = 0.0;
	StartTime = 0.0;
	//}}AFX_DATA_INIT
}


void CPSSetupDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CPSSetupDlg)
	DDX_Control(pDX, IDC_EVENTTYPECOMBO, EventTypeCMBO);
	DDX_Control(pDX, IDC_EVENTLIST, EventList);
	DDX_Control(pDX, IDC_DACPULSELIST, DacPulseList);
	DDX_Text(pDX, IDC_DURATION, Duration);
	DDX_Text(pDX, IDC_FINALVALUE, FinalValue);
	DDX_Text(pDX, IDC_STARTTIME, StartTime);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CPSSetupDlg, CDialog)
	//{{AFX_MSG_MAP(CPSSetupDlg)
	ON_BN_CLICKED(IDC_CLEAREVENTS, OnClearEvents)
	ON_BN_CLICKED(IDC_DELETEEVENT, OnDeleteEvent)
	ON_BN_CLICKED(IDC_NEWEVENT, OnNewEvent)
	ON_LBN_SELCHANGE(IDC_EVENTLIST, OnSelchangeEventlist)
	ON_LBN_SELCHANGE(IDC_DACPULSELIST, OnSelchangeDacpulselist)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CPSSetupDlg message handlers

void CPSSetupDlg::OnClearEvents() 
{
	int i;
	//	set CurrentEvent = NumChannelEvents
	//  call OnDeleteEvent 
	// repeat
	for(i=NumChannelEvents; i>0; i--) {
		CurrentEvent = i;
		OnDeleteEvent();
	}
}

void CPSSetupDlg::OnDeleteEvent() 
{
	// set PS->GenPS[currentevent].type = NO_EVENT
	// set PS->GenPS[currentevent].channel = -1 
	// SortPS->GenPS()
	// decrement PS.TotalEvents
	// decrement NumChannelEvents

	// Call InitEventList()
	// set currentEvent = 0;
	// Call InitEventInfo()	
}

void CPSSetupDlg::OnNewEvent() 
{
	// increment PS.TotalEvents
	// increment NumChannelEvents
	// create a NO-EVENT type event with the CurrentDP
	// append it to PS->GenPS
	// Call InitEventList()
	// set currentEvent = this event;
	// Call InitEventInfo()
}

void CPSSetupDlg::OnSelchangeEventlist() 
{
	CurrentEvent = EventIndices[EventList.GetCurSel()];
	InitEventInfo();
}

void CPSSetupDlg::OnSelchangeDacpulselist() 
{
	CurrentDP = ChannelIndex[DacPulseList.GetCurSel()];
	CurrentEvent=0;
	InitEventList();
}

BOOL CPSSetupDlg::OnInitDialog()
{
	CDialog::OnInitDialog();
	int i;
	CString str;

	// force the DoDataExchange to initialize things
	UpdateData(FALSE);
	SetListBoxFont();

	// set up the combobox
	EventTypeCMBO.InitStorage(7,10);
	EventTypeCMBO.AddString("No Event");
	EventTypeCMBO.AddString("TTL Pulse");
	EventTypeCMBO.AddString("DAC Step");
	EventTypeCMBO.AddString("DAC Ramp");
	EventTypeCMBO.AddString("DAC Pulse");
	EventTypeCMBO.SetCurSel(0);
	//EventTypeCMBO.AddString("DAC Quench");

	// clear and initialize peaklist
	DacPulseList.ResetContent();
	DacPulseList.InitStorage(40,30);

	// set up dacpulselist 
	//	1) search through PS->GenPS and add a string for each one there
	//	2) set the current dacpulse = 0, set the current position
	MaxChannelIndex = 0;
	for(i=0; i<PS->NumEvents; i++) {
		if(IsUsedChannel(PS->GenPS[i].Channel) && !ListedChannel(PS->GenPS[i].Channel)) {
			DacPulseList.AddString(pDoc->sys->ChannelNames[PS->GenPS[i].Channel]);
			ChannelIndex[MaxChannelIndex]=PS->GenPS[i].Channel;
			MaxChannelIndex++;
		}
	}
	// set the initial selection point
	DacPulseList.SetCurSel(0);
	CurrentDP=ChannelIndex[DacPulseList.GetCurSel()];
	InitEventList();
	return TRUE;
}

void CPSSetupDlg::InitEventList()
{
	EventList.ResetContent();
	EventList.InitStorage(MAX_PS_EVENTS,30);
	// set up eventlist
	//	1) search through PS->GenPS and for each event with dacpulse 
	//		= currentdacpulse, addstring 
	//	2) set currentevent = 0, set current position
	NumChannelEvents = 0;
	int i;
	CString str;
	for(i=0; i<PS->NumEvents; i++) {
		if(PS->GenPS[i].Channel == ChannelIndex[DacPulseList.GetCurSel()]) {
			switch (PS->GenPS[i].Type) {
			case TTL_PULSE:
				str.Format("%1.3f : TTL pulse",PS->GenPS[i].Delay);
				break;
			case DAC_INIT:
				str.Format("%1.3f : Initialize DAC",PS->GenPS[i].Delay);
				break;
			case DAC_STEP:
				str.Format("%1.3f : Step DAC to %1.2f V",PS->GenPS[i].Delay,PS->GenPS[i].Value);
				break;
			case DAC_RAMP:
				str.Format("%1.3f : Ramp DAC to %1.2f V",PS->GenPS[i].Delay,PS->GenPS[i].Value);
				break;
			case DAC_PULSE:
				str.Format("%1.3f : Pulse Dac to %1.2f V",PS->GenPS[i].Delay,PS->GenPS[i].Value);
				break;
			case DAC_QUENCH:
				str.Format("%1.3f : Quench",PS->GenPS[i].Delay);
				break;
			default:
				str.Format("No-Event");
				break;
			} // end switch
			EventList.AddString(str);
			EventIndices[NumChannelEvents]=i;
			NumChannelEvents++;
		} 
	}
	if(NumChannelEvents == 0) {
		// no events found for this channel - shouldn't be possible
		EventList.AddString("No Events Defined");
	}
	// set the initial selection point
	EventList.SetCurSel(0);
	CurrentEvent=0;
	InitEventInfo();
}

void CPSSetupDlg::InitEventInfo()
{
	// set up eventinfo
	//	eventtype = PS->GenPS[currentevent].Type, etc...
	EventType = PS->GenPS[CurrentEvent].Type;
	EventTypeCMBO.SetCurSel(EventType);
	FinalValue = PS->GenPS[CurrentEvent].Value;
	StartTime = PS->GenPS[CurrentEvent].Delay;
	Duration = PS->GenPS[CurrentEvent].Width;
	Linking = PS->GenPS[CurrentEvent].Linking;
	Link2Event = PS->GenPS[CurrentEvent].Link2Event;
	UpdateData(FALSE);  // force DoDataExchange
	//Description = PS->GenPS[CurrentEvent].Description;
}

void CPSSetupDlg::SetListBoxFont()
{
   CFont* pFont = GetFont();
   LOGFONT lf;
   if (pFont != NULL)
	   pFont->GetObject(sizeof(LOGFONT), &lf);
   else
	   ::GetObject(GetStockObject(SYSTEM_FONT), sizeof(LOGFONT), &lf);

	lf.lfPitchAndFamily = FIXED_PITCH | FF_MODERN;
	lf.lfWidth = 7;
	lf.lfHeight = 16;
	lf.lfWeight = 10;
	//char fnt[32]="Courier\0";
	//lf.lfFaceName = "Courier                       \0";
		// switch to new font.
	m_font.DeleteObject();
	if (m_font.CreateFontIndirect(&lf))
	{
		//CWaitCursor wait;
		DacPulseList.SetFont(&m_font);
		EventList.SetFont(&m_font);
	}
}

BOOL CPSSetupDlg::IsUsedChannel(int channel)
{
	// this function checks sys->channelnames[channel] to see
	// if it says "UNKNOWN" (case insensitive) 
	// if so it returns false, otherwise it returns true

	CString str;
	str = pDoc->sys->ChannelNames[channel];

	if(channel<0 || channel > 39) return FALSE;
	if(!str.CompareNoCase("UNUSED")) return FALSE;
	else return TRUE;
}

BOOL CPSSetupDlg::ListedChannel(int channel)
{
	// this function checks to see if channel is already listed
	// in channelindex and if so returns true
	// it also checks to see if channel is an unlisted channel
	// and if so also returns true
	int i;
	if(channel < USER1_PULSE) return TRUE;

	for(i=0; i<MaxChannelIndex; i++) {
		if(ChannelIndex[i] == channel) return TRUE;
	}
	return FALSE;
}
