// ParmView.cpp : implementation file
//
/*
ParmView.cpp is the code for the CParmView class.  This class is 
intended to create a window where people can view the current document's
fundamental parameters such as number of data points, DAC rate, 
reference frequencies, etc.  It does not show pulse sequence info.

*/
/****************************************************************

Boston University data analysis (BUDA) package 

Description: Buda is a data analysis program for Fourier 
Transform Mass Spectrometry data.  

by Peter B. O'Connor

    Copyright (C) 2000  Peter B. O'Connor

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/


#include "BudaMaster.h"
#include "ParmView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CParmView

IMPLEMENT_DYNCREATE(CParmView, CEditView)

CParmView::CParmView()
{
		pApp = (CBudaApp*)AfxGetApp();
		PDoc = (CBudaDoc*)GetDocument();

		PS = &(PDoc->PS);
}

CParmView::~CParmView()
{
}


BEGIN_MESSAGE_MAP(CParmView, CEditView)
	//{{AFX_MSG_MAP(CParmView)
		// NOTE - the ClassWizard will add and remove mapping macros here.
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CParmView drawing

void CParmView::OnDraw(CDC* pDC)
{
	CDocument* pDoc = GetDocument();
	SetWindowText(m_ScreenBuffer);
	pDC = pDC;
	pDoc = pDoc;
}

/////////////////////////////////////////////////////////////////////////////
// CParmView diagnostics

#ifdef _DEBUG
void CParmView::AssertValid() const
{
	CEditView::AssertValid();
}

void CParmView::Dump(CDumpContext& dc) const
{
	CEditView::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CParmView message handlers

void CParmView::OnInitialUpdate() 
{
	PDoc = (CBudaDoc *) GetDocument();

	CEditView::OnInitialUpdate();

	CreateParmViewBuffer();
	//SetMIFont();

	SetWindowText(m_ScreenBuffer);
	GetDocument()->UpdateAllViews(NULL);		
}

void CParmView::CreateParmViewBuffer()
{
	CString str;
	CString crlf = "\r\n";

	m_ScreenBuffer = "Current Data Set Parameters";
	m_ScreenBuffer = "-----------------------------------";

	str.Format("Data file version : %d.%d",PS->MajorVersion, PS->MinorVersion);
	m_ScreenBuffer += str+crlf;

	str.Format("User ID : %s",PS->UserID);
	m_ScreenBuffer += str+crlf;

	str.Format("Filename : %s",PS->DataFileName);
	m_ScreenBuffer += str+crlf;

	str.Format("TimeAcquired : %s",PS->TimeAcquired);
	m_ScreenBuffer += str+crlf;

	str.Format("TimeProcessed : %s",PS->TimeProcessed);
	m_ScreenBuffer += str+crlf;

	if(PS->HiResMALDI>0) {
		str.Format("HiResMALDI experiment");
		m_ScreenBuffer += str+crlf;
	}

	if(PS->HiResESI>0) {
		str.Format("HiResESI experiment");
		m_ScreenBuffer += str+crlf;
	}

	str.Format("Memo : %s",PS->ExperimentMemo);
	m_ScreenBuffer += str+crlf;

	str.Format("Notes: %s",PS->ExperimentNotes);
	m_ScreenBuffer += str+crlf;

	if(PS->DetectMode == MixerMode) str.Format("Detect Mode : Heterodyne");
	else str.Format("Detect Mode : Broadband");
	m_ScreenBuffer += str+crlf;

	str.Format("ADC Gain : %d",PS->ADCGain);
	m_ScreenBuffer += str+crlf;

	str.Format("Detection Low Mass Limit : %f",PS->DetectLowMass);
	m_ScreenBuffer += str+crlf;

	str.Format("Detection High Mass Limit : %f",PS->DetectHighMass);
	m_ScreenBuffer += str+crlf;

	str.Format("Narrow Band Monitor Mass : %f",PS->MonitorMass);
	m_ScreenBuffer += str+crlf;

	str.Format("Heterodyne Mixer Frequency : %f",PS->MixerFrequency);
	m_ScreenBuffer += str+crlf;

	str.Format("ADC Acquisition Rate : %f",PS->AdcRate);
	m_ScreenBuffer += str+crlf;

	str.Format("Number of Time Domain data points : %d",PS->TransientPoints);
	m_ScreenBuffer += str+crlf;

	str.Format("Spectra Requested : %d",PS->SpectraRequested);
	m_ScreenBuffer += str+crlf;

	str.Format("SpectraAcquired : %d",PS->SpectraAcquired);
	m_ScreenBuffer += str+crlf;

	str.Format("Scans per spectrum : %d",PS->ScansPerSpectrum);
	m_ScreenBuffer += str+crlf;

	str.Format("Scans Requested : %d",PS->ScansRequested);
	m_ScreenBuffer += str+crlf;

	str.Format("Calibration Version : %d",PS->CalibrationVersion);
	m_ScreenBuffer += str+crlf;

	str.Format("Calibration constant A : %d",PS->A);
	m_ScreenBuffer += str+crlf;

	str.Format("Calibration constant B : %d",PS->B);
	m_ScreenBuffer += str+crlf;

	str.Format("Calibration High Mass Frequency Shift : %d",PS->HighMassShift);
	m_ScreenBuffer += str+crlf;

	str.Format("Calibration Trapping voltage frequency shift : %d",PS->TrapShift);
	m_ScreenBuffer += str+crlf;

	str.Format("Calibration Total Ion Intensity Frequency shift : %d",PS->IntensityShift);
	m_ScreenBuffer += str+crlf;

	str.Format("Internal Calibration : %d",PS->PeakMatching);
	m_ScreenBuffer += str+crlf;

	str.Format("Spectrometer Type : %d",PS->SpectrometerType);
	m_ScreenBuffer += str+crlf;

	str.Format("DC offset removal : %d",PS->DCRemoval);
	m_ScreenBuffer += str+crlf;

	str.Format("Window Function : %d",PS->WindowFunction);
	m_ScreenBuffer += str+crlf;

	str.Format("Number of Zero Fills: %d",PS->ZeroFills);
	m_ScreenBuffer += str+crlf;

	str.Format("Truncated Transient : %d",PS->TruncateTransient);
	m_ScreenBuffer += str+crlf;

	str.Format("Truncated number of data points : %d",PS->TruncatePoints);
	m_ScreenBuffer += str+crlf;

	str.Format("Truncate Offset : %d",PS->TruncateOffset);
	m_ScreenBuffer += str+crlf;

	str.Format("FFT Size : %d",PS->FFTSize);
	m_ScreenBuffer += str+crlf;

	str.Format("Number of Frequency domain data points : %d",PS->SpectrumPoints);
	m_ScreenBuffer += str+crlf;

	str.Format("Frequency Data Type : %d",PS->FrequencyDataType); // complex or real (complex is used in zoom transforms)
	m_ScreenBuffer += str+crlf;

	str.Format("ZoomFactor : %d",PS->ZoomFactor);
	m_ScreenBuffer += str+crlf;

	str.Format("Zoom Low Mass : %d",PS->ZoomLowMass);
	m_ScreenBuffer += str+crlf;

	str.Format("Zoom High Mass: %d",PS->ZoomHighMass);
	m_ScreenBuffer += str+crlf;

	str.Format("ZoomFrequencyShift : %d",PS->ZoomFrequencyShift);
	m_ScreenBuffer += str+crlf;

	str.Format("ZoomTransientPoints : %d",PS->ZoomTransientPoints);
	m_ScreenBuffer += str+crlf;

	str.Format("ZoomPointShift : %d",PS->ZoomPointShift);
	m_ScreenBuffer += str+crlf;

	str.Format("Number of bits per Transient point: %d",PS->TransientIsStored);
	m_ScreenBuffer += str+crlf;

	str.Format("Number of bits per Frequency point: %d",PS->FrequencyIsStored);
	m_ScreenBuffer += str+crlf;

	str.Format("Is MI Table Stored? : %d",PS->MITableIsStored);
	m_ScreenBuffer += str+crlf;

	str.Format("Number of bits per Transient point: %d",PS->TransientIsStored);
	m_ScreenBuffer += str+crlf;
}

void CParmView::SetParmViewFont()
{
  CFont* pFont = GetFont();
   LOGFONT lf;
   if (pFont != NULL)
	   pFont->GetObject(sizeof(LOGFONT), &lf);
   else
	   ::GetObject(GetStockObject(SYSTEM_FONT), sizeof(LOGFONT), &lf);

	lf.lfPitchAndFamily = FIXED_PITCH | FF_MODERN;
	lf.lfWidth = 5;
	lf.lfHeight = 12;
	lf.lfWeight = 10;
	//char fnt[32]="Courier\0";
	//lf.lfFaceName = "Courier                       \0";
		// switch to new font.
	m_font.DeleteObject();
	if (m_font.CreateFontIndirect(&lf))
	{
		CWaitCursor wait;
		SetFont(&m_font);
		m_lfDefFont = lf;
	}
}
