// PeriodTable.cpp : implementation file
//
/****************************************************************

Boston University data analysis (BUDA) package 

Description: Buda is a data analysis program for Fourier 
Transform Mass Spectrometry data.  

by Peter B. O'Connor

    Copyright (C) 2000  Peter B. O'Connor

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "stdafx.h"
#include "BudaMaster.h"
#include "Buda.h"
#include "Instrument.h"
#include "PeriodicTable.h"
#include "ElemDlg.h"
#include <direct.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CPeriodicTable dialog


CPeriodicTable::CPeriodicTable(CWnd* pParent /*=NULL*/)
	: CDialog(CPeriodicTable::IDD, pParent)
{
	//{{AFX_DATA_INIT(CPeriodicTable)
	m_ElementalComposition = _T("");
	//}}AFX_DATA_INIT
	pApp = (CBudaApp*) AfxGetApp();
	Instrument=GetInstrument();
	sys=GetSys();
	ReadInPeriodicTable();
}

void CPeriodicTable::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CPeriodicTable)
	DDX_Text(pDX, IDC_ECOMPOSITION, m_ElementalComposition);
	//}}AFX_DATA_MAP		
}

BEGIN_MESSAGE_MAP(CPeriodicTable, CDialog)
	//{{AFX_MSG_MAP(CPeriodicTable)
	ON_BN_CLICKED(IDC_CARBON, OnCarbon)
	ON_BN_CLICKED(IDC_HYDROGEN, OnHydrogen)
	ON_BN_CLICKED(IDC_LITHIUM, OnLithium)
	ON_BN_CLICKED(IDC_ACTINIUM, OnActinium)
	ON_BN_CLICKED(IDC_ALUMINUM, OnAluminum)
	ON_BN_CLICKED(IDC_ANTIMONY, OnAntimony)
	ON_BN_CLICKED(IDC_ARGON, OnArgon)
	ON_BN_CLICKED(IDC_ARSENIC, OnArsenic)
	ON_BN_CLICKED(IDC_ASTATINE, OnAstatine)
	ON_BN_CLICKED(IDC_BARIUM, OnBarium)
	ON_BN_CLICKED(IDC_BERYLIUM, OnBerylium)
	ON_BN_CLICKED(IDC_BISMUTH, OnBismuth)
	ON_BN_CLICKED(IDC_BORON, OnBoron)
	ON_BN_CLICKED(IDC_BROMINE, OnBromine)
	ON_BN_CLICKED(IDC_CADMIUM, OnCadmium)
	ON_BN_CLICKED(IDC_CALCIUM, OnCalcium)
	ON_BN_CLICKED(IDC_CERIUM, OnCerium)
	ON_BN_CLICKED(IDC_CESIUM, OnCesium)
	ON_BN_CLICKED(IDC_CHLORINE, OnChlorine)
	ON_BN_CLICKED(IDC_CHROMIUM, OnChromium)
	ON_BN_CLICKED(IDC_COBALT, OnCobalt)
	ON_BN_CLICKED(IDC_COPPER, OnCopper)
	ON_BN_CLICKED(IDC_DYSPROSIUM, OnDysprosium)
	ON_BN_CLICKED(IDC_ERBIUM, OnErbium)
	ON_BN_CLICKED(IDC_EUROPIUM, OnEuropium)
	ON_BN_CLICKED(IDC_FLOURINE, OnFlourine)
	ON_BN_CLICKED(IDC_FRANCIUM, OnFrancium)
	ON_BN_CLICKED(IDC_GADOLINIUM, OnGadolinium)
	ON_BN_CLICKED(IDC_GERMANIUM, OnGermanium)
	ON_BN_CLICKED(IDC_GOLD, OnGold)
	ON_BN_CLICKED(IDC_HAFNIUM, OnHafnium)
	ON_BN_CLICKED(IDC_HELIUM, OnHelium)
	ON_BN_CLICKED(IDC_HOLMIUM, OnHolmium)
	ON_BN_CLICKED(IDC_INDIUM, OnIndium)
	ON_BN_CLICKED(IDC_IODINE, OnIodine)
	ON_BN_CLICKED(IDC_IRIDIUM, OnIridium)
	ON_BN_CLICKED(IDC_IRON, OnIron)
	ON_BN_CLICKED(IDC_KRYPTON, OnKrypton)
	ON_BN_CLICKED(IDC_LANTHANUM, OnLanthanum)
	ON_BN_CLICKED(IDC_LEAD, OnLead)
	ON_BN_CLICKED(IDC_LUTETIUM, OnLutetium)
	ON_BN_CLICKED(IDC_MAGNESIUM, OnMagnesium)
	ON_BN_CLICKED(IDC_MANGANESE, OnManganese)
	ON_BN_CLICKED(IDC_MERCURY, OnMercury)
	ON_BN_CLICKED(IDC_MOLYBDENUM, OnMolybdenum)
	ON_BN_CLICKED(IDC_NEODYMIUM, OnNeodymium)
	ON_BN_CLICKED(IDC_NEON, OnNeon)
	ON_BN_CLICKED(IDC_NICKEL, OnNickel)
	ON_BN_CLICKED(IDC_NIOBIUM, OnNiobium)
	ON_BN_CLICKED(IDC_NITROGEN, OnNitrogen)
	ON_BN_CLICKED(IDC_OSMIUM, OnOsmium)
	ON_BN_CLICKED(IDC_OXYGEN, OnOxygen)
	ON_BN_CLICKED(IDC_PALLADIUM, OnPalladium)
	ON_BN_CLICKED(IDC_PHOSPHORUS, OnPhosphorus)
	ON_BN_CLICKED(IDC_PLATINUM, OnPlatinum)
	ON_BN_CLICKED(IDC_POLONIUM, OnPolonium)
	ON_BN_CLICKED(IDC_POTASSIUM, OnPotassium)
	ON_BN_CLICKED(IDC_PRASEODYMIUM, OnPraseodymium)
	ON_BN_CLICKED(IDC_PROMETHIUM, OnPromethium)
	ON_BN_CLICKED(IDC_PROTACTINIUM, OnProtactinium)
	ON_BN_CLICKED(IDC_RADIUM, OnRadium)
	ON_BN_CLICKED(IDC_RADON, OnRadon)
	ON_BN_CLICKED(IDC_RHENIUM, OnRhenium)
	ON_BN_CLICKED(IDC_RHODIUM, OnRhodium)
	ON_BN_CLICKED(IDC_RUBIDIUM, OnRubidium)
	ON_BN_CLICKED(IDC_RUTHENIUM, OnRuthenium)
	ON_BN_CLICKED(IDC_SAMARIUM, OnSamarium)
	ON_BN_CLICKED(IDC_SCANDIUM, OnScandium)
	ON_BN_CLICKED(IDC_SELENIUM, OnSelenium)
	ON_BN_CLICKED(IDC_SILICON, OnSilicon)
	ON_BN_CLICKED(IDC_SILVER, OnSilver)
	ON_BN_CLICKED(IDC_SODIUM, OnSodium)
	ON_BN_CLICKED(IDC_STRONTIUM, OnStrontium)
	ON_BN_CLICKED(IDC_SULFUR, OnSulfur)
	ON_BN_CLICKED(IDC_TANTALUM, OnTantalum)
	ON_BN_CLICKED(IDC_TECHNETIUM, OnTechnetium)
	ON_BN_CLICKED(IDC_TELLURIUM, OnTellurium)
	ON_BN_CLICKED(IDC_TERBIUM, OnTerbium)
	ON_BN_CLICKED(IDC_THALLIUM, OnThallium)
	ON_BN_CLICKED(IDC_THORIUM, OnThorium)
	ON_BN_CLICKED(IDC_THULIUM, OnThulium)
	ON_BN_CLICKED(IDC_TIN, OnTin)
	ON_BN_CLICKED(IDC_TITANIUM, OnTitanium)
	ON_BN_CLICKED(IDC_TUNGSTEN, OnTungsten)
	ON_BN_CLICKED(IDC_URANIUM, OnUranium)
	ON_BN_CLICKED(IDC_VANADIUM, OnVanadium)
	ON_BN_CLICKED(IDC_XENON, OnXenon)
	ON_BN_CLICKED(IDC_YTTERBIUM, OnYtterbium)
	ON_BN_CLICKED(IDC_YTTRIUM, OnYttrium)
	ON_BN_CLICKED(IDC_ZINC, OnZinc)
	ON_BN_CLICKED(IDC_ZIRCONIUM, OnZirconium)
	ON_BN_CLICKED(IDC_GALLIUM, OnGallium)
	ON_BN_CLICKED(IDC_CALIFORNIUM, OnCalifornium)
	ON_BN_CLICKED(IDC_AMERICIUM, OnAmericium)
	ON_BN_CLICKED(IDC_BERKELIUM, OnBerkelium)
	ON_BN_CLICKED(IDC_EINSTEINIUM, OnEinsteinium)
	ON_BN_CLICKED(IDC_CURIUM, OnCurium)
	ON_BN_CLICKED(IDC_FERMIUM, OnFermium)
	ON_BN_CLICKED(IDC_LAWRENCIUM, OnLawrencium)
	ON_BN_CLICKED(IDC_MENDELEVIUM, OnMendelevium)
	ON_BN_CLICKED(IDC_NEPTUNIUM, OnNeptunium)
	ON_BN_CLICKED(IDC_NOBELIUM, OnNobelium)
	ON_BN_CLICKED(IDC_PLUTONIUM, OnPlutonium)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CPeriodicTable message handlers

void CPeriodicTable::OnCarbon() 
{
	OnElement("Carbon");	
}

void CPeriodicTable::OnHydrogen() 
{
	OnElement("Hydrogen");	
}

void CPeriodicTable::OnLithium()
{
	OnElement("Lithium");	
}

void CPeriodicTable::OnHelium()
{
	OnElement("Helium");
}

void CPeriodicTable::OnBerylium()
{
	OnElement("Berylium");	
}

void CPeriodicTable::OnBoron()
{
	OnElement("Boron");	
}

void CPeriodicTable::OnNitrogen()
{
	OnElement("Nitrogen");	
}

void CPeriodicTable::OnOxygen()
{
	OnElement("Oxygen");	
}

void CPeriodicTable::OnFlourine()
{
	OnElement("Flourine");	
}

void CPeriodicTable::OnNeon()
{
	OnElement("Neon");	
}

void CPeriodicTable::OnSodium()
{
	OnElement("Sodium");	
}

void CPeriodicTable::OnMagnesium()
{
	OnElement("Magnesium");	
}

void CPeriodicTable::OnAluminum()
{
	OnElement("Aluminum");	
}

void CPeriodicTable::OnSilicon()
{
	OnElement("Silicon");	
}

void CPeriodicTable::OnPhosphorus()
{
	OnElement("Phosphorus");	
}

void CPeriodicTable::OnSulfur()
{
	OnElement("Sulfur");	
}

void CPeriodicTable::OnChlorine()
{
	OnElement("Chlorine");	
}

void CPeriodicTable::OnArgon()
{
	OnElement("Argon");	
}

void CPeriodicTable::OnPotassium()
{
	OnElement("Potassium");	
}

void CPeriodicTable::OnCalcium()
{
	OnElement("Calcium");	
}

void CPeriodicTable::OnScandium()
{
	OnElement("Scandium");	
}

void CPeriodicTable::OnTitanium()
{
	OnElement("Titanium");	
}

void CPeriodicTable::OnVanadium()
{
	OnElement("Vanadium");	
}

void CPeriodicTable::OnChromium()
{
	OnElement("Chromium");	
}

void CPeriodicTable::OnManganese()
{
	OnElement("Manganese");	
}

void CPeriodicTable::OnIron()
{
	OnElement("Iron");	
}

void CPeriodicTable::OnCobalt()
{
	OnElement("Cobalt");	
}

void CPeriodicTable::OnNickel()
{
	OnElement("Nickel");	
}

void CPeriodicTable::OnCopper()
{
	OnElement("Copper");	
}

void CPeriodicTable::OnZinc()
{
	OnElement("Zinc");	
}

void CPeriodicTable::OnGermanium()
{
	OnElement("Germanium");	
}

void CPeriodicTable::OnArsenic()
{
	OnElement("Arsenic");	
}

void CPeriodicTable::OnSelenium()
{
	OnElement("Selenium");	
}

void CPeriodicTable::OnBromine()
{
	OnElement("Bromine");	
}

void CPeriodicTable::OnKrypton()
{
	OnElement("Krypton");	
}

void CPeriodicTable::OnRubidium()
{
	OnElement("Rubidium");	
}

void CPeriodicTable::OnStrontium()
{
	OnElement("Strontium");	
}

void CPeriodicTable::OnYttrium()
{
	OnElement("Yttrium");	
}

void CPeriodicTable::OnZirconium()
{
	OnElement("Zirconium");	
}

void CPeriodicTable::OnNiobium()
{
	OnElement("Niobium");	
}

void CPeriodicTable::OnMolybdenum()
{
	OnElement("Molybdenum");	
}

void CPeriodicTable::OnTechnetium()
{
	OnElement("Technetium");	
}

void CPeriodicTable::OnRuthenium()
{
		OnElement("Ruthenium");	
}

void CPeriodicTable::OnRhodium()
{
	OnElement("Rhodium");	
}

void CPeriodicTable::OnPalladium()
{
	OnElement("Palladium");	
}

void CPeriodicTable::OnSilver()
{
	OnElement("Silver");	
}

void CPeriodicTable::OnCadmium()
{
	OnElement("Cadmium");	
}

void CPeriodicTable::OnIndium()
{
	OnElement("Indium");	
}

void CPeriodicTable::OnTin()
{
	OnElement("Tin");	
}

void CPeriodicTable::OnAntimony()
{
	OnElement("Antimony");	
}

void CPeriodicTable::OnTellurium()
{
	OnElement("Tellurium");	
}

void CPeriodicTable::OnIodine()
{
	OnElement("Iodine");	
}

void CPeriodicTable::OnXenon()
{
	OnElement("Xenon");	
}

void CPeriodicTable::OnCesium()
{
	OnElement("Cesium");	
}

void CPeriodicTable::OnBarium()
{
	OnElement("Barium");	
}

void CPeriodicTable::OnLanthanum()
{
	OnElement("Lanthanum");	
}

void CPeriodicTable::OnCerium()
{
	OnElement("Cerium");	
}

void CPeriodicTable::OnPraseodymium()
{
	OnElement("Praseodymium");	
}

void CPeriodicTable::OnNeodymium()
{
	OnElement("Neodymium");	
}

void CPeriodicTable::OnPromethium()
{
	OnElement("Promethium");	
}

void CPeriodicTable::OnSamarium()
{
	OnElement("Samarium");	
}

void CPeriodicTable::OnEuropium()
{
	OnElement("Europium");	
}

void CPeriodicTable::OnGadolinium()
{
	OnElement("Gadolinium");	
}

void CPeriodicTable::OnTerbium()
{
	OnElement("Terbium");	
}

void CPeriodicTable::OnDysprosium()
{
	OnElement("Dysprosium");	
}

void CPeriodicTable::OnHolmium()
{
	OnElement("Holmium");	
}

void CPeriodicTable::OnErbium()
{
	OnElement("Erbium");	
}

void CPeriodicTable::OnThulium()
{
	OnElement("Thulium");	
}

void CPeriodicTable::OnYtterbium()
{
	OnElement("Ytterbium");	
}

void CPeriodicTable::OnLutetium()
{
	OnElement("Lutetium");	
}

void CPeriodicTable::OnHafnium()
{
	OnElement("Hafnium");	
}

void CPeriodicTable::OnTantalum()
{
	OnElement("Tantalum");	
}

void CPeriodicTable::OnTungsten()
{
	OnElement("Tungsten");	
}

void CPeriodicTable::OnRhenium()
{
	OnElement("Rhenium");	
}

void CPeriodicTable::OnOsmium()
{
	OnElement("Osmium");	
}

void CPeriodicTable::OnIridium()
{
	OnElement("Iridium");	
}

void CPeriodicTable::OnPlatinum()
{
	OnElement("Platinum");	
}

void CPeriodicTable::OnGold()
{
	OnElement("Gold");	
}

void CPeriodicTable::OnMercury()
{
	OnElement("Mercury");	
}

void CPeriodicTable::OnThallium()
{
	OnElement("Thallium");	
}

void CPeriodicTable::OnLead()
{
	OnElement("Lead");	
}

void CPeriodicTable::OnBismuth()
{
	OnElement("Bismuth");	
}

void CPeriodicTable::OnPolonium()
{
	OnElement("Polonium");	
}

void CPeriodicTable::OnAstatine()
{
	OnElement("Astatine");	
}

void CPeriodicTable::OnRadon()
{
	OnElement("Radon");	
}

void CPeriodicTable::OnFrancium()
{
	OnElement("Francium");	
}

void CPeriodicTable::OnRadium()
{
	OnElement("Radium");	
}

void CPeriodicTable::OnActinium()
{
	OnElement("Actinium");	
}

void CPeriodicTable::OnThorium()
{
	OnElement("Thorium");	
}

void CPeriodicTable::OnProtactinium()
{
	OnElement("Protactinium");	
}

void CPeriodicTable::OnUranium()
{
	OnElement("Uranium");	
}

void CPeriodicTable::OnGallium()
{
	OnElement("Gallium");
}

void CPeriodicTable::OnCalifornium()
{
	OnElement("Californium");
}

void CPeriodicTable::OnAmericium()
{
	OnElement("Americium");
}

void CPeriodicTable::OnBerkelium()
{
	OnElement("Berkelium");
}

void CPeriodicTable::OnEinsteinium()
{
	OnElement("Einsteinium");
}

void CPeriodicTable::OnCurium()
{
	OnElement("Curium");
}

void CPeriodicTable::OnFermium()
{
	OnElement("Fermium");
}

void CPeriodicTable::OnLawrencium()
{
	OnElement("Lawrencium");
}

void CPeriodicTable::OnMendelevium()
{
	OnElement("Mendelevium");
}

void CPeriodicTable::OnNeptunium()
{
	OnElement("Neptunium");
}

void CPeriodicTable::OnNobelium()
{
	OnElement("Nobelium");
}

void CPeriodicTable::OnPlutonium()
{
	OnElement("Plutonium");
}

int CPeriodicTable::CountNumTabs(CString str)
{
	int tabcount=0, i;
	char c;
	for (i=0;i<str.GetLength();i++)
	{
		c=str.GetAt(i);
		if (c=='\t')
			tabcount++;
	}
	return tabcount;
}

struct sys* CPeriodicTable::GetSys()
{
	return Instrument->GetSys();
}

void CPeriodicTable::ReadInPeriodicTable()
{
	CFile file;
	CString str, rightstr;

	if(_chdir(pApp->ProgramDirectory)==-1) 
	{
		str.Format("Error 22c\nCouldn't change to Buda directory\n%s", pApp->ProgramDirectory);
		MessageBox(str, "Directory Error!");
		return;
	}

	CString filename = "Elements.txt";
	if(!file.Open(filename,CFile::modeRead, NULL))
	{
		str.Format("Error 21\nCouldn't open document file:\n%s",filename);
		MessageBox(str,"File Error!");
		return;
	}
	CArchive ar(&file, CArchive::load, 4096, NULL);

	int numtabs = 0;
	int elemcount=-1;
	int isocount=0;
	int tabpos=0;
	int i;
	CString Rightstr,Leftstr,Midstr;
	struct Isotope  TempIsos[10];
	while(ar.ReadString(str))  // while not end of file, read a line
	{
		if (str.GetLength()==0) continue;  // ignore empty lines
		if (str.Left(1)=="#") continue;		// ignore comment lines starting with #

		numtabs = CountNumTabs(str);

		switch (numtabs) 
		{
		case 0:
			if(elemcount>=0) 
			{
				PeriodTable[elemcount].NumIsos=isocount;
				PeriodTable[elemcount].Isos=new struct Isotope[isocount];
				for (i=0; i<isocount; i++)
				{
					PeriodTable[elemcount].Isos[i]=TempIsos[i];
				}
			}
			elemcount++;
			PeriodTable[elemcount].ElementName=str;
			isocount=0;
			//TRACE(str+"\r\n");
			break;
		case 1:
			tabpos = GetTabPosition(str);
			Rightstr=str.Right(str.GetLength()-tabpos-1);
			Leftstr=str.Left(tabpos);
			PeriodTable[elemcount].ElementSymbol=Leftstr;
			//check for '-'
			if(Rightstr.Left(1)=="-") 
			{
				PeriodTable[elemcount].AverageMass=-1;
			} 
			else 
			{
				sscanf(Rightstr,"%lf",&(PeriodTable[elemcount].AverageMass));
			}
			//str.Format("Elementsymbol = %s, Averagemass = %lf\r\n",PeriodTable[elemcount].ElementSymbol,PeriodTable[elemcount].AverageMass);
			//TRACE(str);
			break;
		case 2:
			tabpos = GetTabPosition(str);
			Rightstr=str.Right(str.GetLength()-tabpos-1);
			Leftstr=str.Left(tabpos);
			tabpos = GetTabPosition(Rightstr);
			Midstr=Rightstr.Left(tabpos);
			Rightstr=Rightstr.Right(Rightstr.GetLength()-tabpos-1);
			TempIsos[isocount].Isoname=Leftstr;
			sscanf(Midstr,"%lf",&(TempIsos[isocount].Isomass));
			sscanf(Rightstr,"%lf",&(TempIsos[isocount].IsoAbund));
			//str.Format("Isoname = %s, Isomass = %lf, Isoabund = %lf\r\n",TempIsos[isocount].Isoname,TempIsos[isocount].Isomass,TempIsos[isocount].IsoAbund);
			//TRACE(str);
			TempIsos[isocount].IsoNum=0;
			isocount++;
			break;
		}
	}
	ElementCount=elemcount;
}

void CPeriodicTable::OnElement(CString ElementName)
{
	CElemDlg dlg;
	// then set the dlg's variables according to that PeriodicTable entry	
	dlg.m_ElementName = ElementName;	
	dlg.m_PeriodTable = PeriodTable;
	//double AverageMass = AvgMass;

	dlg.DoModal();
		
	// then read back the IsoNum variable and store it

	// and append an entry to the m_ElementalComposition string
}

int CPeriodicTable::GetTabPosition(CString str)
{
	int i;
	char c;
	for (i=0;i<str.GetLength();i++)
	{
		c=str.GetAt(i);
		if (c=='\t') return i;
	}
	return i;	
}

CInstrument* CPeriodicTable::GetInstrument()
{
	return &(pApp->Instrument);
}

BOOL CPeriodicTable::DestroyWindow()
{
	int i;
	for (i=0; i<ElementCount; i++) 
	{
		delete [] PeriodTable[i].Isos;
	}
	return CDialog::DestroyWindow();
}
