// SaveFileOpsDlg.cpp : implementation file
//
/*
SaveFileOpsDlg.cpp is the code for the CSaveFileOpsDlg property page
class.  This class allows the user to control the save file options
such as default directories, default formats, names, etc.

*/
/****************************************************************

Boston University data analysis (BUDA) package 

Description: Buda is a data analysis program for Fourier 
Transform Mass Spectrometry data.  

by Peter B. O'Connor

    Copyright (C) 2000  Peter B. O'Connor

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "BudaMaster.h"
#include "BudaDoc.h"
#include "SaveFileOpsDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CSaveFileOpsDlg property page

IMPLEMENT_DYNCREATE(CSaveFileOpsDlg, CPropertyPage)

CSaveFileOpsDlg::CSaveFileOpsDlg() : CPropertyPage(CSaveFileOpsDlg::IDD)
{
	//{{AFX_DATA_INIT(CSaveFileOpsDlg)
	m_DefaultDir = _T("");
	//}}AFX_DATA_INIT
}

CSaveFileOpsDlg::~CSaveFileOpsDlg()
{
}

void CSaveFileOpsDlg::DoDataExchange(CDataExchange* pDX)
{
	CPropertyPage::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CSaveFileOpsDlg)
	DDX_Control(pDX, IDC_SAVE_TRANSIENT_CHK, m_SaveTranBtn);
	DDX_Control(pDX, IDC_SAVE_SPECTRUM_CHK, m_SaveSpecBtn);
	DDX_Control(pDX, IDC_SAVE_MITABLE_CHK, m_SaveMIBtn);
	DDX_Control(pDX, IDC_SAVE_TRUNCATED_TRANSIENT, m_SaveTruncatedBtn);
	DDX_Control(pDX, IDC_DEFAULT_FILE_NAME_FORMAT_COMBO, m_FileNameCombo);
	DDX_Control(pDX, IDC_AUTO_SAVE_ON_ACQUIRE, m_AutoSaveBtn);
	DDX_Text(pDX, IDC_EDIT_DEFAULT_SAVE_DIRECTORY, m_DefaultDir);
	DDV_MaxChars(pDX, m_DefaultDir, 100);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CSaveFileOpsDlg, CPropertyPage)
	//{{AFX_MSG_MAP(CSaveFileOpsDlg)
	ON_BN_CLICKED(IDC_AUTO_SAVE_ON_ACQUIRE, OnAutoSaveOnAcquire)
	ON_CBN_SELCHANGE(IDC_DEFAULT_FILE_NAME_FORMAT_COMBO, OnSelchangeFileNameCombo)
	ON_BN_CLICKED(IDC_SAVE_TRUNCATED_TRANSIENT, OnSaveTruncatedTransient)
	ON_BN_CLICKED(IDC_SAVE_MITABLE_CHK, OnSaveMitableChk)
	ON_BN_CLICKED(IDC_SAVE_TRANSIENT_CHK, OnSaveTransientChk)
	ON_BN_CLICKED(IDC_SAVE_SPECTRUM_CHK, OnSaveSpectrumChk)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CSaveFileOpsDlg message handlers

void CSaveFileOpsDlg::OnAutoSaveOnAcquire() 
{
	if(m_AutoSaveBtn.GetCheck()) AutoSave = 1;
	else AutoSave = 0;
}

BOOL CSaveFileOpsDlg::OnKillActive() 
{
	m_DefFileNameType=m_FileNameCombo.GetCurSel();
	
	return CPropertyPage::OnKillActive();
}

BOOL CSaveFileOpsDlg::OnSetActive() 
{
	m_FileNameCombo.ResetContent();
	m_FileNameCombo.InsertString(0,"User ID + Time/Date");
	m_FileNameCombo.InsertString(1,"User ID + Sequential Number");
	m_FileNameCombo.SetCurSel(m_DefFileNameType);

	m_SaveMIBtn.SetCheck(SaveMITable);
	m_SaveSpecBtn.SetCheck(SaveSpectrum);
	m_SaveTranBtn.SetCheck(SaveTransient);
	m_SaveTruncatedBtn.SetCheck(SaveTruncated);
		
	return CPropertyPage::OnSetActive();
}

void CSaveFileOpsDlg::OnSelchangeFileNameCombo() 
{
	// TODO: Add your control notification handler code here
	
}

void CSaveFileOpsDlg::OnSaveTruncatedTransient() 
{
	if(m_SaveTruncatedBtn.GetCheck()) SaveTruncated = 1;
	else SaveTruncated = 0;	
}

void CSaveFileOpsDlg::OnSaveMitableChk() 
{
	if(m_SaveMIBtn.GetCheck()) SaveMITable = 1;
	else SaveMITable = 0;
}

void CSaveFileOpsDlg::OnSaveTransientChk() 
{
	if(m_SaveTranBtn.GetCheck()) SaveTransient = 1;
	else SaveTransient = 0;	
}

void CSaveFileOpsDlg::OnSaveSpectrumChk() 
{
	if(m_SaveSpecBtn.GetCheck()) SaveSpectrum = 1;
	else SaveSpectrum = 0;
}
