// SetupARBDlg.cpp : implementation file
//
/****************************************************************

Boston University data analysis (BUDA) package 

Description: Buda is a data analysis program for Fourier 
Transform Mass Spectrometry data.  

by Peter B. O'Connor

    Copyright (C) 2000  Peter B. O'Connor

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "stdafx.h"
#include "Buda.h"
#include "SetupARBDlg.h"
#include "LinkingDlg.h"
#include "EditARBDlg.h"

#ifdef _DEBUG
	#define new DEBUG_NEW
	#undef THIS_FILE
	static char THIS_FILE[] = __FILE__;
#endif

// Buda debug defines
#ifdef _DEBUG
	#define DEBUG_ARB
#endif

/////////////////////////////////////////////////////////////////////////////
// CSetupARBDlg dialog


CSetupARBDlg::CSetupARBDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CSetupARBDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CSetupARBDlg)
	m_Delay = 0.0;
	m_Voltage = 0.0;
	m_TrapLinkStr = _T("");
	m_StartTimeStr = _T("");
	//}}AFX_DATA_INIT

	// define NullGenPS
	NullGenPS.Channel = NO_CHANNEL;
	NullGenPS.Delay = 0;
	NullGenPS.Link2Event = -1;
	NullGenPS.Linking = NO_LINK;
	NullGenPS.Type = NO_EVENT;
	NullGenPS.Value = 0;
	NullGenPS.Width = 0;

	// define NullARB
	NullARB.WindowLowMass = 0;
	NullARB.WindowHighMass = 0;
	NullARB.DacFixed = 0;
	NullARB.SamplesFixed = 0;
	NullARB.FilterFixed = 0;
	NullARB.AmplitudeFixed = 0;
	NullARB.DacRate = 0;
	NullARB.Samples = 0;
	NullARB.Filter = 0;
	NullARB.Amplitude = 0;
	NullARB.Setup = 0;
	NullARB.PacketSize = 0;
	NullARB.PacketSpacing = 0;
	int i;
	for(i=0; i<MAX_ARB_EVENTS; i++) {
		NullARB.Event[i].Type = ARB_NONE;
		NullARB.Event[i].fit=0;
		NullARB.Event[i].MassA=0;
		NullARB.Event[i].MassB=0;
		NullARB.Event[i].Amplitude=0;
		NullARB.Event[i].reserved=0;
	}

	// clear genps
	//PS.GenPS = 0;

	// clear out eventnum
	for(i=0; i<MAX_PS_EVENTS; i++) Eventnum[i]=0;
}


void CSetupARBDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CSetupARBDlg)
	DDX_Control(pDX, IDC_LINK_CHK, m_LinkChk);
	DDX_Control(pDX, IDC_EVENTLIST, m_EventListBox);
	DDX_Text(pDX, IDC_DELAY, m_Delay);
	DDX_Text(pDX, IDC_VOLTAGE, m_Voltage);
	DDX_Text(pDX, IDC_TRAP_LINK_STR, m_TrapLinkStr);
	DDX_Text(pDX, IDC_START_TIME_STR, m_StartTimeStr);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CSetupARBDlg, CDialog)
	//{{AFX_MSG_MAP(CSetupARBDlg)
	ON_BN_CLICKED(IDC_EDIT_ARBWAVEFORM, OnEditArbwaveform)
	ON_BN_CLICKED(IDC_CLEAR_ALL_EVENTS, OnClearAllEvents)
	ON_BN_CLICKED(IDC_DELETE_EVENT, OnDeleteEvent)
	ON_LBN_SELCHANGE(IDC_EVENTLIST, OnSelchangeEventlist)
	ON_BN_CLICKED(IDC_LINK_EXCITE, OnLinkExcite)
	ON_BN_CLICKED(IDC_ADD_EVENT, OnAddEvent)
	ON_EN_KILLFOCUS(IDC_VOLTAGE, OnKillfocusVoltage)
	ON_EN_KILLFOCUS(IDC_DELAY, OnKillfocusDelay)
	ON_BN_CLICKED(IDC_LINK_CHK, OnLinkChk)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CSetupARBDlg message handlers
BOOL CSetupARBDlg::OnInitDialog()
{
	CDialog::OnInitDialog();
	// make a copy of PS, and in OnOK, copy it back.
	PS.cfp = 0;
	PS.arb = 0;
	PS.dac = 0;
	PS.GenPS = 0;
	PS.Spec = 0;
	NewEvent = -1;

	pDoc->SortGenPS();
	pDoc->CopyPS(&(pDoc->PS), &PS);

	// set the initial selection point
	Event = 0;
	InitEventData();
	
	UpdateData(FALSE);
	return TRUE;
}

void CSetupARBDlg::OnEditArbwaveform() 
{
	if(NumEvents<=0) return;
	CEditARBDlg dlg;
	dlg.pPS = &PS;
	dlg.pDoc = pDoc;
	int i = m_EventListBox.GetCurSel();
	dlg.CurrentArbWaveform = i;

	if(dlg.DoModal() == IDOK) {
		PS.GenPS[Eventnum[i]].Width = float(1000 * PS.arb[i].Samples/PS.arb[i].DacRate);
		if(Eventnum[i] == NewEvent) NewEvent = -1;
	}
	InitEventData();
	
}

void CSetupARBDlg::OnClearAllEvents() 
{
	int i;
	for(i=0; i<NumEvents; i++) {
		m_EventListBox.SetCurSel(i);
		Event=i;
		OnDeleteEvent();
	}
}

void CSetupARBDlg::OnDeleteEvent() 
{
	if(NumEvents<=0) return;
	Event = m_EventListBox.GetCurSel();
	if(Eventnum[Event] == PS.DetectEvent) {
		// trying to delete the excite for detect
		if(MessageBox("Removing the Excite for detect\nRemoving this event will prevent ion detection","Removing Excite for detect",MB_ICONQUESTION|MB_OKCANCEL)!=IDOK) {
			return;
		}
		PS.DetectEvent = -1;
	}
	
	// now, remove this event and decrement pointers
	PS.arb[Event] = NullARB;
	PS.GenPS[Eventnum[Event]] = NullGenPS;

	// bubble up all remaining arbs
	int j;
	for (j=Event; j<PS.NumARB-1; j++) {
		PS.arb[j]=PS.arb[j+1];
	}
	PS.arb[PS.NumARB-1] = NullARB;

	PS.NumEvents = TOTAL_PS_EVENTS;
	NumEvents--;
	// sort GenPS
	pDoc->SortGenPS(&PS);
	Event=0;
	m_Delay=0;
	m_Voltage=0;
	InitEventData();
}

void CSetupARBDlg::OnSelchangeEventlist() 
{
#ifdef DEBUG_ARB
	CString str;
	str.Format("CSetupARBDlg::OnSelchangeEventlist() Event = %d, Eventnum[Event] = %d\n",Event,Eventnum[Event]);
	TRACE(str);
#endif
	int OldEvent = Event;
	Event = m_EventListBox.GetCurSel();
	if(Event < 0 || Event > PS.NumEvents) {
		MessageBox("Invalid Event Selection Change Rejected","Error");
		Event = OldEvent;
	}
	if (NumEvents==0) return;
	m_Delay = PS.GenPS[Eventnum[Event]].Delay;
	m_Voltage = PS.arb[Event].Amplitude;	
	InitEventData();
}

void CSetupARBDlg::OnLinkExcite() 
{
	CLinkingDlg dlg;
	dlg.Linking = PS.GenPS[Eventnum[Event]].Linking;
	dlg.InEvent = PS.GenPS[Eventnum[Event]].Link2Event;
	dlg.pPS = &PS;
	dlg.pSys = pDoc->sys;
	dlg.pDoc = pDoc;

	if( dlg.DoModal() == IDOK) {
		PS.GenPS[Eventnum[Event]].Linking = dlg.Linking;
		PS.GenPS[Eventnum[Event]].Link2Event = dlg.InEvent;
	}		
}

void CSetupARBDlg::InitEventData()
{
#ifdef DEBUG_ARB
	CString str1;
	str1.Format("CSetupARBDlg::InitEventData() Event = %d, Eventnum[Event] = %d\n",Event,Eventnum[Event]);
	TRACE(str1);
#endif
	//initialize the Event list
	m_EventListBox.ResetContent();
	m_EventListBox.InitStorage(40,30);

	// set up Event data array 
	//	search through PS->GenPS and an element to EventData for each ARB event
	int i;
	float start,end;
	CString str;
	NumEvents = 0;
	for(i=0; i<PS.NumEvents; i++) {
		if(PS.GenPS[i].Channel != ARB_TRIGGER) continue;
		pDoc->GetEventEdges(&PS,i,&start,&end);
		if(i == NewEvent) {
			str.Format("New Event");
		} else {
			str.Format("ARB Excitation at %1.2f msec",start);
		}
		m_EventListBox.AddString(str);
		NumEvents++;
	}

	m_EventListBox.SetCurSel(Event);

	// if there are no arb events
	if(NumEvents<=0) {
		str.Format("No Events Defined");
		m_EventListBox.AddString(str);
		m_LinkChk.SetCheck(0);
		m_LinkChk.SetWindowText("Not Linked");
		m_StartTimeStr = "Start Time (msec):";
		m_TrapLinkStr = "";
		m_Delay = 0;
		m_Voltage = 0;
		UpdateData(FALSE);
		return;
	} else {
		//if(Eventnum!=0) delete [] Eventnum;
		//Eventnum = new int[NumEvents];
		NumEvents=0;
		for(i=0; i<PS.NumEvents; i++) {
			if(PS.GenPS[i].Channel == ARB_TRIGGER) {
				Eventnum[NumEvents] = i;
				NumEvents++;
			}
		}
	}

	m_Delay = PS.GenPS[Eventnum[Event]].Delay;
	m_Voltage = PS.arb[Event].Amplitude;
	if(PS.GenPS[Eventnum[Event]].Linking == NO_LINK) {
		m_LinkChk.SetCheck(0);
		m_LinkChk.SetWindowText("Not Linked");
		m_StartTimeStr = "Start Time (msec):";
		m_TrapLinkStr = "";
	} else {
		m_LinkChk.SetCheck(1);
		m_LinkChk.SetWindowText("Linked to");
		m_StartTimeStr = "DELTA Start Time (msec):";
		m_TrapLinkStr = pDoc->GetLinkedToStr(&PS,Eventnum[Event]);
		if(m_TrapLinkStr == "" ) {
			MessageBox("Error 86: A Link is defined to or from an invalid event\nWe will try to remove the link.  Please repair link manually","Error: Link to/from invalid event");
			PS.GenPS[Eventnum[Event]].Linking = NO_LINK;
			PS.GenPS[Eventnum[Event]].Link2Event = 0;
			m_LinkChk.SetCheck(0);
			m_LinkChk.SetWindowText("Not Linked");
			m_StartTimeStr = "Start Time (msec):";
		}
	}

	UpdateData(FALSE);
}

void CSetupARBDlg::CleanupDialog()
{
	// delete all temporary arrays
	//if(Eventnum!=0) delete [] Eventnum;
	//if(PS.arb!=0) delete [] PS.arb;
	//if(PS.GenPS!=0) delete [] PS.GenPS;
}

void CSetupARBDlg::OnAddEvent() 
{
	//create new event at end of GenPS
	PS.GenPS[PS.NumEvents].Channel = ARB_TRIGGER;
	PS.GenPS[PS.NumEvents].Delay = 0.0;
	PS.GenPS[PS.NumEvents].Link2Event = 0;
	PS.GenPS[PS.NumEvents].Linking = NO_LINK;
	PS.GenPS[PS.NumEvents].Type = TTL_PULSE;
	PS.GenPS[PS.NumEvents].Value = 0.0;
	PS.GenPS[PS.NumEvents].Width = 2.0;
	// and create a new arb function packet
	NewEvent = PS.NumEvents;
	PS.NumEvents++;

	// find the last ARfp and tack onto the end of the list.
	// the last ARB function packet is also NumEvents

	PS.arb[NumEvents].WindowLowMass = (float)PS.DetectLowMass;
	PS.arb[NumEvents].WindowHighMass = (float)PS.DetectHighMass;
	PS.arb[NumEvents].DacFixed = 1;
	PS.arb[NumEvents].SamplesFixed = 1;
	PS.arb[NumEvents].FilterFixed = 1;
	PS.arb[NumEvents].AmplitudeFixed = 0;
	PS.arb[NumEvents].DacRate = 500000;
	PS.arb[NumEvents].Samples = 128*1024;
	PS.arb[NumEvents].Filter = 5;
	PS.arb[NumEvents].Amplitude = 0;
	PS.arb[NumEvents].Setup = 0;
	PS.arb[NumEvents].PacketSize = 100;
	PS.arb[NumEvents].PacketSpacing = 10;
	int i;
	for(i=0; i<MAX_ARB_EVENTS; i++) {
		if(i==0) {
			PS.arb[NumEvents].Event[i].Type = ARB_RANGE;
			PS.arb[NumEvents].Event[i].fit=0;
			PS.arb[NumEvents].Event[i].MassA=PS.DetectLowMass;
			PS.arb[NumEvents].Event[i].MassB=PS.DetectHighMass;
			PS.arb[NumEvents].Event[i].Amplitude=100;
			PS.arb[NumEvents].Event[i].reserved=0;
		} else {
			PS.arb[NumEvents].Event[i].Type = ARB_NONE;
			PS.arb[NumEvents].Event[i].fit=0;
			PS.arb[NumEvents].Event[i].MassA=0;
			PS.arb[NumEvents].Event[i].MassB=0;
			PS.arb[NumEvents].Event[i].Amplitude=0;
			PS.arb[NumEvents].Event[i].reserved=0;
		}
	}
	
	// don't resort GenPS.  Do this in OnOK()
	Event = NumEvents;
	NumEvents++;
	InitEventData();
}

void CSetupARBDlg::OnOK() 
{
	OnKillfocusDelay();
	OnKillfocusVoltage();

	// check for multiple FSP events happening at the same time
	if (CheckOverlappingEvents()) {
		return;
	}

	pDoc->CopyPS(&PS, &(pDoc->PS));

	// then delete the temporary arrays
	CleanupDialog();

	pDoc->SortGenPS();
	pDoc->UpdateAllViews(NULL,NULL,NULL);
	CDialog::OnOK();
}

void CSetupARBDlg::OnCancel() 
{
	// delete all temporary arrays
	CleanupDialog();

	CDialog::OnCancel();
}

BOOL CSetupARBDlg::CheckOverlappingEvents()
{
	// cycle through all cfp's and check if they overlap 
	// in time.  Also check to see if they overlap with
	// arb, quad, and impulse events.
	int i,j;
	float starti,endi;
	float startj,endj;
	CString str1,str2,str;
	BOOL Overlapping = FALSE;
	for(i=0; i<PS.NumEvents; i++) {
		if(PS.GenPS[i].Channel != SYN_TRIGGER && 
		   PS.GenPS[i].Channel != ARB_TRIGGER &&
		   PS.GenPS[i].Channel != IMP_TRIGGER) continue;
		for(j=0; j<PS.NumEvents; j++) {
			if(PS.GenPS[j].Channel != SYN_TRIGGER && 
			   PS.GenPS[j].Channel != ARB_TRIGGER &&
			   PS.GenPS[j].Channel != IMP_TRIGGER ) continue;
			if (i==j) continue; // an event can't overlap itself
			// if event i overlaps event j, generate error
			pDoc->GetEventEdges(&PS,i,&starti,&endi);
			pDoc->GetEventEdges(&PS,j,&startj,&endj);
			if(starti<=startj && endi>=startj) Overlapping = TRUE;
			if(starti<=endj && endi>=endj) Overlapping = TRUE;
			if(Overlapping) {
				if(PS.GenPS[i].Channel == SYN_TRIGGER) 
					str1.Format("Frequency Synthesizer event at %1.2f milliseconds\n",PS.GenPS[i].Delay);
				if(PS.GenPS[i].Channel == ARB_TRIGGER) 
					str1.Format("Arbitrary Waveform Generator event at %1.2f milliseconds\n",PS.GenPS[i].Delay);
				if(PS.GenPS[i].Channel == IMP_TRIGGER) 
					str1.Format("Impulse Excite event at %1.2f milliseconds\n",PS.GenPS[i].Delay);
				if(PS.GenPS[j].Channel == SYN_TRIGGER) 
					str2.Format("Frequency Synthesizer event at %1.2f milliseconds\n",PS.GenPS[j].Delay);
				if(PS.GenPS[j].Channel == ARB_TRIGGER) 
					str2.Format("Arbitrary Waveform Generator event at %1.2f milliseconds\n",PS.GenPS[j].Delay);
				if(PS.GenPS[j].Channel == IMP_TRIGGER) 
					str2.Format("Impulse Excite event at %1.2f milliseconds\n",PS.GenPS[j].Delay);
				str = str1 + "     occurs simultaneously with\n" + str2 + "\n\nPlease correct this before continuing";
				MessageBox(str,"Pulse Sequence Error!!");
				return TRUE;
			}  // end if overlapping
		}	// end for j
	}	// end for i
	return FALSE;
}

void CSetupARBDlg::OnKillfocusVoltage() 
{
#ifdef DEBUG_ARB
	CString str;
	str.Format("CSetupARBDlg::OnKillfocusVoltage() Event = %d, Eventnum[Event] = %d\n",Event,Eventnum[Event]);
	TRACE(str);
#endif
	UpdateData(TRUE);
	PS.GenPS[Eventnum[Event]].Delay = float(m_Delay);
	PS.arb[Event].Amplitude = float(m_Voltage);	
	if(Eventnum[Event] == NewEvent) NewEvent = -1;
	InitEventData();
	UpdateData(FALSE);
}

void CSetupARBDlg::OnKillfocusDelay() 
{
#ifdef DEBUG_ARB
	CString str;
	str.Format("CSetupARBDlg::OnKillfocusDelay() Event = %d, Eventnum[Event] = %d\n",Event,Eventnum[Event]);
	TRACE(str);
#endif
	UpdateData(TRUE);
	PS.GenPS[Eventnum[Event]].Delay = float(m_Delay);
	PS.arb[Event].Amplitude = float(m_Voltage);	
	if(Eventnum[Event] == NewEvent) NewEvent = -1;	
	InitEventData();
	UpdateData(FALSE);
}

void CSetupARBDlg::OnLinkChk() 
{
	CLinkingDlg dlg;
	dlg.Linking = PS.GenPS[Eventnum[Event]].Linking;
	dlg.InEvent = PS.GenPS[Eventnum[Event]].Link2Event;
	dlg.CurrentEvent = Eventnum[Event];
	dlg.pPS = &PS;
	dlg.pSys = pDoc->sys;
	dlg.pDoc = pDoc;

	if( dlg.DoModal() == IDOK) {
		if(PS.GenPS[Eventnum[Event]].Link2Event == Eventnum[Event]) {
			MessageBox("Cannot link an event to itself\nRejecting the link","Error: Invalid link");
			PS.GenPS[Eventnum[Event]].Link2Event=0;
			PS.GenPS[Eventnum[Event]].Linking = NO_LINK;
		}
			
		pDoc->RecalcEventTime(&PS, Eventnum[Event],dlg.Linking,dlg.InEvent);		

		m_Delay = PS.GenPS[Eventnum[Event]].Delay;
		m_Voltage = PS.arb[Event].Amplitude;	
		if(Eventnum[Event] == NewEvent) NewEvent = -1;

		InitEventData();
		UpdateData(FALSE);
	}

	if(PS.GenPS[Eventnum[Event]].Linking == NO_LINK) m_LinkChk.SetCheck(0);
	else m_LinkChk.SetCheck(1);
}
