// SetupImpDlg.cpp : implementation file
//
/****************************************************************

Boston University data analysis (BUDA) package 

Description: Buda is a data analysis program for Fourier 
Transform Mass Spectrometry data.  

by Peter B. O'Connor

    Copyright (C) 2000  Peter B. O'Connor

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "stdafx.h"
#include "Buda.h"
#include "SetupImpDlg.h"
#include "LinkingDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CSetupImpDlg dialog


CSetupImpDlg::CSetupImpDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CSetupImpDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CSetupImpDlg)
	m_Delay = 0.0;
	m_StartTimeStr = _T("");
	m_TrapLinkStr = _T("");
	//}}AFX_DATA_INIT

	// define NullGenPS
	NullGenPS.Channel = NO_CHANNEL;
	NullGenPS.Delay = 0;
	NullGenPS.Link2Event = -1;
	NullGenPS.Linking = NO_LINK;
	NullGenPS.Type = NO_EVENT;
	NullGenPS.Value = 0;
	NullGenPS.Width = 0;

	// clear genps
	PS.GenPS = 0;
	Eventnum = 0;
}


void CSetupImpDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CSetupImpDlg)
	DDX_Control(pDX, IDC_LINK_CHK, m_LinkChk);
	DDX_Control(pDX, IDC_EVENTLIST, m_EventListBox);
	DDX_Text(pDX, IDC_DELAY, m_Delay);
	DDX_Text(pDX, IDC_START_TIME_STR, m_StartTimeStr);
	DDX_Text(pDX, IDC_TRAP_LINK_STR, m_TrapLinkStr);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CSetupImpDlg, CDialog)
	//{{AFX_MSG_MAP(CSetupImpDlg)
	ON_BN_CLICKED(IDC_DELETE_EVENT, OnDeleteEvent)
	ON_LBN_SELCHANGE(IDC_EVENTLIST, OnSelchangeEventlist)
	ON_BN_CLICKED(IDC_LINK_EXCITE, OnLinkExcite)
	ON_BN_CLICKED(IDC_CLEAR_ALL_EVENTS, OnClearAllEvents)
	ON_BN_CLICKED(IDC_ADD_EVENT, OnAddEvent)
	ON_EN_CHANGE(IDC_DELAY, OnChangeDelay)
	ON_BN_CLICKED(IDC_LINK_CHK, OnLinkChk)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CSetupImpDlg message handlers
BOOL CSetupImpDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	// create a new cfp array and genps array for PS
	// then copy in the values from the pDoc->PS cfp 
	// and genps arrays
	PS.cfp = 0;
	PS.arb = 0;
	PS.dac = 0;
	PS.GenPS = 0;
	PS.Spec = 0;

	pDoc->SortGenPS();
	pDoc->CopyPS(&(pDoc->PS),&PS);

	InitEventData();
	
	UpdateData(FALSE);
	return TRUE;
}


void CSetupImpDlg::OnDeleteEvent() 
{
	Event = m_EventListBox.GetCurSel();
	if(Eventnum[Event] == PS.DetectEvent) {
		// trying to delete the excite for detect
		if(MessageBox("Removing the Excite for detect\nRemoving this event will prevent ion detection","Removing Excite for detect",MB_ICONQUESTION|MB_OKCANCEL)!=IDOK) {
			return;
		}
		PS.DetectEvent = -1;
	}
	
	// now, remove this event and decrement pointers
	PS.GenPS[Eventnum[Event]] = NullGenPS;

	PS.NumEvents = TOTAL_PS_EVENTS;
	NumEvents--;
	// sort GenPS
	pDoc->SortGenPS(&PS);
	Event=0;
	InitEventData();
}

void CSetupImpDlg::OnSelchangeEventlist() 
{
	Event = m_EventListBox.GetCurSel();
	InitEventData();	
}

void CSetupImpDlg::OnLinkExcite() 
{
	CLinkingDlg dlg;
	dlg.Linking = PS.GenPS[Eventnum[Event]].Linking;
	dlg.InEvent = PS.GenPS[Eventnum[Event]].Link2Event;
	dlg.pPS = &PS;
	dlg.pSys = pDoc->sys;
	dlg.pDoc = pDoc;

	if( dlg.DoModal() == IDOK) {
		PS.GenPS[Eventnum[Event]].Linking = dlg.Linking;
		PS.GenPS[Eventnum[Event]].Link2Event = dlg.InEvent;
	}
}

void CSetupImpDlg::OnClearAllEvents() 
{
	int i;
	for(i=0; i<NumEvents; i++) {
		m_EventListBox.SetCurSel(i);
		OnDeleteEvent();
	}
}

void CSetupImpDlg::OnAddEvent() 
{
	//create new event at end of GenPS
	PS.GenPS[PS.NumEvents].Channel = IMP_TRIGGER;
	PS.GenPS[PS.NumEvents].Delay = 0.0;
	PS.GenPS[PS.NumEvents].Link2Event = 0;
	PS.GenPS[PS.NumEvents].Linking = NO_LINK;
	PS.GenPS[PS.NumEvents].Type = TTL_PULSE;
	PS.GenPS[PS.NumEvents].Value = 0.0;
	PS.GenPS[PS.NumEvents].Width = 2.0;
	// and create a new chirp function packet
	PS.NumEvents++;
	NumEvents++;

	// don't resort GenPS.  Do this in OnOK()
	InitEventData();	
}

void CSetupImpDlg::InitEventData()
{
	// set the initial selection point
	Event = 0;

	//initialize the Event list
	m_EventListBox.ResetContent();
	m_EventListBox.InitStorage(40,30);

	// set up Event data array 
	//	search through PS->GenPS and an element to EventData for each synthesizer event
	int i;
	CString str;
	float start,end;
	NumEvents = 0;
	for(i=0; i<PS.NumEvents; i++) {
		if(PS.GenPS[i].Channel != IMP_TRIGGER) continue;
		pDoc->GetEventEdges(&PS,i,&start,&end);
		str.Format("Impulse at %1.2f msec",start);
		m_EventListBox.AddString(str);
		NumEvents++;
	}

	m_EventListBox.SetCurSel(Event);

	// if there are no impulse events
	if(NumEvents==0) {
		str.Format("No Events Defined");
		m_EventListBox.AddString(str);
		UpdateData(FALSE);
		return;
	} else {
		if(Eventnum!=0) delete [] Eventnum;
		Eventnum = new int[NumEvents];
		NumEvents=0;
		for(i=0; i<PS.NumEvents; i++) {
			if(PS.GenPS[i].Channel == IMP_TRIGGER) {
				Eventnum[NumEvents] = i;
				NumEvents++;
			}
		}
	}

	m_Delay = PS.GenPS[Eventnum[Event]].Delay;
	if(PS.GenPS[Eventnum[Event]].Linking == NO_LINK) {
		m_LinkChk.SetCheck(0);
		m_LinkChk.SetWindowText("Not Linked");
		m_StartTimeStr = "Start Time (msec):";
		m_TrapLinkStr = "";
	} else {
		m_LinkChk.SetCheck(1);
		m_LinkChk.SetWindowText("Linked to");
		m_StartTimeStr = "DELTA Start Time (msec):";
		m_TrapLinkStr = pDoc->GetLinkedToStr(&PS,Eventnum[Event]);
		if(m_TrapLinkStr == "" ) {
			MessageBox("Error 86: A Link is defined to or from an invalid event\nWe will try to remove the link.  Please repair link manually","Error: Link to/from invalid event");
			PS.GenPS[Eventnum[Event]].Linking = NO_LINK;
			PS.GenPS[Eventnum[Event]].Link2Event = 0;
			m_LinkChk.SetCheck(0);
			m_LinkChk.SetWindowText("Not Linked");
			m_StartTimeStr = "Start Time (msec):";
		}
	}

	UpdateData(FALSE);
}

void CSetupImpDlg::OnOK() 
{
	// check for multiple FSP events happening at the same time
	if (CheckOverlappingEvents()) {
		return;
	}

	pDoc->SortGenPS();
	pDoc->CopyPS(&PS,&(pDoc->PS));

	// then delete the temporary arrays
	CleanupDialog();

	CDialog::OnOK();
}

void CSetupImpDlg::OnCancel() 
{
	CleanupDialog();	
	CDialog::OnCancel();
}

void CSetupImpDlg::CleanupDialog()
{
	// delete all temporary arrays
	if(Eventnum!=0) delete [] Eventnum;
	if(PS.GenPS!=0) delete [] PS.GenPS;
}

BOOL CSetupImpDlg::CheckOverlappingEvents()
{
	// cycle through all events and check if they overlap 
	// in time.  Also check to see if they overlap with
	// arb, quad, and impulse events.
	int i,j;
	float starti,endi;
	float startj,endj;
	CString str1,str2,str;
	BOOL Overlapping = FALSE;
	for(i=0; i<PS.NumEvents; i++) {
		if(PS.GenPS[i].Channel != SYN_TRIGGER && 
		   PS.GenPS[i].Channel != ARB_TRIGGER &&
		   PS.GenPS[i].Channel != IMP_TRIGGER) continue;
		for(j=0; j<PS.NumEvents; j++) {
			if(PS.GenPS[j].Channel != SYN_TRIGGER && 
			   PS.GenPS[j].Channel != ARB_TRIGGER &&
			   PS.GenPS[j].Channel != IMP_TRIGGER ) continue;
			if (i==j) continue; // an event can't overlap itself
			// if event i overlaps event j, generate error
			pDoc->GetEventEdges(&PS,i,&starti,&endi);
			pDoc->GetEventEdges(&PS,j,&startj,&endj);
			if(starti<=startj && endi>=startj) Overlapping = TRUE;
			if(starti<=endj && endi>=endj) Overlapping = TRUE;
			if(Overlapping) {
				if(PS.GenPS[i].Channel == SYN_TRIGGER) 
					str1.Format("Frequency Synthesizer event at %1.2f milliseconds\n",PS.GenPS[i].Delay);
				if(PS.GenPS[i].Channel == ARB_TRIGGER) 
					str1.Format("Arbitrary Waveform Generator event at %1.2f milliseconds\n",PS.GenPS[i].Delay);
				if(PS.GenPS[i].Channel == IMP_TRIGGER) 
					str1.Format("Impulse Excite event at %1.2f milliseconds\n",PS.GenPS[i].Delay);
				if(PS.GenPS[j].Channel == SYN_TRIGGER) 
					str2.Format("Frequency Synthesizer event at %1.2f milliseconds\n",PS.GenPS[j].Delay);
				if(PS.GenPS[j].Channel == ARB_TRIGGER) 
					str2.Format("Arbitrary Waveform Generator event at %1.2f milliseconds\n",PS.GenPS[j].Delay);
				if(PS.GenPS[j].Channel == IMP_TRIGGER) 
					str2.Format("Impulse Excite event at %1.2f milliseconds\n",PS.GenPS[j].Delay);
				str = str1 + "     occurs simultaneously with\n" + str2 + "\n\nPlease correct this before continuing";
				MessageBox(str,"Pulse Sequence Error!!");
				return TRUE;
			}  // end if overlapping
		}	// end for j
	}	// end for i
	return FALSE;
}	// end function


void CSetupImpDlg::OnChangeDelay() 
{
	UpdateData(TRUE);
	PS.GenPS[Eventnum[Event]].Delay = (float)m_Delay;
	InitEventData();
}

void CSetupImpDlg::OnLinkChk() 
{
	CLinkingDlg dlg;
	dlg.Linking = PS.GenPS[Eventnum[Event]].Linking;
	dlg.InEvent = PS.GenPS[Eventnum[Event]].Link2Event;
	dlg.CurrentEvent = Eventnum[Event];
	dlg.pPS = &PS;
	dlg.pSys = pDoc->sys;
	dlg.pDoc = pDoc;

	if( dlg.DoModal() == IDOK) {
		if(dlg.Linking != NO_LINK && dlg.InEvent == Eventnum[Event]) {
			MessageBox("Cannot link an event to itself\nRejecting the link","Error: Invalid link");
			PS.GenPS[Eventnum[Event]].Link2Event=0;
			PS.GenPS[Eventnum[Event]].Linking = NO_LINK;			
		} else {
			pDoc->RecalcEventTime(&PS, Eventnum[Event],dlg.Linking,dlg.InEvent);		

			m_Delay = PS.GenPS[Eventnum[Event]].Delay;

			InitEventData();
			UpdateData(FALSE);
		}
	}

	if(PS.GenPS[Eventnum[Event]].Linking == NO_LINK) m_LinkChk.SetCheck(0);
	else m_LinkChk.SetCheck(1);
}
