// SetupQuadDlg.cpp : implementation file
//
/****************************************************************

Boston University data analysis (BUDA) package 

Description: Buda is a data analysis program for Fourier 
Transform Mass Spectrometry data.  

by Peter B. O'Connor

    Copyright (C) 2000  Peter B. O'Connor

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "stdafx.h"
#include "Buda.h"
#include "SetupQuadDlg.h"
#include "LinkingDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CSetupQuadDlg dialog


CSetupQuadDlg::CSetupQuadDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CSetupQuadDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CSetupQuadDlg)
	m_Voltage = 0.0;
	m_Duration = 0.0;
	m_Delay = 0.0;
	m_Frequency = 0.0;
	m_TrapLinkStr = _T("");
	m_StartTimeStr = _T("");
	//}}AFX_DATA_INIT
	EventData = 0;
	Event=0;

	// define NullCFP
	NullCFP.Amplitude=0;
	NullCFP.EndMass = 0;
	NullCFP.StartMass = 0;
	NullCFP.Type = CFP_NONE;

	// define NullGenPS
	NullGenPS.Channel = NO_CHANNEL;
	NullGenPS.Delay = 0;
	NullGenPS.Link2Event = -1;
	NullGenPS.Linking = NO_LINK;
	NullGenPS.Type = NO_EVENT;
	NullGenPS.Value = 0;
	NullGenPS.Width = 0;

	// clear cfp and genps
	PS.cfp = 0;
	PS.GenPS = 0;
}


void CSetupQuadDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CSetupQuadDlg)
	DDX_Control(pDX, IDC_LINK_CHK, m_LinkChk);
	DDX_Control(pDX, IDC_EVENTLIST, m_EventListBox);
	DDX_Text(pDX, IDC_VOLTAGE, m_Voltage);
	DDX_Text(pDX, IDC_DURATION, m_Duration);
	DDX_Text(pDX, IDC_DELAY, m_Delay);
	DDX_Text(pDX, IDC_FREQUENCY, m_Frequency);
	DDX_Text(pDX, IDC_TRAP_LINK_STR, m_TrapLinkStr);
	DDX_Text(pDX, IDC_START_TIME_STR, m_StartTimeStr);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CSetupQuadDlg, CDialog)
	//{{AFX_MSG_MAP(CSetupQuadDlg)
	ON_BN_CLICKED(IDC_ADD_EVENT, OnAddEvent)
	ON_BN_CLICKED(IDC_CLEAR_ALL_EVENTS, OnClearAllEvents)
	ON_BN_CLICKED(IDC_DELETE_EVENT, OnDeleteEvent)
	ON_LBN_SELCHANGE(IDC_EVENTLIST, OnSelchangeEventlist)
	ON_EN_CHANGE(IDC_VOLTAGE, OnChangeVoltage)
	ON_EN_CHANGE(IDC_FREQUENCY, OnChangeFrequency)
	ON_EN_CHANGE(IDC_DURATION, OnChangeDuration)
	ON_EN_CHANGE(IDC_DELAY, OnChangeDelay)
	ON_BN_CLICKED(IDC_LINK_CHK, OnLinkChk)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CSetupQuadDlg message handlers
BOOL CSetupQuadDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	// create a new cfp array and genps array for PS
	// then copy in the values from the pDoc->PS cfp 
	// and genps arrays
	PS.cfp = 0;
	PS.arb = 0;
	PS.dac = 0;
	PS.GenPS = 0;
	PS.Spec = 0;

	pDoc->SortGenPS();
	pDoc->CopyPS(&(pDoc->PS),&PS);

	InitEventData();
	
	UpdateData(FALSE);
	return TRUE;
}

void CSetupQuadDlg::InitEventData()
{

	// set up Event data array 
	//	search through PS->GenPS and an element to EventData for each synthesizer event
	int cfpnum=0,i;
	if(EventData != 0) delete[] EventData;
	EventData = new struct QuadEventData[PS.NumEvents];
	NumEvents = 0;
	for(i=0; i<PS.NumEvents; i++) {
		if(PS.GenPS[i].Channel != SYN_TRIGGER) continue;
		if(PS.cfp[cfpnum].Type != CFP_QUAD) {
			cfpnum++; // gotta skip the non-quad events
			continue;
		}
		EventData[NumEvents].Eventnum=i;
		EventData[NumEvents].Channel=PS.GenPS[i].Channel;
		EventData[NumEvents].pcfp = &(PS.cfp[cfpnum]);
		NumEvents++;
		cfpnum++;
	}

	// set the initial selection point
	Event = 0;
	SelectEventData();
	if(NumEvents == 0) {
		m_LinkChk.SetCheck(0);
		m_LinkChk.SetWindowText("Not Linked");
		m_StartTimeStr = "Start Time (msec):";
		m_TrapLinkStr = "";
		return;
	}

	if(PS.GenPS[EventData[Event].Eventnum].Linking == NO_LINK) {
		m_LinkChk.SetCheck(0);
		m_LinkChk.SetWindowText("Not Linked");
		m_StartTimeStr = "Start Time (msec):";
		m_TrapLinkStr = "";
	} else {
		m_LinkChk.SetCheck(1);
		m_LinkChk.SetWindowText("Linked to");
		m_StartTimeStr = "DELTA Start Time (msec):";
		m_TrapLinkStr = pDoc->GetLinkedToStr(&PS,EventData[Event].Eventnum);
		if(m_TrapLinkStr == "" ) {
			MessageBox("Error 86: A Link is defined to or from an invalid event\nWe will try to remove the link.  Please repair link manually","Error: Link to/from invalid event");
			PS.GenPS[EventData[Event].Eventnum].Linking = NO_LINK;
			PS.GenPS[EventData[Event].Eventnum].Link2Event = 0;
			m_LinkChk.SetCheck(0);
			m_LinkChk.SetWindowText("Not Linked");
			m_StartTimeStr = "Start Time (msec):";
		}
	}

}

void CSetupQuadDlg::OnAddEvent() 
{
	//create new event at end of GenPS
	PS.GenPS[PS.NumEvents].Channel = SYN_TRIGGER;
	PS.GenPS[PS.NumEvents].Delay = 0.0;
	PS.GenPS[PS.NumEvents].Link2Event = 0;
	PS.GenPS[PS.NumEvents].Linking = NO_LINK;
	PS.GenPS[PS.NumEvents].Type = TTL_PULSE;
	PS.GenPS[PS.NumEvents].Value = 0.0;
	PS.GenPS[PS.NumEvents].Width = 10.0;
	// and create a new chirp function packet
	PS.NumEvents++;
	NumEvents++;
	// find the last cfp and tack onto the end of the list.
	// remember that some cfp's are quad events, so you will
	// have to search the whole cfp array.
	int i;
	for (i=0; i<PS.NumCFP; i++) {
		if(PS.cfp[i].Type == CFP_NONE) break;
	}
	PS.cfp[i].Amplitude = 0;
	if(PS.HiResESI && PS.HiResMALDI) {
		MessageBox("This experiment sequence is both MALDI and ESI\nThis is impossible.  Defaulting to ESI","Pulse Sequence Error!!");
		PS.HiResESI = 1;
		PS.HiResMALDI = 0;
	}
	PS.cfp[i].StartMass = pDoc->sys->QuadFrequency;
	
	PS.cfp[i].EndMass = 0;
	PS.cfp[i].Type = CFP_QUAD;
	// don't bother to add it to EventData.
	// InitEventData will find it and add it.

	// don't resort GenPS.  Do this in OnOK()
	InitEventData();
	Event = NumEvents-1;
	SelectEventData();
}

void CSetupQuadDlg::OnClearAllEvents() 
{
	int i;
	for(i=0; i<NumEvents; i++) {
		m_EventListBox.SetCurSel(i);
		OnDeleteEvent();
	}
}

void CSetupQuadDlg::OnDeleteEvent() 
{
	Event = m_EventListBox.GetCurSel();
	if(EventData[Event].Eventnum == PS.DetectEvent) {
		// trying to delete the excite for detect
		if(MessageBox("Removing the Excite for detect\nRemoving this event will prevent ion detection","Removing Excite for detect",MB_ICONQUESTION|MB_OKCANCEL)!=IDOK) {
			return;
		}
		PS.DetectEvent = -1;
	}
	
	// now, remove this event and decrement pointers
	*(EventData[Event].pcfp) = NullCFP;
	PS.GenPS[EventData[Event].Eventnum] = NullGenPS;

	// bubble up all remaining cfps
	int j;
	BOOL B4Event = TRUE;
	for (j=0; j<PS.NumCFP-1; j++) {
		if(&(PS.cfp[j]) == EventData[Event].pcfp) B4Event = FALSE;
		if(B4Event) continue;
		PS.cfp[j]=PS.cfp[j+1];
	}

	PS.NumEvents = TOTAL_PS_EVENTS;
	NumEvents--;
	// sort GenPS
	pDoc->SortGenPS(&PS);
	Event=0;
	InitEventData();
}

void CSetupQuadDlg::OnSelchangeEventlist() 
{
	Event = m_EventListBox.GetCurSel();
	SelectEventData();	
}

/*void CSetupQuadDlg::OnLinkExcite() 
{
	CLinkingDlg dlg;
	dlg.Linking = PS.GenPS[EventData[Event].Eventnum].Linking;
	dlg.InEvent = PS.GenPS[EventData[Event].Eventnum].Link2Event;
	dlg.pPS = &PS;
	dlg.pSys = pDoc->sys;
	dlg.pDoc = pDoc;

	if( dlg.DoModal() == IDOK) {
		PS.GenPS[EventData[Event].Eventnum].Linking = dlg.Linking;
		PS.GenPS[EventData[Event].Eventnum].Link2Event = dlg.InEvent;
	}
}*/

void CSetupQuadDlg::OnChangeVoltage() 
{
	UpdateData(TRUE);
	EventData[Event].pcfp->Amplitude = (float)m_Voltage;
	SelectEventData();	
}

void CSetupQuadDlg::OnChangeFrequency() 
{
	UpdateData(TRUE);
	EventData[Event].pcfp->StartMass = (float)m_Frequency;
	SelectEventData();	
}

void CSetupQuadDlg::OnChangeDuration() 
{
	UpdateData(TRUE);
	PS.GenPS[EventData[Event].Eventnum].Width = (float)m_Duration;
	SelectEventData();	
}

void CSetupQuadDlg::OnChangeDelay() 
{
	UpdateData(TRUE);
	PS.GenPS[EventData[Event].Eventnum].Delay = (float)m_Delay;
	SelectEventData();
}

void CSetupQuadDlg::SelectEventData()
{
	CString str;
	//initialize the Event list
	m_EventListBox.ResetContent();
	m_EventListBox.InitStorage(40,30);
	// loop through the event list and add the strings to the listbox
	int i;
	float start, end;
	for (i=0; i<NumEvents; i++) {
		pDoc->GetEventEdges(&PS,EventData[i].Eventnum,&start,&end);
		str.Format("%1.2f Vbp Quad. RF at %1.2f msec",EventData[i].pcfp->Amplitude,start);
		m_EventListBox.AddString(str);
	}
	m_EventListBox.SetCurSel(Event);

	// if there are no quad events
	if(NumEvents==0) {
		str.Format("No Events Defined");
		m_EventListBox.AddString(str);
		UpdateData(FALSE);
		return;
	}

	m_Delay = PS.GenPS[EventData[Event].Eventnum].Delay;
	m_Duration = PS.GenPS[EventData[Event].Eventnum].Width;
	m_Voltage = EventData[Event].pcfp->Amplitude;
	// for quad events, the cfp.startmass element is interpreted
	// as a frequency to avoid the calibration
	m_Frequency = EventData[Event].pcfp->StartMass;

	UpdateData(FALSE);
}

void CSetupQuadDlg::CleanupDialog()
{
	// delete all temporary arrays
	if(EventData!=0) {
		delete [] EventData;
		EventData = 0;
	}
	if(PS.cfp!=0) {
		delete [] PS.cfp;
		PS.cfp = 0;
	}
	if(PS.GenPS!=0) {
		delete [] PS.GenPS;
		PS.GenPS = 0;
	}
}

void CSetupQuadDlg::OnOK() 
{
	// check for multiple FSP events happening at the same time
	if (CheckOverlappingEvents()) {
		return;
	}

	pDoc->CopyPS(&PS,&(pDoc->PS));
	pDoc->SortGenPS();
	pDoc->UpdateAllViews(NULL,NULL,NULL);

	// then delete the temporary arrays
	CleanupDialog();

	CDialog::OnOK();
}

void CSetupQuadDlg::OnCancel() 
{
	// delete all temporary arrays
	CleanupDialog();
	
	CDialog::OnCancel();
}

BOOL CSetupQuadDlg::CheckOverlappingEvents()
{
	// cycle through all cfp's and check if they overlap 
	// in time.  Also check to see if they overlap with
	// arb, quad, and impulse events.
	int i,j;
	float starti,endi;
	float startj,endj;
	CString str1,str2,str;
	BOOL Overlapping = FALSE;
	for(i=0; i<PS.NumEvents; i++) {
		if(PS.GenPS[i].Channel != SYN_TRIGGER && 
		   PS.GenPS[i].Channel != ARB_TRIGGER &&
		   PS.GenPS[i].Channel != IMP_TRIGGER) continue;
		for(j=0; j<PS.NumEvents; j++) {
			if(PS.GenPS[j].Channel != SYN_TRIGGER && 
			   PS.GenPS[j].Channel != ARB_TRIGGER &&
			   PS.GenPS[j].Channel != IMP_TRIGGER ) continue;
			if (i==j) continue; // an event can't overlap itself
			// if event i overlaps event j, generate error
			pDoc->GetEventEdges(&PS,i,&starti,&endi);
			pDoc->GetEventEdges(&PS,j,&startj,&endj);
			if(starti<=startj && endi>=startj) Overlapping = TRUE;
			if(starti<=endj && endi>=endj) Overlapping = TRUE;
			if(Overlapping) {
				if(PS.GenPS[i].Channel == SYN_TRIGGER) 
					str1.Format("Frequency Synthesizer event at %1.2f milliseconds\n",PS.GenPS[i].Delay);
				if(PS.GenPS[i].Channel == ARB_TRIGGER) 
					str1.Format("Arbitrary Waveform Generator event at %1.2f milliseconds\n",PS.GenPS[i].Delay);
				if(PS.GenPS[i].Channel == IMP_TRIGGER) 
					str1.Format("Impulse Excite event at %1.2f milliseconds\n",PS.GenPS[i].Delay);
				if(PS.GenPS[j].Channel == SYN_TRIGGER) 
					str2.Format("Frequency Synthesizer event at %1.2f milliseconds\n",PS.GenPS[j].Delay);
				if(PS.GenPS[j].Channel == ARB_TRIGGER) 
					str2.Format("Arbitrary Waveform Generator event at %1.2f milliseconds\n",PS.GenPS[j].Delay);
				if(PS.GenPS[j].Channel == IMP_TRIGGER) 
					str2.Format("Impulse Excite event at %1.2f milliseconds\n",PS.GenPS[j].Delay);
				str = str1 + "     occurs simultaneously with\n" + str2 + "\n\nPlease correct this before continuing";
				MessageBox(str,"Pulse Sequence Error!!");
				return TRUE;
			}  // end if overlapping
		}	// end for j
	}	// end for i
	return FALSE;
}	// end function


void CSetupQuadDlg::OnLinkChk() 
{
	CLinkingDlg dlg;
	dlg.Linking = PS.GenPS[EventData[Event].Eventnum].Linking;
	dlg.InEvent = PS.GenPS[EventData[Event].Eventnum].Link2Event;
	dlg.CurrentEvent = EventData[Event].Eventnum;
	dlg.pPS = &PS;
	dlg.pSys = pDoc->sys;
	dlg.pDoc = pDoc;

	if( dlg.DoModal() == IDOK) {
		if(PS.GenPS[EventData[Event].Eventnum].Link2Event == EventData[Event].Eventnum) {
			MessageBox("Cannot link an event to itself\nRejecting the link","Error: Invalid link");
			PS.GenPS[EventData[Event].Eventnum].Link2Event=0;
			PS.GenPS[EventData[Event].Eventnum].Linking = NO_LINK;
		}
			
		pDoc->RecalcEventTime(&PS, EventData[Event].Eventnum,dlg.Linking,dlg.InEvent);		

		m_Delay = PS.GenPS[EventData[Event].Eventnum].Delay;
		m_Voltage = EventData[Event].pcfp->Amplitude;	

		InitEventData();
		UpdateData(FALSE);
	}

	if(PS.GenPS[EventData[Event].Eventnum].Linking == NO_LINK) m_LinkChk.SetCheck(0);
	else m_LinkChk.SetCheck(1);
}
