// SetupSYNDlg.cpp : implementation file
//
/****************************************************************

Boston University data analysis (BUDA) package 

Description: Buda is a data analysis program for Fourier 
Transform Mass Spectrometry data.  

by Peter B. O'Connor

    Copyright (C) 2000  Peter B. O'Connor

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "stdafx.h"
#include "Buda.h"
#include "SetupSYNDlg.h"
#include "LinkingDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CSetupSYNDlg dialog


CSetupSYNDlg::CSetupSYNDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CSetupSYNDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CSetupSYNDlg)
	m_Delay = 0.0f;
	m_Duration = 0.0f;
	m_HighMass = 0.0f;
	m_LowMass = 0.0f;
	m_Voltage = 0.0f;
	m_StartTimeStr = _T("");
	m_TrapLinkStr = _T("");
	m_FreqStr = _T("");
	//}}AFX_DATA_INIT
	EventData = 0;
	Event=0;

	// define NullCFP
	NullCFP.Amplitude=0;
	NullCFP.EndMass = 0;
	NullCFP.StartMass = 0;
	NullCFP.Type = CFP_NONE;

	// define NullGenPS
	NullGenPS.Channel = NO_CHANNEL;
	NullGenPS.Delay = 0;
	NullGenPS.Link2Event = -1;
	NullGenPS.Linking = NO_LINK;
	NullGenPS.Type = NO_EVENT;
	NullGenPS.Value = 0;
	NullGenPS.Width = 0;

	// clear cfp and genps
	PS.cfp = 0;
	PS.GenPS = 0;
}


void CSetupSYNDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CSetupSYNDlg)
	DDX_Control(pDX, IDC_VOLTAGE, m_VoltageEdit);
	DDX_Control(pDX, IDC_DURATION, m_DurationEdit);
	DDX_Control(pDX, IDC_DELAY, m_DelayEdit);
	DDX_Control(pDX, IDC_LINK_CHK, m_LinkChk);
	DDX_Control(pDX, IDC_HIGHMASS, m_HighMassEdit);
	DDX_Control(pDX, IDC_LOWMASS, m_LowMassEdit);
	DDX_Control(pDX, IDC_FREQUENCYBURST, m_FreqBurstBtn);
	DDX_Control(pDX, IDC_FREQUENCYSWEEP, m_FreqSweepBtn);
	DDX_Control(pDX, IDC_EVENTLIST, m_EventListBox);
	DDX_Text(pDX, IDC_DELAY, m_Delay);
	DDX_Text(pDX, IDC_DURATION, m_Duration);
	DDX_Text(pDX, IDC_HIGHMASS, m_HighMass);
	DDX_Text(pDX, IDC_LOWMASS, m_LowMass);
	DDX_Text(pDX, IDC_VOLTAGE, m_Voltage);
	DDX_Text(pDX, IDC_START_TIME_STR, m_StartTimeStr);
	DDX_Text(pDX, IDC_TRAP_LINK_STR, m_TrapLinkStr);
	DDX_Text(pDX, IDC_FREQUENCY, m_FreqStr);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CSetupSYNDlg, CDialog)
	//{{AFX_MSG_MAP(CSetupSYNDlg)
	ON_BN_CLICKED(IDC_FREQUENCYBURST, OnFrequencyburst)
	ON_BN_CLICKED(IDC_FREQUENCYSWEEP, OnFrequencysweep)
	ON_LBN_SELCHANGE(IDC_EVENTLIST, OnSelchangeEventlist)
	ON_EN_CHANGE(IDC_LOWMASS, OnChangeLowmass)
	ON_EN_CHANGE(IDC_VOLTAGE, OnChangeVoltage)
	ON_EN_CHANGE(IDC_HIGHMASS, OnChangeHighmass)
	ON_EN_CHANGE(IDC_DURATION, OnChangeDuration)
	ON_EN_CHANGE(IDC_DELAY, OnChangeDelay)
	ON_BN_CLICKED(IDC_DELETE_EVENT, OnDeleteEvent)
	ON_BN_CLICKED(IDC_CLEAR_ALL_EVENTS, OnClearAllEvents)
	ON_BN_CLICKED(IDC_ADD_EVENT, OnAddEvent)
	ON_BN_CLICKED(IDC_LINK_CHK, OnLinkChk)
	ON_WM_RBUTTONDOWN()
	ON_EN_KILLFOCUS(IDC_DELAY, OnKillfocusDelay)
	ON_EN_KILLFOCUS(IDC_DURATION, OnKillfocusDuration)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

struct EventData {
	int Channel;
	struct ChirpPacket* pcfp;
	int Eventnum;
};

/////////////////////////////////////////////////////////////////////////////
// CSetupSYNDlg message handlers

BOOL CSetupSYNDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	// make a copy of PS, and in OnOK, copy it back.
	PS.cfp = 0;
	PS.arb = 0;
	PS.dac = 0;
	PS.GenPS = 0;
	PS.Spec = 0;
	NewEvent = -1;

	pDoc->SortGenPS();
	pDoc->CopyPS(&(pDoc->PS), &PS);

	InitEventData();
	
	UpdateData(FALSE);
	return TRUE;
}

void CSetupSYNDlg::InitEventData()
{
	// set up Event data array 
	//	search through PS->GenPS and an element to EventData for each synthesizer event
	int cfpnum=0,i;
	if(EventData != 0) delete[] EventData;
	EventData = new struct EventData[PS.NumEvents];
	NumEvents = 0;
	for(i=0; i<PS.NumEvents; i++) {
		if(PS.GenPS[i].Channel != SYN_TRIGGER) continue;
		if(PS.cfp[cfpnum].Type == CFP_QUAD) {
			cfpnum++; // gotta skip the quad events
			continue;
		}
		EventData[NumEvents].Eventnum=i;
		EventData[NumEvents].Channel=PS.GenPS[i].Channel;
		EventData[NumEvents].pcfp = &(PS.cfp[cfpnum]);
		NumEvents++;
		cfpnum++;
	}

	// set the initial selection point
	Event = 0;
	SelectEventData();
	if(NumEvents == 0) {
		m_LinkChk.SetCheck(0);
		m_LinkChk.SetWindowText("Not Linked");
		m_StartTimeStr = "Start Time (msec):";
		m_TrapLinkStr = "";
		return;
	}

	if(PS.GenPS[EventData[Event].Eventnum].Linking == NO_LINK) {
		m_LinkChk.SetCheck(0);
		m_LinkChk.SetWindowText("Not Linked");
		m_StartTimeStr = "Start Time (msec):";
		m_TrapLinkStr = "";
	} else {
		m_LinkChk.SetCheck(1);
		m_LinkChk.SetWindowText("Linked to");
		m_StartTimeStr = "DELTA Start Time (msec):";
		m_TrapLinkStr = pDoc->GetLinkedToStr(&PS,EventData[Event].Eventnum);
		if(m_TrapLinkStr == "" ) {
			MessageBox("Error 86: A Link is defined to or from an invalid event\nWe will try to remove the link.  Please repair link manually","Error: Link to/from invalid event");
			PS.GenPS[EventData[Event].Eventnum].Linking = NO_LINK;
			PS.GenPS[EventData[Event].Eventnum].Link2Event = 0;
			m_LinkChk.SetCheck(0);
			m_LinkChk.SetWindowText("Not Linked");
			m_StartTimeStr = "Start Time (msec):";
		}
	}

}

void CSetupSYNDlg::SelectEventData()
{
	CString str;
	//initialize the Event list
	m_EventListBox.ResetContent();
	m_EventListBox.InitStorage(40,30);
	// loop through the event list and add the strings to the listbox
	int i;

	for (i=0; i<NumEvents; i++) {
		if(EventData[i].Eventnum == NewEvent) {
			str.Format("New Event");
		}
		else if(EventData[i].pcfp->Type == CFP_SWEEP)
			str.Format("Sweep %1.2f m/z to %1.2f m/z",EventData[i].pcfp->StartMass,EventData[i].pcfp->EndMass);
		else if(EventData[i].pcfp->Type == CFP_BURST)
			str.Format("Frequency Burst at %1.2f m/z",EventData[i].pcfp->StartMass);
		m_EventListBox.AddString(str);
	}
	m_EventListBox.SetCurSel(Event);

	// if there are no excites??
	if(NumEvents==0) {
		str.Format("No Events");
		m_EventListBox.AddString(str);
		m_Delay = 0;
		m_FreqBurstBtn.SetCheck(TRUE);
		m_Duration = 0;
		m_LowMass = 1000;
		m_HighMass = 2500;
		m_Voltage = 0;
		m_Frequency = pDoc->MassToFreq(m_LowMass);
		m_FreqStr.Format("%1.1f",m_Frequency);
		UpdateData(FALSE);
		return;
	}

	if(EventData[Event].pcfp->Type == CFP_BURST) {
		m_FreqBurstBtn.SetCheck(TRUE);
		m_FreqSweepBtn.SetCheck(FALSE);
		// disable the high mass edit control
		m_HighMassEdit.EnableWindow(FALSE);
	}
	if(EventData[Event].pcfp->Type == CFP_SWEEP) {
		m_FreqBurstBtn.SetCheck(FALSE);
		m_FreqSweepBtn.SetCheck(TRUE);
		// enable the high mass edit control
		m_HighMassEdit.EnableWindow(TRUE);
	}
	if(EventData[Event].pcfp->Type != CFP_BURST && EventData[Event].pcfp->Type != CFP_SWEEP) {
		// error: synthesizer can only generate frequency sweeps
		// and single frequency excites.
		MessageBox("Error 77 : synthesizer event is not a sweep or a burst\nThere is a problem with the pulse sequence","Pulse Sequence Error!!");
		return;
	}

	m_Delay = PS.GenPS[EventData[Event].Eventnum].Delay;
	m_Duration = PS.GenPS[EventData[Event].Eventnum].Width;
	m_LowMass = EventData[Event].pcfp->StartMass;
	m_HighMass = EventData[Event].pcfp->EndMass;
	m_Voltage = EventData[Event].pcfp->Amplitude;
	m_Frequency = pDoc->MassToFreq(m_LowMass);

	m_FreqStr.Format("%1.1f",m_Frequency);

	UpdateData(FALSE);
}

void CSetupSYNDlg::OnFrequencysweep() 
{
	if(EventData[Event].Eventnum == NewEvent) NewEvent = -1;
	m_FreqBurstBtn.SetCheck(FALSE);
	m_FreqSweepBtn.SetCheck(TRUE);
	EventData[Event].pcfp->Type = CFP_SWEEP;
	// enable the high mass edit control
	m_HighMassEdit.EnableWindow(TRUE);
	SelectEventData();
}

void CSetupSYNDlg::OnFrequencyburst() 
{
	if(EventData[Event].Eventnum == NewEvent) NewEvent = -1;
	m_FreqBurstBtn.SetCheck(TRUE);
	m_FreqSweepBtn.SetCheck(FALSE);	
	EventData[Event].pcfp->Type = CFP_BURST;
	// disable the high mass edit control
	m_HighMassEdit.EnableWindow(FALSE);
	SelectEventData();
}

void CSetupSYNDlg::OnSelchangeEventlist() 
{
	Event = m_EventListBox.GetCurSel();
	SelectEventData();
}

void CSetupSYNDlg::OnChangeLowmass() 
{
	if(this->GetFocus()->m_hWnd != m_LowMassEdit.m_hWnd) return;
	char buffer[100];
	float mass=float(m_LowMass);
	int i;
	for (i=0; i<100; i++) buffer[i] = '\0';
	if(EventData[Event].Eventnum == NewEvent) NewEvent = -1;

	m_LowMassEdit.GetLine(0,buffer,100);

	if(sscanf(buffer,"%f",&mass) <= 0) mass = 0;

	//todo: integerization
	m_LowMass = double(mass);
	EventData[Event].pcfp->StartMass = m_LowMass;
	SelectEventData();
}

void CSetupSYNDlg::OnChangeVoltage() 
{
	char buffer[100];
	float voltage=m_Voltage;
	int i;
	for (i=0; i<100; i++) buffer[i] = '\0';
	if(EventData[Event].Eventnum == NewEvent) NewEvent = -1;

	m_VoltageEdit.GetLine(0,buffer,100);

	if(sscanf(buffer,"%f",&voltage) <= 0) voltage = 0;

	//todo: verify valid values of delay
	m_Voltage = voltage;

	EventData[Event].pcfp->Amplitude = m_Voltage;
	SelectEventData();
}

void CSetupSYNDlg::OnChangeHighmass() 
{
	char buffer[100];
	float mass=float(m_HighMass);
	int i;
	for (i=0; i<100; i++) buffer[i] = '\0';
	if(EventData[Event].Eventnum == NewEvent) NewEvent = -1;

	m_HighMassEdit.GetLine(0,buffer,100);

	if(sscanf(buffer,"%f",&mass) <= 0) mass = 0;

	//todo: verify valid values of delay
	m_HighMass = double(mass);

	EventData[Event].pcfp->EndMass = m_HighMass;
	SelectEventData();
}

void CSetupSYNDlg::OnChangeDuration() 
{
	char buffer[100];
	float duration=m_Duration;
	int i;
	for (i=0; i<100; i++) buffer[i] = '\0';
	if(EventData[Event].Eventnum == NewEvent) NewEvent = -1;

	m_DurationEdit.GetLine(0,buffer,100);

	if(sscanf(buffer,"%f",&duration) <= 0) duration = 0;

	//todo: verify valid values of delay
	m_Duration = duration;

	PS.GenPS[EventData[Event].Eventnum].Width = m_Duration;
	SelectEventData();
}

void CSetupSYNDlg::OnChangeDelay() 
{
	char buffer[100];
	float delay=m_Delay;
	int i;
	for (i=0; i<100; i++) buffer[i] = '\0';
	if(EventData[Event].Eventnum == NewEvent) NewEvent = -1;

	m_DelayEdit.GetLine(0,buffer,100);

	if(sscanf(buffer,"%f",&delay) <= 0) delay = 0;

	//todo: verify valid values of delay
	m_Delay = delay;

	PS.GenPS[EventData[Event].Eventnum].Delay = m_Delay;
	SelectEventData();
}

/*void CSetupSYNDlg::OnLinkExcite() 
{
	CLinkingDlg dlg;
	dlg.Linking = PS.GenPS[EventData[Event].Eventnum].Linking;
	dlg.InEvent = PS.GenPS[EventData[Event].Eventnum].Link2Event;
	dlg.pPS = &PS;
	dlg.pSys = pDoc->sys;
	dlg.pDoc = pDoc;

	if( dlg.DoModal() == IDOK) {
		PS.GenPS[EventData[Event].Eventnum].Linking = dlg.Linking;
		PS.GenPS[EventData[Event].Eventnum].Link2Event = dlg.InEvent;
	}	
}*/

void CSetupSYNDlg::OnDeleteEvent() 
{
	Event = m_EventListBox.GetCurSel();
	if(EventData[Event].Eventnum == PS.DetectEvent) {
		// trying to delete the excite for detect
		if(MessageBox("Removing the Excite for detect\nRemoving this event will prevent ion detection","Removing Excite for detect",MB_ICONQUESTION|MB_OKCANCEL)!=IDOK) {
			return;
		}
		PS.DetectEvent = -1;
	}
	
	// now, remove this event and decrement pointers
	*(EventData[Event].pcfp) = NullCFP;
	PS.GenPS[EventData[Event].Eventnum] = NullGenPS;

	// bubble up all remaining cfps
	int j;
	BOOL B4Event = TRUE;
	for (j=0; j<PS.NumCFP-1; j++) {
		if(&(PS.cfp[j]) == EventData[Event].pcfp) B4Event = FALSE;
		if(B4Event) continue;
		PS.cfp[j]=PS.cfp[j+1];
	}

	PS.NumEvents = TOTAL_PS_EVENTS;
	NumEvents--;
	// sort GenPS
	pDoc->SortGenPS(&PS);
	Event=0;
	InitEventData();
}

void CSetupSYNDlg::OnClearAllEvents() 
{
	int i;
	for(i=0; i<NumEvents; i++) {
		m_EventListBox.SetCurSel(i);
		OnDeleteEvent();
	}
}

void CSetupSYNDlg::OnAddEvent() 
{
	//create new event at end of GenPS
	PS.GenPS[PS.NumEvents].Channel = SYN_TRIGGER;
	PS.GenPS[PS.NumEvents].Delay = 0.0;
	PS.GenPS[PS.NumEvents].Link2Event = 0;
	PS.GenPS[PS.NumEvents].Linking = NO_LINK;
	PS.GenPS[PS.NumEvents].Type = TTL_PULSE;
	PS.GenPS[PS.NumEvents].Value = 0.0;
	PS.GenPS[PS.NumEvents].Width = 10.0;

	// now check to make sure these values are valid.
	int i,j=0;
	float starti,endi;
	float startj,endj,lastend=0;
	BOOL Overlapping = FALSE;
	startj = PS.GenPS[PS.NumEvents].Delay;
	endj = PS.GenPS[PS.NumEvents].Delay + PS.GenPS[PS.NumEvents].Width;
	for(i=0; i<PS.NumEvents; i++) {
		if(PS.GenPS[i].Channel != SYN_TRIGGER && 
		   PS.GenPS[i].Channel != ARB_TRIGGER &&
		   PS.GenPS[i].Channel != IMP_TRIGGER) continue;
		Overlapping = FALSE;
		pDoc->GetEventEdges(&PS,i,&starti,&endi);
		if(j==0) lastend = endi;
		j++;
		if(endi>lastend) lastend = endi;
		if(starti<=startj && endi>=startj) Overlapping = TRUE;
		if(starti<=endj && endi>=endj) Overlapping = TRUE;
		if(Overlapping) {
			PS.GenPS[PS.NumEvents].Delay =  endi+1;
		}
	}	// end for i

	// and create a new chirp function packet
	PS.NumEvents++;
	NumEvents++;
	// find the last cfp and tack onto the end of the list.
	// remember that some cfp's are quad events, so you will
	// have to search the whole cfp array.
	for (i=0; i<PS.NumCFP; i++) {
		if(PS.cfp[i].Type == CFP_NONE) break;
	}
	PS.cfp[i].Amplitude = 0;
	PS.cfp[i].StartMass = PS.DetectLowMass;
	PS.cfp[i].EndMass = PS.DetectLowMass;
	PS.cfp[i].Type = CFP_BURST;
	// don't bother to add it to EventData.
	// InitEventData will find it and add it.

	// don't resort GenPS.  Do this in OnOK()
	NewEvent = PS.NumEvents-1;
	InitEventData();
	Event = NumEvents-1;
	SelectEventData();
}

void CSetupSYNDlg::OnOK() 
{
	// check for multiple FSP events happening at the same time
	if (CheckOverlappingEvents()) {
		return;
	}
	pDoc->CopyPS(&PS, &(pDoc->PS));
	pDoc->SortGenPS();
	pDoc->UpdateAllViews(NULL,NULL,NULL);

	// then delete the temporary arrays
	CleanupDialog();

	CDialog::OnOK();
}

void CSetupSYNDlg::OnCancel() 
{
	// delete all temporary arrays
	CleanupDialog();

	CDialog::OnCancel();
}

void CSetupSYNDlg::CleanupDialog()
{
	// delete all temporary arrays
	if(EventData!=0) {
		delete [] EventData;
		EventData = 0;
	}
	if(PS.cfp!=0) {
		delete [] PS.cfp;
		PS.cfp = 0;
	}
	if(PS.GenPS!=0) {
		delete [] PS.GenPS;
		PS.GenPS = 0;
	}}

BOOL CSetupSYNDlg::CheckOverlappingEvents()
{
	// cycle through all cfp's and check if they overlap 
	// in time.  Also check to see if they overlap with
	// arb, quad, and impulse events.
	int i,j;
	int cpi=-1, cpj=-1;
	float starti,endi;
	float startj,endj;
	CString str1,str2,str;
	BOOL Overlapping = FALSE;
	for(i=0; i<PS.NumEvents; i++) {
		if(PS.GenPS[i].Channel != SYN_TRIGGER && 
		   PS.GenPS[i].Channel != ARB_TRIGGER &&
		   PS.GenPS[i].Channel != IMP_TRIGGER) continue;
		cpi++;
		for(j=i; j<PS.NumEvents; j++) {
			if(PS.GenPS[j].Channel != SYN_TRIGGER && 
			   PS.GenPS[j].Channel != ARB_TRIGGER &&
			   PS.GenPS[j].Channel != IMP_TRIGGER ) continue;
			cpj++;
			if (i==j) continue; // an event can't overlap itself

			// if event i overlaps event j, generate error
			pDoc->GetEventEdges(&PS,i,&starti,&endi);
			pDoc->GetEventEdges(&PS,j,&startj,&endj);
			if(starti<=startj && endi>=startj) Overlapping = TRUE;
			if(starti<=endj && endi>=endj) Overlapping = TRUE;
			if(Overlapping) {
				ASSERT(cpi>=0 && cpi < PS.NumCFP);
				ASSERT(cpj>0 && cpj < PS.NumCFP);
				if(PS.GenPS[i].Channel == SYN_TRIGGER) {
					if(PS.cfp[cpi].Type != CFP_QUAD) {
						str1.Format("Frequency Synthesizer event at %1.2f milliseconds\n",PS.GenPS[i].Delay);
					} else {
						str1.Format("Quadrupole RF event at %1.2f milliseconds\n",PS.GenPS[i].Delay);
					}
				}
				if(PS.GenPS[i].Channel == ARB_TRIGGER) 
					str1.Format("Arbitrary Waveform Generator event at %1.2f milliseconds\n",PS.GenPS[i].Delay);
				if(PS.GenPS[i].Channel == IMP_TRIGGER) 
					str1.Format("Impulse Excite event at %1.2f milliseconds\n",PS.GenPS[i].Delay);

				if(PS.GenPS[j].Channel == SYN_TRIGGER) {
					if(PS.cfp[cpj].Type != CFP_QUAD) {
						str2.Format("Frequency Synthesizer event at %1.2f milliseconds\n",PS.GenPS[j].Delay);
					} else {
						str2.Format("Quadrupole RF event at %1.2f milliseconds\n",PS.GenPS[j].Delay);
					}
				}
				if(PS.GenPS[j].Channel == ARB_TRIGGER) 
					str2.Format("Arbitrary Waveform Generator event at %1.2f milliseconds\n",PS.GenPS[j].Delay);
				if(PS.GenPS[j].Channel == IMP_TRIGGER) 
					str2.Format("Impulse Excite event at %1.2f milliseconds\n",PS.GenPS[j].Delay);
				str = str1 + "     overlaps (in time) with\n" + str2 + "\n\nPlease correct this before continuing";
				MessageBox(str,"Pulse Sequence Error!!");
				return TRUE;
			}  // end if overlapping
		}	// end for j
	}	// end for i
	return FALSE;
}	// end function

void CSetupSYNDlg::OnLinkChk() 
{
	CLinkingDlg dlg;
	dlg.Linking = PS.GenPS[EventData[Event].Eventnum].Linking;
	dlg.InEvent = PS.GenPS[EventData[Event].Eventnum].Link2Event;
	dlg.CurrentEvent = EventData[Event].Eventnum;
	dlg.pPS = &PS;
	dlg.pSys = pDoc->sys;
	dlg.pDoc = pDoc;

	if( dlg.DoModal() == IDOK) {
		if(PS.GenPS[EventData[Event].Eventnum].Link2Event == EventData[Event].Eventnum) {
			MessageBox("Cannot link an event to itself\nRejecting the link","Error: Invalid link");
			PS.GenPS[EventData[Event].Eventnum].Link2Event=0;
			PS.GenPS[EventData[Event].Eventnum].Linking = NO_LINK;
		}
			
		pDoc->RecalcEventTime(&PS, EventData[Event].Eventnum,dlg.Linking,dlg.InEvent);		

		m_Delay = PS.GenPS[EventData[Event].Eventnum].Delay;
		m_Voltage = EventData[Event].pcfp->Amplitude;	

		InitEventData();
		UpdateData(FALSE);
	}

	if(PS.GenPS[EventData[Event].Eventnum].Linking == NO_LINK) m_LinkChk.SetCheck(0);
	else m_LinkChk.SetCheck(1);
}

void CSetupSYNDlg::OnKillfocusDelay() 
{
	if(CheckOverlappingEvents()) {
		m_DelayEdit.SetFocus();
	}
}

void CSetupSYNDlg::OnKillfocusDuration() 
{
	if(CheckOverlappingEvents()) {
		m_DurationEdit.SetFocus();
	}
}

