// SetupTTLDlg.cpp : implementation file
//
/****************************************************************

Boston University data analysis (BUDA) package 

Description: Buda is a data analysis program for Fourier 
Transform Mass Spectrometry data.  

by Peter B. O'Connor

    Copyright (C) 2000  Peter B. O'Connor

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "stdafx.h"
#include "Buda.h"
#include "SetupTTLDlg.h"
#include "LinkingDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CSetupTTLDlg dialog


CSetupTTLDlg::CSetupTTLDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CSetupTTLDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CSetupTTLDlg)
	m_TTLName = _T("");
	m_Duration = 0.0f;
	m_Delay = 0.0f;
	m_StartTimeStr = _T("");
	m_TrapLinkStr = _T("");
	//}}AFX_DATA_INIT
}


void CSetupTTLDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CSetupTTLDlg)
	DDX_Control(pDX, IDC_LINK_CHK, m_LinkChk);
	DDX_Control(pDX, IDC_EVENTLIST, m_EventList);
	DDX_Text(pDX, IDC_TTLNAME, m_TTLName);
	DDX_Text(pDX, IDC_DURATION, m_Duration);
	DDX_Text(pDX, IDC_DELAY, m_Delay);
	DDX_Control(pDX, IDC_DURATION, m_DurationEdit);
	DDX_Control(pDX, IDC_DELAY, m_DelayEdit);
	DDX_Text(pDX, IDC_START_TIME_STR, m_StartTimeStr);
	DDX_Text(pDX, IDC_TRAP_LINK_STR, m_TrapLinkStr);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CSetupTTLDlg, CDialog)
	//{{AFX_MSG_MAP(CSetupTTLDlg)
	ON_EN_CHANGE(IDC_DELAY, OnChangeDelay)
	ON_EN_CHANGE(IDC_DURATION, OnChangeDuration)
	ON_LBN_SELCHANGE(IDC_EVENTLIST, OnSelchangeEventlist)
	ON_BN_CLICKED(IDC_ADD_EVENT, OnAddEvent)
	ON_BN_CLICKED(IDC_CLEAR_ALL_EVENTS, OnClearAllEvents)
	ON_BN_CLICKED(IDC_DELETE_EVENT, OnDeleteEvent)
	ON_BN_CLICKED(IDC_LINK_CHK, OnLinkChk)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CSetupTTLDlg message handlers

BOOL CSetupTTLDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	// force the dialog to update the window
	UpdateData(FALSE);

	// make a copy of PS, and in OnOK, copy it back.
	PS.cfp = 0;
	PS.arb = 0;
	PS.dac = 0;
	PS.GenPS = 0;
	PS.Spec = 0;

	pDoc->SortGenPS();
	pDoc->CopyPS(&(pDoc->PS), &PS);

	//initialize the Event list
	m_EventList.ResetContent();
	m_EventList.InitStorage(40,30);

	// set up the list box
	CurrentEvent = 0;
	SetupEventListBox();

	InitEventInfo();

	return TRUE;
}

void CSetupTTLDlg::OnUpdate()
{
	SetupEventListBox();
	InitEventInfo();
}

void CSetupTTLDlg::InitEventInfo()
{
	CString str;
	// uses the event listed in CurrentEvent 
	if(eventnum==0) {
		str.Format("No Events");
		m_EventList.AddString(str);
		return;
	}
	CurrentEvent = m_EventList.GetCurSel();

	m_Delay = PS.GenPS[EventList[CurrentEvent]].Delay;
	str.Format("%1.3f",m_Delay);

	m_Duration = PS.GenPS[EventList[CurrentEvent]].Width;
	str.Format("%1.3f",m_Duration);

	if(PS.GenPS[EventList[CurrentEvent]].Linking == NO_LINK) {
		m_LinkChk.SetCheck(0);
		m_LinkChk.SetWindowText("Not Linked");
		m_StartTimeStr = "Start Time (msec):";
		m_TrapLinkStr = "";
	} else {
		m_LinkChk.SetCheck(1);
		m_LinkChk.SetWindowText("Linked to");
		m_StartTimeStr = "DELTA Start Time (msec):";
		m_TrapLinkStr = pDoc->GetLinkedToStr(&PS,EventList[CurrentEvent]);
		if(m_TrapLinkStr == "" ) {
			MessageBox("Error 86: A Link is defined to or from an invalid event\nWe will try to remove the link.  Please repair link manually","Error: Link to/from invalid event");
			PS.GenPS[EventList[CurrentEvent]].Linking = NO_LINK;
			PS.GenPS[EventList[CurrentEvent]].Link2Event = 0;
			m_LinkChk.SetCheck(0);
			m_LinkChk.SetWindowText("Not Linked");
			m_StartTimeStr = "Start Time (msec):";
		}
	}


	UpdateData(FALSE);
}

void CSetupTTLDlg::SetupEventListBox()
{
	// this function clears the Eventlistbox and then
	// fills it with new events
	m_EventList.ResetContent();
	//pDoc->SortGenPS();

	float start,end;
	CString str;
	int i;
	eventnum=0;
	for(i=0; i<PS.NumEvents; i++) {
		if(PS.GenPS[i].Channel == Channel) {
			if(Channel>0 && Channel<8) return;	// these are the internal channels, the user doesn't have access to them
			pDoc->GetEventEdges(pPS,i,&start,&end);
			str.Format("Pulse at %1.1f msec",start);
			if(PS.GenPS[i].Type == BLANK_EVENT) str.Format("New Event");
			m_EventList.AddString(str);
			EventList[eventnum]=i;
			eventnum++;
		}
	}

	m_EventList.SetCurSel(CurrentEvent);
}


void CSetupTTLDlg::OnChangeDelay() 
{
	char buffer[100];
	int i;
	for (i=0; i<100; i++) buffer[i] = '\0';

	m_DelayEdit.GetLine(0,buffer,100);

	if(sscanf(buffer,"%f",&m_Delay) <=0) m_Delay = 0;
	PS.GenPS[EventList[CurrentEvent]].Delay = m_Delay;
	PS.GenPS[EventList[CurrentEvent]].Type = TTL_PULSE;
	OnUpdate();
}

void CSetupTTLDlg::OnChangeDuration() 
{
	char buffer[100];
	int i;
	for (i=0; i<100; i++) buffer[i] = '\0';

	m_DurationEdit.GetLine(0,buffer,100);

	if(sscanf(buffer,"%f",&m_Duration) <= 0) m_Duration = 0;
	PS.GenPS[EventList[CurrentEvent]].Width = m_Duration;
	PS.GenPS[EventList[CurrentEvent]].Type = TTL_PULSE;
	OnUpdate();
}

void CSetupTTLDlg::OnSelchangeEventlist() 
{
	InitEventInfo();	
}

void CSetupTTLDlg::OnOK() 
{
	//	search for BLANK events.  If they
	// exist, return after message boxing the error
	int i;
	for(i=0; i<eventnum; i++) {
		if(PS.GenPS[EventList[i]].Type == BLANK_EVENT) {
			// error there is a blank event.
			MessageBox("One or more Events is a BLANK event\nYou must define this event before exiting",
				"Error : Invalid Event");
			return;
		}
	}

	// copy back the modified PS
	pDoc->CopyPS(&PS, pPS);
	pDoc->SortGenPS();

	pDoc->UpdateAllViews(NULL,NULL,NULL);
	CDialog::OnOK();
}

void CSetupTTLDlg::OnDeleteEvent() 
{
	CurrentEvent = m_EventList.GetCurSel();	
	PS.GenPS[EventList[CurrentEvent]].Delay=0;
	PS.GenPS[EventList[CurrentEvent]].Width=0;
	PS.GenPS[EventList[CurrentEvent]].Value=0;
	PS.GenPS[EventList[CurrentEvent]].Type=NO_EVENT;
	PS.GenPS[EventList[CurrentEvent]].Channel=NO_CHANNEL;
	PS.GenPS[EventList[CurrentEvent]].Linking=NO_LINK;
	PS.GenPS[EventList[CurrentEvent]].Link2Event=0;
	CurrentEvent = 0;
	m_EventList.SetCurSel(CurrentEvent);
	OnUpdate();
}

void CSetupTTLDlg::OnAddEvent() 
{
	CurrentEvent = eventnum;
	eventnum++;

	// find a NO_EVENT event in the GenPS
	int i;
	for(i=0; i<PS.NumEvents; i++) {
		if(PS.GenPS[i].Type == NO_EVENT) break;
	}
	EventList[CurrentEvent] = i;

	// add a blank event, but set a flag telling of the existence
	// of a blank event
	PS.GenPS[EventList[CurrentEvent]].Delay=0;
	PS.GenPS[EventList[CurrentEvent]].Width=0;
	PS.GenPS[EventList[CurrentEvent]].Value=0;
	PS.GenPS[EventList[CurrentEvent]].Type=BLANK_EVENT;
	PS.GenPS[EventList[CurrentEvent]].Channel=Channel;
	PS.GenPS[EventList[CurrentEvent]].Linking=NO_LINK;
	PS.GenPS[EventList[CurrentEvent]].Link2Event=0;
	m_EventList.SetCurSel(CurrentEvent);
	PS.NumEvents++;
	OnUpdate();
}

void CSetupTTLDlg::OnClearAllEvents() 
{
	for(CurrentEvent=0; CurrentEvent<eventnum; CurrentEvent++) {
		PS.GenPS[EventList[CurrentEvent]].Delay=0;
		PS.GenPS[EventList[CurrentEvent]].Width=0;
		PS.GenPS[EventList[CurrentEvent]].Value=0;
		PS.GenPS[EventList[CurrentEvent]].Type=NO_EVENT;
		PS.GenPS[EventList[CurrentEvent]].Channel=NO_CHANNEL;
		PS.GenPS[EventList[CurrentEvent]].Linking=NO_LINK;
		PS.GenPS[EventList[CurrentEvent]].Link2Event=0;
	}
	OnUpdate();
}

void CSetupTTLDlg::OnLinkChk() 
{
	CLinkingDlg dlg;
	dlg.Linking = PS.GenPS[EventList[CurrentEvent]].Linking;
	dlg.InEvent = PS.GenPS[EventList[CurrentEvent]].Link2Event;
	dlg.CurrentEvent = EventList[CurrentEvent];
	dlg.pPS = &PS;
	dlg.pSys = pDoc->sys;
	dlg.pDoc = pDoc;

	if( dlg.DoModal() == IDOK) {
		if(dlg.Linking != NO_LINK && dlg.InEvent == EventList[CurrentEvent]) {
			MessageBox("Cannot link an event to itself\nRejecting the link","Error: Invalid link");
			PS.GenPS[EventList[CurrentEvent]].Link2Event=0;
			PS.GenPS[EventList[CurrentEvent]].Linking = NO_LINK;
			
		} else {
			pDoc->RecalcEventTime(&PS, EventList[CurrentEvent],dlg.Linking,dlg.InEvent);		

			m_Delay = PS.GenPS[EventList[CurrentEvent]].Delay;

			InitEventInfo();
			UpdateData(FALSE);
		}
	}

	if(PS.GenPS[EventList[CurrentEvent]].Linking == NO_LINK) m_LinkChk.SetCheck(0);
	else m_LinkChk.SetCheck(1);
}
