// SwirlyView.cpp : implementation file
//
/****************************************************************

Boston University data analysis (BUDA) package 

Description: Buda is a data analysis program for Fourier 
Transform Mass Spectrometry data.  

by Peter B. O'Connor

    Copyright (C) 2000  Peter B. O'Connor

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "stdafx.h"
#include "Buda.h"
#include "SwirlyView.h"
#include <math.h>
#include "BudaDoc.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CSwirlyView

IMPLEMENT_DYNCREATE(CSwirlyView, CView)

CSwirlyView::CSwirlyView()
{
}

CSwirlyView::~CSwirlyView()
{
}


BEGIN_MESSAGE_MAP(CSwirlyView, CView)
	//{{AFX_MSG_MAP(CSwirlyView)
	ON_WM_SIZE()
	ON_COMMAND(ID_SWIRLY_MASS, OnSwirlyMass)
	ON_COMMAND(ID_SWIRLY_FREQUENCY, OnSwirlyFrequency)
	ON_WM_RBUTTONDOWN()
	ON_WM_LBUTTONUP()
	ON_WM_LBUTTONDOWN()
	ON_WM_MOUSEMOVE()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CSwirlyView drawing

void CSwirlyView::OnDraw(CDC* pDeviceContext)
{
	pDoc = (CBudaDoc*)GetDocument();
	pDC = pDeviceContext;

	// Define drawing limits and constants
	PlotArea.top = 32768-6000;
	PlotArea.right = 32768 - 1000;
	PlotArea.bottom = 4000;
	PlotArea.left = 1000;
	AxisLineYpos = PlotArea.bottom;
	if(flags.PlotYAxis) PlotArea.left = 4000;
	if(flags.PlotParameters) PlotArea.right = 32768 - 8000;

	pDC->GetWindow()->GetClientRect(&ViewSize);
	if(!flags.InitializedWindowSize) {
		InitialWindowSize.cx = ViewSize.right - ViewSize.left;
		InitialWindowSize.cy = ViewSize.bottom - ViewSize.top;
		flags.InitializedWindowSize = TRUE;
	}

	// place page title
	CString str = "Pulse and DAC Sequences";
	CPoint pt;
	pt.x = 16384;
	pt.y = 32700;
	//Text(str,pt,CENTER_JUSTIFIED);

		// test the ranging by drawing an x in the screen
	/*MoveTo(0,0);			// lower right
	LineTo(16384,16384);	// center
	LineTo(32768,0);		// upper left
	LineTo(16384,16384);	// center
	LineTo(0,32768);		// lower right
	LineTo(16384,16384);	// center
	LineTo(32768,32768);	// upper right
	MoveTo(PlotArea.right, PlotArea.top);
	LineTo(PlotArea.left, PlotArea.top);
	LineTo(PlotArea.left, PlotArea.bottom);
	LineTo(PlotArea.right, PlotArea.bottom);
	LineTo(PlotArea.right, PlotArea.top);
	*/

	PrepareView();
	DrawAxis();
	DrawParms();
	DrawData();
	DrawVerticalAxis();
	LabelPeaks();
}

/////////////////////////////////////////////////////////////////////////////
// CSwirlyView diagnostics

#ifdef _DEBUG
void CSwirlyView::AssertValid() const
{
	CView::AssertValid();
}

void CSwirlyView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CSwirlyView message handlers


void CSwirlyView::PrepareView()
{
	float FontScaleFactor = float(ViewSize.right - ViewSize.left)/float(InitialWindowSize.cx);
	if(FontScaleFactor > float(ViewSize.bottom - ViewSize.top)/float(InitialWindowSize.cy))
		FontScaleFactor = float(ViewSize.bottom - ViewSize.top)/float(InitialWindowSize.cy);
	// note: how do i rescale the size of bitmap and text when size of window changes?

	// add swirly, Buda HiResMALDI/ESI sign, Memo, default parms
	// first, the swirly
	CBitmap bmp;
	bmp.LoadBitmap(IDB_SMALL_MONO_LOGO);
	CSize sz = bmp.GetBitmapDimension();
	sz.cx = 40;
	sz.cy = 40;
	CPoint pt;
	pt.x = ViewSize.left;
	pt.y = ViewSize.top ;
	
	pDC->SelectObject(&bmp);
	pDC->DrawState(pt,sz,&bmp, DST_BITMAP | DSS_NORMAL, NULL); 

	// now the HiResMALDI/HiResEsi Sign.
	CString str = "Buda HiResMALDI";
	pt.x = pt.x + sz.cx;
	pt.y = pt.y ;
	CFont* pFnt;
	pFnt = pDC->GetCurrentFont();

	CFont SignFont, TextFont;
	SignFont.CreatePointFont(int(FontScaleFactor*120),"Arial Black",pDC);
	pDC->SelectObject(&SignFont);
	pDC->TextOut(pt.x,pt.y,str);	// put the text on the page.
	sz = pDC->GetTextExtent(str);
	pDC->SelectObject(pFnt);

	TextFont.CreatePointFont(int(FontScaleFactor*70),"Times New Roman",pDC);
	pDC->SelectObject(&TextFont);

	// now add memo
	pt.y += sz.cy;
	str.Format("File : %s",pDoc->GetPathName());
	pDC->TextOut(pt.x,pt.y,str);	// put the text on the page.
	sz = pDC->GetTextExtent(str);
	pt.y += sz.cy;
	str.Format("%s",pDoc->PS.ExperimentMemo);
	pDC->TextOut(pt.x,pt.y,str);	// put the text on the page.

	// now add top parameters.
	sz = pDC->GetTextExtent(" 99-DEC-2000  ");	// dummy string to get the right spacing
	pt.x = ViewSize.right - sz.cx - 5;
	pt.y = ViewSize.top;
	CPoint pt1 = pt;	// for alignment of the ':' characters

	CString str1;
	str1.Format("%s",pDoc->PS.TimeAcquired);
	int space = str1.Find(" ",0);

	str = "Date : ";
	sz = pDC->GetTextExtent(str);	// right align the field name.
	pt1.x = pt.x - sz.cx;
	pt1.y = pt.y;
	pDC->TextOut(pt1.x,pt1.y,str);	// put the text on the page.
	pDC->TextOut(pt.x,pt.y,str1.Left(space));	// put the text on the page.
	sz = pDC->GetTextExtent(str);	// shift down pt for the next one.
	pt.y += long(0.9*sz.cy);

	str = "Time : ";
	sz = pDC->GetTextExtent(str);	// right align the field name.
	pt1.x = pt.x - sz.cx;
	pt1.y = pt.y;
	pDC->TextOut(pt1.x,pt1.y,str);	// put the text on the page.
	pDC->TextOut(pt.x,pt.y,str1.Right(str1.GetLength() - space - 1));	// put the text on the page.
	sz = pDC->GetTextExtent(str);	// shift down pt for the next one.
	pt.y += long(0.9*sz.cy);

	if(pDoc->TrapVoltsOnDetect >= 0) str1.Format(" Positive ");
	else str1.Format(" Negative ");
	str = "Mode : ";
	sz = pDC->GetTextExtent(str);	// right align the field name.
	pt1.x = pt.x - sz.cx;
	pt1.y = pt.y;
	pDC->TextOut(pt1.x,pt1.y,str);	// put the text on the page.
	pDC->TextOut(pt.x,pt.y,str1);	// put the text on the page.
	sz = pDC->GetTextExtent(str);	// shift down pt for the next one.
	pt.y += long(0.9*sz.cy);

	str1.Format(" %d",pDoc->PS.ScansPerSpectrum);
	str = "Scans : ";
	sz = pDC->GetTextExtent(str);	// right align the field name.
	pt1.x = pt.x - sz.cx;
	pt1.y = pt.y;
	pDC->TextOut(pt1.x,pt1.y,str);	// put the text on the page.
	pDC->TextOut(pt.x,pt.y,str1);	// put the text on the page.
	sz = pDC->GetTextExtent(str);	// shift down pt for the next one.
	pt.y += long(0.9*sz.cy);

	str1.Format(" %1.2f",VScale);
	str = "Scale : ";
	sz = pDC->GetTextExtent(str);	// right align the field name.
	pt1.x = pt.x - sz.cx;
	pt1.y = pt.y;
	pDC->TextOut(pt1.x,pt1.y,str);	// put the text on the page.
	pDC->TextOut(pt.x,pt.y,str1);	// put the text on the page.
	sz = pDC->GetTextExtent(str);	// shift down pt for the next one.

}

void CSwirlyView::DrawTimeAxis()
{
	DrawGenericAxis(PlotArea.left, PlotArea.right, 0, float(pDoc->PS.TransientPoints/pDoc->PS.AdcRate), "Seconds", "%1.1f");
}

void CSwirlyView::Text(CString str, CPoint pt, int just)
{
	CSize sz = pDC->GetTextExtent(str);

	CPoint pt1,pt2;
	pt1 = PTtoViewPT(pt);
	if(just == CENTER_JUSTIFIED) pt1.x = pt1.x-sz.cx/2;
	if(just == LEFT_JUSTIFIED) pt1.y = pt1.y - sz.cy/2;
	if(just == CENTER_JUSTIFIED2) {
		pt1.x = pt1.x-sz.cx/2;
		pt1.y = pt1.y - sz.cy/2;
	}
	if(just == RIGHT_JUSTIFIED) {
		pt1.x = pt1.x - sz.cx;
		pt1.y = pt1.y - sz.cy/2;
	}

	pDC->TextOut(pt1.x,pt1.y,str);
}

void CSwirlyView::DrawParms()
{

}

void CSwirlyView::DrawData()
{
	// call the various DrawData functions.
	// only DrawTransientData() available right now...
	if(PlotType == TRANSIENT)
		DrawTransientData();
	else if(PlotType == FREQUENCY) 
		DrawFrequencyData();
	else if(PlotType == MASSSPEC) 
		DrawMassData();
	else if(PlotType == TOTAL_ION_CHROMATOGRAM) 
		DrawTICData();
	else if(PlotType == STICKPLOT) 
		DrawStickplotData();
	else if(PlotType == RECONSTRUCTED_ION_CHROMATOGRAM) 
		DrawRICData();
	else if(PlotType == DECONVOLVED_MASS_DATA) 
		DrawDeconData();
}

void CSwirlyView::LabelPeaks()
{

}

void CSwirlyView::MoveTo(int x, int y)
{
	pDC->MoveTo(PTtoViewPT(x,y));
}

void CSwirlyView::MoveTo(CPoint pt)
{
	pDC->MoveTo(PTtoViewPT(pt));
}

void CSwirlyView::LineTo(int x, int y)
{
	pDC->LineTo(PTtoViewPT(x,y));
}

void CSwirlyView::LineTo(CPoint pt)
{
	pDC->LineTo(PTtoViewPT(pt));
}

CPoint CSwirlyView::PTtoViewPT(CPoint pt)
{
	// this function maps the window specified by ViewSize
	// into a logical window from lower-left = 0,0 to 
	// upper right = 32767,32767

	// the input variable pt is assumed to be in the 0-32767
	// window, while the output is in the viewsize window.

	if (pt.x>32767) pt.x = 32767;
	if (pt.y>32767) pt.y = 32767;
	if (pt.x<0) pt.x = 0;
	if (pt.y<0) pt.y = 0;

	CPoint outpt;
	double x,y,mx,my,by;
	my = ((double)ViewSize.bottom-(double)ViewSize.top)/32768.0;
	mx = ((double)ViewSize.right-(double)ViewSize.left)/32768.0;
	by = (double)ViewSize.bottom-(double)ViewSize.top;
	y = by - my*(double)pt.y;
	x = mx*(double)pt.x;

	outpt.y = (short)y;
	outpt.x = (short)x;
	return outpt;
}

CPoint CSwirlyView::PTtoViewPT(int x, int y)
{
	CPoint pt;
	pt.x=x;
	pt.y=y;
	return PTtoViewPT(pt);
}	

void CSwirlyView::DrawAxis()
{
	// only time plot available right now.  plot that one.
	if(PlotType == TRANSIENT)
		DrawTimeAxis();
	else if(PlotType == FREQUENCY) 
		DrawFrequencyAxis();
	else if(PlotType == MASSSPEC) 
		DrawMassAxis();
	else if(PlotType == TOTAL_ION_CHROMATOGRAM) 
		DrawTICAxis();
	else if(PlotType == STICKPLOT) 
		DrawMassAxis();
	else if(PlotType == RECONSTRUCTED_ION_CHROMATOGRAM) 
		DrawRICAxis();
	else if(PlotType == DECONVOLVED_MASS_DATA) 
		DrawDeconAxis();
}

void CSwirlyView::DrawTransientData()
{
	ViewLineLength = 6000;
	long i;

	// clearing the viewline array.
	if(flags.ClearViewLine) {
		struct ViewLinePt NullPt;
		NullPt.x = 0;
		NullPt.y = 0;
		for(i=0; i<MAX_VIEWLINE_LENGTH; i++) ViewLine[i] = NullPt;
		flags.ClearViewLine = FALSE;
	}

	if(flags.CalcViewLine) {
		CalculateTransientViewLine();
	}

	float m1, b1, m2, b2;
	if(ViewLineMaxy != ViewLineMiny) 
		m1 = (PlotArea.top - PlotArea.bottom)/(ViewLineMaxy - ViewLineMiny);
	else m1 = 1;
	b1 = -1*m1*ViewLineMiny + PlotArea.bottom;

	if(ViewLineMaxx != ViewLineMinx)
		m2 = (PlotArea.right - PlotArea.left)/(ViewLineMaxx - ViewLineMinx);
	else m2 = 1;
	b2 = -1*m2*ViewLineMinx + PlotArea.left;

	MoveTo(DataToViewPoint(ViewLine[0], m1, b1, m2, b2));
	for(i=1; i<ViewLineLength; i++) {
		LineTo(DataToViewPoint(ViewLine[i], m1, b1, m2, b2));
	}
}

void CSwirlyView::CalculateTransientViewLine()
{
	long *data;
	long i,j;
	float rangepts; // points per sample range
	long beginpt;
	double maxdata, mindata, avg;
	double globalmax, globalmin;
	long minx, maxx, temp, maxpts;
	float miny, maxy;

	data = pDoc->TimeArray;
	maxpts=pDoc->PS.TransientPoints;
	avg = 0;
	for(i=0; i<maxpts; i++) avg+=double(data[i])/double(maxpts);
	DCOffset = long(avg);
	if(flags.RemoveDC) {
		for(i=0; i<maxpts; i++) data[i] -= DCOffset;
	}

	minx=pDoc->TimeToPt(zoomstack[zoompointer].minx);
	maxx=pDoc->TimeToPt(zoomstack[zoompointer].maxx);
	miny=zoomstack[zoompointer].miny;
	maxy=zoomstack[zoompointer].maxy;

	if(minx>maxx) {
		temp=minx;
		minx=maxx;
		maxx=temp;
	}

	int mini = 0;
	int maxi = 0;
	// check the initial and final points and set them if necessary
	//ASSERT(beginpt>=0 && beginpt<=maxpts);
	if(minx<0) {
		mini = 2;
		ViewLine[0].y = 0;
		ViewLine[1].y = 0;
		ViewLine[0].x =zoomstack[zoompointer].minx;
		ViewLine[1].x =float(1.0/pDoc->PS.AdcRate);
		minx = 0;
	}
	if((long)minx>pDoc->PS.TransientPoints)
		minx = 0;
	if(maxx<0)
		maxx = pDoc->PS.TransientPoints;
	if((long)maxx>pDoc->PS.TransientPoints) {
		maxi=2;
		maxx = pDoc->PS.TransientPoints;
		ViewLine[ViewLineLength-1].y =0;
		ViewLine[ViewLineLength-2].y =0;
		ViewLine[ViewLineLength-1].x =zoomstack[zoompointer].maxx;
		ViewLine[ViewLineLength-2].x =float(pDoc->PS.TransientPoints/pDoc->PS.AdcRate);
	}

	rangepts = (maxx-minx)/float(ViewLineLength-mini-maxi);
	scale = 0;
	
	globalmax = globalmin = data[mini];
	if ((maxx-minx)<ViewLineLength) {  // if zoomed in far enough, plot all the points
		ViewLineLength=maxx-minx;
		for(i=mini;i<ViewLineLength;i++) {
			globalmax = (globalmax > data[minx+i]) ? globalmax : data[minx+i];
			globalmin = (globalmin < data[minx+i]) ? globalmin : data[minx+i];
			ViewLine[i].y =float(data[minx+i]);
			ViewLine[i].x =(float) ((minx+i)/(float)pDoc->PS.AdcRate); 
			if (flags.IsZoomingY) {
				if (ViewLine[i].y >maxy) ViewLine[i].y =float(maxy);
				if (ViewLine[i].y <miny) ViewLine[i].y =float(miny);
			}
		}
		rangepts=1;
	} else { // plot both minimum and maximum points
		for (i=mini; i<ViewLineLength-maxi; i+=2) {
			beginpt = long(i*rangepts)+minx;
		// now find the data for each individual range of points
			maxdata = data[beginpt];
			mindata = data[beginpt];
			for (j=0; j<rangepts; j++) { // find max and min point on range
				maxdata = (maxdata > data[beginpt+j]) ? maxdata : data[beginpt+j];
				mindata = (mindata < data[beginpt+j]) ? mindata : data[beginpt+j];
				}
			globalmax = (globalmax > maxdata) ? globalmax : maxdata;
			globalmin = (globalmin < mindata) ? globalmin : mindata;

			ViewLine[i].y  = float(maxdata);
			ViewLine[i+1].y  = float(mindata);
			ViewLine[i].x =float((minx+i*rangepts)/pDoc->PS.AdcRate);
			ViewLine[i+1].x =float((minx+(i+1)*rangepts)/pDoc->PS.AdcRate);
			if (flags.IsZoomingY) {
				if (ViewLine[i].y >maxy) ViewLine[i].y=float(maxy);
				if (ViewLine[i].y<miny) ViewLine[i].y=float(miny);
				if (ViewLine[i+1].y>maxy) ViewLine[i+1].y=float(maxy);
				if (ViewLine[i+1].y<miny) ViewLine[i+1].y=float(miny);
			}
		} // end for
	} // end if
	ScaleViewLine();
	ViewLineMaxx = float(pDoc->PS.TransientPoints/pDoc->PS.AdcRate);
	ViewLineMinx = 0;
	flags.CalcViewLine = FALSE;
}

void CSwirlyView::ScaleViewLine()
{
	// now, scale the TempViewLine into integers for ViewLine.
	int i;
	float maxdata, mindata;

	if (flags.IsZoomingY) {
		for (i=0; i<ViewLineLength; i++) {
			if (ViewLine[i].y >zoomstack[zoompointer].maxy) ViewLine[i].y =float(zoomstack[zoompointer].maxy);
			if (ViewLine[i].y <zoomstack[zoompointer].miny) ViewLine[i].y =float(zoomstack[zoompointer].miny);
		}
	}

	maxdata = ViewLine[0].y;
	mindata = ViewLine[0].y;
	for (i=0; i<ViewLineLength; i++) {
		if (maxdata < ViewLine[i].y) maxdata = ViewLine[i].y;
		if (mindata > ViewLine[i].y) mindata = ViewLine[i].y;
	}
	scale = float(maxdata-mindata)/float(32767.0);
	ViewLineMaxy = float(maxdata);
	ViewLineMiny = float(mindata);

	zoomstack[zoompointer].maxy = float(ViewLineMaxy/scale);
	zoomstack[zoompointer].miny = float(ViewLineMiny/scale);
}

void CSwirlyView::InitializeZoomStack()
{
	int i;

	switch(PlotType) {
		case TRANSIENT:
			for(i=0; i<20; i++) {
				zoomstack[i].minx=0.0;
				zoomstack[i].maxx=(float)pDoc->PtToTime(pDoc->PS.TransientPoints-1);
				zoomstack[i].miny=-1.0;
				zoomstack[i].maxy=1.0;
			}
			break;
		case FREQUENCY:
			for(i=0; i<20; i++) {
				zoomstack[i].minx=(float)pDoc->MassToFreq(pDoc->PS.DetectHighMass, pDoc->PS.A, pDoc->PS.B);
				zoomstack[i].maxx=(float)pDoc->MassToFreq(pDoc->PS.DetectLowMass, pDoc->PS.A, pDoc->PS.B);
				zoomstack[i].miny=0;
				zoomstack[i].maxy=1.0;
			}
			break;
		case MASSSPEC:
			for(i=0; i<20; i++) {
				zoomstack[i].minx=(float)pDoc->PS.DetectLowMass;
				zoomstack[i].maxx=(float)pDoc->PS.DetectHighMass;
				zoomstack[i].miny=0;
				zoomstack[i].maxy=120.0;
			}
			break;
		case STICKPLOT:
			for(i=0; i<20; i++) {
				zoomstack[i].minx=(float)pDoc->PS.DetectLowMass;
				zoomstack[i].maxx=(float)pDoc->PS.DetectHighMass;
				zoomstack[i].miny=0;
				zoomstack[i].maxy=1.0;
			}
			break;
		case TOTAL_ION_CHROMATOGRAM:
//			for(i=0; i<20; i++) {
//				zoomstack[i].minx=(float)TwoDSpecnumToMin(0);
//				zoomstack[i].maxx=(float)TwoDSpecnumToMin(pDoc->PS.SpectraAcquired-1);
//				zoomstack[i].miny=0;
//				zoomstack[i].maxy=1.0;
//			}
			break;
		case DECONVOLVED_MASS_DATA:
			for(i=0; i<20; i++) {
				zoomstack[i].minx=(float)pDoc->sys->PeakPickerLowMass;
				zoomstack[i].maxx=(float)pDoc->sys->PeakPickerHighMass;
				zoomstack[i].miny=0;
				zoomstack[i].maxy=1.0;
			}
			break;
	} // end switch(PlotType)
	flags.UpdateScaleFactor = TRUE;
	flags.ViewIsSetup=FALSE;
	zoompointer = 0;
	//ViewLineLength=Buda_VIEWLINELENGTH;
}

void CSwirlyView::OnInitialUpdate() 
{
	CView::OnInitialUpdate();
	pDoc = (CBudaDoc*)GetDocument();

	flags.Zooming = FALSE;
	flags.ClearViewLine = TRUE;
	flags.CalcViewLine = TRUE;
	flags.RemoveDC = TRUE;
	flags.IsZoomingY = FALSE;
	flags.PlotYAxis = TRUE;
	flags.PlotParameters = FALSE;
	flags.InitializedWindowSize = FALSE;
	flags.UpdateScaleFactor = TRUE;
	flags.ViewIsSetup = TRUE;
	
	PlotType = TRANSIENT;
	InitializeZoomStack();
	VScale = 1.0;
}



void CSwirlyView::DrawVerticalAxis()
{
	if(!flags.PlotYAxis) return;

	// this function draws a time axis out to the last event
	// in the pulse sequence
	int AxisLineYstart = PlotArea.bottom;
	int AxisLineYend = PlotArea.top;
	int AxisLineXpos = PlotArea.left;
	long i;

	double ybase = (double)ViewLineMaxy - (double)ViewLineMiny;

	// second draw the axis line 
	MoveTo(AxisLineXpos,PlotArea.bottom);
	LineTo(AxisLineXpos,PlotArea.top);

	// place tics
	// there will be <=10 tics
	int ticpos[10];
	double ticy[10];
	for(i=0; i<10; i++) {
		ticpos[i] = 0;
		ticy[i] = 0;
	}

	// there will be 5-10 tics  
	// ticsteps can be (order of magnitude of timebase) / 1, 2, 5 
	double a = log10(ybase);
	double b = floor(a);	// b = order of magnitude of timebase
	double ticstep = pow(10,b);
	double nt;	
	int numtics;
	nt = ybase/ ticstep;

	if(nt>=5) numtics = (int)nt;
	if(nt>2 && nt<5) {
		ticstep=pow(10,b)/2;
		numtics = (int)(ybase/ticstep);
	}
	if(nt<=2) {
		ticstep = pow(10,b)/5;
		numtics = (int)(ybase/ticstep);
	}

	// find the position of the first tic.
	// if zero is in the range, I want zero to be one of the tics
	double NumTixb4Zero = (double)ViewLineMiny/ticstep;
	if(NumTixb4Zero>0) {// GenPS starts after zero
		ticy[0]=(floor(NumTixb4Zero)+1) * ticstep;
	}
	if (NumTixb4Zero == 0) ticy[0] = 0;
	if (NumTixb4Zero < 0) { // GenPS starts before zero
		ticy[0] = -1 * floor(-1*NumTixb4Zero) * ticstep;
	}

	// find the position of each tic relative to ticy[0]
	for(i=1; i<10; i++) {
		ticy[i]=ticstep*i+ticy[0];
		if(ticy[i]>ViewLineMaxy) break;
	}
	numtics = i-1;

	// find tic x positions
	// remember that position 1000 = FirstEventStart
	// and 31768 = LastEventEnd;
				// create a linear eqn y = a*x + b
				// where y's are the plotspace #'s and 
				// x's are the timebase space #'s
	a = (AxisLineYend-AxisLineYstart)/(ViewLineMaxy - ViewLineMiny);
	b = AxisLineYstart - a*ViewLineMiny;
	for(i=0; i<numtics; i++) {
		ticpos[i] = int(a*ticy[i] + b);
	}

	// now actually plot the tics
	for(i=0; i<numtics; i++) {
		MoveTo(AxisLineXpos,ticpos[i]);
		LineTo(AxisLineXpos-500,ticpos[i]);
	}

	// lable tics
	CString str;
	CPoint pt;
	CSize sz;

	str.Format("Raw ADC Value");
	pt.y = 16384;
	pt.y = AxisLineXpos - 2000;
	//VerticalText(str,pt,CENTER_JUSTIFIED);

	for(i=0; i<numtics; i++) {
		str.Format("%1.0f",ticy[i]);
		pt.y = ticpos[i];
		pt.x = AxisLineXpos - 600 ;
		Text(str,pt,RIGHT_JUSTIFIED);
	}

	// end of draw vertical axis
}

CPoint CSwirlyView::DataToViewPoint(struct ViewLinePt inpt, float yslope, float yintercept, float xslope, float xintercept)
{
	CPoint outpt;
	// rescale ViewLinePt to PlotArea
	outpt.y = long(yslope*inpt.y + yintercept);
	outpt.x = long(xslope*inpt.x + xintercept);

	return outpt;
}

void CSwirlyView::VerticalText(CString str, CPoint pt, int just)
{
	CSize sz = pDC->GetTextExtent(str);

	CPoint pt1,pt2;
	pt1 = PTtoViewPT(pt);
	if(just == CENTER_JUSTIFIED) pt1.x = pt1.x-sz.cx/2;
	if(just == LEFT_JUSTIFIED) pt1.y = pt1.y - sz.cy/2;
	if(just == CENTER_JUSTIFIED2) {
		pt1.x = pt1.x-sz.cx/2;
		pt1.y = pt1.y - sz.cy/2;
	}
	if(just == RIGHT_JUSTIFIED) {
		pt1.x = pt1.x - sz.cx;
		pt1.y = pt1.y - sz.cy/2;
	}
	// rotate text
	//don't know how to do this one yet.
	pDC->TextOut(pt1.x,pt1.y,str);

}

void CSwirlyView::OnSize(UINT nType, int cx, int cy) 
{
	CView::OnSize(nType, cx, cy);
}

void CSwirlyView::DrawFrequencyData()
{
	ViewLineLength = 2000;
	long i;

	// clearing the viewline array.
	if(flags.ClearViewLine) {
		struct ViewLinePt NullPt;
		NullPt.x = 0;
		NullPt.y = 0;
		for(i=0; i<MAX_VIEWLINE_LENGTH; i++) ViewLine[i] = NullPt;
		flags.ClearViewLine = FALSE;
	}

	if(flags.CalcViewLine) {
		CalculateFrequencyViewLine();
	}

	float m1, b1, m2, b2;
	if(ViewLineMaxy != ViewLineMiny) 
		m1 = (PlotArea.top - PlotArea.bottom)/(ViewLineMaxy - ViewLineMiny);
	else m1 = 1;
	b1 = -1*m1*ViewLineMiny + PlotArea.bottom;

	if(ViewLineMaxx != ViewLineMinx)
		m2 = (PlotArea.right - PlotArea.left)/(ViewLineMinx - ViewLineMaxx);
	else m2 = 1;
	b2 = -1*m2*ViewLineMinx + PlotArea.right;

	MoveTo(DataToViewPoint(ViewLine[0], m1, b1, m2, b2));
	for(i=1; i<ViewLineLength; i++) {
		LineTo(DataToViewPoint(ViewLine[i], m1, b1, m2, b2));
	}

}

void CSwirlyView::DrawMassData()
{
	ViewLineLength = 2000;
	long i;

	// clearing the viewline array.
	if(flags.ClearViewLine) {
		struct ViewLinePt NullPt;
		NullPt.x = 0;
		NullPt.y = 0;
		for(i=0; i<MAX_VIEWLINE_LENGTH; i++) ViewLine[i] = NullPt;
		flags.ClearViewLine = FALSE;
	}

	if(flags.CalcViewLine) {
		CalculateMassViewLine();
	}

	float m1, b1, m2, b2;
	if(ViewLineMaxy != ViewLineMiny) 
		m1 = (PlotArea.top - PlotArea.bottom)/(ViewLineMaxy - ViewLineMiny);
	else m1 = 1;
	b1 = -1*m1*ViewLineMiny + PlotArea.bottom;

	if(ViewLineMaxx != ViewLineMinx)
		m2 = (PlotArea.right - PlotArea.left)/(ViewLineMaxx - ViewLineMinx);
	else m2 = 1;
	b2 = -1*m2*ViewLineMinx + PlotArea.left;

	MoveTo(DataToViewPoint(ViewLine[0], m1, b1, m2, b2));
	for(i=1; i<ViewLineLength; i++) {
		LineTo(DataToViewPoint(ViewLine[i], m1, b1, m2, b2));
	}
}

void CSwirlyView::DrawTICData()
{

}

void CSwirlyView::DrawStickplotData()
{

}

void CSwirlyView::DrawRICData()
{

}

void CSwirlyView::DrawDeconData()
{

}

void CSwirlyView::DrawDeconAxis()
{

}

void CSwirlyView::DrawFrequencyAxis()
{
	float beginfreq, endfreq;
	beginfreq = float(zoompointer[zoomstack].minx);
	endfreq = float(zoompointer[zoomstack].maxx);
	DrawGenericAxis(PlotArea.left, PlotArea.right, beginfreq, endfreq, "Hz", "%1.1f");
}

void CSwirlyView::DrawMassAxis()
{
	DrawGenericAxis(PlotArea.left, PlotArea.right, float(pDoc->PS.DetectLowMass), float(pDoc->PS.DetectHighMass), "m/z", "%1.1f");
}

void CSwirlyView::DrawTICAxis()
{

}

void CSwirlyView::DrawRICAxis()
{

}

void CSwirlyView::CalculateFrequencyViewLine()
{
	float *data;
	int i,j;
	float rangepts; // points per sample range
	long beginpt;
	float maxdata;
	long minx, maxx, temp, maxpts;
	float miny, maxy;
	
	data=pDoc->FreqArray;
	maxpts=pDoc->PS.SpectrumPoints;

	minx=pDoc->FreqToPt(zoomstack[zoompointer].minx);
	maxx=pDoc->FreqToPt(zoomstack[zoompointer].maxx);
	miny=zoomstack[zoompointer].miny;
	maxy=zoomstack[zoompointer].maxy;

	if(minx>maxx) {
		temp=minx;
		minx=maxx;
		maxx=temp;
	}

	// test that the minx and maxx values are good and reset if not
	int mini = 0;
	int maxi = 0;
	// check the initial and final points and set them if necessary
	//ASSERT(beginpt>=0 && beginpt<=maxpts);
	if(minx<=0) { // if negative point numbers or DC
		mini = 2;
		ViewLine[0].y = (float)0.01;
		ViewLine[1].y = (float)0.01;
		ViewLine[0].x = zoomstack[zoompointer].minx;
		ViewLine[1].x = (float)pDoc->PtToFreq(1);
		minx = 1;
	}
	// if "lowest" frequency > freq of lowest mass
	// Lowest frequency = freq of highest mass
	if(pDoc->PS.DetectMode != MixerMode) {
		if(minx>pDoc->MassToPt(pDoc->PS.DetectLowMass))
			minx = pDoc->MassToPt(pDoc->PS.DetectHighMass);
		// if highest frequency is negative or DC
		// highest freq = freq of lowest mass
		if(maxx<=0)
			maxx = pDoc->MassToPt(pDoc->PS.DetectLowMass);
		// if highest frequency > freq of lowest mass, 
		if(maxx>pDoc->MassToPt(pDoc->PS.DetectLowMass)) {
			maxi=2;
			maxx = pDoc->MassToPt(pDoc->PS.DetectLowMass);
			ViewLine[ViewLineLength-1].y =(float)0.01;
			ViewLine[ViewLineLength-2].y =(float)0.01;
			ViewLine[ViewLineLength-1].x =zoomstack[zoompointer].maxx;
			ViewLine[ViewLineLength-2].x =(float)pDoc->MassToFreq(pDoc->PS.DetectLowMass);
		}
		// finished testing minx and maxx
	}

	rangepts = (maxx-minx)/float(ViewLineLength-mini-maxi);
	ASSERT(rangepts<=(maxpts/ViewLineLength));
	scale=0;

	if ((maxx-minx)<ViewLineLength) {  // if zoomed in far enough, plot all the points
		ViewLineLength=maxx-minx;
		for(i=mini;i<ViewLineLength-maxi;i++) {
			ViewLine[i].y =float(data[minx+i]);
			ViewLine[i].x =(float)pDoc->PtToFreq(minx+i);
			scale= (scale>ViewLine[i].y) ? scale : ViewLine[i].y;
			if (flags.IsZoomingY) {
				if (ViewLine[i].y >maxy) ViewLine[i].y =maxy;
				if (ViewLine[i].y <miny) ViewLine[i].y =miny;
			}
		}
		rangepts=1;
	} else { // plot both minimum and maximum points
		for (i=mini; i<ViewLineLength-maxi; i++) {
			beginpt = long(i*rangepts)+minx;
			ASSERT(beginpt>=0 && beginpt<=maxpts);
			maxdata = data[beginpt];
			for (j=0; j<rangepts; j++) { // find max point on range
				maxdata = (maxdata > data[beginpt+j]) ? maxdata : data[beginpt+j];
				}
			ViewLine[i].y = float(maxdata);
			ViewLine[i].x = (float)pDoc->PtToFreq(beginpt+j/2);
			scale= (scale>maxdata) ? scale : maxdata;
			if (flags.IsZoomingY) {
				if (ViewLine[i].y>maxy) ViewLine[i].y=maxy;
				if (ViewLine[i].y<miny) ViewLine[i].y=miny;
			}
		} // end for
	}
	ScaleViewLine();
	ViewLineMaxx = float(zoompointer[zoomstack].minx);
	ViewLineMinx = float(zoompointer[zoomstack].maxx);
	flags.CalcViewLine = FALSE;
  // end CSwirlyView::CalcFrequencyViewLine()
}

void CSwirlyView::CalculateMassViewLine()
{
	float *data,noise;
	int i, avptr=0;
	float rangepts; // points per sample range
	long currentpt=0,beginpt,endpt, oldendpt, k, tempm;
	double maxdata, mindata, avgmax[10];
	double beginmass, endmass, massstep;
	long minx, maxx, temp, maxpts,nullpts;
	float miny, maxy;
	
	for (k=0; k<10; k++) avgmax[k]=0;

	data=pDoc->FreqArray;
	// trace the actual data values here
	CString str;

	// diagnostic check of the data.  Should always be positive due to magnitude
	// mode calculation
#ifdef _DEBUG
	for(temp=0; temp<pDoc->PS.SpectrumPoints; temp++) {
		if(data[temp]<0) {
			str.Format("Buda::CalcMassSpecViewLine data error: data value negative = %e\n", data[temp]);
			TRACE(str);
		}
	}
#endif
	// end trace

	maxpts=pDoc->PS.SpectrumPoints;
	noise=float(pDoc->PS.Spec[pDoc->CurrentSpectrumFocus].NoiseLevel);

	beginmass = zoomstack[zoompointer].minx;
	endmass = zoomstack[zoompointer].maxx;
	if (beginmass<pDoc->PS.DetectLowMass) 
		beginmass=pDoc->PS.DetectLowMass;
	if (endmass>100000.0 && pDoc->PS.DetectMode != MixerMode) 
		endmass=100000.0;
	if (endmass>pDoc->PS.DetectHighMass && pDoc->PS.DetectMode == MixerMode) 
		endmass=pDoc->PS.DetectHighMass;
	if (endmass<pDoc->PS.DetectLowMass) endmass = pDoc->PS.DetectHighMass;
	
	minx=pDoc->MassToPt(endmass);
	maxx=pDoc->MassToPt(beginmass);
	miny=zoomstack[zoompointer].miny;
	maxy=zoomstack[zoompointer].maxy;

	if(minx>maxx) {
		temp=minx;
		minx=maxx;
		maxx=temp;
	}

	// test that the minx and maxx values are good and reset if not
	massstep=(endmass-beginmass)/ViewLineLength;
	rangepts = (maxx-minx)/float(ViewLineLength);
	ASSERT(rangepts<=(maxpts/ViewLineLength));
	scale=0;
	nullpts=0;
	oldendpt=pDoc->MassToPt(beginmass);

	if ((maxx-minx)<ViewLineLength) {  // if zoomed in far enough, plot all the points
		ViewLineLength=maxx-minx;
		for(i=0;i<ViewLineLength;i++) {
			ViewLine[i].y = float(data[minx+i]);
			ViewLine[i].x = (float)pDoc->PtToMass(minx+i);
			scale= (scale>ViewLine[i].y) ? scale : ViewLine[i].y;
		}
		rangepts=1;
	} else { // plot it spaced linearly in mass, not in points
		for (i=0; i<ViewLineLength; i++) {
			if (i==0) {
				endpt=pDoc->MassToPt(beginmass);
				oldendpt = endpt;
			}
			else endpt = oldendpt;
			beginpt = pDoc->MassToPt(beginmass+((i+nullpts+1)*massstep));
			oldendpt = beginpt;
			if(beginpt>endpt) {
				tempm=beginpt;
				beginpt=endpt;
				endpt=tempm;
			}
			if(abs(beginpt-endpt)<2) { // if only one point in the range 
				if (currentpt!=beginpt) {  // if the first time we were at this point
					currentpt=beginpt;
					ViewLine[i].y = float(data[currentpt]);
					ViewLine[i].x = float(beginmass+(i*massstep));
					scale= (scale>ViewLine[i].y) ? scale : ViewLine[i].y;
				} else { 
					// if not the first time at this point, decrement ViewLineLength
					// and increment nullpts, and decrement i so that you won't have
					// an undefined point in the ViewLine
					ViewLineLength--;
					nullpts++;
					i--;
				}
			} else {
				maxdata=data[beginpt];
				mindata=data[beginpt];
				for (k=beginpt; k<endpt; k++) { // find max point on range
					maxdata = (maxdata > data[k]) ? maxdata : data[k];
					mindata = (mindata < data[k]) ? mindata : data[k];
				}
				if (maxdata>noise) {
					// if a peak, plot min and max, except:
					//    for min, plot greater of min or 
					//    a rolling array which contains the maxdata
					//    of the last 10 pts which were not peaks
					ViewLine[i].y = float(maxdata);
					ViewLine[i].x = float(beginmass+(i*massstep));
					scale= (scale>ViewLine[i].y) ? scale : ViewLine[i].y;	
					i++;
					if (mindata<noise) {
						if (mindata<avgmax[avptr]) {
							mindata = avgmax[avptr];
							avptr++;
							if (avptr>=10) avptr=0;
						}
					}
					ViewLine[i].y = float(mindata);
					ViewLine[i].x =float(beginmass+(i*massstep));
				} else {  // not a peak
					// this is not a peak because maxdata<noise
					// so, store maxdata and increment ptr being
					// careful not to increment above 10 pts
					ViewLine[i].y = float(maxdata);
					ViewLine[i].x =float(beginmass+(i*massstep));
					scale= (scale>ViewLine[i].y) ? scale : ViewLine[i].y;	
					avgmax[avptr]=maxdata;
					avptr++;
					if (avptr>=10) avptr=0;
				}
			}
		} // end for i
	} // end if zoomed in far enough

	// now zoom y if necessary
	ScaleViewLine();
	ViewLineMaxx = float(pDoc->PS.DetectHighMass);
	ViewLineMinx = float(pDoc->PS.DetectLowMass);
	flags.CalcViewLine = FALSE;
	//CurrentFreqArrayPtr = pDoc->CurrentSpectrumFocus;
} // end CSwirlyView::CalcMassSpecViewLine()


void CSwirlyView::CalculateRICViewLine()
{

}

void CSwirlyView::CalculateTICViewLine()
{

}

void CSwirlyView::CalculateStickplotViewLine()
{

}

void CSwirlyView::CalculateDeconViewLine()
{

}

void CSwirlyView::OnSwirlyMass() 
{
	flags.CalcViewLine = TRUE;
	flags.PlotYAxis = FALSE;
	PlotType = MASSSPEC;	
	InitializeZoomStack();
	Invalidate(TRUE);
}

void CSwirlyView::OnSwirlyFrequency() 
{
	flags.CalcViewLine = TRUE;
	flags.PlotYAxis = FALSE;
	PlotType = FREQUENCY;
	InitializeZoomStack();
	Invalidate(TRUE);
}

void CSwirlyView::DrawGenericAxis(int PlotStart, int PlotEnd, float DataStart, float DataEnd, CString caption, CString labelformat)
{
	// this function draws a time axis out to the last event
	// in the pulse sequence
	int AxisLineXstart = PlotStart;
	int AxisLineXend = PlotEnd;
	long i;
	double MassStart = DataStart;
	double MassEnd = DataEnd;

	double massbase = (double)MassEnd - (double)MassStart;

	// second draw the axis line 
	MoveTo(AxisLineXstart,AxisLineYpos);
	LineTo(AxisLineXend,AxisLineYpos);

	// place tics
	// there will be <=10 tics
	int ticpos[10];
	double ticmass[10];
	for(i=0; i<10; i++) {
		ticpos[i] = 0;
		ticmass[i] = 0;
	}

	// there will be 5-10 tics  
	// ticsteps can be (order of magnitude of massbase) / 1, 2, 5 
	double a = log10(massbase);
	double b = floor(a);	// b = order of magnitude of massbase
	double ticstep = pow(10,b);
	double nt;	
	int numtics;
	nt = massbase/ ticstep;

	if(nt>=5) numtics = (int)nt;
	if(nt>2 && nt<5) {
		ticstep=pow(10,b)/2;
		numtics = (int)(massbase/ticstep);
	}
	if(nt<=2) {
		ticstep = pow(10,b)/5;
		numtics = (int)(massbase/ticstep);
	}

	// find the position of the first tic.
	// if zero is in the range, I want zero to be one of the ticx
	double NumTixb4Zero = (double)MassStart/ticstep;
	if(NumTixb4Zero>0) {// GenPS starts after zero
		ticmass[0]=(floor(NumTixb4Zero)+1) * ticstep;
	}
	if (NumTixb4Zero == 0) ticmass[0] = 0;
	if (NumTixb4Zero < 0) { // GenPS starts before zero
		ticmass[0] = -1 * floor(-1*NumTixb4Zero) * ticstep;
	}

	// find the position of each tic relative to ticmass[0]
	for(i=1; i<10; i++) {
		ticmass[i]=ticstep*i+ticmass[0];
		if(ticmass[i]>MassEnd) break;
	}
	numtics = i-1;

	// find tic x positions
	// remember that position 1000 = FirstEventStart
	// and 31768 = LastEventEnd;
				// create a linear eqn y = a*x + b
				// where y's are the plotspace #'s and 
				// x's are the massbase space #'s
	a = (AxisLineXend-AxisLineXstart)/(MassEnd - MassStart);
	b = AxisLineXstart - a*MassStart;
	for(i=0; i<numtics; i++) {
		ticpos[i] = int(a*ticmass[i] + b);
	}

	// now actually plot the tics
	for(i=0; i<numtics; i++) {
		MoveTo(ticpos[i],AxisLineYpos);
		LineTo(ticpos[i],AxisLineYpos-500);
	}

	// lable tics
	CString str;
	CPoint pt;
	CSize sz;

	pt.x = 16384;
	pt.y = AxisLineYpos - 2000;
	Text(caption,pt,CENTER_JUSTIFIED);

	// probably will want to decide on how to print these numbers.
	// won't want %1.1f each time.
	for(i=0; i<numtics; i++) {
		str.Format(labelformat,ticmass[i]);
		pt.x = ticpos[i];
		pt.y = AxisLineYpos - 600;
		Text(str,pt,CENTER_JUSTIFIED);
	}
	// end of draw axis
}

void CSwirlyView::OnRButtonDown(UINT nFlags, CPoint point) 
{
	// TODO: Add your message handler code here and/or call default
	
	CView::OnRButtonDown(nFlags, point);
}

void CSwirlyView::OnLButtonUp(UINT nFlags, CPoint point) 
{
	flags.Zooming = FALSE;	
	ZoomEndPt = point;
	// now calculate the new zoomstack points, reset flag.CalcViewLine, and redraw.
	CView::OnLButtonUp(nFlags, point);
}

void CSwirlyView::OnLButtonDown(UINT nFlags, CPoint point) 
{
	flags.Zooming = TRUE;	
	ZoomInitPt = point;
	CView::OnLButtonDown(nFlags, point);
}

void CSwirlyView::OnMouseMove(UINT nFlags, CPoint point) 
{
	if(flags.Zooming) {
		CPoint pt;
		CPen ErasePen(PS_SOLID, 1, pDC->GetBkColor());
		CPen* pDrawPen;
		pDrawPen = pDC->GetCurrentPen();

		// first, remove the previous MouseMove box
		// how do i switch pens?  make it a dotted pen with color = background color
		pDC->SelectObject(&ErasePen);
		pt = ZoomInitPt;
		pDC->MoveTo(pt);
		pt.y = ZoomEndPt.y;
		pDC->LineTo(pt);
		pt.x = ZoomEndPt.x;
		pDC->LineTo(pt);
		pt.y = ZoomInitPt.y;
		pDC->LineTo(pt);
		pt.x = ZoomInitPt.x;
		pDC->LineTo(pt);

		// second, draw a new MouseMove box
		// change pens to color = foreground color
		pDC->SelectObject(pDrawPen);
		ZoomEndPt = point;
		pt.y = ZoomEndPt.y;
		pDC->LineTo(pt);
		pt.x = ZoomEndPt.x;
		pDC->LineTo(pt);
		pt.y = ZoomInitPt.y;
		pDC->LineTo(pt);
		pt.x = ZoomInitPt.x;
		pDC->LineTo(pt);
	}

	// finally, clean up and save this point.
	CView::OnMouseMove(nFlags, point);
}
