/****************************************************************

Boston University data analysis (BUDA) package 

Description: Buda is a data analysis program for Fourier 
Transform Mass Spectrometry data.  

by Peter B. O'Connor

    Copyright (C) 2000  Peter B. O'Connor

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/
#if !defined(AFX_SWIRLYVIEW_H__2D525020_EB3C_11D2_B907_004005A3C9E9__INCLUDED_)
#define AFX_SWIRLYVIEW_H__2D525020_EB3C_11D2_B907_004005A3C9E9__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000
// SwirlyView.h : header file
//

/////////////////////////////////////////////////////////////////////////////
// CSwirlyView view
#define CENTER_JUSTIFIED 0
#define LEFT_JUSTIFIED 1
#define CENTER_JUSTIFIED2 2
#define RIGHT_JUSTIFIED 3
#define MAX_VIEWLINE_LENGTH 6000

#if !defined(STRUCT_VIEWLINEPT_DEFINED)
#define STRUCT_VIEWLINEPT_DEFINED
struct ViewLinePt {
	float x;
	float y;
};
#endif

class CSwirlyView : public CView
{
protected:
	CSwirlyView();           // protected constructor used by dynamic creation
	DECLARE_DYNCREATE(CSwirlyView)

// Attributes
public:

// Operations
public:
	CPoint ZoomEndPt;
	CPoint ZoomInitPt;
	void DrawGenericAxis(int PlotStart, int PlotEnd, float DataStart, float DataEnd, CString caption, CString labelformat);
	int PlotType;
	CSize InitialWindowSize;
	void VerticalText(CString str, CPoint pt, int just);
	CPoint DataToViewPoint(struct ViewLinePt inpt, float yslope, float yintercept, float xslope, float xintercept);
	void DrawVerticalAxis();
	long DCOffset;
	float ViewLineMiny;
	float ViewLineMaxy;
	float ViewLineMinx;
	float ViewLineMaxx;
	float scale;
	float VScale;
	void InitializeZoomStack();
	void ScaleViewLine();
	void CalculateTransientViewLine();
	void CalculateFrequencyViewLine();
	void CalculateMassViewLine();
	void CalculateTICViewLine();
	void CalculateRICViewLine();
	void CalculateDeconViewLine();
	void CalculateStickplotViewLine();
	long ViewLineLength;
	struct ViewLinePt ViewLine[MAX_VIEWLINE_LENGTH];
	void DrawTransientData();
	void DrawFrequencyData();
	void DrawMassData();
	void DrawTICData();
	void DrawStickplotData();
	void DrawRICData();
	void DrawDeconData();
	void DrawFrequencyAxis();
	void DrawMassAxis();
	void DrawTICAxis();
	void DrawRICAxis();
	void DrawDeconAxis();
	void DrawTimeAxis();
	void LabelPeaks();
	void DrawData();
	void DrawParms();
	void DrawAxis();
	void PrepareView();
	CDC* pDC;
	CBudaDoc* pDoc;
	int AxisLineYpos;
	CRect ViewSize;
	CRect PlotArea;
	void MoveTo(int x, int y);
	void MoveTo(CPoint pt);
	void LineTo(int x, int y);
	void LineTo(CPoint pt);
	CPoint PTtoViewPT(CPoint pt);
	CPoint PTtoViewPT(int x, int y);
	void Text(CString str, CPoint pt, int just);
	int zoompointer;
	struct zoomwindow {
		//unsigned long minx;
		//unsigned long maxx;
		float minx;
		float maxx;
		float miny;
		float maxy;
	} zoomstack[20];
	struct FlagValues {
	public:
		BOOL Zooming;
		BOOL ViewIsSetup;
		BOOL UpdateScaleFactor;
		BOOL ClearViewLine;
		BOOL CalcViewLine;
		BOOL PlotParameters;
		BOOL RemoveDC;
		BOOL PlotYAxis;
		BOOL IsZoomingY;
		BOOL InitializedWindowSize;
	} flags;

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CSwirlyView)
	public:
	virtual void OnInitialUpdate();
	protected:
	virtual void OnDraw(CDC* pDC);      // overridden to draw this view
	//}}AFX_VIRTUAL

// Implementation
protected:
	virtual ~CSwirlyView();
#ifdef _DEBUG
	virtual void AssertValid() const;
	virtual void Dump(CDumpContext& dc) const;
#endif

	// Generated message map functions
protected:
	//{{AFX_MSG(CSwirlyView)
	afx_msg void OnSize(UINT nType, int cx, int cy);
	afx_msg void OnSwirlyMass();
	afx_msg void OnSwirlyFrequency();
	afx_msg void OnRButtonDown(UINT nFlags, CPoint point);
	afx_msg void OnLButtonUp(UINT nFlags, CPoint point);
	afx_msg void OnLButtonDown(UINT nFlags, CPoint point);
	afx_msg void OnMouseMove(UINT nFlags, CPoint point);
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

/////////////////////////////////////////////////////////////////////////////

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_SWIRLYVIEW_H__2D525020_EB3C_11D2_B907_004005A3C9E9__INCLUDED_)
