// TTLChannelList.cpp : implementation file
//
/****************************************************************

Boston University data analysis (BUDA) package 

Description: Buda is a data analysis program for Fourier 
Transform Mass Spectrometry data.  

by Peter B. O'Connor

    Copyright (C) 2000  Peter B. O'Connor

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "stdafx.h"
#include "Buda.h"
#include "TTLChannelList.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CTTLChannelList dialog


CTTLChannelList::CTTLChannelList(CWnd* pParent /*=NULL*/)
	: CDialog(CTTLChannelList::IDD, pParent)
{
	//{{AFX_DATA_INIT(CTTLChannelList)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
}


void CTTLChannelList::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CTTLChannelList)
	DDX_Control(pDX, IDC_LIST1, m_TTLList);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CTTLChannelList, CDialog)
	//{{AFX_MSG_MAP(CTTLChannelList)
	ON_LBN_DBLCLK(IDC_LIST1, OnDblclkList1)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CTTLChannelList message handlers

BOOL CTTLChannelList::OnInitDialog()
{
	BOOL retval = CDialog::OnInitDialog();
	int i;
	BOOL ChannelsListed[40];

	for(i=0; i<40; i++) {
		ChannelsListed[i]=FALSE;
		ChannelList[i]=0;
	}

	for(i=0; i<pPS->NumEvents; i++) {
		if(pPS->GenPS[i].Channel>=0 && pPS->GenPS[i].Channel<16) {
			ChannelsListed[pPS->GenPS[i].Channel]=TRUE;
		}
	}

	CString str;
	int chnum=0;
	for(i=8; i<16; i++) { 
		// note that ttls 0-7 are reserved for internal use
		if(i==QUADRF_GATE) continue;

		if(InFlags == SELECT_UNUSED) {
			if(!ChannelsListed[i]) {
				str = pDoc->sys->ChannelNames[i];
				if(str == "UNUSED") break;
				m_TTLList.AddString((LPCTSTR)str);
				ChannelList[chnum]=i;
				chnum++;
			}
		}
		if(InFlags == SELECT_CURRENT) {
			if(ChannelsListed[i]) {
				str = pDoc->sys->ChannelNames[i];
				m_TTLList.AddString((LPCTSTR)str);
				ChannelList[chnum]=i;
				chnum++;
			}
		}
	}

	if(chnum==0) {
		m_TTLList.AddString("No TTL's Available");
		m_TTLList.EnableWindow(FALSE);
		OutFlag = CANCEL;
	} else {
		if(Channel>=0 && Channel<chnum) {
			m_TTLList.SetCurSel(Channel);
		} else {
			Channel = 0;
			m_TTLList.SetCurSel(Channel);
		}
	}

	return retval;
}

void CTTLChannelList::OnCancel() 
{
	OutFlag = CANCEL;	
	Channel = -1;
	CDialog::OnCancel();
}

void CTTLChannelList::OnOK() 
{
	Channel = ChannelList[m_TTLList.GetCurSel()];	
	CDialog::OnOK();
}

void CTTLChannelList::OnDblclkList1() 
{
	OnOK();	
}
