// TextView.cpp : implementation file
//
/*
TextView.cpp is the code that implements the CTextView class.  
It is intended as an ascii text listing of the data currently
displayed in the CBudaView that can be cut-and-pasted over
to a spreadsheet for working on it directly.  It should not show
more than CBudaView.ViewLineLength data points.  
*/
/****************************************************************

Boston University data analysis (BUDA) package 

Description: Buda is a data analysis program for Fourier 
Transform Mass Spectrometry data.  

by Peter B. O'Connor

    Copyright (C) 2000  Peter B. O'Connor

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "BudaMaster.h"
#include "Buda.h"
#include "BudaDoc.h"
#include "BudaView.h"

#include "TextView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CTextView

IMPLEMENT_DYNCREATE(CTextView, CEditView)

CTextView::CTextView()
{
	pApp = (CBudaApp*)AfxGetApp();
}

CTextView::~CTextView()
{
}


BEGIN_MESSAGE_MAP(CTextView, CEditView)
	//{{AFX_MSG_MAP(CTextView)
		// NOTE - the ClassWizard will add and remove mapping macros here.
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CTextView drawing

void CTextView::OnDraw(CDC* pDC)
{
	CDocument* pDoc = GetDocument();
	SetWindowText(m_ScreenBuffer);
	pDC = pDC;
	pDoc = pDoc;
}

/////////////////////////////////////////////////////////////////////////////
// CTextView diagnostics

#ifdef _DEBUG
void CTextView::AssertValid() const
{
	CEditView::AssertValid();
}

void CTextView::Dump(CDumpContext& dc) const
{
	CEditView::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CTextView message handlers

void CTextView::OnInitialUpdate() 
{
	PDoc = (CBudaDoc *) GetDocument();

	CEditView::OnInitialUpdate();

	CreateTextViewBuffer();
	//SetMIFont();

	SetWindowText(m_ScreenBuffer);
	GetDocument()->UpdateAllViews(NULL);	
}

BOOL CTextView::Create(LPCTSTR lpszClassName, LPCTSTR lpszWindowName, DWORD dwStyle, const RECT& rect, CWnd* pParentWnd, UINT nID, CCreateContext* pContext) 
{
	dwStyle |= (WS_VSCROLL | ES_MULTILINE | ES_READONLY | ES_AUTOHSCROLL | ES_AUTOVSCROLL);
	return CWnd::Create(lpszClassName, lpszWindowName, dwStyle, rect, pParentWnd, nID, pContext);
}

void CTextView::CreateTextViewBuffer()
{
	CBudaView* vw;
	vw = (CBudaView*)pApp->m_pCurrentActiveView;
	double minx, maxx;
	minx=vw->zoomstack[vw->zoompointer].minx;
	maxx=vw->zoomstack[vw->zoompointer].maxx;

	CString str;
	CString crlf="\r\n";

	switch(vw->PlotType) {
	case TRANSIENT:
		m_ScreenBuffer="Not Currently Supported for Time Domain Plots";
		break;
	case FREQUENCY:
		m_ScreenBuffer="Not Currently Supported for Frequency Domain Plots";
		break;
	case MASSSPEC:
		str.Format("   Mass      Intensity ");
		m_ScreenBuffer = str + crlf;
		str.Format("----------  -----------");
		m_ScreenBuffer += str + crlf;
	
		long i;
		double mass, intensity;
		long minxpt,maxxpt;
		minxpt = PDoc->MassToPt(minx);
		maxxpt = PDoc->MassToPt(maxx);
		for (i=maxxpt; i<minxpt; i++) {
			mass = PDoc->PtToMass(i);
			intensity = PDoc->FreqArray[i];

			str.Format("%5.4f",mass);
			while(str.GetLength()<10) str = " "+str;
			m_ScreenBuffer += str;

			str.Format("%2.2f",intensity);
			while(str.GetLength()<8) str = " "+str;
			m_ScreenBuffer += str + crlf;
		}
		break;
	}
}

void CTextView::SetTextViewFont()
{
   CFont* pFont = GetFont();
   LOGFONT lf;
   if (pFont != NULL)
	   pFont->GetObject(sizeof(LOGFONT), &lf);
   else
	   ::GetObject(GetStockObject(SYSTEM_FONT), sizeof(LOGFONT), &lf);

	lf.lfPitchAndFamily = FIXED_PITCH | FF_MODERN;
	lf.lfWidth = 5;
	lf.lfHeight = 12;
	lf.lfWeight = 10;
	//char fnt[32]="Courier\0";
	//lf.lfFaceName = "Courier                       \0";
		// switch to new font.
	m_font.DeleteObject();
	if (m_font.CreateFontIndirect(&lf))
	{
		CWaitCursor wait;
		SetFont(&m_font);
		m_lfDefFont = lf;
	}
}
