// Instrument.cpp: implementation of the CInstrument class.
//
//////////////////////////////////////////////////////////////////////
/*

The Instrument.cpp file is the code for the CInstrument class.  
This class is intended to include all the code for reading and
writing data to the hardware drivers, keeping information on the
capabilities of the current instrument (i.e. is it MALDI or ESI
or Both), 

Instrument.cpp revision history:
Created by PBO, 1999

  PBO   3/29/2003, removed multiple sys structure format, converting
	CInstrument to one sys.
  PBO	3/29/2003, started modifying CInstrument to allow BUDA to run
  the National Instruments based data acquistion system.

*/
/****************************************************************

Boston University data analysis (BUDA) package 

Description: Buda is a data analysis program for Fourier 
Transform Mass Spectrometry data.  

by Peter B. O'Connor

    Copyright (C) 2000  Peter B. O'Connor

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "stdafx.h"
#include "Buda.h"
#include "BudaMaster.h"
#include "Instrument.h"
#include <direct.h>
#include <math.h>
#include <sys\Timeb.h>
#include "PSSetupDlg.h"
#include "StartupDiagsDlg.h"

#ifdef _DEBUG
	#undef THIS_FILE
	static char THIS_FILE[]=__FILE__;
	#define new DEBUG_NEW
#endif

// Buda debug defines
#ifdef _DEBUG
	#define DEBUG_INSTRUMENT_THREAD
	#define CHKMEM _CrtCheckMemory()
#endif

/////////////////////////////////////////////////////////////////////////////
// CInstrument

IMPLEMENT_DYNCREATE(CInstrument, CDocument)

BEGIN_MESSAGE_MAP(CInstrument, CDocument)
	//{{AFX_MSG_MAP(CInstrument)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CInstrument::CInstrument()
{
	// Diagnostic flags
	PSG_Tested = FALSE;
	VSP_Tested = FALSE;
	FSP_Tested = FALSE;
	ARB_Tested = FALSE;
	TD_Tested = FALSE;
	PSG_Passed = FALSE;
	VSP_Passed = FALSE;
	FSP_Passed = FALSE;
	ARB_Passed = FALSE;
	TD_Passed = FALSE;
	ThreadInfo.m_hwndNotifyDiagDataUpdated = 0;
	TuneMode = FALSE;
	Running = FALSE;

#ifdef ENCODE_SERIAL_NUMBER
	SerialNumber = "12345678901234567890";
#endif
	ScrambleFactor = SERIAL_NUMBER_SCRAMBLE_FACTOR;
	EncryptedSerialNumber = "*3210/.-,+*3210/.-,";
	pApp = (CBudaApp*)AfxGetApp();
	pWnd = (CMainFrame*)AfxGetApp()->m_pMainWnd;

	int j;
	sys.UserID = "Luke Skywalker";		
	sys.SystemName = "Luke Skywalker";			
	sys.SystemSerialNumber = "Luke Skywalker";
	sys.DataPath = "Luke Skywalker";	
	sys.BudaPath = "Luke Skywalker";
	sys.TempPath = "Luke Skywalker";	
	sys.UserIDFile = "Luke Skywalker";	
	sys.UserSetupFile = "Luke Skywalker";  
	sys.UserDir = "Luke Skywalker";
	sys.CalibrationFile = "Luke Skywalker";	
	sys.CalDate = "Luke Skywalker";	
	sys.CalDescription = "Luke Skywalker";
	sys.DefaultSetupFile = "Luke Skywalker";
	for(j=0; j<40; j++) sys.ChannelNames[j] = "Luke Skywalker";

	// setup sys.ADC rate lists
	sys.ADCRateList[0] = 16e6;
	sys.ADCRateList[1] = 8e6;
	sys.ADCRateList[2] = 4e6;
	sys.ADCRateList[3] = 2e6;
	sys.ADCRateList[4] = 1e6;
	sys.ADCRateList[5] = 500e3;
	sys.ADCRateList[6] = 200e3;
	sys.ADCRateList[7] = 100e3;
	sys.ADCRateList[8] = 50e3;
	sys.ADCRateList[9] = 20e3;
	sys.ADCRateList[10] = 10e3;
	sys.ADCRateList[11] = 1e3;
	sys.ADCRateList[12] = 5e3;
	sys.ADCRateList[13] = 2e3;
	sys.ADCRateList[14] = -1;
	sys.ADCRateList[15] = -1;

	sys.ADCRateString[0] = "16 MHz";
	sys.ADCRateString[1] = "8 MHz";
	sys.ADCRateString[2] = "4 MHz";
	sys.ADCRateString[3] = "2 MHz";
	sys.ADCRateString[4] = "1 MHz";
	sys.ADCRateString[5] = "500 kHz";
	sys.ADCRateString[6] = "200 kHz";
	sys.ADCRateString[7] = "100 kHz";
	sys.ADCRateString[8] = "50 kHz";
	sys.ADCRateString[9] = "20 kHz";
	sys.ADCRateString[10] = "10 kHz";
	sys.ADCRateString[11] = "1 kHz";
	sys.ADCRateString[12] = "5 kHz";
	sys.ADCRateString[13] = "2 kHz";
	sys.ADCRateString[14] = "Unknown";
	sys.ADCRateString[15] = "Unknown";

	sys.ADCRateAvailable[0] = FALSE;
	sys.ADCRateAvailable[1] = FALSE;
	sys.ADCRateAvailable[2] = FALSE;
	sys.ADCRateAvailable[3] = FALSE;
	sys.ADCRateAvailable[4] = FALSE;
	sys.ADCRateAvailable[5] = FALSE;
	sys.ADCRateAvailable[6] = FALSE;
	sys.ADCRateAvailable[7] = FALSE;
	sys.ADCRateAvailable[8] = FALSE;
	sys.ADCRateAvailable[9] = FALSE;
	sys.ADCRateAvailable[10] = FALSE;
	sys.ADCRateAvailable[11] = FALSE;
	sys.ADCRateAvailable[12] = FALSE;
	sys.ADCRateAvailable[13] = FALSE;
	sys.ADCRateAvailable[14] = FALSE;
	sys.ADCRateAvailable[15] = FALSE;

	PS.cfp = 0;
	PS.arb = 0;
	PS.dac = 0;
	PS.GenPS = 0;
	PS.Spec = 0;

	SourceName = "HiPressureMALDI";
	ActiveSystem = 0;
	acqScansInMemory = 0;
	acqScansInTD=0;

}

BOOL CInstrument::OnNewDocument()
{
	if (!CDocument::OnNewDocument())
		return FALSE;
	return TRUE;
}

CInstrument::~CInstrument()
{
	if(PS.cfp != 0) {
		delete [] PS.cfp;
		PS.cfp = 0;
	}
	if(PS.dac != 0) {
		delete [] PS.dac;
		PS.dac = 0;
	}
	if(PS.arb != 0) {
		delete [] PS.arb;
		PS.arb = 0;
	}
	if(PS.GenPS != 0) {
		delete [] PS.GenPS;
		PS.GenPS = 0;
	}
	if(PS.Spec != 0) {
		delete [] PS.Spec;
		PS.Spec = 0;
	}
}

/////////////////////////////////////////////////////////////////////////////
// CInstrument diagnostics

#ifdef _DEBUG
void CInstrument::AssertValid() const
{
	CDocument::AssertValid();
}

void CInstrument::Dump(CDumpContext& dc) const
{
	CDocument::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CInstrument serialization

void CInstrument::Serialize(CArchive& ar)
{
	if (ar.IsStoring())
	{
		// TODO: add storing code here
	}
	else
	{
		// TODO: add loading code here
	}
}

/////////////////////////////////////////////////////////////////////////////
// CInstrument commands

struct sys* CInstrument::GetSys()
{
	return &(sys);
}

BOOL CInstrument::Setup()
{
	ThreadInfo.ThreadStatus = -1;
	InstrumentAvailable = FALSE;

	TRACE("CInstrument::Setup()\n");

	///////////////////////////////////////////
	//			Serial number stuff          //
	///////////////////////////////////////////
	//SetSerialNumber();

	// read and parse configuration files and define sys

	// check the compiled serial number with the one read in 
	// from the configuration file
	//CString SerNum;
	//if(!CheckSerialNumber(SerNum)) {
		// shut down the program
		//return FALSE;
	//}

	///////////////////////////////////////////
	//			Setup SYS structure			 //
	///////////////////////////////////////////
	CurrentInstrument = DEFAULT_INSTRUMENT;
	InitializeSys();
	// read calibration table into sys.caldata etc
	ReadCalibrationFile();
		
	if (!ReadConfigurationFile()) {
		MessageBox("Error 1021. \nError Reading Config File Buda.cnf","Config File Error");
		return FALSE;
	}

	///////////////////////////////////////////
	//		Setup Instrument PS structure    //
	///////////////////////////////////////////
	InitializePS();
	//if(!ReadDefaultSetupFile()) {
		//MessageBox("Error 1022. \nError Reading Default Setup File","Setup File Error");
	//	return FALSE;
	//}
	return TRUE;
}

void CInstrument::MessageBox(CString str1, CString str2)
{
	pWnd->MessageBox(str1,str2, MB_OK);
}

BOOL CInstrument::ReadCalibrationFile()
{
	int i,j;
	BOOL test;
	double f, m;
	float amp, v;
	char date[40];
	CFile calfile;
	long calhdr,incl;
	CString errorstr;
	CString filestr = "File Open Error";

	char* pathname;
	pathname = new char[256];
	strcpy(pathname,pApp->ProgramDirectory);
	strcat(pathname,"MALDI.cal");

	test = calfile.Open(pathname,  CFile::modeRead, NULL);
	if (!test) {
		errorstr.Format("Couldn't open Calibration file \n %s",pathname);
		MessageBox(errorstr,filestr);
		delete pathname;
		return FALSE;
	} else {
		sys.CalibrationFile = pathname;
	}

	CArchive ar(&calfile,CArchive::load,4096,NULL);

	ar.Read(&calhdr,4);
	if (calhdr != 0xcafd) {
		errorstr.Format("File %s is not a valid calibration file",pathname);
		MessageBox(errorstr,filestr);
		return FALSE;
	}

	delete pathname;

	char cdte[40], cdesc[64];
	ar.Read(&(sys.CalibrationVersion),4);
	ar.Read(&(sys.A),8);
	ar.Read(&(sys.B),8);
	ar.Read(&(sys.HighMassShift),8);
	ar.Read(&(sys.TrapShift),8);
	ar.Read(&(sys.IntensityShift),8);
	ar.Read(&(sys.CalibrationPoints),4);
	ar.Read(&(cdte),40);
	sys.CalDate=cdte;
	ar.Read(&(cdesc),64);
	sys.CalDescription=cdesc;

	test = TRUE;
	for(i=0; i<sys.CalibrationPoints; i++) {
		if(ar.Read(&m,8)<8) {
			test = FALSE;
			break;
		}
		if(ar.Read(&f,8)<8){
			test = FALSE;
			break;
		}
		if(ar.Read(&v,4)<4) {
			test = FALSE;
			break;
		}
		if(ar.Read(&amp,4)<4) {
			test = FALSE;
			break;
		}
		if(ar.Read(&incl,4)<4) {
			test = FALSE;
			break;
		}
		if(ar.Read(&date,40)<40) {
			test = TRUE;
			break;
		}
		sys.CalData[i].ActualMass = m;
		sys.CalData[i].MeasuredFrequency = f;
		sys.CalData[i].TrapVoltage = v;
		sys.CalData[i].TotalIntensity = amp;
		sys.CalData[i].IncludeInCalc = incl;
		for (j=0; j<40; j++) {
			sys.CalData[i].TimeEntered[j] = date[j];
			if (date[j]==NULL) break;
		}
	}
	if (!test) {
		errorstr.Format("Premature End of File :\n %s",pathname);
		MessageBox(errorstr,filestr);
		return FALSE;
	}
	ar.Close();

	return TRUE;
}


BOOL CInstrument::CheckSerialNumber(CString cnfSN)
{
	CString sernum;
	sernum = GetSerialNumber();

	if (sernum != cnfSN) return FALSE;
	else return TRUE;
}

CString CInstrument::GetSerialNumber()
{
	int i, len;
	CString temp, str = EncryptedSerialNumber;
	char *s1, *s2;

	len = str.GetLength();

	s1 = str.GetBuffer(len);
	s2 = temp.GetBuffer(len);

	for(i=0; i<len; i++) {
		s2[i]=char(int(s1[len-i])+ScrambleFactor);
	}
	return temp;
}

void CInstrument::SetSerialNumber()
{
#ifdef ENCODE_SERIAL_NUMBER
	int i, len;
	CString temp, str = SerialNumber, junk;
	char *s1, *s2;

	len = str.GetLength();

	s1 = str.GetBuffer(len);
	s2 = temp.GetBuffer(len);

	for(i=0; i<len; i++) {
		s2[i]=char(int(s1[len-i])-ScrambleFactor);
	}
	junk.Format("Encrypted Serial Number = :%s:\n",temp);
	TRACE(junk);
#endif
}

inline void CInstrument::DumpToDiagFile(CString str)
{
	CHKMEM;
#ifdef DEBUG_DUMP_DIAGS_TO_FILE
	str=str+"\r\n";
	diagfile->Write(str,str.GetLength());
	TRACE(str);
#endif
}

BOOL CInstrument::ReadConfigurationFile()
{
	DumpToDiagFile("entering CInstrument::ReadConfigurationFile()");
	CString readstr, str, rightstr;

	// first open the configuration file
	if(_chdir(sys.BudaPath)==-1) {
		str.Format("Error 22j\nCouldn't change to Buda directory\n%s",sys.BudaPath);
		MessageBox(str,"Directory error!");
		return FALSE;
	}

	char* pathname;
	pathname = new char[256];
	strcpy(pathname,sys.BudaPath);
	strcat(pathname,"Buda.cnf");

	//open an archive
	if(!ConfigFile.Open(pathname,CFile::modeRead,NULL)) {
		str.Format("Error 21\nCouldn't open configuration file:\n%s",pathname);
		MessageBox(str,"File error!");
		delete pathname;
		return FALSE;
	}
	CArchive ar(&ConfigFile,CArchive::load,4096,NULL);

	//////////////////////////////////////////
	// Read in Config file.
	//
	//loop through config file reading in one line at a time and 
	//interpreting it  
	//

	sys.HiResESI = -1;	// i.e. undefined
	sys.HiResMALDI = -1;

	while(ar.ReadString(readstr)) {	// breaks if EOF
		str=CompressString(readstr);
		if (str.GetLength()==0) continue; // empty lines
		if (str.Left(1)=="#") continue; // comment lines
	
		if(str.Left(7) == "SOURCE[") {
			rightstr = GetRightString(str);
			int num;
			sscanf(str.Mid(7,1),"%d",&num);
			if(num>MAX_NUM_VACUUM_SYSTEMS || num <1) {
				MessageBox("Couldn't interpret config string :"+readstr+"\nInvalid Source number","Config File Error");
				return FALSE;
			}
			SourceName = rightstr;
		}

		// check for a source name "[name]" and stop reading if we find one
		if(str.Left(1) == "[" ) break;

		// the main engine of this function is here
		InterpretConfigFileString(readstr);

	}//end while (non-specific var loop)
	
	// initialize all sys structures by copying the default values 
	// already read into sys to all sys structures

	// now read in all the specific source info
	CString name,endname;
	BOOL GoodRegion;
	int namelength;

	ConfigFile.SeekToBegin();
	ASSERT(ConfigFile.GetPosition() == 0);
	//CArchive ar(&ConfigFile,CArchive::load,4096,NULL);

	name = "[" + SourceName + "]" ;
	endname = "[" + SourceName + "]";
	name = CompressString(name);
	endname = CompressString(endname);

	namelength = name.GetLength();
	GoodRegion = FALSE;
	if (str.Left(namelength) == name)  {
		GoodRegion = TRUE;
	}

	//loop till find ion source then read it 
	while(ar.ReadString(readstr)) {
		str=CompressString(readstr);
		if (str.Left(namelength) == name)  {
			GoodRegion = TRUE;
		}
		if(!GoodRegion) continue;
		if (str.Left(namelength+1) == endname)  {
			GoodRegion = FALSE;
			break;
		}

		InterpretConfigFileString(readstr);
	}//end while

	ar.Close();
	ConfigFile.Close();

	delete pathname;

	DumpToDiagFile("leaving CInstrument::ReadConfigurationFile()");
	return TRUE;
}

int CInstrument::FindEqualSign(CString str)
{
	int i=0;
	for(i=0; i<str.GetLength(); i++) {
		if(str.Mid(i,1)=="=") break;
	}
	return i;
}

CString CInstrument::CompressString(CString str)
{
	CString strleft, strright, outstring;
	outstring = str;
	outstring.MakeUpper();	// make string uppercase
	int i;
	for (i=0; i<outstring.GetLength(); i++) {
		if(outstring.GetAt(i) == ' ' || outstring.GetAt(i) == '\t') {
			// remove spaces and tabs from string
			strleft = outstring.Left(i);
			strright= outstring.Right(outstring.GetLength()-i-1);
			outstring = strleft + strright;
		}
	}
	return outstring;
}

CString CInstrument::GetRightString(CString str)
{
	int equalpos;
	equalpos = FindEqualSign(str)+1;
	return str.Right(str.GetLength()-equalpos);
}

CString CInstrument::CommaParse(CString str)
{
	int i = FindComma(str);
	return str.Left(i);
}

CString CInstrument::CommaReturn(CString	str)
{
	int i = FindComma(str);
	return str.Right(str.GetLength()-i-1);
}



void CInstrument::InitializeSys()
{
	int j,k;
	// this function merely sets the default sys parameters
	// so that the variables at least are defined to SOMETHING.  
	// these are overwritten and read in from the Buda.cnf
	// file if they exist = this is in ReadConfigFile()
	sys.MajorVersion = CURRENT_MAJOR_VERSION_NUMBER;
	sys.MinorVersion = CURRENT_MINOR_VERSION_NUMBER;
	sys.UserID = "Pete";
	sys.SystemName = "Pete's Development System";
	//sys.SystemSerialNumber = "9696969696969696969696969696969";
	sys.HaveImpulse			= 0;
	sys.HaveSynthesizer		= 0;
	sys.HaveArbGenerator	= 0;
	sys.HaveNBMonitor		= 0;
	sys.ProgrammableFilter	= 0;
	sys.ArbGeneratorMemory	= 1;
	sys.ArrayProcessorMemory= 2;
	sys.AdcBits				= 12;
	sys.MaxAdcRate			= 16.0e6;
	sys.DigitizerMemory		= 256;
	sys.HiResMALDI			= 0;
	sys.HiResESI			= 0;
	sys.BootMALDI			= 1;
	sys.BootESI			= 0;
	sys.ProgrammableGain	= 0;
	sys.MagnetDrift		= 0.00;
	sys.FSPType			= 2;
	sys.LaserTriggerDelay = 0.0;
	sys.FlipFlopRFPhase				= 0;
	sys.DCRemoval					= 1;
	sys.WindowFunction				= 2; // cosine window
	sys.TransientPointsUsed			= -1;
	sys.ZeroFills					= -1;
	sys.FrequencyDataType			= 1;
	sys.PeakPickerThreshold			= -1.0;
	sys.MinimumAutoThreshold		= pApp->MinimumAutoThreshold;
	sys.PeakPickerMeanFactor		= 4.0;
	sys.PeakPickerLevel				= 0.25;
	sys.PeakPickerLoopCount			= 6;
	sys.PeakPickerLowMass			= -1.0;
	sys.PeakPickerHighMass			= -1.0;

	sys.UserDir = " ";
	sys.UserIDFile	= " ";
	sys.UserSetupFile = " ";

	sys.DefaultSetupFile ="";
	sys.BudaPath = pApp->ProgramDirectory;
	sys.DataPath	= sys.BudaPath + "FTData\\";
	sys.TempPath	= sys.BudaPath + "FTData\\";
	sys.CalibrationFile = "NoCalFile";
	sys.DefaultSetupFile = "";

	sys.SystemLowMass = 10.0; // this should really be calculated from adcrate and A and B
	sys.SystemHighMass = 50000.0; // arbitrarily assigned
	sys.SystemLowFrequency = 1000.0; // arbitrarily assigned
	sys.SystemHighFrequency = sys.MaxAdcRate/2;  // nyquist limit

	sys.TransientMemoryLimit = 2*1024*1024; // 2 megapoints
	sys.TransientFFTLimit	= sys.TransientMemoryLimit;
	sys.APFFTLimit	=  1024*1024; // maximum the Array Processor can do

	sys.PsgTicksPerMillisecond = 1000;	

	sys.TrapShift		= 0.0;
	sys.IntensityShift	= 0.0;
	sys.HighMassShift	= 0.0;
	sys.AutoLabelPeaks	= 1;
	sys.AutoLabelCharge	= 0;
	sys.LabelCharge1	= 0;
	sys.ChargeLimit		= 30;
	sys.CorrectTotalIntensity = 0;
	sys.ExcludeNoResPeaks = 1;

	sys.PeakRFAmpVoltage			= 180.0;
	sys.MinRFAmpVoltage				= 1.0;
	sys.PeakQuadRFVoltage			= 600.0;
	sys.RFAmpFixedGain				= 200.0;
	sys.QuadRFFixedGain				= 200.0;
	sys.GainControlVoltageDivider	= 6.0;
	sys.SynOutputVoltage			= 0.75;
	sys.ArbOutputVoltage			= 0.60;
	sys.QuadFrequency			= 900000.0;
	sys.MixerCorrection				= 0.0;

	sys.A = 70000000.0;
	sys.B = 1.0;

	sys.CalibrationLoaded = 0;  // is there a calibration from the startup file?
	sys.CalibrationVersion = 1;
	sys.CalibrationPoints = 3;
	sys.CalDate = "12-23-68";
	sys.CalDescription = "Undefined Calibration = variables set to defaults";

		// setup the ChannelNames
	CString str;
	for(j=0; j<40; j++) {
		if(j<16) {
			str.Format("TTL Channel #%d",j+1);
		} else {
			str.Format("DAC Channel #%d",j-15);
		}
		sys.ChannelNames[j]= str;
	} //end for loop

	// set up calibration points
	for(j=0; j<MAX_CALIB_POINTS; j++) {
		sys.CalData[j].ActualMass = 0;
		sys.CalData[j].MeasuredFrequency = 0;
		sys.CalData[j].TrapVoltage = 0;
		sys.CalData[j].TotalIntensity = 0;
		sys.CalData[j].IncludeInCalc = 0;
		for(k=0; k<40; k++) sys.CalData[j].TimeEntered[k] = '\0';
	}

	// set up dacs
	for(j=0; j<MAX_DACS; j++) {
		sys.dac[j].Units = 0;
		sys.dac[j].MaxValue = +10;
		sys.dac[j].MinValue = -10;
		sys.dac[j].MaxDac = 2047;
		sys.dac[j].MinDac = -2048;
		sys.dac[j].MaxGraph = 10;
		sys.dac[j].MinGraph = -10;
		sys.dac[j].DacAvailable = 0;
	}

	// set up the ADC rate info.
	for(j=0; j<16; j++) {
		sys.ADCRateAvailable[j]=FALSE;
	}

	// setup sys.ADC rate lists
	sys.ADCRateList[0] = 16e6;
	sys.ADCRateList[1] = 8e6;
	sys.ADCRateList[2] = 4e6;
	sys.ADCRateList[3] = 2e6;
	sys.ADCRateList[4] = 1e6;
	sys.ADCRateList[5] = 500e3;
	sys.ADCRateList[6] = 200e3;
	sys.ADCRateList[7] = 100e3;
	sys.ADCRateList[8] = 50e3;
	sys.ADCRateList[9] = 20e3;
	sys.ADCRateList[10] = 10e3;
	sys.ADCRateList[11] = 1e3;
	sys.ADCRateList[12] = 5e3;
	sys.ADCRateList[13] = 2e3;
	sys.ADCRateList[14] = -1;
	sys.ADCRateList[15] = -1;

	sys.ADCRateString[0] = "16 MHz";
	sys.ADCRateString[1] = "8 MHz";
	sys.ADCRateString[2] = "4 MHz";
	sys.ADCRateString[3] = "2 MHz";
	sys.ADCRateString[4] = "1 MHz";
	sys.ADCRateString[5] = "500 kHz";
	sys.ADCRateString[6] = "200 kHz";
	sys.ADCRateString[7] = "100 kHz";
	sys.ADCRateString[8] = "50 kHz";
	sys.ADCRateString[9] = "20 kHz";
	sys.ADCRateString[10] = "10 kHz";
	sys.ADCRateString[11] = "1 kHz";
	sys.ADCRateString[12] = "5 kHz";
	sys.ADCRateString[13] = "2 kHz";
	sys.ADCRateString[14] = "Unknown";
	sys.ADCRateString[15] = "Unknown";

}


int CInstrument::FindComma(CString str)
{
	for(int i=0; i<str.GetLength()-1; i++) {
		if(str[i]==',') break;
	}
	return i;

}

void CInstrument::OnSetupPulsesequence() 
{
	CPSSetupDlg dlg;

	CDocument* curdoc;
	CFrameWnd* frame;
	CWnd* activewnd;
	CBudaApp* pApp;
	pApp = (CBudaApp*)AfxGetApp();

	activewnd=pApp->m_pMainWnd->GetFocus();
	frame = activewnd->GetParentFrame();
	curdoc = frame->GetActiveDocument();

	// first, give CPSSetupDlg the PS structure
	dlg.PS = &PS;
	dlg.pDoc = (CBudaDoc*)curdoc;
	dlg.sys = &sys;

	// then domodal
	dlg.DoModal();	
}

void CInstrument::SortGenPS()
{
	// sort genps in channel order removing no_events
	int i,j;
	struct PSDesc temp;
	
	// first move everything that isn't a no_event to the top.
	for(i=0; i<PS.NumEvents; i++) {
		if(PS.GenPS[i].Type != NO_EVENT) {
			// for each event that is not a NO_EVENT
			for(j=i; j>0; j--) {
				// bubble that event up until it finds the
				// the first non NO_EVENT event
				if(j==0) break;
				if(PS.GenPS[j-1].Type != NO_EVENT) break;
				ShiftLinks(j,j-1);
				temp = PS.GenPS[j-1];
				PS.GenPS[j-1] = PS.GenPS[j];
				PS.GenPS[j]=temp;
			}
		}
	}

	// before the channel sort, survey the GenPS, and set each NO_EVENT
	// to Channel #40, set each bogus event (channel#'s <0 or >40) = channel#40
	for(i=0; i<PS.NumEvents; i++) {
		if(PS.GenPS[i].Channel<0 || PS.GenPS[i].Channel>40) PS.GenPS[i].Channel=40;
		if(PS.GenPS[i].Type == NO_EVENT) PS.GenPS[i].Channel=40;
	}

	// second, sort the events in increaing order by channel
	for(i=0; i<PS.NumEvents; i++) {
		if(PS.GenPS[i].Channel < PS.GenPS[i-1].Channel) {
			// if current event channel < last event channel
			for(j=i; j>0; j--) {
				// bubble the current event up until it is 
				// no longer < last event channel
				if(PS.GenPS[j].Channel >= PS.GenPS[j-1].Channel) break;
				ShiftLinks(j,j-1);
				temp = PS.GenPS[j-1];
				PS.GenPS[j-1] = PS.GenPS[j];
				PS.GenPS[j]=temp;
			}
		}
	}

	// third, for each channel, sort the events by time
	int currentchannel = 0;
	int firstindex = 0;
	for(i=0; i<PS.NumEvents; i++) {
		if(PS.GenPS[i].Channel != currentchannel) {
			// if we have come to a channel boundary, 
			// reset current channel and increment i loop
			currentchannel = PS.GenPS[i].Channel;
			firstindex = i;
			continue;
		}

		if(EventTime(i) < EventTime(i-1)) {
			// if current event time < last event event time
			for(j=i; j>firstindex+1; j--) {
				// bubble the current event up until it is 
				// no longer < last event time or until it reaches
				// channel boundary
				if(EventTime(j) >= EventTime(j-1)) break;
				ShiftLinks(j,j-1);
				temp = PS.GenPS[j-1];
				PS.GenPS[j-1] = PS.GenPS[j];
				PS.GenPS[j]=temp;
			}
		}
	}

	// finally, run through and count the number of events
	// that are not No_Event and reset PS.NumEvents
	int Events = 0;
	for (i=0; i<MAX_PS_EVENTS; i++) {
		if(PS.GenPS[i].Type != NO_EVENT) Events++;
	}
	// ok, now, if there is some problem with NumEvents, this
	// should fix it.
	if(Events>PS.NumEvents) {
		PS.NumEvents = TOTAL_PS_EVENTS;
		SortGenPS();
	} else {
		PS.NumEvents = Events;
	}
}

float CInstrument::EventTime(int index)
{
	// this function returns the absolute time of the
	// start GenPS event given by index
	float start, finish;

	GetEventEdges(index, &start, &finish);
	return start;
}

int CInstrument::ShiftLinks(int event1, int event2)
{
	// this function checks GenPS for links to event1
	//  and if it finds any, shifts them to event2

	int i, numlinksfound = 0;
	for (i=0; i<PS.NumEvents; i++) {
		if(PS.GenPS[i].Linking != NO_LINK && PS.GenPS[i].Link2Event == event1) {
			PS.GenPS[i].Link2Event = event2;
			numlinksfound++;
		}
	}
	return numlinksfound;
}

void CInstrument::GetEventEdges(long EventIndex, float *start, float *finish)
{
	// this function searches the GenPS event array to return the 
	// start and end time of event EventIndex
	// usually, this is trivial, but if the events are linked,
	// then we need to find them

	*start = 0.0;
	*finish = 0.0;

	// return on bad events
	if(EventIndex < 0) return;
	if(EventIndex > PS.NumEvents) return;
	if(EventIndex > TOTAL_PS_EVENTS) return;
	if(PS.GenPS[EventIndex].Type == NO_EVENT) return;

	// if not linked, then we are done
	if(PS.GenPS[EventIndex].Linking == NO_LINK) {
		*start = PS.GenPS[EventIndex].Delay;
		*finish = *start + PS.GenPS[EventIndex].Width;
		return;
	}

	// if linked, then we need to get the edges of the linked pulse
	// this part is a recursive function call
	float parentstart, parentend;
	// check for the simplest infinite loop situation
	if(EventIndex == PS.GenPS[EventIndex].Link2Event) return;

	GetEventEdges(PS.GenPS[EventIndex].Link2Event,&parentstart,&parentend);

	// if linking to the beginning of the link2event
	if(PS.GenPS[EventIndex].Linking == LINK_BEG) {
		*start = parentstart + PS.GenPS[EventIndex].Delay;
		*finish = *start + PS.GenPS[EventIndex].Width;
	}

	// if linking to the end of the lin2event
	if(PS.GenPS[EventIndex].Linking == LINK_END) {
		*start = parentend + PS.GenPS[EventIndex].Delay;
		*finish = *start + PS.GenPS[EventIndex].Width;
	}	
}

void CInstrument::InitializePS()
{
	// this function defines the default PS structure
	// for the system.  It is expected that this structure
	// will be overwritten by ReadDefaultSetupFile(), but 
	// if there is an error on reading, these values will
	// remain

	// the CInstrument::PS structure is the one that will
	// be used for data acquisition.  It will be different
	// from the CBudaDoc::PS's that are around, but the
	// user will be able to copy the CBudaDoc::PS to 
	// this one for data acquisition

	CString str;
	int i,j;

	PS.FileType = SETUP_FILE;
	PS.MajorVersion = sys.MajorVersion;
	PS.MinorVersion = sys.MinorVersion;

	//// fixed length strings ////
	//fixed length string SystemName
	for(i=0; i<128; i++) {
		if(i>=sys.SystemName.GetLength()) break;;
		PS.SystemName[i]=sys.SystemName.GetAt(i);
	}
	PS.SystemName[i]='\0';
	//fixed length string SystemSerialNumber
	for(i=0; i<32; i++) {
		if(i>=sys.SystemSerialNumber.GetLength()) break;;
		PS.SystemSerialNumber[i]=sys.SystemSerialNumber.GetAt(i);
	}
	PS.SystemSerialNumber[i]='\0';
	//fixed length string UserID
	for(i=0; i<8; i++) {
		if(i>=sys.UserID.GetLength()) break;;
		PS.UserID[i]=sys.UserID.GetAt(i);
	}
	PS.UserID[i]='\0';
	//fixed length string DataFileName
/*	for(i=0; i<64; i++) {
		if(i>=sys.DefaultSetupFile->GetLength()) break;;
		PS.DataFileName[i]=sys.DefaultSetupFile->GetAt(i);
	}
*/
	PS.DataFileName[i]='\0';
	str = "time acquired init'd to none  ";
	//fixed length string TimeAcquired
	for(i=0; i<40; i++) {
		if(i>=str.GetLength()) break;;
		PS.TimeAcquired[i]=str.GetAt(i);
	}
	PS.TimeAcquired[i]='\0';
	str = "time processed init'd to none  ";
	//fixed length string TimeAcquired
	for(i=0; i<32; i++) {
		if(i>=str.GetLength()) break;;
		PS.TimeProcessed[i]=str.GetAt(i);
	}
	PS.TimeAcquired[i]='\0';
	str = "No Experiment Defined - Using setup defaults";
	//fixed length string ExperimentMemo
	for(i=0; i<60; i++) {
		if(i>=str.GetLength()) break;;
		PS.ExperimentMemo[i]=str.GetAt(i);
	}
	PS.ExperimentMemo[i]='\0';
	str = "No Experiment Defined - Using setup defaults";
	//fixed length string ExperimentNotes
	for(i=0; i<600; i++) {
		if(i>=str.GetLength()) break;;
		PS.ExperimentNotes[i]=str.GetAt(i);
	}
	PS.ExperimentNotes[i]='\0';
	//// end fixed length strings ////

	PS.HiResMALDI = 0;
	PS.HiResESI = 0;
	PS.DetectMode = DirectMode;
	PS.ADCGain = 0;
	PS.DetectEvent = 0;
	PS.DetectLowMass = 100;
	PS.DetectHighMass = 10000;
	PS.MonitorMass = 1000;
	PS.MixerFrequency = 100000.0;
	PS.AdcRate = 1000000.0;
	PS.ScanMode = 0;
	PS.TransientPoints = 16*1024;
	PS.SpectraRequested = 1;
	PS.SpectraAcquired = 1;
	PS.CurrentSpectrum = 0;
	PS.ScansPerSpectrum = 1;
	PS.ScansRequested = 1;
	PS.TriggerMode = 0;
	PS.CalibrationVersion = sys.CalibrationVersion;
	PS.A = sys.A;
	PS.B = sys.B;
	PS.HighMassShift = sys.HighMassShift;
	PS.TrapShift = sys.TrapShift;
	PS.IntensityShift = sys.IntensityShift;
	PS.PeakMatching = 0;
	PS.PeakMatchFactor = 0;
	str = "time acquired init'd to none  ";
	for(i=0; i<MAX_REF_PEAK; i++) {
		PS.RefPeak[i].ActualMass = 1000;
		PS.RefPeak[i].MeasuredFrequency = 100000;
		//fixed length string TimeEntered
		for(j=0; j<40; j++) {
		if(j>=str.GetLength()) break;;
			PS.RefPeak[i].TimeEntered[j]=str.GetAt(j);
		}
		PS.RefPeak[j].TimeEntered[i]='\0';
		PS.RefPeak[i].TotalIntensity = 1;
		PS.RefPeak[i].TrapVoltage = (float)0.46066;
	}
	PS.SpectrometerType = 0;
	PS.DCRemoval = 1;
	PS.WindowFunction = 1;
	PS.ZeroFills = 0;
	PS.TruncateTransient = 0;
	PS.TruncatePoints = 0;
	PS.TruncateOffset = 0;
	PS.FFTSize = PS.TransientPoints;
	PS.SpectrumPoints = PS.FFTSize/2;
	PS.FrequencyDataType = 0; // what is this
	PS.ZoomFactor = 1;
	PS.ZoomLowMass = 1000;
	PS.ZoomHighMass = 1010;
	PS.ZoomFrequencyShift = 1;
	PS.ZoomTransientPoints = 1024*4;
	PS.ZoomPointShift = 1;
	PS.TransientIsStored = 0;
	PS.FrequencyIsStored = 0;
	PS.MITableIsStored = 0;
	PS.NumEvents = 1;

	// fill in ChirpPacket, DacPacket, ArbPacket, GenPS
	// first, create the arrays
	if(PS.cfp != 0) {
		delete [] PS.cfp;
		PS.cfp = 0;
	}
	if(PS.dac != 0) {
		delete [] PS.dac;
		PS.dac = 0;
	}
	if(PS.arb != 0) {
		delete [] PS.arb;
		PS.arb = 0;
	}
	if(PS.GenPS != 0) {
		delete [] PS.GenPS;
		PS.GenPS = 0;
	}
	PS.cfp = new struct ChirpPacket[MAX_SYN_TRIGGERS];
	PS.dac = new struct DacPacket[MAX_DACS];
	PS.arb = new struct ArbPacket[MAX_ARB_TRIGGERS];
	PS.GenPS = new struct PSDesc[MAX_PS_EVENTS];

	for(i=0; i<MAX_SYN_TRIGGERS; i++) {
		PS.cfp[i].Type = CFP_NONE;
		PS.cfp[i].Amplitude = 0;
		PS.cfp[i].StartMass = 1000;
		PS.cfp[i].EndMass = 2000;
	}

	for(i=0; i<MAX_ARB_TRIGGERS; i++) {
		PS.arb[i].WindowLowMass = 400;
		PS.arb[i].WindowHighMass = 1000;
		PS.arb[i].DacFixed = 0;
		PS.arb[i].SamplesFixed = 0;
		PS.arb[i].FilterFixed = 0;
		PS.arb[i].AmplitudeFixed = 0;
		PS.arb[i].DacRate = 100000;
		PS.arb[i].Samples = 16*1024;
		PS.arb[i].Filter = 0;
		PS.arb[i].Amplitude = 0;
		PS.arb[i].Setup = 0;
		PS.arb[i].PacketSize = 0;
		PS.arb[i].PacketSpacing = 1;
		for(j=0; j<MAX_ARB_EVENTS; j++) {
			PS.arb[i].Event[j].Type = ARB_NONE;
			PS.arb[i].Event[j].fit = 0;
			PS.arb[i].Event[j].MassA = sys.A;
			PS.arb[i].Event[j].MassB = sys.B;
			PS.arb[i].Event[j].Amplitude = 0;
			PS.arb[i].Event[j].reserved = 0;
		}
	}

	for(i=0; i<MAX_DACS; i++) {
		PS.dac[i].Units = 0;
		PS.dac[i].MaxValue = 10;
		PS.dac[i].MinValue = -10;
		PS.dac[i].MaxDac = 2048;
		PS.dac[i].MinDac = -2047;
		PS.dac[i].MaxGraph = 10;
		PS.dac[i].MinGraph = -10;
		PS.dac[i].DacAvailable = 0;
	}

	for(i=0; i<MAX_PS_EVENTS; i++) {
		PS.GenPS[i].Channel = NO_CHANNEL; // the NO_CHANNEL channel
		PS.GenPS[i].Type = NO_EVENT;
		PS.GenPS[i].Linking = NO_LINK;
		PS.GenPS[i].Link2Event = NO_CHANNEL;
		PS.GenPS[i].Delay = 0;
		PS.GenPS[i].Width = 0;
		PS.GenPS[i].Value = 0;
	}

	// create struct Spectrum and fill it in
	if(PS.Spec != 0) {
		delete [] PS.Spec;
		PS.Spec = 0;
	}
	PS.Spec = new struct Spectrum[PS.SpectraAcquired];
	for(i=0; i<PS.SpectraAcquired; i++) {
		PS.Spec[i].ElapsedTime = 0;
		PS.Spec[i].IsMIed = FALSE;
		PS.Spec[i].IsFFTed = FALSE;
		PS.Spec[i].NoiseLevel = 0;
		PS.Spec[i].NumPeaks = 0;
		PS.Spec[i].ScaleFactor = 100;
		PS.Spec[i].TotalIonInt = 0;
		PS.Spec[i].FITable = 0;
		PS.Spec[i].TArrayFileOffset = 0;
	}
	//////////////////////////////////////////////////////////
	// finished creating the instrument's default PS structure
	// hopefully, these values will never be seen
}

BOOL CInstrument::ReadDefaultSetupFile()
{
/*	long MajorVersion, MinorVersion, FileType, inpts=0; 
	CFile arf;
	CString str;

	// get the default setup file name from sys
	// sys got it from the configuration file

	if(sys.DefaultSetupFile->IsEmpty()) {
		return FALSE;	// filename undefined
	}

	// open the file and create the archive
	if(!arf.Open(LPCTSTR(sys.DefaultSetupFile),CFile::modeRead,NULL)) {
		str.Format("Error 23\nCouldn't open default setup file:\n%s",sys.DefaultSetupFile);
		MessageBox(str,"File error!");
		return FALSE;
	}

	// first read in version info
	inpts+=arf.Read(&FileType,4);
	inpts+=arf.Read(&MajorVersion,4);
	inpts+=arf.Read(&MinorVersion,4);

	if(inpts<12) {
		MessageBox("Error Reading Version Number\nFile truncated?","File error!");
	}

	// then reset file to point to beginning of file
	arf.SeekToBegin();
		
	// then read in the particular version
	switch(MajorVersion) {
	case 6:
		ReadVersion6SetupFile(&arf);
		break;
		// end of loading version 6
	case 8:
		ReadVersion8SetupFile(&arf);
		break;
		// end of loading version 8
	default:
		MessageBox("Cannot Recognize Version Number","File error!");
		return FALSE;
	}
*/
	return TRUE;
}

BOOL CInstrument::StartupDiagnostics()
{	
	//  This is the function for running all electronics
	//  and software diagnostics.  If they fail so that 
	//  the instrument cannot be run, return FALSE.
	DiagnosticsComplete = FALSE;
	CString crlf = "\r\n";

	// future method is simpler.  
	int ErrorCode = 0;
	CString ErrorString = "No Error";
	//ErrorCode = RunDiagnostics();
	switch (ErrorCode) {
	case 0:
		ErrorString = "";
		break;
	case 101:
		ErrorString = "PSG Error E101 - PSG reset did not clear busy flag";
		break;
	case 102:
		ErrorString = "PSG Error E102 - PSG memory has bit stuck low";
		break;
	case 103:
		ErrorString = "PSG Error E103 - PSG memory has bit stuck high";
		break;
	case 104:
		ErrorString = "PSG Error E104 - PSG did not start from host command";
		break;
	case 105:
		ErrorString = "PSG Error E105 - PSG timing program finished early";
		break;
	case 106:
		ErrorString = "PSG Error E106 - PSG timing program finished late or never";
		break;
	case 201:
		ErrorString = "VSP Error E201 - VSP reset did not clear busy flag";
		break;
	case 202:
		ErrorString = "VSP Error E202 - VSP memory has bit stuck low";
		break;
	case 203:
		ErrorString = "VSP Error E203 - VSP memory has bit stuck high.";
		break;
	case 204:
		ErrorString = "VSP Error E204 - VSP did not start from host command";
		break;
	case 205:
		ErrorString = "VSP Error E205 - VSP timing program finished early";
		break;
	case 206:
		ErrorString = "VSP Error E206 - VSP timing program finished late or never";
		break;
	case 207:
		ErrorString = "VSP Error E207 - VSP did not start from PSG trigger";
		break;
	case 301:
		ErrorString = "FSP Error E301 - FSP reset did not clear busy flag";
		break;
	case 302:
		ErrorString = "FSP Error E302 - FSP memory has bit stuck low";
		break;
	case 303:
		ErrorString = "FSP Error E303 - FSP memory has bit stuck high.";
		break;
	case 304:
		ErrorString = "FSP Error E304 - FSP did not start from host command";
		break;
	case 305:
		ErrorString = "FSP Error E305 - FSP timing program finished early";
		break;
	case 306:
		ErrorString = "FSP Error E306 - FSP timing program finished late or never";
		break;
	case 307:
		ErrorString = "FSP Error E307 - FSP did not start from PSG trigger";
		break;
	case 401:
		ErrorString = "ARB Error E401 - ARB reset did not clear busy flag";
		break;
	case 402:
		ErrorString = "ARB Error E402 - ARB memory has bit stuck low";
		break;
	case 403:
		ErrorString = "ARB Error E403 - ARB memory has bit stuck high.";
		break;
	case 404:
		ErrorString = "ARB Error E404 - ARB did not start from host command";
		break;
	case 405:
		ErrorString = "ARB Error E405 - ARB timing program finished early";
		break;
	case 406:
		ErrorString = "ARB Error E406 - ARB timing program finished late or never";
		break;
	case 407:
		ErrorString = "ARB Error E407 - ARB did not start from PSG trigger";
		break;
	case 501:
		ErrorString = "TDA Error E501 - TDA reset did not clear acquire busy flag";
		break;
	case 502:
		ErrorString = "TDA Error E502 - TDA reset did not clear alu busy flag";
		break;
	case 503:
		ErrorString = "TDA Error E503 - TDA loading word counter did not reset EOS flag";
		break;
	case 504:
		ErrorString = "TDA Error E504 - TDA incrementing word counter did not set EOS";
		break;
	case 505:
		ErrorString = "TDA Error E505 - TDA acquisition memory has bit stuck low";
		break;
	case 506:
		ErrorString = "TDA Error E506 - TDA acquisition memory has bit stuck high";
		break;
	case 507:
		ErrorString = "TDA Error E507 - TDA accumulating memory has bit stuck low";
		break;
	case 508:
		ErrorString = "TDA Error E508 - TDA accumulating memory has bit stuck high";
		break;
	case 509:
		ErrorString = "TDA Error E509 - TDA no error 509";
		break;
	case 510:
		ErrorString = "TDA Error E510 - TDA timing program did not start from host command";
		break;
	case 511:
		ErrorString = "TDA Error E511 - TDA timing program finished early";
		break;
	case 512:
		ErrorString = "TDA Error E512 - TDA timing program finished late or never";
		break;
	case 513:
		ErrorString = "TDA Error E513 - TDA alu operation did not start";
		break;
	case 514:
		ErrorString = "TDA Error E514 - TDA alu operation did not complete";
		break;
	case 515:
		ErrorString = "TDA Error E515 - TDA transfer alu operation failed data check";
		break;
	case 516:
		ErrorString = "TDA Error E516 - TDA summation alu uperation failed data check";
		break;
	case 517:
		ErrorString = "TDA Error E517 - TDA program did not start from FSP immediate trigger";
		break;
	case 518:
		ErrorString = "TDA Error E518 - TDA program did not start from FSP delayed trigger";
		break;
	default:
		ErrorString = "Unknown Error code returned from Diagnostic routine";
		break;
	}
	if(ErrorCode != 0) MessageBox(ErrorString,"Diagnostics Failed!!");

	// reset all the flags
	PSG_Tested = FALSE;
	VSP_Tested = FALSE;
	FSP_Tested = FALSE;
	ARB_Tested = FALSE;
	TD_Tested = FALSE;
	PSG_Passed = FALSE;
	VSP_Passed = FALSE;
	FSP_Passed = FALSE;
	ARB_Passed = FALSE;
	TD_Passed = FALSE;
	if(ThreadInfo.m_hwndNotifyDiagDataUpdated != 0)
		::PostMessage(ThreadInfo.m_hwndNotifyDiagDataUpdated,WM_DIAGDATAUPDATE,0,0);
	
	// test the instrument and set the flags
	PSG_Passed = RunPSGDiagnostic();
	PSG_Tested = TRUE;
	if(ThreadInfo.m_hwndNotifyDiagDataUpdated != 0)
		::PostMessage(ThreadInfo.m_hwndNotifyDiagDataUpdated,WM_DIAGDATAUPDATE,0,0);

	VSP_Passed = RunVSPDiagnostic();
	VSP_Tested = TRUE;
	if(ThreadInfo.m_hwndNotifyDiagDataUpdated != 0)
		::PostMessage(ThreadInfo.m_hwndNotifyDiagDataUpdated,WM_DIAGDATAUPDATE,0,0);

	FSP_Passed = RunFSPDiagnostic();
	FSP_Tested = TRUE;
	if(ThreadInfo.m_hwndNotifyDiagDataUpdated != 0)
		::PostMessage(ThreadInfo.m_hwndNotifyDiagDataUpdated,WM_DIAGDATAUPDATE,0,0);

	ARB_Passed = RunARBDiagnostic();
	ARB_Tested = TRUE;
	if(ThreadInfo.m_hwndNotifyDiagDataUpdated != 0)
		::PostMessage(ThreadInfo.m_hwndNotifyDiagDataUpdated,WM_DIAGDATAUPDATE,0,0);

	TD_Passed = RunTDDiagnostic();
	TD_Tested = TRUE;
	if(ThreadInfo.m_hwndNotifyDiagDataUpdated != 0)
		::PostMessage(ThreadInfo.m_hwndNotifyDiagDataUpdated,WM_DIAGDATAUPDATE,0,0);

	// finished, so if we passed all the tests, return TRUE, otherwise FALSE
	DiagnosticsComplete = TRUE;

	// for temporary testing purposes, 
	InstrumentAvailable = TRUE;
	return TRUE;

	if(PSG_Passed && VSP_Passed && FSP_Passed && ARB_Passed && TD_Passed) {
		InstrumentAvailable = TRUE;
		return TRUE;
	} else {
		InstrumentAvailable = FALSE;
		return FALSE;
	}
}

void CInstrument::ReadVersion6SetupFile(CFile* arf)
{
	long i;
	long inpts;
	char* junk;
	struct PSDesc6 event[TOTAL_PS_EVENTS];

	// read in the individual elements of PS that have not
	// changed from version 6 to version 8
	inpts=0;
	inpts+=arf->Read(&(PS.FileType),4);
	inpts+=arf->Read(&(PS.MajorVersion),4);
	inpts+=arf->Read(&(PS.MinorVersion),4);
	inpts+=arf->Read(&(PS.UserID),8);
	inpts+=arf->Read(&(PS.DataFileName),64);
	inpts+=arf->Read(&(PS.TimeAcquired),40);
	inpts+=arf->Read(&(PS.TimeProcessed),32);
	inpts+=arf->Read(&(PS.HiResMALDI),4);
	inpts+=arf->Read(&(PS.HiResESI),4);
	inpts+=arf->Read(&(PS.ExperimentMemo),60);
	inpts+=arf->Read(&(PS.ExperimentNotes),600);
	inpts+=arf->Read(&(PS.DetectMode),2);
	inpts+=arf->Read(&(PS.ADCGain),2);
	inpts+=arf->Read(&(PS.DetectEvent),4);
	inpts+=arf->Read(&(PS.DetectLowMass),8);
	inpts+=arf->Read(&(PS.DetectHighMass),8);
	inpts+=arf->Read(&(PS.MonitorMass),8);
	inpts+=arf->Read(&(PS.MixerFrequency),8);
	inpts+=arf->Read(&(PS.AdcRate),8);
	inpts+=arf->Read(&(PS.ScanMode),4);
	//PS.ElapsedTime = new long[MAX_SPECTRA];
	//inpts+=arf->Read(PS.ElapsedTime,4*MAX_SPECTRA);
	inpts+=arf->Read(&(PS.TransientPoints),4);
	inpts+=arf->Read(&(PS.SpectraRequested),4);
	inpts+=arf->Read(&(PS.SpectraAcquired),4);
	inpts+=arf->Read(&(PS.CurrentSpectrum),4);
	inpts+=arf->Read(&(PS.ScansPerSpectrum),4);
	inpts+=arf->Read(&(PS.ScansRequested),4);
	inpts+=arf->Read(&(PS.TriggerMode),4);
	inpts+=arf->Read(&(PS.CalibrationVersion),4);
	inpts+=arf->Read(&(PS.A),8);
	inpts+=arf->Read(&(PS.B),8);
	inpts+=arf->Read(&(PS.HighMassShift),8);
	inpts+=arf->Read(&(PS.TrapShift),8);
	inpts+=arf->Read(&(PS.IntensityShift),8);
	inpts+=arf->Read(&(PS.PeakMatching),4);
	inpts+=arf->Read(&(PS.PeakMatchFactor),8);
	inpts+=arf->Read(&(PS.RefPeak),sizeof(struct PeakMatchPoint)*MAX_REF_PEAK);
	inpts+=arf->Read(&(PS.SpectrometerType),4);
	inpts+=arf->Read(&(PS.DCRemoval),4);
	inpts+=arf->Read(&(PS.WindowFunction),4);
	inpts+=arf->Read(&(PS.TruncatePoints),4);
	inpts+=arf->Read(&(PS.ZeroFills),4);
	inpts+=arf->Read(&(PS.FFTSize),4);
	inpts+=arf->Read(&(PS.SpectrumPoints),4);
	inpts+=arf->Read(&(PS.FrequencyDataType),4);
	inpts+=arf->Read(&(PS.ZoomFactor),4);
	inpts+=arf->Read(&(PS.ZoomLowMass),8);
	inpts+=arf->Read(&(PS.ZoomHighMass),8);
	inpts+=arf->Read(&(PS.ZoomFrequencyShift),8);
	inpts+=arf->Read(&(PS.ZoomTransientPoints),4);
	inpts+=arf->Read(&(PS.ZoomPointShift),4);
	// create and fill in PeakCount
	//PS.PeakCount = new long[MAX_SPECTRA];
	//inpts+=arf->Read(PS.PeakCount,4*MAX_SPECTRA);

	// create and fill in cfp, dac, arb
	if(PS.cfp != 0) {
		delete [] PS.cfp;
		PS.cfp = 0;
	}
	if(PS.dac != 0) {
		delete [] PS.dac;
		PS.dac = 0;
	}
	if(PS.arb != 0) {
		delete [] PS.arb;
		PS.arb = 0;
	}
	PS.cfp = new struct ChirpPacket[MAX_SYN_TRIGGERS];
	PS.dac = new struct DacPacket[MAX_DACS];
	PS.arb = new struct ArbPacket[MAX_ARB_TRIGGERS];
	inpts+=arf->Read(PS.cfp,sizeof(struct ChirpPacket)*MAX_SYN_TRIGGERS);
	inpts+=arf->Read(PS.dac,sizeof(struct DacPacket)*MAX_DACS);
	inpts+=arf->Read(PS.arb,sizeof(struct ArbPacket)*MAX_ARB_TRIGGERS);

	// create and fill in GenPS
	inpts+=arf->Read(event,sizeof(struct PSDesc6)*TOTAL_PS_EVENTS);
	if(PS.GenPS != 0) {
		delete [] PS.GenPS;
		PS.GenPS = 0;
	}
	PS.GenPS = new struct PSDesc[MAX_PS_EVENTS];
	for(i=0; i<TOTAL_PS_EVENTS; i++) {
		PS.GenPS[i].Channel=GetV6ChannelNumber(i);
		PS.GenPS[i].Type = event[i].Type;
		PS.GenPS[i].Linking = event[i].Linking;
		PS.GenPS[i].Link2Event = event[i].Link2Event;
		PS.GenPS[i].Delay = event[i].Delay;
		PS.GenPS[i].Width = event[i].Width;
		PS.GenPS[i].Value = event[i].Value;
	}
	PS.NumEvents = TOTAL_PS_EVENTS;
	SortGenPS();

	PS.SpectraAcquired = 0;

	if (inpts != PS6_NUM_BYTES) {
		MessageBox("WARNING!\nError 01:\nError Reading Version 6 setup file\nHeader Incorrect","File Error!");
		junk = new char[PS6_NUM_BYTES-inpts];
		arf->Read(junk,PS6_NUM_BYTES-inpts);
		delete [] junk;
	}
	// now define the new data elements that weren't defined in version 6
	PS.TransientIsStored = 1;
	PS.MITableIsStored = 0;
	PS.FrequencyIsStored = 0;
	PS.TruncateTransient = 0;
	PS.TruncatePoints = PS.TruncatePoints/2;
	PS.TruncateOffset = 0;
	char c[8] = "Unknown";
	for (i=0; i<128; i++) {
		if(i<7) {
			PS.SystemName[i]=c[i];
			PS.SystemSerialNumber[i]=c[i];
		}
		else if (i<32) {
			PS.SystemName[i]=0;
			PS.SystemSerialNumber[i]=0;
		}
		else PS.SystemName[i]=0;
	}

	PS.TransientIsStored = 0;
	PS.MITableIsStored = 0;
	PS.FrequencyIsStored = 0;

	// close arf
	arf->Close();
}	// end ReadVersion6SetupFile

void CInstrument::ReadVersion8SetupFile(CFile* arf)
{
	// read in main header
	arf->Read(&PS, sizeof(PS));

	// create and read time and peakcount arrays
	//PS.ElapsedTime = new long [PS.SpectraAcquired];
	//PS.PeakCount = new long [PS.SpectraAcquired];
	//arf->Read(PS.ElapsedTime, 4*PS.SpectraAcquired);
	//arf->Read(PS.PeakCount, 4*PS.SpectraAcquired);

	// read in DacPacket, ChirpPacket, ArbPacket, and GenPS arrays
	// create and fill in cfp, dac, arb
	if(PS.cfp != 0) {
		delete [] PS.cfp;
		PS.cfp = 0;
	}
	if(PS.dac != 0) {
		delete [] PS.dac;
		PS.dac = 0;
	}
	if(PS.arb != 0) {
		delete [] PS.arb;
		PS.arb = 0;
	}
	PS.cfp = new struct ChirpPacket[PS.NumCFP];
	PS.dac = new struct DacPacket[PS.NumDAC];
	PS.arb = new struct ArbPacket[PS.NumARB];
	arf->Read(PS.cfp,sizeof(struct ChirpPacket)*PS.NumCFP);
	arf->Read(PS.dac,sizeof(struct DacPacket)*PS.NumDAC);
	arf->Read(PS.arb,sizeof(struct ArbPacket)*PS.NumARB);
	
	// create and fill in GenPS
	if(PS.GenPS != 0) {
		delete [] PS.GenPS;
		PS.GenPS = 0;
	}
	PS.GenPS = new struct PSDesc[PS.NumEvents];
	arf->Read(PS.GenPS,sizeof(struct PSDesc)*PS.NumEvents);

	// close arf
	arf->Close();
}	// end ReadVersion8SetupFile

int CInstrument::GetV6ChannelNumber(int index)
{
	// this function takes the index and figures out what
	// channel that would correspond to in the version 6
	// GenPS array

	//TRACE("FIRST_EXCITE_DISABLE = %d\n",FIRST_EXCITE_DISABLE);
	//TRACE("LAST_EXCITE_DISABLE = %d\n",LAST_EXCITE_DISABLE);
	//TRACE("FIRST_EXCITE_SOURCEE = %d\n",FIRST_EXCITE_SOURCE);
	//TRACE("LAST_EXCITE_DISABLE = %d\n",LAST_EXCITE_DISABLE);
	if(index>= FIRST_EXCITE_DISABLE && index<=LAST_EXCITE_DISABLE) {
		return EXCITE_DISABLE;
	}
	else if(index>= FIRST_EXCITE_SOURCE && index<=LAST_EXCITE_SOURCE) {
		return EXCITE_SOURCE;
	}
	else if(index>= FIRST_RF_SOURCE && index<=LAST_RF_SOURCE) {
		return RF_SOURCE;
	}
	else if(index>= FIRST_RF_PHASE && index<=LAST_RF_PHASE) {
		return ULTIMA_SWITCH;
	}
	else if(index>= FIRST_IMP_TRIGGER && index<=LAST_IMP_TRIGGER) {
		return IMP_TRIGGER;
	}
	else if(index>= FIRST_SYN_TRIGGER && index<=LAST_SYN_TRIGGER) {
		return SYN_TRIGGER;
	}
	else if(index>= FIRST_ARB_TRIGGER && index<=LAST_ARB_TRIGGER) {
		return ARB_TRIGGER;
	}
	else if(index>= FIRST_VSP_TRIGGER && index<=LAST_VSP_TRIGGER) {
		return VSP_TRIGGER;
	}
	else if(index>= FIRST_LASER_PULSE && index<=LAST_LASER_PULSE) {
		return LASER_PULSE;
	}
	else if(index>= FIRST_QUADRF_GATE && index<=LAST_QUADRF_GATE) {
		return QUADRF_GATE;
	}
	else if(index>= FIRST_VALVE_PULSE && index<=LAST_VALVE_PULSE) {
		return VALVE_PULSE;
	}
	else if(index>= FIRST_SHUTTER_GATE && index<=LAST_SHUTTER_GATE) {
		return SHUTTER_GATE;
	}
	else if(index>= FIRST_USER5_PULSE && index<=LAST_USER5_PULSE) {
		return USER5_PULSE;
	}
	else if(index>= FIRST_USER6_PULSE && index<=LAST_USER6_PULSE) {
		return USER6_PULSE;
	}
	else if(index>= FIRST_USER7_PULSE && index<=LAST_USER7_PULSE) {
		return USER7_PULSE;
	}
	else if(index>= FIRST_USER8_PULSE && index<=LAST_USER8_PULSE) {
		return USER8_PULSE;
	}
	else if(index>= FIRST_XMIT1 && index<=LAST_XMIT1) {
		return DAC_XMIT1;
	}
	else if(index>= FIRST_XMIT2 && index<=LAST_XMIT2) {
		return DAC_XMIT2;
	}
	else if(index>= FIRST_RCVR1 && index<=LAST_RCVR1) {
		return DAC_RCVR1;
	}
	else if(index>= FIRST_RCVR2 && index<=LAST_RCVR2) {
		return DAC_RCVR2;
	}
	else if(index>= FIRST_TRAP1 && index<=LAST_TRAP1) {
		return DAC_TRAP1;
	}
	else if(index>= FIRST_TRAP2 && index<=LAST_TRAP2) {
		return DAC_TRAP2;
	}
	else if(index>= FIRST_EE && index<=LAST_EE) {
		return DAC_EE;
	}
	else if(index>= FIRST_GAIN && index<=LAST_GAIN) {
		return DAC_GAIN;
	}
	else if(index>= FIRST_USER1_DAC && index<=LAST_USER1_DAC) {
		return DAC_USER1;
	}
	else if(index>= FIRST_USER2_DAC && index<=LAST_USER2_DAC) {
		return DAC_USER2;
	}
	else if(index>= FIRST_USER3_DAC && index<=LAST_USER3_DAC) {
		return DAC_USER3;
	}
	else if(index>= FIRST_USER4_DAC && index<=LAST_USER4_DAC) {
		return DAC_USER4;
	}
	else if(index>= FIRST_USER5_DAC && index<=LAST_USER5_DAC) {
		return DAC_USER5;
	}
	else if(index>= FIRST_USER6_DAC && index<=LAST_USER6_DAC) {
		return DAC_USER6;
	}
	else if(index>= FIRST_USER7_DAC && index<=LAST_USER7_DAC) {
		return DAC_USER7;
	}
	else if(index>= FIRST_USER8_DAC && index<=LAST_USER8_DAC) {
		return DAC_USER8;
	}
	else if(index>= FIRST_USER9_DAC && index<=LAST_USER9_DAC) {
		return DAC_USER9;
	}
	else if(index>= FIRST_USER10_DAC && index<=LAST_USER10_DAC) {
		return DAC_USER10;
	}
	else if(index>= FIRST_USER11_DAC && index<=LAST_USER11_DAC) {
		return DAC_USER11;
	}
	else if(index>= FIRST_USER12_DAC && index<=LAST_USER12_DAC) {
		return DAC_USER12;
	}
	else if(index>= FIRST_USER13_DAC && index<=LAST_USER13_DAC) {
		return DAC_USER13;
	}
	else if(index>= FIRST_USER14_DAC && index<=LAST_USER14_DAC) {
		return DAC_USER14;
	}
	else if(index>= FIRST_USER15_DAC && index<=LAST_USER15_DAC) {
		return DAC_USER15;
	}
	else if(index>= FIRST_USER16_DAC && index<=LAST_USER16_DAC) {
		return DAC_USER16;
	}
	else if(index == LAST_USER16_DAC+1) {
		// the is the temp event
		return NO_EVENT_CHANNEL;
	}
	// if we get here, then we didn't have a valid index
	return -1;
}

CString CInstrument::RemoveSpaces(CString instr)
{
	// removes the leading and trailing spaces from CString
	int i;

	// remove leading spaces
	i = 0;
	while (instr.GetAt(i) == ' ') {
		i++;
	}
	instr = instr.Right(instr.GetLength()-i);

	// remove trailing spaces
	i = instr.GetLength();
	while (instr.GetAt(i-1) == ' ') {
		i--;
	}
	instr = instr.Left(i);
	return instr;
}

BOOL CInstrument::ReadCNFFileMALDI()
{
	CString readstr, str, rightstr;

	char* pathname;
	pathname = new char[256];
	strcpy(pathname,pApp->ProgramDirectory);
	strcat(pathname,pApp->ConfigFileName);

	if(!ConfigFile.Open(pathname,CFile::modeRead,NULL)) {
		str.Format("Error 21\nCouldn't open configuration file:\n%s",pathname);
		MessageBox(str,"File error!");
		delete pathname;
		return FALSE;
	}
	delete pathname;

	CArchive ar3(&ConfigFile,CArchive::load,4096,NULL);

	//loop till find START OF MALDI, then break 
	while(ar3.ReadString(readstr)) {
		str=CompressString(readstr);
		if (str.GetLength()==0) continue; // empty lines
		if (str.Left(1)=="#") continue; // comment lines

		//found start of MALDI, set flag and break
		if (str.Left(12)=="[HIRESMALDI]")  {
			break;
		}
	}//end while
			
	//loop and read MALDI config info 
	while (ar3.ReadString(readstr) ) {
			str=CompressString(readstr);
			if (str.GetLength()==0) continue; // empty lines
			if (str.Left(1)=="#") continue; // comment lines

			//found start of ESI, set flag and break MALDI loop 
			if (str.Left(10)=="[HIRESESI]")  break;
		
			/*...........start reading in information for MALDI............*/

			if (str.Left(16)=="PEAKRFAMPVOLTAGE") {
				rightstr = GetRightString(str);
				sscanf(rightstr,"%lf",&(sys.PeakRFAmpVoltage));
			}
			else if (str.Left(15)=="MINRFAMPVOLTAGE") {
				rightstr = GetRightString(str);
				sscanf(rightstr,"%lf",&(sys.MinRFAmpVoltage));
			}
			else if (str.Left(17)=="PEAKQUADRFVOLTAGE") {
				rightstr = GetRightString(str);
				sscanf(rightstr,"%lf",&(sys.PeakQuadRFVoltage));
			}
			else if (str.Left(16)=="SYNOUTPUTVOLTAGE") {
				rightstr = GetRightString(str);
				sscanf(rightstr,"%lf",&(sys.SynOutputVoltage));
			}
			else if (str.Left(16)=="ARBOUTPUTVOLTAGE") {
				rightstr = GetRightString(str);
				sscanf(rightstr,"%lf",&(sys.ArbOutputVoltage));
			}
			else if (str.Left(14)=="RFAMPFIXEDGAIN") {
				rightstr = GetRightString(str);
				sscanf(rightstr,"%lf",&(sys.RFAmpFixedGain));
			}
			else if (str.Left(15)=="QUADRFFIXEDGAIN") {
				rightstr = GetRightString(str);
				sscanf(rightstr,"%lf",&(sys.QuadRFFixedGain));
			}
			else if (str.Left(19)=="RFAMPVOLTAGEDIVIDER") {
				rightstr = GetRightString(str);
				sscanf(rightstr,"%lf",&(sys.GainControlVoltageDivider));
			}
			else if (str.Left(18)=="MALDIQUADFREQUENCY") {
				rightstr = GetRightString(str);
				sscanf(rightstr,"%lf",&(sys.QuadFrequency));
			}
			else if (str.Left(15)=="MIXERCORRECTION") {
				rightstr = GetRightString(str);
				sscanf(rightstr,"%lf",&(sys.MixerCorrection));
			}
			else if (str.Left(8)=="DATAPATH") {
				rightstr = GetRightString(str);
				sys.DataPath = rightstr;
			}
			else if (str.Left(8)=="TEMPPATH") {
				rightstr = GetRightString(str);
				sys.TempPath = rightstr;
			}
			else if (str.Left(17)=="INITIALSETUPFILE") {
				rightstr = GetRightString(str);
				sys.DefaultSetupFile = rightstr;
			}
			else if (str.Left(15)=="CALIBRATIONFILE") {
				rightstr = GetRightString(str);
				sys.CalibrationFile = rightstr;
			}
			else if (str.Left(15)=="TRAPSHIFTFACTOR") {
				rightstr = GetRightString(str);
				sscanf(rightstr,"%lf",&(sys.TrapShift));
			}
			else if (str.Left(20)=="INTENSITYSHIFTFACTOR") {
				rightstr = GetRightString(str);
				sscanf(rightstr,"%lf",&(sys.IntensityShift));
			}
			else if (str.Left(19)=="HIGHMASSSHIFTFACTOR") {
				rightstr = GetRightString(str);
				sscanf(rightstr,"%lf",&(sys.HighMassShift));
			}
			else if (str.Left(21)=="CORRECTTOTALINTENSITY") {
				rightstr = GetRightString(str);
				if (str == "YES")
					sys.CorrectTotalIntensity = 1;
				else 
					sys.CorrectTotalIntensity = 0;
			}
			else if (str.Left(17)=="EXCLUDENORESPEAKS") {
				rightstr = GetRightString(str);
				if (rightstr == "YES")
					sys.ExcludeNoResPeaks = 1;
				else 
					sys.ExcludeNoResPeaks = 0;
			}
			else if (str.Left(14)=="AUTOLABELPEAKS") {
				rightstr = GetRightString(str);
				if (rightstr == "YES")
					sys.AutoLabelPeaks = 1;
				else 
					sys.AutoLabelPeaks = 0;
			}
			else if (str.Left(15)=="AUTOLABELCHARGE") {
				rightstr = GetRightString(str);
				if (rightstr == "YES")
					sys.AutoLabelCharge = 1;
				else 
					sys.AutoLabelCharge = 0;
			}
			// get ttl and dac names here *********
			else if (str.Left(10)=="TTLPULSE#1") {
				rightstr=GetRightString(readstr);
				rightstr=rightstr.Left(FindComma(rightstr));
				rightstr=RemoveSpaces(rightstr);
				sys.ChannelNames[8] = rightstr;
			}
			else if (str.Left(10)=="TTLPULSE#2") {
				rightstr=GetRightString(readstr);
				rightstr=rightstr.Left(FindComma(rightstr));
				rightstr=RemoveSpaces(rightstr);
				sys.ChannelNames[9] = rightstr;
			}
			else if (str.Left(10)=="TTLPULSE#3") {
				rightstr=GetRightString(readstr);
				rightstr=rightstr.Left(FindComma(rightstr));
				rightstr=RemoveSpaces(rightstr);
				sys.ChannelNames[10] = rightstr;
			}
			else if (str.Left(10)=="TTLPULSE#4") {
				rightstr=GetRightString(readstr);
				rightstr=rightstr.Left(FindComma(rightstr));
				rightstr=RemoveSpaces(rightstr);
				sys.ChannelNames[11] = rightstr;
			}
			else if (str.Left(10)=="TTLPULSE#5") {
				rightstr=GetRightString(readstr);
				rightstr=rightstr.Left(FindComma(rightstr));
				rightstr=RemoveSpaces(rightstr);
				sys.ChannelNames[12] = rightstr;
			}
			else if (str.Left(10)=="TTLPULSE#6") {
				rightstr=GetRightString(readstr);
				rightstr=rightstr.Left(FindComma(rightstr));
				rightstr=RemoveSpaces(rightstr);
				sys.ChannelNames[13] = rightstr;
			}
			else if (str.Left(10)=="TTLPULSE#7") {
				rightstr=GetRightString(readstr);
				rightstr=rightstr.Left(FindComma(rightstr));
				rightstr=RemoveSpaces(rightstr);
				sys.ChannelNames[14] = rightstr;
			}
			else if (str.Left(10)=="TTLPULSE#8") {
				rightstr=GetRightString(readstr);
				rightstr=rightstr.Left(FindComma(rightstr));
				rightstr=RemoveSpaces(rightstr);
				sys.ChannelNames[15] = rightstr;
			}
			else if (str.Left(12)=="DACCHANNEL#5") {
				rightstr=GetRightString(readstr);
				rightstr=rightstr.Left(FindComma(rightstr));
				rightstr=RemoveSpaces(rightstr);
				sys.ChannelNames[15+5] = rightstr;
			}
			else if (str.Left(12)=="DACCHANNEL#6") {
				rightstr=GetRightString(readstr);
				rightstr=rightstr.Left(FindComma(rightstr));
				rightstr=RemoveSpaces(rightstr);
				sys.ChannelNames[15+6] = rightstr;
			}
			else if (str.Left(12)=="DACCHANNEL#9") {
				rightstr=GetRightString(readstr);
				rightstr=rightstr.Left(FindComma(rightstr));
				rightstr=RemoveSpaces(rightstr);
				sys.ChannelNames[15+9] = rightstr;
			}
			else if (str.Left(13)=="DACCHANNEL#10") {
				rightstr=GetRightString(readstr);
				rightstr=rightstr.Left(FindComma(rightstr));
				rightstr=RemoveSpaces(rightstr);
				sys.ChannelNames[15+10] = rightstr;
			}
			else if (str.Left(13)=="DACCHANNEL#11") {
				rightstr=GetRightString(readstr);
				rightstr=rightstr.Left(FindComma(rightstr));
				rightstr=RemoveSpaces(rightstr);
				sys.ChannelNames[15+11] = rightstr;
			}
			else if (str.Left(13)=="DACCHANNEL#12") {
				rightstr=GetRightString(readstr);
				rightstr=rightstr.Left(FindComma(rightstr));
				rightstr=RemoveSpaces(rightstr);
				sys.ChannelNames[15+12] = rightstr;
			}
			else if (str.Left(13)=="DACCHANNEL#13") {
				rightstr=GetRightString(readstr);
				rightstr=rightstr.Left(FindComma(rightstr));
				rightstr=RemoveSpaces(rightstr);
				sys.ChannelNames[15+13] = rightstr;
			}
			else if (str.Left(13)=="DACCHANNEL#14") {
				rightstr=GetRightString(readstr);
				rightstr=rightstr.Left(FindComma(rightstr));
				rightstr=RemoveSpaces(rightstr);
				sys.ChannelNames[15+14] = rightstr;
			}
			else if (str.Left(13)=="DACCHANNEL#15") {
				rightstr=GetRightString(readstr);
				rightstr=rightstr.Left(FindComma(rightstr));
				rightstr=RemoveSpaces(rightstr);
				sys.ChannelNames[15+15] = rightstr;
			}
			else if (str.Left(13)=="DACCHANNEL#16") {
				rightstr=GetRightString(readstr);
				rightstr=rightstr.Left(FindComma(rightstr));
				rightstr=RemoveSpaces(rightstr);
				sys.ChannelNames[15+16] = rightstr;
			}
			else if (str.Left(13)=="DACCHANNEL#17") {
				rightstr=GetRightString(readstr);
				rightstr=rightstr.Left(FindComma(rightstr));
				rightstr=RemoveSpaces(rightstr);
				sys.ChannelNames[15+17] = rightstr;
			}
			else if (str.Left(13)=="DACCHANNEL#18") {
				rightstr=GetRightString(readstr);
				rightstr=rightstr.Left(FindComma(rightstr));
				rightstr=RemoveSpaces(rightstr);
				sys.ChannelNames[15+18] = rightstr;
			}
			else if (str.Left(13)=="DACCHANNEL#19") {
				rightstr=GetRightString(readstr);
				rightstr=rightstr.Left(FindComma(rightstr));
				rightstr=RemoveSpaces(rightstr);
				sys.ChannelNames[15+19] = rightstr;
			}
			else if (str.Left(13)=="DACCHANNEL#20") {
				rightstr=GetRightString(readstr);
				rightstr=rightstr.Left(FindComma(rightstr));
				rightstr=RemoveSpaces(rightstr);
				sys.ChannelNames[15+20] = rightstr;
			}
			else if (str.Left(13)=="DACCHANNEL#21") {
				rightstr=GetRightString(readstr);
				rightstr=rightstr.Left(FindComma(rightstr));
				rightstr=RemoveSpaces(rightstr);
				sys.ChannelNames[15+21] = rightstr;
			}
			else if (str.Left(13)=="DACCHANNEL#22") {
				rightstr=GetRightString(readstr);
				rightstr=rightstr.Left(FindComma(rightstr));
				rightstr=RemoveSpaces(rightstr);
				sys.ChannelNames[15+22] = rightstr;
			}
			else if (str.Left(13)=="DACCHANNEL#23") {
				rightstr=GetRightString(readstr);
				rightstr=rightstr.Left(FindComma(rightstr));
				rightstr=RemoveSpaces(rightstr);
				sys.ChannelNames[15+23] = rightstr;
			}
			else if (str.Left(13)=="DACCHANNEL#24") {
				rightstr=GetRightString(readstr);
				rightstr=rightstr.Left(FindComma(rightstr));
				rightstr=RemoveSpaces(rightstr);
				sys.ChannelNames[15+24] = rightstr;
			}

			//SETUP DAC stuff here...................................
			else if (str.Left(11)=="SETUPDAC #1") {
				str = GetRightString(str);

				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[0].MaxValue));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[0].MaxDac));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[0].MinValue));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[0].MinDac));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[0].MaxGraph));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[0].MinGraph));
				//all that's left in str is what we want...
				sscanf(str,"%ld",&(sys.dac[0].Units));
			}
			else if (str.Left(11)=="SETUPDAC #2") {
				str = GetRightString(str);

				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[1].MaxValue));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[1].MaxDac));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[1].MinValue));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[1].MinDac));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[1].MaxGraph));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[1].MinGraph));
				//all that's left in str is what we want...
				sscanf(str,"%ld",&(sys.dac[1].Units));
			}
			else if (str.Left(11)=="SETUPDAC #3") {
				str = GetRightString(str);

				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[2].MaxValue));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[2].MaxDac));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[2].MinValue));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[2].MinDac));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[2].MaxGraph));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[2].MinGraph));
				//all that's left in str is what we want...
				sscanf(str,"%ld",&(sys.dac[2].Units));
			}
			else if (str.Left(11)=="SETUPDAC #4") {
				str = GetRightString(str);

				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[3].MaxValue));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[3].MaxDac));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[3].MinValue));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[3].MinDac));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[3].MaxGraph));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[3].MinGraph));
				//all that's left in str is what we want...
				sscanf(str,"%ld",&(sys.dac[3].Units));
			}
			else if (str.Left(11)=="SETUPDAC #5") {
				str = GetRightString(str);

				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[4].MaxValue));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[4].MaxDac));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[4].MinValue));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[4].MinDac));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[4].MaxGraph));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[4].MinGraph));
				//all that's left in str is what we want...
				sscanf(str,"%ld",&(sys.dac[4].Units));
			}
			else if (str.Left(11)=="SETUPDAC #6") {
				str = GetRightString(str);

				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[5].MaxValue));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[5].MaxDac));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[5].MinValue));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[5].MinDac));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[5].MaxGraph));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[5].MinGraph));
				//all that's left in str is what we want...
				sscanf(str,"%ld",&(sys.dac[5].Units));
			}
			else if (str.Left(11)=="SETUPDAC #7") {
				str = GetRightString(str);

				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[6].MaxValue));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[6].MaxDac));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[6].MinValue));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[6].MinDac));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[6].MaxGraph));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[6].MinGraph));
				//all that's left in str is what we want...
				sscanf(str,"%ld",&(sys.dac[6].Units));
			}
			else if (str.Left(11)=="SETUPDAC #8") {
				str = GetRightString(str);

				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[7].MaxValue));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[7].MaxDac));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[7].MinValue));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[7].MinDac));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[7].MaxGraph));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[7].MinGraph));
				//all that's left in str is what we want...
				sscanf(str,"%ld",&(sys.dac[7].Units));
			}
			else if (str.Left(11)=="SETUPDAC #9") {
				str = GetRightString(str);

				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[8].MaxValue));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[8].MaxDac));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[8].MinValue));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[8].MinDac));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[8].MaxGraph));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[8].MinGraph));
				//all that's left in str is what we want...
				sscanf(str,"%ld",&(sys.dac[8].Units));
			}
			else if (str.Left(11)=="SETUPDAC#10") {
				str = GetRightString(str);

				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[9].MaxValue));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[9].MaxDac));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[9].MinValue));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[9].MinDac));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[9].MaxGraph));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[9].MinGraph));
				//all that's left in str is what we want...
				sscanf(str,"%ld",&(sys.dac[9].Units));
			}
			else if (str.Left(11)=="SETUPDAC#11") {
				str = GetRightString(str);

				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[10].MaxValue));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[10].MaxDac));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[10].MinValue));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[10].MinDac));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[10].MaxGraph));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[10].MinGraph));
				//all that's left in str is what we want...
				sscanf(str,"%ld",&(sys.dac[10].Units));
			}
			else if (str.Left(11)=="SETUPDAC#12") {
				str = GetRightString(str);

				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[11].MaxValue));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[11].MaxDac));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[11].MinValue));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[11].MinDac));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[11].MaxGraph));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[11].MinGraph));
				//all that's left in str is what we want...
				sscanf(str,"%ld",&(sys.dac[11].Units));
			}
			else if (str.Left(11)=="SETUPDAC#13") {
				str = GetRightString(str);

				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[12].MaxValue));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[12].MaxDac));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[12].MinValue));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[12].MinDac));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[12].MaxGraph));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[12].MinGraph));
				//all that's left in str is what we want...
				sscanf(str,"%ld",&(sys.dac[12].Units));
			}
			else if (str.Left(11)=="SETUPDAC#14") {
				str = GetRightString(str);

				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[13].MaxValue));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[13].MaxDac));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[13].MinValue));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[13].MinDac));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[13].MaxGraph));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[13].MinGraph));
				//all that's left in str is what we want...
				sscanf(str,"%ld",&(sys.dac[13].Units));
			}
			else if (str.Left(11)=="SETUPDAC#15") {
				str = GetRightString(str);

				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[14].MaxValue));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[14].MaxDac));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[14].MinValue));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[14].MinDac));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[14].MaxGraph));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[14].MinGraph));
				//all that's left in str is what we want...
				sscanf(str,"%ld",&(sys.dac[14].Units));
			}
			else if (str.Left(11)=="SETUPDAC#16") {
				str = GetRightString(str);

				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[15].MaxValue));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[15].MaxDac));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[15].MinValue));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[15].MinDac));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[15].MaxGraph));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[15].MinGraph));
				//all that's left in str is what we want...
				sscanf(str,"%ld",&(sys.dac[15].Units));
			}
			else if (str.Left(11)=="SETUPDAC#17") {
				str = GetRightString(str);

				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[16].MaxValue));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[16].MaxDac));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[16].MinValue));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[16].MinDac));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[16].MaxGraph));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[16].MinGraph));
				//all that's left in str is what we want...
				sscanf(str,"%ld",&(sys.dac[16].Units));
			}
			else if (str.Left(11)=="SETUPDAC#18") {
				str = GetRightString(str);

				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[17].MaxValue));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[17].MaxDac));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[17].MinValue));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[17].MinDac));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[17].MaxGraph));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[17].MinGraph));
				//all that's left in str is what we want...
				sscanf(str,"%ld",&(sys.dac[17].Units));
			}
			else if (str.Left(11)=="SETUPDAC#19") {
				str = GetRightString(str);

				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[18].MaxValue));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[18].MaxDac));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[18].MinValue));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[18].MinDac));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[18].MaxGraph));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[18].MinGraph));
				//all that's left in str is what we want...
				sscanf(str,"%ld",&(sys.dac[18].Units));
			}
			else if (str.Left(11)=="SETUPDAC#20") {
				str = GetRightString(str);

				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[19].MaxValue));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[19].MaxDac));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[19].MinValue));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[19].MinDac));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[19].MaxGraph));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[19].MinGraph));
				//all that's left in str is what we want...
				sscanf(str,"%ld",&(sys.dac[19].Units));
			}
			else if (str.Left(11)=="SETUPDAC#21") {
				str = GetRightString(str);

				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[20].MaxValue));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[20].MaxDac));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[20].MinValue));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[20].MinDac));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[20].MaxGraph));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[20].MinGraph));
				//all that's left in str is what we want...
				sscanf(str,"%ld",&(sys.dac[20].Units));
			}
			else if (str.Left(11)=="SETUPDAC#22") {
				str = GetRightString(str);

				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[21].MaxValue));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[21].MaxDac));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[21].MinValue));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[21].MinDac));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[21].MaxGraph));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[21].MinGraph));
				//all that's left in str is what we want...
				sscanf(str,"%ld",&(sys.dac[21].Units));
			}
			else if (str.Left(11)=="SETUPDAC#23") {
				str = GetRightString(str);

				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[22].MaxValue));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[22].MaxDac));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[22].MinValue));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[22].MinDac));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[22].MaxGraph));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[22].MinGraph));
				//all that's left in str is what we want...
				sscanf(str,"%ld",&(sys.dac[22].Units));
			}
			else if (str.Left(11)=="SETUPDAC#24") {
				str = GetRightString(str);

				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[23].MaxValue));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[23].MaxDac));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[23].MinValue));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[23].MinDac));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[23].MaxGraph));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[23].MinGraph));
				//all that's left in str is what we want...
				sscanf(str,"%ld",&(sys.dac[23].Units));
			}

	}//end while (loop and read MALDI info)
	
	ar3.Close();
	ConfigFile.Close();
	return TRUE;
}

BOOL CInstrument::ReadCNFFileESI()
{
	char* pathname;
	pathname = new char[256];
	strcpy(pathname,pApp->ProgramDirectory);
	strcat(pathname,pApp->ConfigFileName);

	CString readstr, str, rightstr;
	if(!ConfigFile.Open(pathname,CFile::modeRead,NULL)) {
		str.Format("Error 21\nCouldn't open configuration file:\n%s",pathname);
		MessageBox(str,"File error!");
		delete pathname;
		return FALSE;
	}
	CArchive ar3(&ConfigFile,CArchive::load,4096,NULL);
	delete pathname;

	//loop till find START OF ESI, then break 
	while(ar3.ReadString(readstr)) {
			str=CompressString(readstr);
			if (str.GetLength()==0) continue; // empty lines
			if (str.Left(1)=="#") continue; // comment lines

			//found start of ESI, set flag
			if (str.Left(10)=="[HIRESESI]")  {
				break;
			}	
		}//end while (find start of ESI)

		//loop and read ESI info
		while (ar3.ReadString(readstr) ) {
			str=CompressString(readstr);
			if (str.GetLength()==0) continue; // empty lines
			if (str.Left(1)=="#") continue; // comment lines

			//found start of MALDI, break ESI loop 
			if (str.Left(12)=="[HIRESMALDI]")  break;
		
			//start reading in information for ESI..............
			if (str.Left(16)=="PEAKRFAMPVOLTAGE") {
				rightstr = GetRightString(str);
				sscanf(rightstr,"%lf",&(sys.PeakRFAmpVoltage));
			}
			else if (str.Left(15)=="MINRFAMPVOLTAGE") {
				rightstr = GetRightString(str);
				sscanf(rightstr,"%lf",&(sys.MinRFAmpVoltage));
			}
			else if (str.Left(17)=="PEAKQUADRFVOLTAGE") {
				rightstr = GetRightString(str);
				sscanf(rightstr,"%lf",&(sys.PeakQuadRFVoltage));
			}
			else if (str.Left(16)=="SYNOUTPUTVOLTAGE") {
				rightstr = GetRightString(str);
				sscanf(rightstr,"%lf",&(sys.SynOutputVoltage));
			}
			else if (str.Left(16)=="ARBOUTPUTVOLTAGE") {
				rightstr = GetRightString(str);
				sscanf(rightstr,"%lf",&(sys.ArbOutputVoltage));
			}
			else if (str.Left(14)=="RFAMPFIXEDGAIN") {
				rightstr = GetRightString(str);
				sscanf(rightstr,"%lf",&(sys.RFAmpFixedGain));
			}
			else if (str.Left(15)=="QUADRFFIXEDGAIN") {
				rightstr = GetRightString(str);
				sscanf(rightstr,"%lf",&(sys.QuadRFFixedGain));
			}
			else if (str.Left(19)=="RFAMPVOLTAGEDIVIDER") {
				rightstr = GetRightString(str);
				sscanf(rightstr,"%lf",&(sys.GainControlVoltageDivider));
			}
			else if (str.Left(16)=="ESIQUADFREQUENCY") {
				rightstr = GetRightString(str);
				sscanf(rightstr,"%lf",&(sys.QuadFrequency));
			}
			else if (str.Left(15)=="MIXERCORRECTION") {
				rightstr = GetRightString(str);
				sscanf(rightstr,"%lf",&(sys.MixerCorrection));
			}
			else if (str.Left(8)=="DATAPATH") {
				rightstr = GetRightString(str);
				sys.DataPath = rightstr;
			}
			else if (str.Left(8)=="TEMPPATH") {
				rightstr = GetRightString(str);
				sys.TempPath = rightstr;
			}
			else if (str.Left(17)=="INITIALSETUPFILE") {
				rightstr = GetRightString(str);
				sys.DefaultSetupFile = rightstr;
			}
			else if (str.Left(15)=="CALIBRATIONFILE") {
				rightstr = GetRightString(str);
				sys.CalibrationFile = rightstr;
			}
			else if (str.Left(15)=="TRAPSHIFTFACTOR") {
				rightstr = GetRightString(str);
				sscanf(rightstr,"%lf",&(sys.TrapShift));
			}
			else if (str.Left(20)=="INTENSITYSHIFTFACTOR") {
				rightstr = GetRightString(str);
				sscanf(rightstr,"%lf",&(sys.IntensityShift));
			}
			else if (str.Left(19)=="HIGHMASSSHIFTFACTOR") {
				rightstr = GetRightString(str);
				sscanf(rightstr,"%lf",&(sys.HighMassShift));
			}
			else if (str.Left(21)=="CORRECTTOTALINTENSITY") {
				rightstr = GetRightString(str);
				if (str == "YES")
					sys.CorrectTotalIntensity = 1;
				else 
					sys.CorrectTotalIntensity = 0;
			}
			else if (str.Left(17)=="EXCLUDENORESPEAKS") {
				rightstr = GetRightString(str);
				if (rightstr == "YES")
					sys.ExcludeNoResPeaks = 1;
				else 
					sys.ExcludeNoResPeaks = 0;
			}
			else if (str.Left(14)=="AUTOLABELPEAKS") {
				rightstr = GetRightString(str);
				if (rightstr == "YES")
					sys.AutoLabelPeaks = 1;
				else 
					sys.AutoLabelPeaks = 0;
			}
			else if (str.Left(15)=="AUTOLABELCHARGE") {
				rightstr = GetRightString(str);
				if (rightstr == "YES")
					sys.AutoLabelCharge = 1;
				else 
					sys.AutoLabelCharge = 0;
			}
			else if (str.Left(12)=="LABELCHARGE1") {
				rightstr = GetRightString(str);
				if (rightstr == "YES")
					sys.LabelCharge1 = 1;
				else 
					sys.LabelCharge1 = 0;
			}
			else if (str.Left(11)=="CHARGELIMIT") {
				rightstr = GetRightString(str);
				sscanf(rightstr,"%ld",&(sys.ChargeLimit));
			}

			// get ttl and dac names here *********
			else if (str.Left(10)=="TTLPULSE#1") {
				rightstr=GetRightString(readstr);
				rightstr=rightstr.Left(FindComma(rightstr));
				rightstr=RemoveSpaces(rightstr);
				sys.ChannelNames[8] =rightstr;
			}
			else if (str.Left(10)=="TTLPULSE#2") {
				rightstr=GetRightString(readstr);
				rightstr=rightstr.Left(FindComma(rightstr));
				rightstr=RemoveSpaces(rightstr);
				sys.ChannelNames[9] = rightstr;
			}
			else if (str.Left(10)=="TTLPULSE#3") {
				rightstr=GetRightString(readstr);
				rightstr=rightstr.Left(FindComma(rightstr));
				rightstr=RemoveSpaces(rightstr);
				sys.ChannelNames[10] = rightstr;
			}
			else if (str.Left(10)=="TTLPULSE#4") {
				rightstr=GetRightString(readstr);
				rightstr=rightstr.Left(FindComma(rightstr));
				rightstr=RemoveSpaces(rightstr);
				sys.ChannelNames[11] = rightstr;
			}
			else if (str.Left(10)=="TTLPULSE#5") {
				rightstr=GetRightString(readstr);
				rightstr=rightstr.Left(FindComma(rightstr));
				rightstr=RemoveSpaces(rightstr);
				sys.ChannelNames[12] = rightstr;
			}
			else if (str.Left(10)=="TTLPULSE#6") {
				rightstr=GetRightString(readstr);
				rightstr=rightstr.Left(FindComma(rightstr));
				rightstr=RemoveSpaces(rightstr);
				sys.ChannelNames[13] = rightstr;
			}
			else if (str.Left(10)=="TTLPULSE#7") {
				rightstr=GetRightString(readstr);
				rightstr=rightstr.Left(FindComma(rightstr));
				rightstr=RemoveSpaces(rightstr);
				sys.ChannelNames[14] = rightstr;
			}
			else if (str.Left(10)=="TTLPULSE#8") {
				rightstr=GetRightString(readstr);
				rightstr=rightstr.Left(FindComma(rightstr));
				rightstr=RemoveSpaces(rightstr);
				sys.ChannelNames[15] = rightstr;
			}
			else if (str.Left(12)=="DACCHANNEL#5") {
				rightstr=GetRightString(readstr);
				rightstr=rightstr.Left(FindComma(rightstr));
				rightstr=RemoveSpaces(rightstr);
				sys.ChannelNames[15+5] = rightstr;
			}
			else if (str.Left(12)=="DACCHANNEL#6") {
				rightstr=GetRightString(readstr);
				rightstr=rightstr.Left(FindComma(rightstr));
				rightstr=RemoveSpaces(rightstr);
				sys.ChannelNames[15+6] = rightstr;
			}
			else if (str.Left(12)=="DACCHANNEL#9") {
				rightstr=GetRightString(readstr);
				rightstr=rightstr.Left(FindComma(rightstr));
				rightstr=RemoveSpaces(rightstr);
				sys.ChannelNames[15+9] = rightstr;
			}
			else if (str.Left(13)=="DACCHANNEL#10") {
				rightstr=GetRightString(readstr);
				rightstr=rightstr.Left(FindComma(rightstr));
				rightstr=RemoveSpaces(rightstr);
				sys.ChannelNames[15+10] = rightstr;
			}
			else if (str.Left(13)=="DACCHANNEL#11") {
				rightstr=GetRightString(readstr);
				rightstr=rightstr.Left(FindComma(rightstr));
				rightstr=RemoveSpaces(rightstr);
				sys.ChannelNames[15+11] = rightstr;
			}
			else if (str.Left(13)=="DACCHANNEL#12") {
				rightstr=GetRightString(readstr);
				rightstr=rightstr.Left(FindComma(rightstr));
				rightstr=RemoveSpaces(rightstr);
				sys.ChannelNames[15+12] = rightstr;
			}
			else if (str.Left(13)=="DACCHANNEL#13") {
				rightstr=GetRightString(readstr);
				rightstr=rightstr.Left(FindComma(rightstr));
				rightstr=RemoveSpaces(rightstr);
				sys.ChannelNames[15+13] = rightstr;
			}
			else if (str.Left(13)=="DACCHANNEL#14") {
				rightstr=GetRightString(readstr);
				rightstr=rightstr.Left(FindComma(rightstr));
				rightstr=RemoveSpaces(rightstr);
				sys.ChannelNames[15+14] = rightstr;
			}
			else if (str.Left(13)=="DACCHANNEL#15") {
				rightstr=GetRightString(readstr);
				rightstr=rightstr.Left(FindComma(rightstr));
				rightstr=RemoveSpaces(rightstr);
				sys.ChannelNames[15+15] = rightstr;
			}
			else if (str.Left(13)=="DACCHANNEL#16") {
				rightstr=GetRightString(readstr);
				rightstr=rightstr.Left(FindComma(rightstr));
				rightstr=RemoveSpaces(rightstr);
				sys.ChannelNames[15+16] = rightstr;
			}
			else if (str.Left(13)=="DACCHANNEL#17") {
				rightstr=GetRightString(readstr);
				rightstr=rightstr.Left(FindComma(rightstr));
				rightstr=RemoveSpaces(rightstr);
				sys.ChannelNames[15+17] = rightstr;
			}
			else if (str.Left(13)=="DACCHANNEL#18") {
				rightstr=GetRightString(readstr);
				rightstr=rightstr.Left(FindComma(rightstr));
				rightstr=RemoveSpaces(rightstr);
				sys.ChannelNames[15+18] = rightstr;
			}
			else if (str.Left(13)=="DACCHANNEL#19") {
				rightstr=GetRightString(readstr);
				rightstr=rightstr.Left(FindComma(rightstr));
				rightstr=RemoveSpaces(rightstr);
				sys.ChannelNames[15+19] = rightstr;
			}
			else if (str.Left(13)=="DACCHANNEL#20") {
				rightstr=GetRightString(readstr);
				rightstr=rightstr.Left(FindComma(rightstr));
				rightstr=RemoveSpaces(rightstr);
				sys.ChannelNames[15+20] = rightstr;
			}
			else if (str.Left(13)=="DACCHANNEL#21") {
				rightstr=GetRightString(readstr);
				rightstr=rightstr.Left(FindComma(rightstr));
				rightstr=RemoveSpaces(rightstr);
				sys.ChannelNames[15+21] = rightstr;
			}
			else if (str.Left(13)=="DACCHANNEL#22") {
				rightstr=GetRightString(readstr);
				rightstr=rightstr.Left(FindComma(rightstr));
				rightstr=RemoveSpaces(rightstr);
				sys.ChannelNames[15+22] = rightstr;
			}
			else if (str.Left(13)=="DACCHANNEL#23") {
				rightstr=GetRightString(readstr);
				rightstr=rightstr.Left(FindComma(rightstr));
				rightstr=RemoveSpaces(rightstr);
				sys.ChannelNames[15+23] = rightstr;
			}
			else if (str.Left(13)=="DACCHANNEL#24") {
				rightstr=GetRightString(readstr);
				rightstr=rightstr.Left(FindComma(rightstr));
				rightstr=RemoveSpaces(rightstr);
				sys.ChannelNames[15+24] = rightstr;
			}

            //....DAC stuff here......................
			else if (str.Left(10)=="SETUPDAC#1") {
				str = GetRightString(str);
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[0].MaxValue));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[0].MaxDac));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[0].MinValue));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[0].MinDac));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[0].MaxGraph));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[0].MinGraph));
				//all that's left in str is what we want...
				sscanf(str,"%ld",&(sys.dac[0].Units));
			}
			else if (str.Left(11)=="SETUPDAC #2") {
				str = GetRightString(str);

				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[1].MaxValue));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[1].MaxDac));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[1].MinValue));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[1].MinDac));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[1].MaxGraph));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[1].MinGraph));
				//all that's left in str is what we want...
				sscanf(str,"%ld",&(sys.dac[1].Units));
			}
			else if (str.Left(11)=="SETUPDAC #3") {
				str = GetRightString(str);

				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[2].MaxValue));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[2].MaxDac));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[2].MinValue));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[2].MinDac));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[2].MaxGraph));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[2].MinGraph));
				//all that's left in str is what we want...
				sscanf(str,"%ld",&(sys.dac[2].Units));
			}
			else if (str.Left(11)=="SETUPDAC #4") {
				str = GetRightString(str);

				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[3].MaxValue));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[3].MaxDac));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[3].MinValue));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[3].MinDac));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[3].MaxGraph));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[3].MinGraph));
				//all that's left in str is what we want...
				sscanf(str,"%ld",&(sys.dac[3].Units));
			}
			else if (str.Left(11)=="SETUPDAC #5") {
				str = GetRightString(str);

				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[4].MaxValue));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[4].MaxDac));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[4].MinValue));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[4].MinDac));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[4].MaxGraph));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[4].MinGraph));
				//all that's left in str is what we want...
				sscanf(str,"%ld",&(sys.dac[4].Units));
			}
			else if (str.Left(11)=="SETUPDAC #6") {
				str = GetRightString(str);

				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[5].MaxValue));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[5].MaxDac));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[5].MinValue));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[5].MinDac));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[5].MaxGraph));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[5].MinGraph));
				//all that's left in str is what we want...
				sscanf(str,"%ld",&(sys.dac[5].Units));
			}
			else if (str.Left(11)=="SETUPDAC #7") {
				str = GetRightString(str);

				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[6].MaxValue));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[6].MaxDac));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[6].MinValue));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[6].MinDac));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[6].MaxGraph));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[6].MinGraph));
				//all that's left in str is what we want...
				sscanf(str,"%ld",&(sys.dac[6].Units));
			}
			else if (str.Left(11)=="SETUPDAC #8") {
				str = GetRightString(str);

				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[7].MaxValue));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[7].MaxDac));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[7].MinValue));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[7].MinDac));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[7].MaxGraph));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[7].MinGraph));
				//all that's left in str is what we want...
				sscanf(str,"%ld",&(sys.dac[7].Units));
			}
			else if (str.Left(11)=="SETUPDAC #9") {
				str = GetRightString(str);

				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[8].MaxValue));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[8].MaxDac));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[8].MinValue));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[8].MinDac));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[8].MaxGraph));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[8].MinGraph));
				//all that's left in str is what we want...
				sscanf(str,"%ld",&(sys.dac[8].Units));
			}
			else if (str.Left(11)=="SETUPDAC#10") {
				str = GetRightString(str);

				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[9].MaxValue));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[9].MaxDac));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[9].MinValue));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[9].MinDac));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[9].MaxGraph));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[9].MinGraph));
				//all that's left in str is what we want...
				sscanf(str,"%ld",&(sys.dac[9].Units));
			}
			else if (str.Left(11)=="SETUPDAC#11") {
				str = GetRightString(str);

				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[10].MaxValue));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[10].MaxDac));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[10].MinValue));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[10].MinDac));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[10].MaxGraph));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[10].MinGraph));
				//all that's left in str is what we want...
				sscanf(str,"%ld",&(sys.dac[10].Units));
			}
			else if (str.Left(11)=="SETUPDAC#12") {
				str = GetRightString(str);

				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[11].MaxValue));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[11].MaxDac));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[11].MinValue));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[11].MinDac));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[11].MaxGraph));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[11].MinGraph));
				//all that's left in str is what we want...
				sscanf(str,"%ld",&(sys.dac[11].Units));
			}
			else if (str.Left(11)=="SETUPDAC#13") {
				str = GetRightString(str);

				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[12].MaxValue));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[12].MaxDac));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[12].MinValue));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[12].MinDac));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[12].MaxGraph));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[12].MinGraph));
				//all that's left in str is what we want...
				sscanf(str,"%ld",&(sys.dac[12].Units));
			}
			else if (str.Left(11)=="SETUPDAC#14") {
				str = GetRightString(str);

				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[13].MaxValue));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[13].MaxDac));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[13].MinValue));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[13].MinDac));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[13].MaxGraph));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[13].MinGraph));
				//all that's left in str is what we want...
				sscanf(str,"%ld",&(sys.dac[13].Units));
			}
			else if (str.Left(11)=="SETUPDAC#15") {
				str = GetRightString(str);

				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[14].MaxValue));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[14].MaxDac));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[14].MinValue));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[14].MinDac));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[14].MaxGraph));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[14].MinGraph));
				//all that's left in str is what we want...
				sscanf(str,"%ld",&(sys.dac[14].Units));
			}
			else if (str.Left(11)=="SETUPDAC#16") {
				str = GetRightString(str);

				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[15].MaxValue));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[15].MaxDac));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[15].MinValue));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[15].MinDac));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[15].MaxGraph));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[15].MinGraph));
				//all that's left in str is what we want...
				sscanf(str,"%ld",&(sys.dac[15].Units));
			}
			else if (str.Left(11)=="SETUPDAC#17") {
				str = GetRightString(str);

				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[16].MaxValue));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[16].MaxDac));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[16].MinValue));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[16].MinDac));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[16].MaxGraph));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[16].MinGraph));
				//all that's left in str is what we want...
				sscanf(str,"%ld",&(sys.dac[16].Units));
			}
			else if (str.Left(11)=="SETUPDAC#18") {
				str = GetRightString(str);

				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[17].MaxValue));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[17].MaxDac));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[17].MinValue));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[17].MinDac));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[17].MaxGraph));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[17].MinGraph));
				//all that's left in str is what we want...
				sscanf(str,"%ld",&(sys.dac[17].Units));
			}
			else if (str.Left(11)=="SETUPDAC#19") {
				str = GetRightString(str);

				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[18].MaxValue));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[18].MaxDac));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[18].MinValue));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[18].MinDac));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[18].MaxGraph));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[18].MinGraph));
				//all that's left in str is what we want...
				sscanf(str,"%ld",&(sys.dac[18].Units));
			}
			else if (str.Left(11)=="SETUPDAC#20") {
				str = GetRightString(str);

				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[19].MaxValue));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[19].MaxDac));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[19].MinValue));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[19].MinDac));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[19].MaxGraph));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[19].MinGraph));
				//all that's left in str is what we want...
				sscanf(str,"%ld",&(sys.dac[19].Units));
			}
			else if (str.Left(11)=="SETUPDAC#21") {
				str = GetRightString(str);

				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[20].MaxValue));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[20].MaxDac));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[20].MinValue));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[20].MinDac));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[20].MaxGraph));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[20].MinGraph));
				//all that's left in str is what we want...
				sscanf(str,"%ld",&(sys.dac[20].Units));
			}
			else if (str.Left(11)=="SETUPDAC#22") {
				str = GetRightString(str);

				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[21].MaxValue));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[21].MaxDac));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[21].MinValue));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[21].MinDac));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[21].MaxGraph));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[21].MinGraph));
				//all that's left in str is what we want...
				sscanf(str,"%ld",&(sys.dac[21].Units));
			}
			else if (str.Left(11)=="SETUPDAC#23") {
				str = GetRightString(str);

				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[22].MaxValue));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[22].MaxDac));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[22].MinValue));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[22].MinDac));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[22].MaxGraph));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[22].MinGraph));
				//all that's left in str is what we want...
				sscanf(str,"%ld",&(sys.dac[22].Units));
			}
			else if (str.Left(11)=="SETUPDAC#24") {
				str = GetRightString(str);

				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[23].MaxValue));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[23].MaxDac));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[23].MinValue));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[23].MinDac));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[23].MaxGraph));
				rightstr = CommaParse(str);
				str = CommaReturn(str);
				sscanf(rightstr,"%f",&(sys.dac[23].MinGraph));
				//all that's left in str is what we want...
				sscanf(str,"%ld",&(sys.dac[23].Units));
			}
	}//end while (read ESI info)

	ar3.Close();
	ConfigFile.Close();
	return TRUE;
}

void CInstrument::CopySys(/* int from, int to */)
{
	// PBO 3/29/2003 : no longer necessary, only one sys per instrument now.
//	Lock();
/*	int j,k;
	// copies sys[from] to sys[to]
		sys[to].MajorVersion				= sys[from].MajorVersion;
		sys[to].MinorVersion				= sys[from].MinorVersion;
		sys[to].UserID						= sys[from].UserID;
		sys[to].SystemName					= sys[from].SystemName;
		sys[to].SystemSerialNumber			= sys[from].SystemSerialNumber;
		sys[to].HaveImpulse					= sys[from].HaveImpulse;
		sys[to].HaveSynthesizer				= sys[from].HaveSynthesizer;
		sys[to].HaveArbGenerator			= sys[from].HaveArbGenerator;
		sys[to].HaveNBMonitor				= sys[from].HaveNBMonitor;
		sys[to].ProgrammableFilter			= sys[from].ProgrammableFilter;
		sys[to].ArbGeneratorMemory			= sys[from].ArbGeneratorMemory;

		sys[to].ArrayProcessorMemory		= sys[from].ArrayProcessorMemory;
		sys[to].AdcBits						= sys[from].AdcBits;
		sys[to].MaxAdcRate					= sys[from].MaxAdcRate;
		sys[to].DigitizerMemory				= sys[from].DigitizerMemory;
		sys[to].HiResMALDI					= sys[from].HiResMALDI;
		sys[to].HiResESI					= sys[from].HiResESI;
		sys[to].BootMALDI					= sys[from].BootMALDI;
		sys[to].BootESI						= sys[from].BootESI;
		sys[to].ProgrammableGain			= sys[from].ProgrammableGain;
		sys[to].MagnetDrift					= sys[from].MagnetDrift;
		sys[to].FSPType						= sys[from].FSPType;
		sys[to].LaserTriggerDelay			= sys[from].LaserTriggerDelay;
		sys[to].FlipFlopRFPhase				= sys[from].FlipFlopRFPhase;
		sys[to].DCRemoval					= sys[from].DCRemoval;
		sys[to].WindowFunction				= sys[from].WindowFunction; // cosine window
		sys[to].TransientPointsUsed			= sys[from].TransientPointsUsed;
		sys[to].ZeroFills					= sys[from].ZeroFills;
		sys[to].FrequencyDataType			= sys[from].FrequencyDataType;

		sys[to].PeakPickerThreshold			= sys[from].PeakPickerThreshold;
		sys[to].MinimumAutoThreshold		= sys[from].MinimumAutoThreshold;
		sys[to].PeakPickerMeanFactor		= sys[from].PeakPickerMeanFactor;
		sys[to].PeakPickerLevel				= sys[from].PeakPickerLevel;
		sys[to].PeakPickerLoopCount			= sys[from].PeakPickerLoopCount;
		sys[to].PeakPickerLowMass			= sys[from].PeakPickerLowMass;
		sys[to].PeakPickerHighMass			= sys[from].PeakPickerHighMass;

		sys[to].UserDir = sys[from].UserDir;
		sys[to].UserIDFile	= sys[from].UserIDFile;
		sys[to].UserSetupFile = sys[from].UserSetupFile;

		sys[to].DefaultSetupFile = sys[from].DefaultSetupFile;
		sys[to].BudaPath	= sys[from].BudaPath;
		sys[to].DataPath	= sys[from].DataPath;
		sys[to].TempPath	= sys[from].TempPath;
		sys[to].CalibrationFile = sys[from].CalibrationFile;
		sys[to].DefaultSetupFile = sys[from].DefaultSetupFile;

		sys[to].SystemLowMass = sys[from].SystemLowMass; 
		sys[to].SystemHighMass = sys[from].SystemHighMass; 
		sys[to].SystemLowFrequency = sys[from].SystemLowFrequency; 
		sys[to].SystemHighFrequency = sys[from].SystemHighFrequency ;  

		sys[to].TransientMemoryLimit = sys[from].TransientMemoryLimit; 
		sys[to].TransientFFTLimit	= sys[from].TransientFFTLimit;
		sys[to].APFFTLimit	=  sys[from].APFFTLimit; 

		sys[to].PsgTicksPerMillisecond = sys[from].PsgTicksPerMillisecond;	

		sys[to].TrapShift		= sys[from].TrapShift;
		sys[to].IntensityShift	= sys[from].IntensityShift;
		sys[to].HighMassShift	= sys[from].HighMassShift;
		sys[to].AutoLabelPeaks	= sys[from].AutoLabelPeaks;
		sys[to].AutoLabelCharge	= sys[from].AutoLabelCharge;
		sys[to].LabelCharge1	= sys[from].LabelCharge1;
		sys[to].ChargeLimit		= sys[from].ChargeLimit;
		sys[to].CorrectTotalIntensity = sys[from].CorrectTotalIntensity;
		sys[to].ExcludeNoResPeaks = sys[from].ExcludeNoResPeaks;

		sys[to].PeakRFAmpVoltage			= sys[from].PeakRFAmpVoltage;
		sys[to].MinRFAmpVoltage				= sys[from].MinRFAmpVoltage;
		sys[to].PeakQuadRFVoltage			= sys[from].PeakQuadRFVoltage;
		sys[to].RFAmpFixedGain				= sys[from].RFAmpFixedGain;
		sys[to].QuadRFFixedGain				= sys[from].QuadRFFixedGain;
		sys[to].GainControlVoltageDivider	= sys[from].GainControlVoltageDivider;
		sys[to].SynOutputVoltage			= sys[from].SynOutputVoltage;
		sys[to].ArbOutputVoltage			= sys[from].ArbOutputVoltage;
		sys[to].QuadFrequency			= sys[from].QuadFrequency;
		sys[to].MixerCorrection				= sys[from].MixerCorrection;

		sys[to].A = sys[from].A;
		sys[to].B = sys[from].B;

		sys[to].CalibrationLoaded = sys[from].CalibrationLoaded;  
		sys[to].CalibrationVersion = sys[from].CalibrationVersion;
		sys[to].CalibrationPoints = sys[from].CalibrationPoints;
		sys[to].CalDate = sys[from].CalDate;
		sys[to].CalDescription = sys[from].CalDescription;

		// setup the ChannelNames
		for(j=0; j<40; j++) {
			sys[to].ChannelNames[j]= sys[from].ChannelNames[j];
		} //end for loop

		// set up calibration points
		for(j=0; j<MAX_CALIB_POINTS; j++) {
			sys[to].CalData[j].ActualMass = sys[from].CalData[j].ActualMass;
			sys[to].CalData[j].MeasuredFrequency = sys[from].CalData[j].MeasuredFrequency;
			sys[to].CalData[j].TrapVoltage = sys[from].CalData[j].TrapVoltage;
			sys[to].CalData[j].TotalIntensity = sys[from].CalData[j].TotalIntensity;
			sys[to].CalData[j].IncludeInCalc = sys[from].CalData[j].IncludeInCalc;
			for(k=0; k<40; k++) sys[to].CalData[j].TimeEntered[k] = sys[from].CalData[j].TimeEntered[k];
		}

		// set up dacs
		for(j=0; j<MAX_DACS; j++) {
			sys[to].dac[j].Units = sys[from].dac[j].Units;
			sys[to].dac[j].MaxValue = sys[from].dac[j].MaxValue;
			sys[to].dac[j].MinValue = sys[from].dac[j].MinValue;
			sys[to].dac[j].MaxDac = sys[from].dac[j].MaxDac;
			sys[to].dac[j].MinDac = sys[from].dac[j].MinDac;
			sys[to].dac[j].MaxGraph = sys[from].dac[j].MaxGraph;
			sys[to].dac[j].MinGraph = sys[from].dac[j].MinGraph;
			sys[to].dac[j].DacAvailable = sys[from].dac[j].DacAvailable;
		}	

		// set the ADC rate stuff
		for(j=0; j<16; j++) {
			sys[to].ADCRateAvailable[j] =	sys[from].ADCRateAvailable[j];
			sys[to].ADCRateString[j]	=	sys[from].ADCRateString[j];
			sys[to].ADCRateList[j]		=	sys[from].ADCRateList[j];
		}
*/
//		Unlock();
}

//---------------------------------------------------------------
//
// The main Instrument thread procedure
//
//---------------------------------------------------------------
#ifdef DEBUG_INSTRUMENT_THREAD

UINT InstrumentThreadProc(LPVOID pInstrument) 
{
	CInstrument* pSys = (CInstrument*)pInstrument;
	struct InstrumentThreadInfo* pInfo;	// pointer to the current FFT info
	pInfo = &(pSys->ThreadInfo);

	//WaitForSingleObject(pInfo->m_hStartThread, INFINITE);
	pInfo->pMainWnd = pSys->pApp->m_pMainWnd;

	// read in configuration files and set up sys and ps arrays
	pSys->Setup();

	// initialize the instrument
	pSys->InitializeInstrument();

	// run start up diagnostics
	pSys->StartupDiagnostics();

	BOOL Kill_Thread = FALSE;
	while (TRUE) // infinite loop
	{
		// first check if this thread should be killed i.e. m_hEventFFTThreadKilled
		if( Kill_Thread ) {
			// if here, then m_hKillThread was signalled, so we must break out of
			// the infinite loop and exit/terminate the thread
			ResetEvent(pInfo->m_hKillThread);
			break;	// break the infinite loop
		}

		pInfo->ThreadStatus = 0; // fft in stage 0 (idle)

		// reset StartFFTEvent then idle and wait for new ExecuteCommand
		ResetEvent(pInfo->m_hProcessCommand);

		/// *** WAITPOINT ***
		// this is where the thread waits unit a command is entered and it is
		// given the m_hProcessCommand event
		WaitForSingleObject(pInfo->m_hProcessCommand, INFINITE);
		ResetEvent(pInfo->m_hProcessCommand);

		switch(pInfo->InstrumentCommands) {
		case IDLE:
			pInfo->ThreadStatus = IDLE;
			// idle command;
			break;
		case KILL_THREAD:
			// Kill the thread
			Kill_Thread = TRUE;
			break;
		case RUN_EXPERIMENT:
			// acquire spectrum
			pSys->RunExperiment();
			break;
		case ABORT_EXPERIMENT:
			// run the three startup diagnostics tests
			pSys->AbortExperiment();
			break;
		case RESET_INSTRUMENT:
			pSys->ResetInstrument();
			break;
		case UPDATE_STATUS:
			pSys->UpdateStatus();
			break;
		case CHANGE_SYSTEM:
			//pSys->CopySys(pSys->ActiveSystem,0);
			break;
		}		
		pInfo->InstrumentCommands = IDLE;
	}

	// only get here if a pInfo->m_hKillThread event was signaled
	SetEvent(pInfo->m_hThreadDead);
	TRACE("Buda::InstrumentThread Exited Normally\n");
	return 0; // function exited normally
}	// end InstrumentThreadProc();

#endif //DEBUG_INSTRUMENT_THREAD

void CInstrument::StartInstrumentThread()
{
	Access = new CCriticalSection();
	DataLock = new CSingleLock(Access,FALSE);

	// FFTThrdInfo
	ThreadInfo.m_hStartThread = CreateEvent(NULL, TRUE, FALSE, NULL	);	// event initially reset, manual reset
	ThreadInfo.m_hKillThread = CreateEvent(NULL, TRUE, FALSE, NULL	);	// event initially reset, manual reset
	ThreadInfo.m_hProcessCommand = CreateEvent(NULL, TRUE, FALSE, NULL	);	// event initially reset, manual reset
	ThreadInfo.m_hAbortExperiment = CreateEvent(NULL, TRUE, FALSE, NULL	);	// event initially reset, manual reset
	ThreadInfo.m_hStopAcquisition = CreateEvent(NULL, TRUE, FALSE, NULL	);	// event initially reset, manual reset
	ThreadInfo.m_hThreadDead = CreateEvent(NULL, TRUE, FALSE, NULL	);	// event initially reset, manual reset

	// create FFTThread
	InstrumentThread = AfxBeginThread(InstrumentThreadProc, this);
}

void CInstrument::KillInstrumentThread()
{
	ThreadInfo.ThreadStatus = KILL_THREAD;

	ThreadInfo.InstrumentCommands = KILL_THREAD;
	SetEvent(ThreadInfo.m_hProcessCommand);
	WaitForSingleObject(ThreadInfo.m_hThreadDead, INFINITE);

	if(Access != 0) {
		delete Access;
		Access = 0;
	}
	if(DataLock != 0) {
		delete DataLock;
		DataLock = 0;
	}
}

void CInstrument::InitializeInstrument()
{
	ThreadInfo.ThreadStatus = RESET_INSTRUMENT;
	ResetInstrument();
}

void CInstrument::RunExperiment()
{
	ThreadInfo.ThreadStatus = RUN_EXPERIMENT;
	pWnd = (CMainFrame*) pApp->m_pMainWnd;
	//pWnd->OnUpdateStatus(0);

	Running = TRUE;
	TDAstat = 0;
	VSPstat = 0;
	PSGstat = 0;
	ARBstat = 0;
	FSPstat = 0;

	if(pDoc->PSisDirty) {
		//Lock();	// no need, already in CopyPS
		CopyPS(&(pDoc->PS),&(PS));
		//Unlock();
	}

	// second, if number of datapoints has changed, delete and
	// recreate timearray
	if(pDoc->TDisDirty) {
		pDoc->DataPointSize = 32;
		if(TimeArray != 0) delete [] TimeArray;
		TimeArray = new long[PS.TransientPoints];
	}

	acqScansCompleted = 0;
	acqScansInMemory = 0;
	acqScansInTD = 0;

	long i;
	acqGroupSize = 1;
	for (i=0; i<(16 - sys.AdcBits); i++) acqGroupSize *= 2;
	acqGroupSize --;

	BuildPS();
	RateCode = ADCRateCode(PS.AdcRate);
	TriggerCode = PS.TriggerMode;

	StartTimer();	// only needed for scanduration calculation
	double dtime = 0;
	
	acqAbort = FALSE;
	acqStop = FALSE;
	acqTRASH = FALSE;
//	long Address = 0;
//	long Samples = PS.TransientPoints;
	while(!acqStop && !acqAbort && acqScansCompleted < PS.ScansRequested) {
		// reset everything and start.
//		tdrst();
//		vsprst();
//		fsprst();
//		arbrst();
//		psgrst();
		start = clock();
		now = clock();
//		ftacq16(&Address,&Samples, &RateCode, &TriggerCode);

		while(!acqAbort) {
			// check the instrument status, if done, break;
//			bxstat(&TDAstat,&FSPstat,&PSGstat,&ARBstat,&VSPstat);
			if(PSGstat == 0 && FSPstat == 0 && TDAstat == 1) break;

			//update the progressbar
			now = clock(); // note this clock has a 55 millisecond resolution
			dtime = (double)(now - start) / CLOCKS_PER_SEC;

			// for now, use dtime to break out of the loop
			if(dtime >= scanduration) break;
			//pWnd->OnUpdateStatus(int(100*dtime/scanduration));

			//wait 1 millisecond
			WaitForSingleObject(ThreadInfo.m_hAbortExperiment,1);
		}

		if(!acqAbort) {
			acqScansCompleted++;
			acqScansInTD++;

			if(PS.ScansRequested > 1) {
/*				if(acqScansInTD == 1) 
				//	tdxfer(&Address,&Samples);
				else
				//	tdsum(&Address,&Samples);
*/			}

			if(acqScansInTD >= acqGroupSize) {
				ReadTD();
				acqScansInTD = 0;
			}
		} // if(!acqAbort)
	} // while(!acqAbort && acqScansCompleted < PS.ScansRequested)
	
	if(acqTRASH) acqScansCompleted = 0;

	PS.ScansPerSpectrum = acqScansCompleted;

	Lock();
	if(acqScansCompleted > 0) {
		ReadTD();
		PS.SpectraAcquired = 1;
		// copy back the acquired data and changed elements of PS
		for(i=0; i<PS.TransientPoints; i++) {
			pDoc->TimeArray[i] = TimeArray[i];
		}
		for(i=0; i<40; i++) pDoc->PS.TimeAcquired[i] = PS.TimeAcquired[i];
		pDoc->PS.SpectraAcquired = PS.SpectraAcquired;
		pDoc->PS.ScansPerSpectrum = PS.ScansPerSpectrum;
		pDoc->TransientIsDefined = TRUE;
		pDoc->IsDefined = TRUE;

	} else {
		pDoc->IsDefined = FALSE;
		pDoc->TransientIsDefined = FALSE;
	}
	Unlock();
	// signal to main app that new data is ready and to refresh the views
	::PostMessage(ThreadInfo.pMainWnd->m_hWnd,WM_NEWDATAREADY,0,0);
	

	Running = FALSE;
	if(TuneMode && !acqAbort) RunExperiment();
}

void CInstrument::ResetInstrument()
{
	ThreadInfo.ThreadStatus = RESET_INSTRUMENT;
	// reset merely calls the instrument InitTD() high level
	// driver dll call. - that command does what is necessary
	//InitTD();
}

BOOL CInstrument::Lock()
{
	int i=0;
	while(DataLock->IsLocked()) {
		i++;
	}
	return DataLock->Lock(INFINITE);
}

BOOL CInstrument::Unlock()
{
	return DataLock->Unlock();
}

BOOL CInstrument::RunPSGDiagnostic()
{
	return FALSE;
}

BOOL CInstrument::RunVSPDiagnostic()
{
	// this should build up the VSPDiagnostic PS and run with it.
	return FALSE;
}

BOOL CInstrument::RunFSPDiagnostic()
{
	return FALSE;
}

BOOL CInstrument::RunARBDiagnostic()
{
	return FALSE;
}

BOOL CInstrument::RunTDDiagnostic()
{
	return FALSE;
}

void CInstrument::CopyPS(struct PS* from, struct PS* to)
{
	Lock();
	// first save the PS pointers
	struct ChirpPacket*	cfp = to->cfp;
	struct DacPacket*	dac = to->dac;
	struct ArbPacket*	arb = to->arb;
	struct PSDesc*		GenPS = to->GenPS;
	struct Spectrum*	Spec = to->Spec;

	// copy the data from from to to (note: this also changes the 
	// pointers, but doesn't copy the data from one e.g cfp array
	// to the other one, so then copy those individually)
	*to = *from;

	// fix the PS pointers
	to->cfp = cfp;
	to->dac = dac;
	to->arb = arb;
	to->GenPS = GenPS;
	to->Spec = Spec;

	// now copy over the data pointed to.
	long i;
	for(i=0; i<from->NumCFP; i++) to->cfp[i] = from->cfp[i];
	for(i=0; i<from->NumDAC; i++) to->dac[i] = from->dac[i];
	for(i=0; i<from->NumARB; i++) to->arb[i] = from->arb[i];
	for(i=0; i<from->NumEvents; i++) to->GenPS[i] = from->GenPS[i];
	for(i=0; i<from->SpectraRequested; i++) to->Spec[i] = from->Spec[i];
	Unlock();
}

float CInstrument::FlightTime(double Mass, double QuadOffset)
{
	// returns the flight time in milliseconds
	float flighttime;

	// old version - thru 12-17-98
	/*
	if (Mass > 0) 
		flighttime = 0.0161 * sqrt(Mass);
	else
		flighttime = 0;
	*/
	// since sqrt's are involved, we must take abolute values
	Mass = fabs(Mass);
	QuadOffset = fabs(QuadOffset);

	if(Mass > 0 && QuadOffset > 1) 
		flighttime = float(0.10 + 0.0864*sqrt(Mass/QuadOffset));
	else
		flighttime = float(0.2);
	return flighttime;
}

float CInstrument::QuadRFNeeded(double Mass)
{
	//	computes the rf amplitude (Vbp) needed to efficiently pass
	//	ions of m/z Mass

	// since sqrt's are involved, we must take abolute values
	Mass = fabs(Mass);

	float amplitude;
	amplitude = float(0.086 * Mass + 70.0);
	return amplitude;
}

int CInstrument::SetBudaPath(CString pathname)
{
	// sets the path name to BudaPath for instrument
	sys.BudaPath = pathname;

	// also setup datapaths and etc?
	return 1;
}


void CInstrument::AbortExperiment()
{
	ThreadInfo.ThreadStatus = ABORT_EXPERIMENT;	
	// this routine just calls the high level dll driver routine
	// AbortTD();
	//AbortTD();
}

void CInstrument::UpdateStatus()
{
}

void CInstrument::ReadTD()
{
	// read the data.
//	long Address = 0;
//	long Samples = PS.TransientPoints;

	if(acqScansInMemory == 0) {
		//if first scan, just transfer it to memory
/*		if(acqScansInTD == 1) 
//			rdamem(&Address,&Samples,TimeArray);
		else
//			rdbm16(&Address,&Samples,TimeArray);
*/	} else {
/*		// if not first scan, add it to memory
		if(acqScansInTD == 1) 
//			rdamema(&Address,&Samples,TimeArray);
		else
//			rdbm16a(&Address,&Samples,TimeArray);
*/	}
}

BOOL CInstrument::InterpretConfigFileString(CString readstr)
{
	BOOL Identified = FALSE;
	CString rightstr,str;
	str=CompressString(readstr);
	int i;

		// set the Magnet drift per hour
		if (str.Left(11)=="MAGNETDRIFT") {
			rightstr = GetRightString(str);
			sscanf(rightstr,"%lf",&(sys.MagnetDrift));
			Identified = TRUE;
		}
		// check for maldi
		if (str.Left(10)=="HIRESMALDI") {
			if(str.Right(3)=="YES") {
				sys.HiResMALDI = 1;
			} else {
				sys.HiResMALDI = 0;
			}
			Identified = TRUE;
		}
		// check for esi 
		else if (str.Left(8)=="HIRESESI") {
			if(str.Right(3)=="YES") {
				sys.HiResESI = 1;
			} else {
				sys.HiResESI = 0;
			}
			Identified = TRUE;
		}
		// do we boot into MALDI or ESI as a default?
		else if (str.Left(11)=="DEFAULTBOOT") {
			if(sys.HiResESI && sys.HiResMALDI) {
				if(str.Right(5)=="MALDI") {
					sys.BootMALDI = 1;
					sys.BootESI = 0;
				} else {
					sys.BootMALDI = 0;
					sys.BootESI = 1;
				}
			}
			if(sys.HiResESI && !sys.HiResMALDI) {
				sys.BootMALDI = 0;
				sys.BootESI = 1;
			}
			if(!sys.HiResESI && sys.HiResMALDI) {
				sys.BootMALDI = 1;
				sys.BootESI = 0;
			}
			if(!sys.HiResESI && !sys.HiResMALDI) {
				MessageBox("Config file must have EITHER:\nHiResESI = yes	OR\nHiResMALDI = yes","Error in Config file");
				return FALSE;
			}		
			Identified = TRUE;
		}

		// Label Charge 1
		else if (str.Left(12)=="LABELCHARGE1") {
			rightstr = GetRightString(str);
			if (rightstr == "YES")
				sys.LabelCharge1 = 1;
			else 
				sys.LabelCharge1 = 0;
		}

		// ADC rates.
		else if (str.Left(17) == "ADCRATESAVAILABLE") {
			// need to chop up the rightstr.
			rightstr = GetRightString(str);
			for(i=0; i<12; i++) {
				if(rightstr.GetAt(i) == '1') 
					sys.ADCRateAvailable[i] = TRUE;
				else sys.ADCRateAvailable[i] = FALSE;
			}
		}

		// do we have an impulse amplifier?
		else if (str.Left(10)=="IMPULSEAMP") {
			if(str.Right(3)=="YES") {
				sys.HaveImpulse = 1;
			}
			else {
				sys.HaveImpulse = 0;
			}
			Identified = TRUE;
		}
		// do we have a frequency synthesizer?
		else if (str.Left(11)=="SYNTHESIZER") {
			if(str.Right(3)=="YES") {
				sys.HaveSynthesizer = 1;
			}
			else {
				sys.HaveSynthesizer = 0;
			}
			Identified = TRUE;
		}
		// do we have an Arbitrary Waveform Generator?
		else if (str.Left(12)=="ARBGENERATOR") {
			if(str.Right(3)=="YES") {
				sys.HaveArbGenerator = 1;
			}
			else {
				sys.HaveArbGenerator = 0;
			}
			Identified = TRUE;
		}
		// how much arb memory?
		else if (str.Left(9)=="ARBMEMORY") {
			rightstr = GetRightString(str);
			sscanf(rightstr,"%ld",&(sys.ArbGeneratorMemory));
			Identified = TRUE;
		}
		// what type of synthesizer?
		else if (str.Left(7)=="FSPTYPE") {
			rightstr = GetRightString(str);
			sscanf(rightstr,"%ld",&(sys.FSPType));
			Identified = TRUE;
		}
		// do we have a programmable filter?
		else if (str.Left(18)=="PROGRAMMABLEFILTER") {
			if(str.Right(3)=="YES") {
				sys.ProgrammableFilter = 1;
			}
			else {
				sys.ProgrammableFilter = 0;
			}
			Identified = TRUE;
		}
		// do we have programmable gain?
		else if (str.Left(16)=="PROGRAMMABLEGAIN") {
			if(str.Right(3)=="YES") {
				sys.ProgrammableGain = 1;
			}
			else {
				sys.ProgrammableGain = 0;
			}
			Identified = TRUE;
		}
		// do we have Narrow band monitor?
		else if (str.Left(19)=="NBMONITOR") {
			if(str.Right(3)=="YES") {
				sys.HaveNBMonitor = 1;
			}
			else {
				sys.HaveNBMonitor = 0;
			}
			Identified = TRUE;
		}
		// how many bits to the ADC?
		else if (str.Left(13)=="ADCRESOLUTION") {
			rightstr = GetRightString(str);
			sscanf(rightstr,"%ld",&(sys.AdcBits));
			Identified = TRUE;
		}
		// what is the maximum adc rate?
		else if (str.Left(10)=="MAXADCRATE") {
			rightstr = GetRightString(str);
			sscanf(rightstr,"%lf",&(sys.MaxAdcRate));
			Identified = TRUE;
		}
		// what is the maximum number of data points per transient?
		else if (str.Left(15)=="DIGITIZERMEMORY") {
			rightstr = GetRightString(str);
			sscanf(rightstr,"%ld",&(sys.DigitizerMemory));
			Identified = TRUE;
		}
		// do we automatically remove the DC offset?
		else if (str.Left(12)=="FFTDCREMOVAL") {
			if(str.Right(3)=="YES") {
				sys.DCRemoval = 1;
			}
			else {
				sys.DCRemoval = 0;
			}
			Identified = TRUE;
		}
		// what type of apodization function?
		else if (str.Left(17)=="FFTWINDOWFUNCTION") {
			if(str.Right(6)=="COSINE") {
				sys.WindowFunction = 1;
			}
			else if(str.Right(4) == "NONE") {
				sys.WindowFunction = 0;
			}
			else {
				sys.WindowFunction = 0;
			}
			Identified = TRUE;
		}
		// how many zero fills?
		else if (str.Left(12)=="FFTZEROFILLS") {
			if(str.Right(5)=="TABLE") {
				sys.ZeroFills = -1; // 
			}
			else {
				rightstr = GetRightString(str);
				sscanf(rightstr,"%ld",&(sys.ZeroFills));
			}
			Identified = TRUE;
		}
		// do we automatically remove the DC offset?
		else if (str.Left(17)=="EXCLUDENORESPEAKS") {
			if(str.Right(3)=="YES") {
				sys.ExcludeNoResPeaks = 1;
			}
			else {
				sys.ExcludeNoResPeaks = 0;
			}
			Identified = TRUE;
		}
		//put peak picker stuff here (towards end of sys vars...)
		else if (str.Left(19)=="PEAKPICKERTHRESHOLD") {
			if (str.Right(4)=="AUTO")
				sys.PeakPickerThreshold = -1;
			else {
				rightstr = GetRightString(str);
				sscanf(rightstr,"%lf",&(sys.PeakPickerThreshold));
			}
			Identified = TRUE;
		}
		else if (str.Left(19)=="PEAKPICKERLOOPCOUNT") {
			rightstr = GetRightString(str);
			sscanf(rightstr,"%ld",&(sys.PeakPickerLoopCount));
			Identified = TRUE;
		}
		else if (str.Left(22)=="PEAKPICKERMINTHRESHOLD") {
			rightstr = GetRightString(str);
			sscanf(rightstr,"%lf",&(sys.MinimumAutoThreshold));
			Identified = TRUE;
		}
		else if (str.Left(20)=="PEAKPICKERMEANFACTOR") {
			rightstr = GetRightString(str);
			sscanf(rightstr,"%lf",&(sys.PeakPickerMeanFactor));
			Identified = TRUE;
		}
		else if (str.Left(15)=="PEAKPICKERLEVEL") {
			rightstr = GetRightString(str);
			//get rid of percent sign if applicable
			if (rightstr.Right(1)=="%") {
				rightstr = rightstr.Left(rightstr.GetLength()-1);
			}
			
			sscanf(rightstr,"%lf",&(sys.PeakPickerLevel));
			Identified = TRUE;
		}
		else if (str.Left(17)=="PEAKPICKERLOWMASS") {
			if (str.Right(4)=="AUTO")
				sys.PeakPickerThreshold = 0;
			else {
				rightstr = GetRightString(str);
				sscanf(rightstr,"%lf",&(sys.PeakPickerLowMass));
			}
			Identified = TRUE;
		}
		else if (str.Left(18)=="PEAKPICKERHIGHMASS") {
			if (str.Right(4)=="AUTO")
				sys.PeakPickerThreshold = 0;
			else {
				rightstr = GetRightString(str);
				sscanf(rightstr,"%lf",&(sys.PeakPickerHighMass));
			}
			Identified = TRUE;
		}

			if (str.Left(16)=="PEAKRFAMPVOLTAGE") {
				rightstr = GetRightString(str);
				sscanf(rightstr,"%lf",&(sys.PeakRFAmpVoltage));
			Identified = TRUE;
			}
			else if (str.Left(15)=="MINRFAMPVOLTAGE") {
				rightstr = GetRightString(str);
				sscanf(rightstr,"%lf",&(sys.MinRFAmpVoltage));
			Identified = TRUE;
			}
			else if (str.Left(17)=="PEAKQUADRFVOLTAGE") {
				rightstr = GetRightString(str);
				sscanf(rightstr,"%lf",&(sys.PeakQuadRFVoltage));
			Identified = TRUE;
			}
			else if (str.Left(16)=="SYNOUTPUTVOLTAGE") {
				rightstr = GetRightString(str);
				sscanf(rightstr,"%lf",&(sys.SynOutputVoltage));
				Identified = TRUE;
			}
			else if (str.Left(16)=="ARBOUTPUTVOLTAGE") {
				rightstr = GetRightString(str);
				sscanf(rightstr,"%lf",&(sys.ArbOutputVoltage));
				Identified = TRUE;
			}
			else if (str.Left(14)=="RFAMPFIXEDGAIN") {
				rightstr = GetRightString(str);
				sscanf(rightstr,"%lf",&(sys.RFAmpFixedGain));
				Identified = TRUE;
			}
			else if (str.Left(15)=="QUADRFFIXEDGAIN") {
				rightstr = GetRightString(str);
				sscanf(rightstr,"%lf",&(sys.QuadRFFixedGain));
				Identified = TRUE;
			}
			else if (str.Left(19)=="RFAMPVOLTAGEDIVIDER") {
				rightstr = GetRightString(str);
				sscanf(rightstr,"%lf",&(sys.GainControlVoltageDivider));
				Identified = TRUE;
			}
			else if (str.Left(18)=="MALDIQUADFREQUENCY") {
				rightstr = GetRightString(str);
				sscanf(rightstr,"%lf",&(sys.QuadFrequency));
				Identified = TRUE;
			}
			else if (str.Left(15)=="MIXERCORRECTION") {
				rightstr = GetRightString(str);
				sscanf(rightstr,"%lf",&(sys.MixerCorrection));
				Identified = TRUE;
			}
			else if (str.Left(8)=="DATAPATH") {
				rightstr = GetRightString(str);
				sys.DataPath = rightstr;
				Identified = TRUE;
			}
			else if (str.Left(8)=="TEMPPATH") {
				rightstr = GetRightString(str);
				sys.TempPath = rightstr;
				Identified = TRUE;
			}
			else if (str.Left(16)=="INITIALSETUPFILE") {
				rightstr = GetRightString(str);
				sys.DefaultSetupFile = rightstr;
				Identified = TRUE;
			}
			else if (str.Left(15)=="CALIBRATIONFILE") {
				rightstr = GetRightString(str);
				sys.CalibrationFile = rightstr;
				Identified = TRUE;
			}
			else if (str.Left(15)=="TRAPSHIFTFACTOR") {
				rightstr = GetRightString(str);
				sscanf(rightstr,"%lf",&(sys.TrapShift));
				Identified = TRUE;
			}
			else if (str.Left(20)=="INTENSITYSHIFTFACTOR") {
				rightstr = GetRightString(str);
				sscanf(rightstr,"%lf",&(sys.IntensityShift));
				Identified = TRUE;
			}
			else if (str.Left(19)=="HIGHMASSSHIFTFACTOR") {
				rightstr = GetRightString(str);
				sscanf(rightstr,"%lf",&(sys.HighMassShift));
				Identified = TRUE;
			}
			else if (str.Left(21)=="CORRECTTOTALINTENSITY") {
				rightstr = GetRightString(str);
				if (str == "YES")
					sys.CorrectTotalIntensity = 1;
				else 
					sys.CorrectTotalIntensity = 0;
				Identified = TRUE;
			}
			else if (str.Left(17)=="EXCLUDENORESPEAKS") {
				rightstr = GetRightString(str);
				if (rightstr == "YES")
					sys.ExcludeNoResPeaks = 1;
				else 
					sys.ExcludeNoResPeaks = 0;
				Identified = TRUE;
			}
			else if (str.Left(14)=="AUTOLABELPEAKS") {
				rightstr = GetRightString(str);
				if (rightstr == "YES")
					sys.AutoLabelPeaks = 1;
				else 
					sys.AutoLabelPeaks = 0;
				Identified = TRUE;
			}
			else if (str.Left(15)=="AUTOLABELCHARGE") {
				rightstr = GetRightString(str);
				if (rightstr == "YES")
					sys.AutoLabelCharge = 1;
				else 
					sys.AutoLabelCharge = 0;
				Identified = TRUE;
			}
			// get ttl and dac names here *********
			else if (str.Left(10)=="TTLPULSE#1") {
				rightstr=GetRightString(readstr);
				rightstr=rightstr.Left(FindComma(rightstr));
				rightstr=RemoveSpaces(rightstr);
				sys.ChannelNames[8] = rightstr;
				Identified = TRUE;
			}
			else if (str.Left(10)=="TTLPULSE#2") {
				rightstr=GetRightString(readstr);
				rightstr=rightstr.Left(FindComma(rightstr));
				rightstr=RemoveSpaces(rightstr);
				sys.ChannelNames[9] = rightstr;
				Identified = TRUE;
			}
			else if (str.Left(10)=="TTLPULSE#3") {
				rightstr=GetRightString(readstr);
				rightstr=rightstr.Left(FindComma(rightstr));
				rightstr=RemoveSpaces(rightstr);
				sys.ChannelNames[10] = rightstr;
				Identified = TRUE;
			}
			else if (str.Left(10)=="TTLPULSE#4") {
				rightstr=GetRightString(readstr);
				rightstr=rightstr.Left(FindComma(rightstr));
				rightstr=RemoveSpaces(rightstr);
				sys.ChannelNames[11] = rightstr;
				Identified = TRUE;
			}
			else if (str.Left(10)=="TTLPULSE#5") {
				rightstr=GetRightString(readstr);
				rightstr=rightstr.Left(FindComma(rightstr));
				rightstr=RemoveSpaces(rightstr);
				sys.ChannelNames[12] = rightstr;
				Identified = TRUE;
			}
			else if (str.Left(10)=="TTLPULSE#6") {
				rightstr=GetRightString(readstr);
				rightstr=rightstr.Left(FindComma(rightstr));
				rightstr=RemoveSpaces(rightstr);
				sys.ChannelNames[13] = rightstr;
				Identified = TRUE;
			}
			else if (str.Left(10)=="TTLPULSE#7") {
				rightstr=GetRightString(readstr);
				rightstr=rightstr.Left(FindComma(rightstr));
				rightstr=RemoveSpaces(rightstr);
				sys.ChannelNames[14] = rightstr;
				Identified = TRUE;
			}
			else if (str.Left(10)=="TTLPULSE#8") {
				rightstr=GetRightString(readstr);
				rightstr=rightstr.Left(FindComma(rightstr));
				rightstr=RemoveSpaces(rightstr);
				sys.ChannelNames[15] = rightstr;
				Identified = TRUE;
			}
			else if (str.Left(12)=="DACCHANNEL#5") {
				rightstr=GetRightString(readstr);
				rightstr=rightstr.Left(FindComma(rightstr));
				rightstr=RemoveSpaces(rightstr);
				sys.ChannelNames[15+5] = rightstr;
				Identified = TRUE;
			}
			else if (str.Left(12)=="DACCHANNEL#6") {
				rightstr=GetRightString(readstr);
				rightstr=rightstr.Left(FindComma(rightstr));
				rightstr=RemoveSpaces(rightstr);
				sys.ChannelNames[15+6] = rightstr;
				Identified = TRUE;
			}
			else if (str.Left(12)=="DACCHANNEL#9") {
				rightstr=GetRightString(readstr);
				rightstr=rightstr.Left(FindComma(rightstr));
				rightstr=RemoveSpaces(rightstr);
				sys.ChannelNames[15+9] = rightstr;
				Identified = TRUE;
			}
			else if (str.Left(13)=="DACCHANNEL#10") {
				rightstr=GetRightString(readstr);
				rightstr=rightstr.Left(FindComma(rightstr));
				rightstr=RemoveSpaces(rightstr);
				sys.ChannelNames[15+10] = rightstr;
				Identified = TRUE;
			}
			else if (str.Left(13)=="DACCHANNEL#11") {
				rightstr=GetRightString(readstr);
				rightstr=rightstr.Left(FindComma(rightstr));
				rightstr=RemoveSpaces(rightstr);
				sys.ChannelNames[15+11] = rightstr;
				Identified = TRUE;
			}
			else if (str.Left(13)=="DACCHANNEL#12") {
				rightstr=GetRightString(readstr);
				rightstr=rightstr.Left(FindComma(rightstr));
				rightstr=RemoveSpaces(rightstr);
				sys.ChannelNames[15+12] = rightstr;
				Identified = TRUE;
			}
			else if (str.Left(13)=="DACCHANNEL#13") {
				rightstr=GetRightString(readstr);
				rightstr=rightstr.Left(FindComma(rightstr));
				rightstr=RemoveSpaces(rightstr);
				sys.ChannelNames[15+13] = rightstr;
				Identified = TRUE;
			}
			else if (str.Left(13)=="DACCHANNEL#14") {
				rightstr=GetRightString(readstr);
				rightstr=rightstr.Left(FindComma(rightstr));
				rightstr=RemoveSpaces(rightstr);
				sys.ChannelNames[15+14] = rightstr;
				Identified = TRUE;
			}
			else if (str.Left(13)=="DACCHANNEL#15") {
				rightstr=GetRightString(readstr);
				rightstr=rightstr.Left(FindComma(rightstr));
				rightstr=RemoveSpaces(rightstr);
				sys.ChannelNames[15+15] = rightstr;
				Identified = TRUE;
			}
			else if (str.Left(13)=="DACCHANNEL#16") {
				rightstr=GetRightString(readstr);
				rightstr=rightstr.Left(FindComma(rightstr));
				rightstr=RemoveSpaces(rightstr);
				sys.ChannelNames[15+16] = rightstr;
				Identified = TRUE;
			}
			else if (str.Left(13)=="DACCHANNEL#17") {
				rightstr=GetRightString(readstr);
				rightstr=rightstr.Left(FindComma(rightstr));
				rightstr=RemoveSpaces(rightstr);
				sys.ChannelNames[15+17] = rightstr;
				Identified = TRUE;
			}
			else if (str.Left(13)=="DACCHANNEL#18") {
				rightstr=GetRightString(readstr);
				rightstr=rightstr.Left(FindComma(rightstr));
				rightstr=RemoveSpaces(rightstr);
				sys.ChannelNames[15+18] = rightstr;
				Identified = TRUE;
			}
			else if (str.Left(13)=="DACCHANNEL#19") {
				rightstr=GetRightString(readstr);
				rightstr=rightstr.Left(FindComma(rightstr));
				rightstr=RemoveSpaces(rightstr);
				sys.ChannelNames[15+19] = rightstr;
				Identified = TRUE;
			}
			else if (str.Left(13)=="DACCHANNEL#20") {
				rightstr=GetRightString(readstr);
				rightstr=rightstr.Left(FindComma(rightstr));
				rightstr=RemoveSpaces(rightstr);
				sys.ChannelNames[15+20] = rightstr;
				Identified = TRUE;
			}
			else if (str.Left(13)=="DACCHANNEL#21") {
				rightstr=GetRightString(readstr);
				rightstr=rightstr.Left(FindComma(rightstr));
				rightstr=RemoveSpaces(rightstr);
				sys.ChannelNames[15+21] = rightstr;
				Identified = TRUE;
			}
			else if (str.Left(13)=="DACCHANNEL#22") {
				rightstr=GetRightString(readstr);
				rightstr=rightstr.Left(FindComma(rightstr));
				rightstr=RemoveSpaces(rightstr);
				sys.ChannelNames[15+22] = rightstr;
				Identified = TRUE;
			}
			else if (str.Left(13)=="DACCHANNEL#23") {
				rightstr=GetRightString(readstr);
				rightstr=rightstr.Left(FindComma(rightstr));
				rightstr=RemoveSpaces(rightstr);
				sys.ChannelNames[15+23] = rightstr;
				Identified = TRUE;
			}
			else if (str.Left(13)=="DACCHANNEL#24") {
				rightstr=GetRightString(readstr);
				rightstr=rightstr.Left(FindComma(rightstr));
				rightstr=RemoveSpaces(rightstr);
				sys.ChannelNames[15+24] = rightstr;
				Identified = TRUE;
			}

			//SETUP DAC stuff here...................................
			else if (str.Left(11)=="SETUPDAC #1") {
				InterpretDACString(str,1);
			}
			else if (str.Left(11)=="SETUPDAC #2") {
				InterpretDACString(str,2);
			}
			else if (str.Left(11)=="SETUPDAC #3") {
				InterpretDACString(str,3);
			}
			else if (str.Left(11)=="SETUPDAC #4") {
				InterpretDACString(str,4);
			}
			else if (str.Left(11)=="SETUPDAC #5") {
				InterpretDACString(str,5);
			}
			else if (str.Left(11)=="SETUPDAC #6") {
				InterpretDACString(str,6);
			}
			else if (str.Left(11)=="SETUPDAC #7") {
				InterpretDACString(str,7);
			}
			else if (str.Left(11)=="SETUPDAC #8") {
				InterpretDACString(str,8);
			}
			else if (str.Left(11)=="SETUPDAC #9") {
				InterpretDACString(str,9);
			}
			else if (str.Left(11)=="SETUPDAC#10") {
				InterpretDACString(str,10);
			}
			else if (str.Left(11)=="SETUPDAC#11") {
				InterpretDACString(str,11);
			}
			else if (str.Left(11)=="SETUPDAC#12") {
				InterpretDACString(str,12);
			}
			else if (str.Left(11)=="SETUPDAC#13") {
				InterpretDACString(str,13);
			}
			else if (str.Left(11)=="SETUPDAC#14") {
				InterpretDACString(str,14);
			}
			else if (str.Left(11)=="SETUPDAC#15") {
				InterpretDACString(str,15);
			}
			else if (str.Left(11)=="SETUPDAC#16") {
				InterpretDACString(str,16);
			}
			else if (str.Left(11)=="SETUPDAC#17") {
				InterpretDACString(str,17);
			}
			else if (str.Left(11)=="SETUPDAC#18") {
				InterpretDACString(str,18);
			}
			else if (str.Left(11)=="SETUPDAC#19") {
				InterpretDACString(str,19);
			}
			else if (str.Left(11)=="SETUPDAC#20") {
				InterpretDACString(str,20);
			}
			else if (str.Left(11)=="SETUPDAC#21") {
				InterpretDACString(str,21);
			}
			else if (str.Left(11)=="SETUPDAC#22") {
				InterpretDACString(str,22);
			}
			else if (str.Left(11)=="SETUPDAC#23") {
				InterpretDACString(str,23);
			}
			else if (str.Left(11)=="SETUPDAC#24") {
				InterpretDACString(str,24);
			}

	return Identified;
}

void CInstrument::SetActiveSystem(int index)
{
	ActiveSystem = index;
	ThreadInfo.InstrumentCommands = CHANGE_SYSTEM;
	SetEvent(ThreadInfo.m_hProcessCommand);
}

void CInstrument::SetUserID(CString ID)
{
	sys.UserID = ID;
	sys.UserID = ID;
	sys.UserID = ID;
}


void CInstrument::StartTimer()
{
	// set TimeAcquired flag
	CTime t = CTime::GetCurrentTime();
	CString str = t.Format("%d-%m-%Y %H:%M:%S");
	int i;
	for(i=0; i<40; i++) PS.TimeAcquired[i] = '\0';
	for(i=0; i<39 && i<str.GetLength(); i++) {
		PS.TimeAcquired[i] = str.GetAt(i);
	}

	// calculate the number of seconds it will take to run a scan.
	float s=0, f=0, FirstEventStart=0, LastEventEnd=0;
	BOOL StartStopUndefined = TRUE;
	for(i=0; i<pDoc->PS.NumEvents; i++) {
		//	First, skip all the events that are in channels
		// that will not be plotted
		if(pDoc->PS.GenPS[i].Channel == EXCITE_DISABLE) continue;
		if(pDoc->PS.GenPS[i].Channel == EXCITE_SOURCE) continue;
		if(pDoc->PS.GenPS[i].Channel == RF_SOURCE) continue;
		if(pDoc->PS.GenPS[i].Channel == ULTIMA_SWITCH) continue;
		if(pDoc->PS.GenPS[i].Channel == RF_PHASE) continue;
		//if(pDoc->PS.GenPS[i].Channel == IMP_TRIGGER) continue;
		//if(pDoc->PS.GenPS[i].Channel == SYN_TRIGGER) continue;
		//if(pDoc->PS.GenPS[i].Channel == ARB_TRIGGER) continue;
		//if(pDoc->PS.GenPS[i].Channel == QUADRF_GATE) continue;
		if(pDoc->PS.GenPS[i].Channel == VSP_TRIGGER) continue;
		if(pDoc->PS.GenPS[i].Channel == DAC_GAIN) continue;
		if(pDoc->PS.GenPS[i].Type == DAC_INIT) continue;

		pDoc->GetEventEdges(i,&s,&f);
		if(StartStopUndefined) {
			FirstEventStart = s;
			LastEventEnd = f;
			StartStopUndefined = FALSE;
		}
		if(s<FirstEventStart) {
			FirstEventStart = s;
		}
		if(f>LastEventEnd) {
			LastEventEnd = f;
		}
	}
	// check to see if the dectection is actually the last thing
	// in time, if so, correct LastEventEnd for this
	pDoc->GetEventEdges(pDoc->PS.DetectEvent,&s,&f);
	float DetectEnd = f + (float)(1000*(double)pDoc->PS.TransientPoints/pDoc->PS.AdcRate); 
	if(LastEventEnd<DetectEnd) LastEventEnd = DetectEnd;

	scanduration = (LastEventEnd-FirstEventStart)/1000;
}

long CInstrument::ADCRateCode(double rate)
{
	int i;
	// maximum rate is 16 MHz

	if(rate > 16e6) rate = 16e6;

	for(i=12; i>0; i--) {
		if(rate <= sys.ADCRateList[i]) {
			rate = sys.ADCRateList[i];
			break;
		}
	}

	return long((16e6 / rate)-1.0);
}


void CInstrument::InterpretDACString(CString str, int dacnum)
{
	dacnum--;

	str = GetRightString(str);

	CString rightstr = CommaParse(str);
	str = CommaReturn(str);
	sscanf(rightstr,"%f",&(sys.dac[dacnum].MaxValue));
	rightstr = CommaParse(str);
	str = CommaReturn(str);
	sscanf(rightstr,"%f",&(sys.dac[dacnum].MaxDac));
	rightstr = CommaParse(str);
	str = CommaReturn(str);
	sscanf(rightstr,"%f",&(sys.dac[dacnum].MinValue));
	rightstr = CommaParse(str);
	str = CommaReturn(str);
	sscanf(rightstr,"%f",&(sys.dac[dacnum].MinDac));
	rightstr = CommaParse(str);
	str = CommaReturn(str);
	sscanf(rightstr,"%f",&(sys.dac[dacnum].MaxGraph));
	rightstr = CommaParse(str);
	str = CommaReturn(str);
	sscanf(rightstr,"%f",&(sys.dac[dacnum].MinGraph));
	//all that's left in str is what we want...
	sscanf(str,"%ld",&(sys.dac[dacnum].Units));
	if(sys.ChannelNames[16+dacnum] != "UNUSED") {
		sys.dac[dacnum].DacAvailable = 1;
		sys.DacAvailable[dacnum] = TRUE;
	} else {
		sys.dac[dacnum].DacAvailable = 0;
		sys.DacAvailable[dacnum] = FALSE;
	}
}

void CInstrument::FitCFP(long* pEvent, long *pPacket, double *pStartMass, double *pEndMass, double *pStartFreq, double *pEndFreq, long *pIncrFreq, long *pNumSteps)
{
	long packet, incrfreq, numsteps, event;
	double startmass, endmass, startfreq, endfreq;

	// make a copy of the input data. copy it back at the end.
	event = *pEvent;
	packet = *pPacket;
	startmass = *pStartMass;
	endmass = *pEndMass;
	startfreq = *pStartFreq;
	endfreq = *pEndFreq;
	incrfreq = *pIncrFreq;
	numsteps = *pNumSteps;

	if(packet < 0 || packet > MAX_SYN_TRIGGERS) return;

	// need volts at this time for calculation of cyclotron frequencies
	float start,finish;
	pDoc->GetEventEdges(event,&start,&finish);
	float volts = pDoc->DacValueAtTime(DAC_TRAP2 - 16, start);

	/*
	Ok, here's the explanation:

	for a frequency burst, since we have input a mass for the burst,
	not a frequency, we need to back calculate a resonance frequency
	for this mass, intergerize it (the hardware only accepts integer
	#'s of Hz) and then back calculate the adjusted burst mass.  

	for this resonance frequency calc, we need volts (above) and assume
	no total-ion-intensity shift.

	*/

	if(PS.cfp[packet].Type == CFP_BURST) {
		startmass = PS.cfp[packet].StartMass;

		startfreq = pDoc->MassToFreq(startmass);
		startfreq = pDoc->CyclotronFrequency(startfreq,volts,0,1, 1, -1);
		startfreq = int(startfreq + 0.5);

		endfreq = startfreq;
		startmass = pDoc->FreqToMass(pDoc->CyclotronFrequency(startfreq,volts,0,1,1,1));
		incrfreq = 0;
		numsteps = 0;
	}

	// for quad events, the start mass is actually a starting frequency.
	// don't fuck with it!
	else if(PS.cfp[packet].Type == CFP_QUAD) {
		incrfreq = 0;
		numsteps = 0;
		endmass = startmass;
	}

	// for sweeps, we need to do the same as with bursts, but do it with 
	// both start and end masses.
	else if(PS.cfp[packet].Type == CFP_SWEEP) {
		startmass = PS.cfp[packet].StartMass;
		endmass = PS.cfp[packet].EndMass;

		startfreq = pDoc->MassToFreq(startmass);
		startfreq = pDoc->CyclotronFrequency(startfreq,volts,0,1, 1, -1);
		startfreq = long(startfreq + 0.5);

		endfreq = pDoc->MassToFreq(endmass);
		endfreq = pDoc->CyclotronFrequency(startfreq,volts,0,1, 1, -1);
		endfreq = int(startfreq + 0.5);

		double range = fabs(endfreq - startfreq);
		bool scanup = TRUE;

		if(endmass > startmass) scanup = FALSE;

		numsteps = long(1000.0 * PS.GenPS[event].Width);
		incrfreq = long(range/double(numsteps) + 0.5);
		double availrange = double(incrfreq) * double(numsteps);

		long correction = long((range - availrange)/2.0);
		if(!scanup) {
			correction *= -1;
			availrange *= -1;
			incrfreq *= -1;
		}

		startfreq += correction;
		endfreq = startfreq + availrange;

		startmass = pDoc->FreqToMass(pDoc->CyclotronFrequency(startfreq,volts,0,1,1,1));
		endmass   = pDoc->FreqToMass(pDoc->CyclotronFrequency(endfreq,volts,0,1,1,1));
		
	}

	// for impulse trigger, not much to do, just clear things up.
	else if(PS.cfp[packet].Type == CFP_IMPTRIG) {
		startmass = 0;
		endmass = 0;
		startfreq = 0;
		endfreq = 0;
		incrfreq = 0;
		numsteps = 1;
	}

}

void CInstrument::BuildPS()
{
// variables. 
	long ArbActive = 65536;
	long SynActive = ArbActive * 2;
	long ImpActive = SynActive * 2;

	long RfPhaseCount = 0;
	//long PulseNumberByEvent;

	struct triplet triplt[800];
	long event[1600];
	long PulseTable[500];
	long TickTable[500];

	long bnc[16], ntrip, PulseWord, TickWord;
	long nevent, Time, RFchk, Impchk, Allchk, Newchk, Arbchk, Synchk, CurrentBits;
	long ierr, i,j,k,itmp, NumPulse;
	BOOL UsesRF, UsesImp, UsesArb, UsesSyn, HiRes;
	float start, finish, tpms;

	struct sys* pSys;
	struct PS* pPS;

	pSys = pDoc->sys;
	pPS = &(pDoc->PS);

	HiRes = pSys->BootESI | pSys->BootMALDI;

	// first program the dacs and generate all the vsp triggers
	ProgramVSP();

	tpms = float(pSys->PsgTicksPerMillisecond);

	// pulse bits are bits 0 - 16.  get their values.
	bnc[0]=1;
	for(i=1; i<16; i++) bnc[i] = bnc[i-1]*2;

	// build triplt table.  Ontime, offtime, pulseword
	//		note that times are expressed in pit ticks, use 
	//		pDoc->GetEventEdges() to get the start and stop times.
	//		
	//	cycle through all known events and create the appropriate triplet
	//	for that channel of event.
	long cfpcounter = 0;
	ntrip = 0;
	for(i=0; i<pPS->NumEvents; i++) { // cycle through GenPS events
		if(pPS->GenPS[i].Type == NO_EVENT) continue;
		if(pPS->GenPS[i].Channel == NO_EVENT_CHANNEL) continue;
		if(pPS->GenPS[i].Channel >= FIRST_DAC) continue; // already handled the dacs in ProgramVSP();
		if(pPS->GenPS[i].Channel < 0 || pPS->GenPS[i].Channel > 40) {
			MessageBox("Error in Pulse Sequence, invalid channel","Pulse Sequence Error!");
			ierr = -1;
			return;
		}
		
		pDoc->GetEventEdges(pPS, i, &start, &finish);

		if(HiRes && pPS->GenPS[i].Channel > USER1_PULSE  && pPS->GenPS[i].Channel != QUADRF_GATE) {
			// user definable pulses
			triplt[ntrip].timeon = int(start*tpms);
			triplt[ntrip].timeoff = int(finish*tpms);
			triplt[ntrip].bitpattern = bnc[pPS->GenPS[i].Channel];
			ntrip++;
		}

		if( pPS->GenPS[i].Channel == VSP_TRIGGER) {
			// vsp stuff, always make these triggers 2 us wide
			triplt[ntrip].timeon = int(start*tpms);
			triplt[ntrip].timeoff = triplt[ntrip].timeon +2;
			triplt[ntrip].bitpattern = VSP_TRIGGER_BITS;
			ntrip++;
		}

		if( pPS->GenPS[i].Channel == SYN_TRIGGER) {
			// syn trigger stuff, always make these triggers 2 us wide
			triplt[ntrip].timeon = int(start*tpms);
			triplt[ntrip].timeoff = triplt[ntrip].timeon +2;
			triplt[ntrip].bitpattern = SYN_TRIGGER_BITS;
			ntrip++;

			// rf bursts require 2 triggers, one to start, and one to stop.
			if(pPS->cfp[cfpcounter].Type == CFP_BURST || pPS->cfp[cfpcounter].Type == CFP_QUAD) {
				triplt[ntrip].timeon = int(finish*tpms);
				triplt[ntrip].timeoff = triplt[ntrip].timeon +2;
				triplt[ntrip].bitpattern = SYN_TRIGGER_BITS;
				ntrip++;
			}

			// for quadrf bursts, also program the quadrf gate bits.  allow 5 ms
			// on either side of these events for the relay to open/close
			if(pPS->cfp[cfpcounter].Type == CFP_QUAD) {
				triplt[ntrip].timeon = int((start - 5.0)*tpms);
				triplt[ntrip].timeoff = int((finish + 5.0)*tpms);
				triplt[ntrip].bitpattern = QUADRF_GATE_BITS;
				ntrip++;
			}
			
			// record a pseudo-pulse that is the full width of the event.
			if(pPS->cfp[cfpcounter].Type != CFP_IMPTRIG) {
				triplt[ntrip].timeon = int(start*tpms);
				triplt[ntrip].timeoff = int(finish*tpms);
				triplt[ntrip].bitpattern = SynActive;
				ntrip++;
			}

			// also, for the cylindrical cells, add an ExciteEnable event
			// (see below under excitedisable) to stop grounding the excite channel
			// remember to give the relay 5 ms settling time beforehand
			// note, for syn events, do not do this for cfp_quad's
			if(pSys->CellType == CYLINDRICAL) {
				if(pPS->cfp[cfpcounter].Type == CFP_BURST || pPS->cfp[cfpcounter].Type == CFP_SWEEP) {
					triplt[ntrip].timeon = int((start-5.0)*tpms);
					triplt[ntrip].timeoff = int(finish*tpms);
					triplt[ntrip].bitpattern = EXCITE_DISABLE_BITS;
					ntrip++;
				}
			}

			// finally, increment the cfpcounter
			cfpcounter++;
		}

		if( pPS->GenPS[i].Channel == ARB_TRIGGER) {
			// for arb triggers, always make them 2 us wide.
			triplt[ntrip].timeon = int(start*tpms);
			triplt[ntrip].timeoff = triplt[ntrip].timeon +2;
			triplt[ntrip].bitpattern = ARB_TRIGGER_BITS;
			ntrip++;
	
			// record a pseudo-pulse that is the full width of the event.
			triplt[ntrip].timeon = int(start*tpms);
			triplt[ntrip].timeoff = int(finish*tpms);
			triplt[ntrip].bitpattern = ArbActive;
			ntrip++;

			// also, for the cylindrical cells, add an ExciteEnable event
			// (see below under excitedisable) to stop grounding the excite channel
			// remember to give the relay 5 ms settling time beforehand
			if(pSys->CellType == CYLINDRICAL) {
				triplt[ntrip].timeon = int((start-5.0)*tpms);
				triplt[ntrip].timeoff = int(finish*tpms);
				triplt[ntrip].bitpattern = EXCITE_DISABLE_BITS;
				ntrip++;
			}
		}

		if( pPS->GenPS[i].Channel == IMP_TRIGGER) {
			// for arb triggers, always make them 2 us wide.
			triplt[ntrip].timeon = int(start*tpms);
			triplt[ntrip].timeoff = triplt[ntrip].timeon +2;
			triplt[ntrip].bitpattern = IMP_TRIGGER_BITS;
			ntrip++;
	
			// record a pseudo-pulse that is the full width of the event.
			triplt[ntrip].timeon = int(start*tpms);
			triplt[ntrip].timeoff = int(finish*tpms);
			triplt[ntrip].bitpattern = ImpActive;
			ntrip++;

			// also, for the cylindrical cells, add an ExciteEnable event
			// (see below under excitedisable) to stop grounding the excite channel
			// remember to give the relay 5 ms settling time beforehand
			if(pSys->CellType == CYLINDRICAL) {
				triplt[ntrip].timeon = int((start-5.0)*tpms);
				triplt[ntrip].timeoff = int(finish*tpms);
				triplt[ntrip].bitpattern = EXCITE_DISABLE_BITS;
				ntrip++;
			}
		}

		if( pPS->GenPS[i].Channel == EXCITE_DISABLE) {
			// handle excite-disable events
			// must also add excite disable events for each syn, arb, imp, quad event 
			//
			// cylindrical cell events are handled above, only cubic cell events will
			// be handled here.
			//
			// for the elongated-cubic cell, we need to disable the output to the transmitter
			// plates (via ExciteDisable) at all times except when we are applying an impulse
			// or signal to them.  The ExciteDisable Pulse drives a mechanical relay, so we
			// must remember to switch it 5 ms before we need it (and release it immediately
			// when we are done with it).
			//
			// for programming simplicity, we will actually program this initially inverted
			// as an ExciteEnable pulse and then invert it at the end of BuildPS, just before
			// actually programming hardware
			if(pSys->CellType != CYLINDRICAL) {
				triplt[ntrip].timeon = int(start*tpms);
				triplt[ntrip].timeoff = int(finish*tpms);
				triplt[ntrip].bitpattern = EXCITE_DISABLE_BITS;
				ntrip++;
			}
		}

	}	// end cycle through GenPS events

	// triplet table done, now build an event vector for each on, off time.
	nevent = 0;
	for(i=0; i<ntrip; i++) {
		event[nevent] = triplt[i].timeon;
		nevent++;

		event[nevent] = triplt[i].timeoff;
		nevent++;
	}

	// sort this event vector,
	for(i=1; i<nevent; i++) {
		if(event[i] < event[i-1]) {
			j = i;
			while(1) {
				itmp = event[j];
				event[j]=event[j-1];
				event[j-1] = itmp;
				j--;
				if(j<1 || event[j] > event[j-1]) break;
			}
		}
	}

	// remove redundancies in event vector
	for(i=1; i<nevent; i++) {
		if(event[i] == event[i-1]) {
			for(j=i; j<nevent; j++) event[j-1] = event[j];
			nevent--;
		}
	}
	event[nevent+1] = event[nevent];

	// add dummy event at time 0 if first event is before time = 0.
	if(event[0] > 0) {
		for(i=nevent+1; i>0; i--) {
			event[i+1] = event[i];
		}
		event[0] = 0;
		nevent++;
	}

	//  OK, now build the pulse and tick tables.
	// scan through the event vector (which listst all the times any pulse changes state)
	// for each event entry, find all the triplets for which pulse are currently on 
	// and use these to build the pulse word (the logical or of their bitpatterns)
	// tick count comes from the difference between this event vector and the next

	NumPulse = 0;
	PulseTable[NumPulse] = 0;
	TickTable[NumPulse] = 20;
	NumPulse++;
	for(i=0; i<nevent; i++) {
		Time = event[i];
		PulseWord = 0;

		for(j = 0; j<ntrip; j++) {
			if(triplt[j].timeon <= Time && triplt[j].timeoff >= Time) 
				PulseWord |= triplt[j].bitpattern;
		}

		TickWord = event[i+1] - Time;
		if (TickWord == 0) TickWord = 2;
		PulseTable[NumPulse] = PulseWord;
		TickTable[NumPulse] = TickWord;
	}
	// mark the end of the list;
	PulseTable[NumPulse] = 0;
	TickTable[NumPulse] = 0;
	NumPulse++;

	// now we must handle the Excite Source bit.  This bit selects whether
	// the output is from the RF (either syn or arb) or from the impulse generator.
	// 
	// again, this controls a relay, so we must remember to give it 5 ms to settle 
	// before use.  
	//
	// there are 3 cases: 1) only impulse events are used (so always set the excite 
	// source bit to impulse mode) 2) only rf pulses are used (do nothing, default for 
	// the excite source bit is to RF mode) 3) both impulse and RF used (toggle the state
	// between events)
	UsesRF = FALSE;
	UsesSyn = FALSE;
	UsesArb = FALSE;
	UsesImp = FALSE;

	for(i=0; i < pPS->NumEvents; i++) {
		if(pPS->GenPS[i].Type == NO_EVENT) continue;
		if(pPS->GenPS[i].Channel == NO_EVENT_CHANNEL) continue;
		if(pPS->GenPS[i].Channel >= FIRST_DAC) continue; // already handled the dacs in ProgramVSP();
		if(pPS->GenPS[i].Channel < 0 || pPS->GenPS[i].Channel > 40) {
			MessageBox("Error in Pulse Sequence, invalid channel","Pulse Sequence Error!");
			ierr = -1;
			return;
		}
	
		if(pPS->GenPS[i].Channel == IMP_TRIGGER) UsesImp = TRUE;
		if(pPS->GenPS[i].Channel == ARB_TRIGGER) UsesArb = TRUE;
		if(pPS->GenPS[i].Channel == SYN_TRIGGER) UsesSyn = TRUE;
		UsesRF = UsesArb + UsesSyn;
	}

	Allchk = Impchk = RFchk = 0;
	if(UsesImp) {
		if(!UsesRF) {
			// simple case (1) just .or. all pulsetable events with ExciteSourceBits
			for(i=0; i<NumPulse; i++) PulseTable[i] |= EXCITE_SOURCE_BITS;
		} else {
			// both rf and imp used (case 3) need to toggle it appropriately
			CurrentBits = 0;
			for(i=0; i<NumPulse;) {
				for(j=i; j<NumPulse; i++) {
					RFchk = PulseTable[j] & (SynActive | ArbActive);
					Impchk = PulseTable[j] & ImpActive;
					Allchk = PulseTable[j] & (SynActive | ArbActive | ImpActive);

					if(RFchk != 0) {
						CurrentBits = 0;
						break;
					}
					if(Impchk != 0) {
						CurrentBits = EXCITE_SOURCE_BITS;
						break;
					}
				}

				for(k=i ; k<j; k++) {
					PulseTable[k] |= CurrentBits;
				}
				i = j+1;

				while(i<NumPulse) {
					Newchk = PulseTable[i] & (SynActive | ArbActive | ImpActive);
					if(Newchk != Allchk) break;
					PulseTable[i] |= CurrentBits;
					i++;
				} // end while;
			} // end for;
		}//end if uses rf
	}// end if uses imp

	// now we must handle the selector relay for choosing between arb and syn
	// in the same way.  the default position is for there to be only syn (no arb) events
	Allchk = Arbchk = Synchk = 0;
	if(UsesArb) {
		if(!UsesSyn) {
			// simple case (1) just .or. all pulsetable events with ExciteSourceBits
			for(i=0; i<NumPulse; i++) PulseTable[i] |= RF_SOURCE_BITS;
		} else {
			// both rf and imp used (case 3) need to toggle it appropriately
			CurrentBits = 0;
			for(i=0; i<NumPulse;) {
				for(j=i; j<NumPulse; i++) {
					Arbchk = PulseTable[j] & ArbActive;
					Synchk = PulseTable[j] & SynActive;
					Allchk = PulseTable[j] & (SynActive | ArbActive);

					if(Arbchk != 0) {
						CurrentBits = RF_SOURCE_BITS;
						break;
					}
					if(Synchk != 0) {
						CurrentBits = 0;
						break;
					}
				}

				for(k=i ; k<j; k++) {
					PulseTable[k] |= CurrentBits;
				}
				i = j+1;

				while(i<NumPulse) {
					Newchk = PulseTable[i] & (SynActive | ArbActive);
					if(Newchk != Allchk) break;
					PulseTable[i] |= CurrentBits;
					i++;
				} // end while;
			} // end for;
		}//end if uses syn
	}// end if uses arb

	// now program the ultima switch.  used to be the RF phase bit.
	if(pSys->UltimaSwitch) {
		if(pSys->BootESI) {
			for(i=0; i< NumPulse; i++) {
				PulseTable[i] |= ULTIMA_SWITCH_BITS;
			}
		} else {
			// check for flip-flopping the phase.  if there is a phase
			// coherent noise in the spectra, flipflopping the phase 180 
			// degrees will cancel it out on signal averaging.  
			//
			// this isn't used very often.
			if(pSys->FlipFlopRFPhase && (RfPhaseCount%2) == 1) {
				for(i=0; i<NumPulse; i++) PulseTable[i] |= RF_PHASE_BITS;
			}
		}
	}

	// finally, invert the ExciteEnable making it into the real ExciteDisable
	if(pSys->CellType == CYLINDRICAL) {
		for(i=0; i<NumPulse; i++) {
			if((PulseTable[i] & EXCITE_DISABLE_BITS) == 0) PulseTable[i] |= EXCITE_DISABLE_BITS;
			else PulseTable[i] &= !EXCITE_DISABLE_BITS;
		}
	}

	// program PSG, FSP, ARB
	ProgramPSG(PulseTable, TickTable);
	if (pSys->FSPType == 1) ProgramFSP();
	if (pSys->FSPType == 2) ProgramFSP2();
	if (pSys->FSPType == 3) ProgramFSP3();
	ProgramARB();
}

void CInstrument::ProgramVSP()
{
	// variables
	long vspEOP = -1;
	long vspNOP = -2;

	float EventTime[1000];
	long DacValue[1000];
	long DacChannel[1000];

	long TrigPtr, i, j, Nstep;
	long Pointer, k;
	long DacC, LinesInPacket;
	long Address, DacV;
	float start, finish, Time, Step;
	float Scale, V, V1, Vstep, Vgain, Offset;

	V = V1 = 0;

	struct sys* pSys = pDoc->sys;
	struct PS* pPS = &(pDoc->PS);

	// end of variables

	// first step is to program the RFGain Dac for use with
	// the synthesizer and Arb events.  Fill in syn events first, then arbs

	// clear all GainDacEvents and set initial value (gain to off)
	PrepDac(DAC_GAIN, 1, float(+9.995), 2047, float(-10.000), -2048, float(+10), float(-10), float(pSys->GainControlVoltageDivider * 0.5));
	
	// synthesizer events.
	long cfpcounter = 0;
	if(pSys->HaveSynthesizer) {
		for(i=0; i < pPS->NumEvents; i++) {
			if (pPS->GenPS[i].Channel != SYN_TRIGGER) continue;
			if (pPS->GenPS[i].Type != TTL_PULSE) continue;
			pDoc->GetEventEdges(pPS, i, &start, &finish);

			if(pPS->cfp[cfpcounter].Type == CFP_QUAD) {
				Vgain = GainDacVoltage(pPS->cfp[cfpcounter].Amplitude, float(pSys->SynOutputVoltage), float(pSys->QuadRFFixedGain), float(pSys->GainControlVoltageDivider));
			} else {
				Vgain = GainDacVoltage(pPS->cfp[cfpcounter].Amplitude, float(pSys->SynOutputVoltage), float(pSys->RFAmpFixedGain), float(pSys->GainControlVoltageDivider));
			}
			// add dac event
			pPS->GenPS[pPS->NumEvents].Channel = DAC_GAIN;
			pPS->GenPS[pPS->NumEvents].Type = DAC_PULSE;
			pPS->GenPS[pPS->NumEvents].Delay = start;
			pPS->GenPS[pPS->NumEvents].Width = finish - start;
			pPS->GenPS[pPS->NumEvents].Link2Event = -1;
			pPS->GenPS[pPS->NumEvents].Linking = NO_LINK;
			pPS->GenPS[pPS->NumEvents].Value = Vgain;
			pPS->NumEvents++;
			cfpcounter++;
		}
	} // endif

	// arb events.
	long arbcounter = 0;
	if(pSys->HaveArbGenerator) {
		for(i=0; i < pPS->NumEvents; i++) {
			if (pPS->GenPS[i].Channel != ARB_TRIGGER) continue;
			if (pPS->GenPS[i].Type != TTL_PULSE) continue;
			pDoc->GetEventEdges(pPS, i, &start, &finish);

			Vgain = GainDacVoltage(pPS->arb[arbcounter].Amplitude, float(pSys->ArbOutputVoltage), float(pSys->RFAmpFixedGain), float(pSys->GainControlVoltageDivider));

			// add dac event
			pPS->GenPS[pPS->NumEvents].Channel = DAC_GAIN;
			pPS->GenPS[pPS->NumEvents].Type = DAC_PULSE;
			pPS->GenPS[pPS->NumEvents].Delay = start;
			pPS->GenPS[pPS->NumEvents].Width = finish - start;
			pPS->GenPS[pPS->NumEvents].Link2Event = -1;
			pPS->GenPS[pPS->NumEvents].Linking = NO_LINK;
			pPS->GenPS[pPS->NumEvents].Value = Vgain;
			pPS->NumEvents++;
			arbcounter++;
		}	
	}	// end havearb

	Pointer = 0;

	for(k = 0; k < MAX_DACS; k++) {
		Scale = float(pPS->dac[k].MaxDac - pPS->dac[k].MinDac) / (pPS->dac[k].MaxValue - pPS->dac[k].MinValue);
		Offset = float(pPS->dac[k].MaxDac) - Scale*pPS->dac[k].MaxValue;
		for(i=0; i<pPS->NumEvents; i++) {
			// ignore unused events and init events
			if(pPS->GenPS[i].Channel != FIRST_DAC + k) continue;
			if(pPS->GenPS[i].Type == NO_EVENT) continue;
			pDoc->GetEventEdges(pPS,i,&start,&finish);
			if(pPS->GenPS[i].Type == DAC_INIT) V = pPS->GenPS[i].Value;

			// step events get a single trigger
			else if(pPS->GenPS[i].Type == DAC_STEP) {
				EventTime[Pointer] = start;
				DacChannel[Pointer] = k;
				DacValue[Pointer] = ValueToDac(pPS->GenPS[i].Value,Scale,Offset);
				Pointer++;
				V = pPS->GenPS[i].Value;
			}
			// pulse events get 2 triggers, one to start, one to stop.
			else if(pPS->GenPS[i].Type == DAC_PULSE) {
				EventTime[Pointer] = start;
				DacChannel[Pointer] = k;
				DacValue[Pointer] = ValueToDac(pPS->GenPS[i].Value,Scale,Offset);
				Pointer++;

				EventTime[Pointer] = finish;
				DacChannel[Pointer] = k;
				DacValue[Pointer] = ValueToDac(V,Scale,Offset);
				Pointer++;
			}
			// ramp events get multiple triggers
			else if(pPS->GenPS[i].Type == DAC_RAMP) {
				Nstep = 40;
				Step = float(int((finish-start)*100.0)/float(Nstep)*10);

				if(Step < 0.01) {
					Step = float(0.01);
					Nstep = long((finish -start) / Step);
				}
				Vstep = (pPS->GenPS[i].Value-V) / float(Nstep);

				for(j=0; j < Nstep; j++) {
					Time = start + float(j) * Step;
					V1 = V + float(j) * Vstep;
					if(j == Nstep) V1 = pPS->GenPS[i].Value;

					EventTime[Pointer] = Time;
					DacChannel[Pointer] = k;
					DacValue[Pointer] = ValueToDac(V1,Scale,Offset);
					Pointer++;
				}
				V = pPS->GenPS[i].Value;
			} // end if DAC_RAMP
		} // end for i 
	} // end for k

	
	// now sort the work vectors by time.
	Pointer--;
	for(i=0; i<Pointer; i++) {
		if(EventTime[i] < EventTime[i-1]) {
			j=i;
			while(1) {
				Time = EventTime[j];
				DacC = DacChannel[j];
				DacV = DacValue[j];

				EventTime[j] = EventTime[j-1];
				DacChannel[j] = DacChannel[j-1];
				DacValue[j] = DacValue[j-1];

				EventTime[j-1] = Time;
				DacChannel[j-1] = DacC;
				DacValue[j-1] = DacV;
				j--;
				if(j < 1) break;
				if(EventTime[j] > EventTime[j-1]) break;
			}// end while
		}// end if
	}// end for

	// now program the vsp memory and at the same time, fill in the
	// vsp trigger pulses for BuildPS.  Note that there is a minimum
	// VSP packet size of three programming points.  NOPS are added
	// as needed.

	// first, clear the whole event list.
	for(i=0; i<pPS->NumEvents; i++) {
		if(pPS->GenPS[i].Channel != VSP_TRIGGER) continue;
		
		pPS->GenPS[i].Channel = NO_CHANNEL;
		pPS->GenPS[i].Type = NO_EVENT;
		pPS->GenPS[i].Linking = NO_LINK;
		pPS->GenPS[i].Link2Event = 0;
		pPS->GenPS[i].Delay = 0;
		pPS->GenPS[i].Width = 0;
		pPS->GenPS[i].Value = 0;
	} // end for

	Time = EventTime[0];
	TrigPtr = 0;
	LinesInPacket = 0;
	Address = 0;

	// first program all the DAC initial values.
	BOOL FoundInit = FALSE;
	for(i=0; i< MAX_DACS; i++) {
		for(j=0; j<pPS->NumEvents; j++) {
			if(pPS->GenPS[j].Channel != i + FIRST_DAC) continue;
			if(pPS->GenPS[j].Type == DAC_INIT) {
				FoundInit = TRUE;
				break;
			}
		}

		if(!FoundInit) {
			CString str;
			str.Format("Couldn't find a DAC_INIT event for DAC #%d",i);
			MessageBox(str,"PulseSequence Error");
			return;
		}

		Scale = (pPS->dac[i].MaxDac - pPS->dac[i].MinDac) / (pPS->dac[i].MaxValue - pPS->dac[i].MinValue);
		Offset = float(pPS->dac[i].MaxDac) - Scale*pPS->dac[i].MaxValue;
		V = pPS->GenPS[j].Value;

		ProgramVSPword(&Address, i, ValueToDac(V, Scale, Offset));
	}// end program dac init's

	// now program the rest.
	for(i=0; i<Pointer; i++) {
		// check for a change in time.  this indicates an end to a packet
		// check that the packet is long enough.  write a stop command.
		if(EventTime[i] > Time) {
			for(j=LinesInPacket+1; i<3; i++) {
				ProgramVSPword(&Address, vspNOP, 0);
			}
			LinesInPacket = 0;
			ProgramVSPword(&Address, vspEOP, 0);

			// create a VSP_TRIGGER event for this VSP packet.
			pPS->GenPS[pPS->NumEvents].Channel = VSP_TRIGGER;
			pPS->GenPS[pPS->NumEvents].Type = TTL_PULSE;
			pPS->GenPS[pPS->NumEvents].Linking = NO_LINK;
			pPS->GenPS[pPS->NumEvents].Link2Event = 0;
			pPS->GenPS[pPS->NumEvents].Delay = Time;
			pPS->GenPS[pPS->NumEvents].Width = float(0.005); // 5 microseconds
			pPS->GenPS[pPS->NumEvents].Value = 0;
			pPS->NumEvents++;
		}// end if

		// program the newest voltage
		ProgramVSPword(&Address, DacChannel[i], DacValue[i]);
		Time = EventTime[i];
		LinesInPacket++;
	} // end for

	// finish off the "last" packet
	for(j=LinesInPacket+1; j<3; j++) {
		ProgramVSPword(&Address, vspNOP, 0);
	}
	LinesInPacket = 0;
	ProgramVSPword(&Address, vspEOP, 0);
	pPS->GenPS[pPS->NumEvents].Channel = VSP_TRIGGER;
	pPS->GenPS[pPS->NumEvents].Type = TTL_PULSE;
	pPS->GenPS[pPS->NumEvents].Linking = NO_LINK;
	pPS->GenPS[pPS->NumEvents].Link2Event = 0;
	pPS->GenPS[pPS->NumEvents].Delay = Time;
	pPS->GenPS[pPS->NumEvents].Width = float(0.005); // 5 microseconds
	pPS->GenPS[pPS->NumEvents].Value = 0;
	pPS->NumEvents++;

	//flag.VSPloaded = TRUE
	flags.VSPLoaded = TRUE;
}

void CInstrument::ProgramFSP()
{
	//double cfreq;
	long Packet, Address, i, NumSteps, IncrFreq;
	double f1,f2,StartFreq, EndFreq, StartMass, EndMass;
	BOOL ZeroPhase, FreqValid, EndSeq, Trigger;

	struct sys* pSys = pDoc->sys;
	struct PS* pPS = &(pDoc->PS);

	if(!pSys->HaveSynthesizer) {
		flags.FSPLoaded = TRUE;
		return;
	}

	// scan through the synthesizer trigger events, programming the FSP 
	// for each valid one.
	Address = 0;
//	fspadr(&Address);

	// record a simple FTP preset program to clear the phase and attenuator
	// registers and to set zero phase
	f1 = 0.0;
	FreqValid = FALSE;
	EndSeq = FALSE;
	Trigger = FALSE;
	ZeroPhase = TRUE;
	ProgramFSPword(&Address, f1, ZeroPhase,FreqValid,EndSeq, Trigger);
	EndSeq = TRUE;
	ProgramFSPword(&Address, f1, ZeroPhase,FreqValid,EndSeq, Trigger);

	Packet = 0; 
	long j;
	for(i=0; i<pPS->NumEvents; i++) {
		if(pPS->GenPS[i].Channel != SYN_TRIGGER) continue;
		
		FitCFP(&i, &Packet, &StartMass, &EndMass, &StartFreq, &EndFreq, &IncrFreq, &NumSteps);

		if(pPS->GenPS[i].Type == TTL_PULSE) {
			if(pPS->cfp[Packet].Type == CFP_BURST) {
				// packet type is an RF burst.  requires 2 programs: first to turn
				// on thr rf, and a second to turn it off again.  if this is the detect
				// rf and we are in MixerMode, we need to program the mixer frequency
				// last and not turn off the rf at the end.

				f1 = StartFreq;
				// first, turn on the pulse of RF
				FreqValid = TRUE;
				EndSeq = FALSE;
				Trigger = FALSE;
				ZeroPhase = TRUE;
				ProgramFSPword(&Address, f1, ZeroPhase,FreqValid,EndSeq, Trigger);

				EndSeq = TRUE;
				ProgramFSPword(&Address, f1, ZeroPhase,FreqValid,EndSeq, Trigger);

				// second, turn off the pulse of RF
				FreqValid = FALSE;
				EndSeq = FALSE;
				Trigger = FALSE;
				ZeroPhase = TRUE;
				if(pPS->DetectEvent == i) {
					Trigger = TRUE;
					if(pPS->DetectMode == DIRECT_MODE) {
						f1 = pDoc->MassToFreq(pPS->MonitorMass) - 1.0;
					} else {
						f1 = pPS->MixerFrequency;
					}
					FreqValid = TRUE;
					ZeroPhase = FALSE;
				} // end if detectevent
				ProgramFSPword(&Address, f1, ZeroPhase,FreqValid,EndSeq, Trigger);

				FreqValid = FALSE;
				EndSeq = TRUE;
				Trigger = FALSE;
				ProgramFSPword(&Address, f1, ZeroPhase,FreqValid,EndSeq, Trigger);
			} else if(pPS->cfp[Packet].Type == CFP_QUAD) { 
				// packet is a quadrf burst.  requires 2 programs: first to
				// turn on the rf, second to turn it off again.
				f1 = pPS->cfp[Packet].StartMass;

				// first, turn on the pulse of RF
				FreqValid = TRUE;
				EndSeq = FALSE;
				Trigger = FALSE;
				ZeroPhase = TRUE;
				ProgramFSPword(&Address, f1, ZeroPhase,FreqValid,EndSeq, Trigger);

				FreqValid = TRUE;
				ZeroPhase = FALSE;
				ProgramFSPword(&Address, f1, ZeroPhase,FreqValid,EndSeq, Trigger);

				EndSeq = TRUE;
				ProgramFSPword(&Address, f1, ZeroPhase,FreqValid,EndSeq, Trigger);

				// second, turn off the pulse of RF
				FreqValid = FALSE;
				EndSeq = FALSE;
				Trigger = FALSE;
				ZeroPhase = TRUE;
				ProgramFSPword(&Address, f1, ZeroPhase,FreqValid,EndSeq, Trigger);

				FreqValid = FALSE;
				EndSeq = TRUE;
				Trigger = FALSE;
				ProgramFSPword(&Address, f1, ZeroPhase,FreqValid,EndSeq, Trigger);
			} else if(pPS->cfp[Packet].Type == CFP_SWEEP) {
				// packet is an rf sweep.  Requires a longer program to set all
				// the frequenices over the specified range.  If this is the detect
				// rf and we are in Mixer Mode, we need to program the mixer 
				// frequency last and not turn off the rf at the end.

				f1 = StartFreq;

				FreqValid = TRUE;
				EndSeq = FALSE;
				Trigger = FALSE;
				ZeroPhase = TRUE;
				ProgramFSPword(&Address, f1, ZeroPhase,FreqValid,EndSeq, Trigger);

				ZeroPhase = FALSE;
				for(j=0; j<NumSteps; j++) {
					f2 = f1 + double(IncrFreq) * double(j);
					ProgramFSPword(&Address, f2, ZeroPhase,FreqValid,EndSeq, Trigger);
				}

				FreqValid = FALSE;
				EndSeq = FALSE;
				Trigger = FALSE;
				ZeroPhase = TRUE;
				if(pPS->DetectEvent == i) {
					Trigger = TRUE;
					if(pPS->DetectMode == DIRECT_MODE) 
						f1 = pDoc->MassToFreq(pPS->MonitorMass) - 1;
					else
						f1 = pPS->MixerFrequency;
					FreqValid = TRUE;
					ZeroPhase = FALSE;
				}
				ProgramFSPword(&Address, f1, ZeroPhase,FreqValid,EndSeq, Trigger);
				FreqValid = FALSE;
				EndSeq = TRUE;
				Trigger = FALSE;
				ProgramFSPword(&Address, f1, ZeroPhase,FreqValid,EndSeq, Trigger);
			} else if(pPS->cfp[Packet].Type == CFP_IMPTRIG) {
				// Packet is an impulse trigger.  only when this is the detect
				// event.  if we are in mixer mode, we need to program the mixer
				// frequency and not turn off the RF at the end.
				f1 = 0.0;

				FreqValid = FALSE;
				EndSeq = FALSE;
				Trigger = TRUE;
				ZeroPhase = TRUE;
				if(pPS->DetectMode == DIRECT_MODE) 
					f1 = pDoc->MassToFreq(pPS->MonitorMass) - 1;
				else
					f1 = pPS->MixerFrequency;
				FreqValid = TRUE;
				ZeroPhase = FALSE;
				ProgramFSPword(&Address, f1, ZeroPhase,FreqValid,EndSeq, Trigger);

				FreqValid = FALSE;
				EndSeq = TRUE;
				Trigger = FALSE;
				ProgramFSPword(&Address, f1, ZeroPhase,FreqValid,EndSeq, Trigger);
			} // end if
		} // end if
		Packet++;
	} // end event loop.
}	// end programFSP

void CInstrument::ProgramFSP2()
{
	//double cfreq;
	long Packet, Address, i, j, NumSteps, IncrFreq;
	double f1,f2,StartFreq, EndFreq, StartMass, EndMass;
	//BOOL ZeroPhase, FreqValid, EndSeq, Trigger;

	struct sys* pSys = pDoc->sys;
	struct PS* pPS = &(pDoc->PS);

	if(!pSys->HaveSynthesizer) {
		flags.FSPLoaded = TRUE;
		return;
	}

	Address = 0;
//	fsprst();
//	fspadr(&Address);

	ProgramFSPzerophase(&Address);
	ProgramFSPend(&Address);

	Packet = 0;
	for(i=0; i<pPS->NumEvents; i++) {
		if(pPS->GenPS[i].Channel != SYN_TRIGGER) continue;
		
		FitCFP(&i, &Packet, &StartMass, &EndMass, &StartFreq, &EndFreq, &IncrFreq, &NumSteps);

		if(pPS->GenPS[i].Type == TTL_PULSE) {
			if(pPS->cfp[Packet].Type == CFP_BURST) {
				// packet type is an RF burst.  requires 2 programs: first to turn
				// on thr rf, and a second to turn it off again.  if this is the detect
				// rf and we are in MixerMode, we need to program the mixer frequency
				// last and not turn off the rf at the end.

				f1 = StartFreq;
				// first, turn on the pulse of RF
				ProgramFSPzerophase(&Address);
				ProgramFSPfrequency(&Address,f1,FALSE);
				ProgramFSPfrequency(&Address,f1,TRUE);
				ProgramFSPend(&Address);

				// second turn it off
				if(pPS->DetectEvent == i) {
					if(pPS->DetectMode == DIRECT_MODE) {
						f1 = pDoc->MassToFreq(pPS->MonitorMass) - 1000.0;
						if(flags.scopeMode || !pSys->HaveNBMonitor) 
							ProgramFSPfrequency(&Address,f1,FALSE);
						else 
							ProgramFSPzerophase(&Address);
					} else {
						f1 = pPS->MixerFrequency * (1 + pSys->MixerCorrection/1e6);
						ProgramFSPfrequency(&Address, f1, FALSE);
					}
					ProgramFSPdelaytrigger(&Address);
				} else 
					ProgramFSPzerophase(&Address);
				ProgramFSPend(&Address);
			} else if (pPS->cfp[Packet].Type == CFP_QUAD) {
				// packet is a quadrf burst.  requires 2 programs: first to
				// turn on the rf, second to turn it off again.
				f1 = pPS->cfp[Packet].StartMass;

				// first, turn on the pulse of RF
				ProgramFSPzerophase(&Address);
				ProgramFSPfrequency(&Address, f1, FALSE);
				ProgramFSPfrequency(&Address, f1, TRUE);
				ProgramFSPend(&Address);

				// second, turn it off.
				ProgramFSPzerophase(&Address);
				ProgramFSPend(&Address);
			} else if (pPS->cfp[Packet].Type == CFP_SWEEP) {
				// packet is an rf sweep.  Requires a longer program to set all
				// the frequenices over the specified range.  If this is the detect
				// rf and we are in Mixer Mode, we need to program the mixer 
				// frequency last and not turn off the rf at the end.

				f1 = StartFreq;
				ProgramFSPzerophase (&Address);
				ProgramFSPfrequency (&Address, f1, FALSE);
				ProgramFSPfrequency (&Address, f1, TRUE);

				for(j = 0; j < NumSteps; j++) {
					f2 = f1 + double(IncrFreq) * double(j+1);
					ProgramFSPfrequency(&Address, f2, FALSE);
				}

				if(pPS->DetectEvent == i) {
					if(pPS->DetectMode == DIRECT_MODE) {
						f1 = pDoc->MassToFreq(pPS->MonitorMass) - 1000.0;
						if(flags.scopeMode || !pSys->HaveNBMonitor) 
							ProgramFSPfrequency(&Address,f1,FALSE);
						else 
							ProgramFSPzerophase(&Address);
					} else {
						f1 = pPS->MixerFrequency * (1 + pSys->MixerCorrection/1e6);
						ProgramFSPfrequency(&Address, f1, FALSE);
					}
					ProgramFSPdelaytrigger(&Address);
				} else 
					ProgramFSPzerophase(&Address);
				ProgramFSPend(&Address);
			} else if(pPS->cfp[Packet].Type == CFP_IMPTRIG) {
				// Packet is an impulse trigger.  only when this is the detect
				// event.  if we are in mixer mode, we need to program the mixer
				// frequency and not turn off the RF at the end.
				ProgramFSPzerophase(&Address);
				if(pPS->DetectMode == DIRECT_MODE) {
					f1 = pDoc->MassToFreq(pPS->MonitorMass) - 1000.0;
					if(flags.scopeMode || !pSys->HaveNBMonitor) 
						ProgramFSPfrequency(&Address,f1,FALSE);
					else 
						ProgramFSPzerophase(&Address);
				} else {
					f1 = pPS->MixerFrequency * (1 + pSys->MixerCorrection/1e6);
					ProgramFSPfrequency(&Address, f1, FALSE);
				}
				ProgramFSPdelaytrigger(&Address);
				ProgramFSPend(&Address);
			} // end if packet type
		} //end if event type
	}// end loop thru events
	flags.FSPLoaded = TRUE;
}

void CInstrument::ProgramFSPzerophase(long *Address)
{
	// bitpattern constants
	//long FSP_FRQ  = 0x08000000;
	//long FSP_END  = 0x10000000;
	//long FSP_ZNO  = 0x20000000;
//	long FSP_ZYES = 0x40000000;
	//long FSP_IMMT = 0x80000000;
	//long FSP_DLYT = 0xc0000000;
//	long number = 1;

//	wrf2mem(Address, &number, &FSP_ZYES);
	(*Address)++;
}

void CInstrument::ProgramFSPend(long *Address)
{
	// bitpattern constants
	//long FSP_FRQ  = 0x08000000;
//	long FSP_END  = 0x10000000;
	//long FSP_ZNO  = 0x20000000;
	//long FSP_ZYES = 0x40000000;
	//long FSP_IMMT = 0x80000000;
	//long FSP_DLYT = 0xc0000000;
//	long number = 1;

//	wrf2mem(Address, &number, &FSP_END);
	(*Address)++;

}

void CInstrument::ProgramFSPdelaytrigger(long *Address)
{
	// bitpattern constants
	//long FSP_FRQ  = 0x08000000;
	//long FSP_END  = 0x10000000;
	//long FSP_ZNO  = 0x20000000;
	//long FSP_ZYES = 0x40000000;
	//long FSP_IMMT = 0x80000000;
//	long FSP_DLYT = 0xc0000000;
//	long number = 1;

//	wrf2mem(Address, &number, &FSP_DLYT);
	(*Address)++;
}

void CInstrument::ProgramFSPfrequency(long *Address, double f, BOOL ReleaseZeroPhase)
{
	// bitpattern constants
	long FSP_FRQ  = 0x08000000;
	//long FSP_END  = 0x10000000;
	long FSP_ZNO  = 0x20000000;
	//long FSP_ZYES = 0x40000000;
	//long FSP_IMMT = 0x80000000;
	//long FSP_DLYT = 0xc0000000;
//	long number = 1;

	long frq, wrd;

	wrd = 0;
	frq = long(f+0.5);
	bcd32(&frq,&wrd);
	wrd += FSP_FRQ;
	if(ReleaseZeroPhase) wrd += FSP_ZNO;

//	wrf2mem(Address, &number, &wrd);
	(*Address)++;
}

void CInstrument::ProgramFSP3()
{

}

void CInstrument::ProgramFSPword(long *Address, double f, BOOL ZeroPhase, BOOL FreqValid, BOOL EndSeq, BOOL Trigger)
{
	// bitpattern constants
	long FSP_FRQ  = 0x0800;
	long FSP_ATN  = 0x1000;
	long FSP_PH1  = 0x2000;
	long FSP_END  = 0x4000;
	long FSP_DLYT = 0x8000;
	//long FSP_IMMT = 0x8000;
	long FSP_PH2  = 0x2000;
	long FSP_ZYES = 0x4000;
	//long FSP_ZNO  = 0x0000;

	//int wrd[2];
	long wrd0, wrd1, wrd2, wrd3, frq;

	wrd0 = 0;
	wrd1 = 0;
	wrd2 = 0;
	wrd3 = 0;
	long wrd;

	if(FreqValid) {
		frq = long(f + 0.5);
		bcd32(&frq,&wrd);
		wrd0 = wrd & 0xff00;
		wrd1 = wrd & 0x00ff;
	}

	wrd1 += FSP_PH1;	// enable zphase check
	wrd1 += FSP_ATN;	// load zero attenuation
	wrd2 += FSP_PH2;	// load the no phase shift
	if(ZeroPhase) wrd2 += FSP_ZYES;		// set/reset zphase bit
	if(FreqValid) wrd1 += FSP_FRQ;		// program frequency
	if(EndSeq) wrd1 += FSP_END;			// stop command
	if(Trigger)	wrd1 += FSP_DLYT;		// Delayed TD trigger

//	wrfmem4(&wrd0,&wrd1,&wrd2,&wrd3);
	*(Address)++;
}

void CInstrument::ProgramARB()
{

}

void CInstrument::ProgramPSG(long *PulseTable, long *TickTable)
{
	long MaxTick = 32767;
	long pointer, pulse, count, address,tick,i,ng;

	// scan through the pulse and tick tables and putput the information to 
	// the PSG hardware.  Note that when a tick count exceeds 32767, 
	// it must be broken into multiple words since the PSG tick memory
	// is only 16 bits wide.

	address = 0;
//	psgadr(&address);

	pointer = 0;

	while(1) {
		pulse = PulseTable[pointer];
		count = TickTable[pointer];
		if(count < MaxTick) { //wrpsg(&pulse,&count);
		}
		else {
			ng = count/MaxTick;
			if((count%MaxTick) > 0) ng++;
			for(i=0; i<ng; i++) {
				tick = MaxTick;
				if (i == ng) tick = count%MaxTick;
				if(tick == 0) tick = MaxTick;
//				wrpsg(&pulse,&tick);
			}// end for
		} // end else if
		if(count == 0) break;
		pointer++;
	}
	flags.PSGLoaded = TRUE;
}

long CInstrument::ValueToDac(float Value, float Scale, float Offset)
{
	// calculates the long integer value from a voltage.  
	// e.g. changing 5.4 volts into -2048...2047 scale for a 12 bit dac.
	if(Value >= 0) return long(Value*Scale + Offset + 0.5);
	else return long(Value*Scale + Offset - 0.5);
}

void CInstrument::ProgramVSPword(long *Addr, long Channel, long DACvalue)
{
	long vspEOP = -1;
	long vspNOP = -2;

	long DACvsp, Chip, SubChannel;

	// -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
	// VSP memory is 24 bits wide.  the format is:
	//
	// -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
	//
	//	bits  0-11: DAC value (2's complement in 12 bits)
	//		 12-13: DAC subunit number (0-3)
	//		    14: EOP (end of program)
	//			15: (reserved)
	//			16: Use subunit in bits 12-13 of dac#0
	//			17: Use subunit in bits 12-13 of dac#1
	//			18: Use subunit in bits 12-13 of dac#2
	//			19: Use subunit in bits 12-13 of dac#3
	//			20: Use subunit in bits 12-13 of dac#4
	//			21: Use subunit in bits 12-13 of dac#5
	//			22: Load DAC(s) with bits 0-11 (LDDAC)
	//			23: (reserved)
	//
	// note: multiple bits 16-21 can be set, but all will reference
	//		 the same subunit number specified in bits 12-13
	// -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-

	long i;
//	long numelements = 1;
//	long EOP = 0x4000;

	if(Channel == vspEOP) { //wrvmem(Addr, &numelements, &EOP);
	}
	else if(Channel == vspNOP) {
		DACvsp = 0;
//		wrvmem(Addr, &numelements, &DACvsp);
	}
	else {
		SubChannel = (long(Channel) & 3) * 0x1000;
		Chip = 1;
		for(i=0; i<Channel/4; i++) Chip *= 2;
		DACvsp = DACvalue & 0x0fff + SubChannel + Chip;
//		wrvmem(Addr,&numelements, &DACvsp);
	}
	(*Addr)++;
}

void CInstrument::PrepDac(long DacNumber, long Units, float vMax, long codeMax, float vMin, long codeMin, float dispMax, float dispMin, float Vinit)
{
	//long FirstEvent, MaxEvents, Event, ne, ierr;
	long i;

	struct sys* pSys = pDoc->sys;
	struct PS* pPS = &(pDoc->PS);

	pSys->dac[DacNumber].Units = Units;
	pSys->dac[DacNumber].MaxValue = vMax;
	pSys->dac[DacNumber].MinValue = vMin;
	pSys->dac[DacNumber].MaxDac = float(codeMax);
	pSys->dac[DacNumber].MinDac = float(codeMin);
	pSys->dac[DacNumber].MaxGraph = dispMax;
	pSys->dac[DacNumber].MinGraph = dispMin;
	pSys->dac[DacNumber].DacAvailable = TRUE;

	pPS->dac[DacNumber] = pSys->dac[DacNumber];

	for(i=0; i<pPS->NumEvents; i++) {
		if(pPS->GenPS[i].Channel != DacNumber + FIRST_DAC) continue;
		pPS->GenPS[i].Channel = NO_CHANNEL;
		pPS->GenPS[i].Type = NO_EVENT;
		pPS->GenPS[i].Linking = NO_LINK;
		pPS->GenPS[i].Link2Event = 0;
		pPS->GenPS[i].Delay = 0;
		pPS->GenPS[i].Width = 0;
		pPS->GenPS[i].Value = 0;
	}

	// add dac channel
	pPS->GenPS[pPS->NumEvents].Channel = DacNumber + FIRST_DAC;
	pPS->GenPS[pPS->NumEvents].Type = DAC_INIT;
	pPS->GenPS[pPS->NumEvents].Delay = -100;
	pPS->GenPS[pPS->NumEvents].Width = 0;
	pPS->GenPS[pPS->NumEvents].Value = Vinit;
	pPS->GenPS[pPS->NumEvents].Linking = NO_LINK;
	pPS->GenPS[pPS->NumEvents].Link2Event = 0;
	pPS->NumEvents++;
}

float CInstrument::GainDacVoltage(float Vout, float Vin, float FixedGain, float Divisor)
{
	// =============================================================================
	// the variable gain devince int he rf amplifier is programmed with a 
	// small voltage (slightly positive for -60 db to about -2V for maximum gain.
	//  the equation is:
	//			G(v/v) = 10**[-2*(Vctl+1)]
	// where G(v/v) = voltage gain factor
	//		  Vctl = control voltage
	//
	//  experiments have shown good linearit for Vctl from 0.0 V to -1.3 V.  
	// or gains from 0.010 to 3.981.  we will therefore limit gains to 4.
	// 
	//
	// the dac will actually program a voltage that is "Divisor" times larger
	// than Vctl to better utilize the range and resolution of the dac.  This
	// output voltage is divided down by a resistor network in the amplifier.
	//
	// the amplifer also contains a fixed gain portion providing X (fixedGain)
	// voltage amplification.  the signal into the amplifier box is Vin.
	// =============================================================================
	float VariableGainNeeded, GainControlVoltage;

	VariableGainNeeded = Vout / (Vin * FixedGain);
	if(VariableGainNeeded > 0.001) {
		GainControlVoltage = float(-0.5*log10(VariableGainNeeded) - 1.0);
	} else {
		GainControlVoltage = float(0.5);
	}
	return Divisor * GainControlVoltage;
}


void CInstrument::bcd32(long *BinWord, long *BcdWord)
{
	long locBin, i, BCDdigits[8];

	locBin = *BinWord;
	if(locBin < 0) locBin = 0;
	if(locBin > 99999999) locBin = 99999999;

	for(i=7; i>=0; i--) {
		BCDdigits[i] = locBin % 10;
		locBin = locBin / 10;
	}

	*BcdWord = 0;
	for(i=0; i<8; i++) {
		*BcdWord = *BcdWord * 16 + BCDdigits[i];
	}
}
